unit FILTERS; {Unit 13 of NEWIDEOLOG.PAS}

interface

  uses
    GLOBALS, MATHS, UTILS, POLYNOME, LEGENDS, SCREEN, POST, Graph, Dos, Crt;

  procedure Laurent (var mu, gamma: vector;
                  q: integer);

  procedure InverseRootsToCoeffs (p: integer;
                  var mu, alpha: complexVector);

  procedure ButterworthDigitalPoles (n: integer;
                  digitalCut: real;
                  var poles: complexVector);

  procedure ButterworthDigitalZeros (n: integer;
                  var zeros: complexVector);

  procedure SeasonalPoles (var s: integer;
                  var rho: real;
                  var poles: complexVector);

  procedure SeasonalZeros (var s: integer;
                  var zeros: complexVector);

  function FIRFilterGain (omega: real;
                  var filterSpec: FilterRecord;
                  var firVec: vector): real;

  function DigitalButterGain (omega: real): real;

  function HPGain (omega, lambda: real): real;

  function SeasFilterGain (omega: real;
                  s: integer;
                  var numerator, denominator: vector): real;
{phi => numerator, chi => denominator}

  procedure PrescottFilter (lambda: real;
                  var gamma, delta: vector;
                  filterSpec: filterRecord);

  procedure CombinedCoefficients (var lambda, rho, factor: real;
                  var s: integer;
                  var numerator, denominator: vector);
{phi => numerator, chi => denominator}

  procedure DiffCoefficients (var alpha: vector;
                  d: integer);

  procedure DeltaPrimeY (var y: longVector;
                  var Tcap: integer;
                  d: integer);

  procedure DeltaY (var y: longVector;
                  var Tcap: integer;
                  d: integer);

  procedure FormGamma (var gamma: vector;
                  mu: vector;
                  r: integer);

  procedure SumDifference (var theta: vector;
                  n: integer;
                  sign: integer);

  procedure FormMvec (var mVec: vector;
                  n: integer);

  procedure FormSvec (var sVec: vector;
                  n: integer);

  procedure FormWvec (var wVec: vector;
                  n: integer;
                  lamOne, lamTwo: real);

  procedure QprimeY (var y: longVector;
                  Tcap: integer);

  procedure GammaY (gamma: vector;
                  var y: longVector;
                  Tcap, n: integer);

  procedure FormQy (var y: longVector;
                  Tcap: integer);

  procedure FormTrend (var y: longVector;
                  var x: longVector;
                  lamOne: real;
                  Tcap: integer);

  procedure ButterworthFilter (n, Tcap: integer;
                  omegaCut: real;
                  var x: longVector;
                  var y: longVector);

  procedure FormQuinDiag (var wVec: vector;
                  lambda: real);

  procedure HPFilter (Tcap: integer;
                  lambda: real;
                  var x: longVector;
                  var y: longVector);

  procedure SprimeY (var y: longVector;
                  s, Tcap: integer);

  procedure FormkappaSy (var s, Tcap: integer;
                  var factor: real;
                  var y: longVector);

  procedure SeasonalFilter (var s, Tcap: integer;
                  var factor: real;
                  var denominator: vector;
                  var x, y: longVector);

  procedure GetFilterGainVector (var filterSpec: FilterRecord;
                  var filterBOX: RBox;
                  var SCREENBOX: Ibox;
                  var fIRVec: vector;
                  var dataVector: longVector);

  procedure TrendCycleComponent (var polyParas: polyParameters;
                  var dataBOX, trendBOX, cycleBOX: rBOX;
                  var dataVector, trendVector, cycleVec: longVector;
                  var SCREENBOX, POSTBOX: IBOX);

  procedure TrendPlusSeason (var polyParas: polyParameters;
                  var dataBOX, residueBOX: rBOX;
                  var dataVector, residueVec: longVector;
                  var SCREENBOX, POSTBOX: IBOX);

implementation

{_________________________________________________}

  procedure Laurent (var mu, gamma: vector;
                  q: integer);

    var
      i, j: integer;

  begin
    for i := 0 to q do
      begin {i}
        gamma[i] := 0.0;
        for j := 0 to q - i do
          gamma[i] := gamma[i] + mu[j] * mu[j + i];
      end; {i}
  end; {Laurent}

 {_________________________________________________}

  procedure InverseRootsToCoeffs (p: integer;
                  var mu, alpha: complexVector);

    var
      j, k: integer;
      store: complex;


  begin {RootToCoefficient}
    alpha[0].re := 1.0;
    alpha[0].im := 0.0;

    for k := 1 to p do
      begin {k}
        alpha[k].im := 0.0;
        alpha[k].re := 0.0;
        for j := k downto 1 do
          begin {j}
            ComplexMult(mu[k], alpha[j - 1], store);
            ComplexSub(alpha[j], store, alpha[j]);
          end; {j}
      end; {k}

  end; {InverseRootsToCoeffs}

   {_________________________________________________}

  procedure ButterworthDigitalPoles (n: integer;
                  digitalCut: real;
                  var poles: complexVector);

    var
      k: integer;
      omegaCut, theta, delta: real;

  begin
    omegaCut := Tan(digitalCut / 2);
    for k := 1 to n do
      begin
        theta := pi * (2 * k - 1) / (2 * n);
        delta := 1 + omegaCut * (2 * Sin(theta) + omegaCut);
        poles[k].re := (1 - omegaCut * omegaCut) / delta;
        poles[k].im := 2 * omegaCut * Cos(theta) / delta;
      end;
  end; {ButterworthDigitalPoles}

 {_________________________________________________}

  procedure ButterworthDigitalZeros (n: integer;
                  var zeros: complexVector);
    var
      k: integer;

  begin
    zeros[0].re := n;
    for k := 1 to n do
      begin
        if filterSpec.filterType = lowPass then
          zeros[k].re := -1
        else
          zeros[k].re := 1;
        zeros[k].im := 0;
      end;
  end; {ButterworthDigitalZeros}

{_________________________________________________}

  procedure SeasonalPoles (var s: integer;
                  var rho: real;
                  var poles: complexVector);

    var
      theta: real;

  begin
    if (s = 12) then
      begin
        theta := (2.0 * Arctan(1.0)) / 3.0;

        poles[1].re := rho * Cos(theta);
        poles[1].im := rho * Sin(theta);

        poles[5].re := -poles[1].re;
        poles[5].im := poles[1].im;

        poles[7].re := -poles[1].re;
        poles[7].im := -poles[1].im;

        poles[11].re := poles[1].re;
        poles[11].im := -poles[1].im;

        poles[2].re := rho * Cos(2.0 * theta);
        poles[2].im := rho * Sin(2.0 * theta);

        poles[4].re := -poles[2].re;
        poles[4].im := poles[2].im;

        poles[8].re := -poles[2].re;
        poles[8].im := -poles[2].im;

        poles[10].re := poles[2].re;
        poles[10].im := -poles[2].im;

        poles[3].re := 0.0;
        poles[3].im := rho;

        poles[6].re := rho;
        poles[6].im := 0.0;

        poles[9].re := 0.0;
        poles[9].im := -rho;
      end;

    if (s = 4) then
      begin
        poles[1].re := 0.0;
        poles[1].im := rho;

        poles[2].re := -rho;
        poles[2].im := 0.0;

        poles[3].re := 0.0;
        poles[3].im := -rho;
      end;

  end; {SeasonalPoles}

{_________________________________________________}

  procedure SeasonalZeros (var s: integer;
                  var zeros: complexVector);

    var
      theta: real;

  begin
    if s = 12 then
      begin
        theta := (2.0 * Arctan(1.0)) / 3.0; {30 degrees}

        zeros[1].re := Cos(theta);
        zeros[1].im := Sin(theta);

        zeros[5].re := -zeros[1].re;
        zeros[5].im := zeros[1].im;

        zeros[7].re := -zeros[1].re;
        zeros[7].im := -zeros[1].im;

        zeros[11].re := zeros[1].re;
        zeros[11].im := -zeros[1].im;

        zeros[2].re := Cos(2.0 * theta);
        zeros[2].im := Sin(2.0 * theta);

        zeros[4].re := -zeros[2].re;
        zeros[4].im := zeros[2].im;

        zeros[8].re := -zeros[2].re;
        zeros[8].im := -zeros[2].im;

        zeros[10].re := zeros[2].re;
        zeros[10].im := -zeros[2].im;

        zeros[3].re := 0.0;
        zeros[3].im := 1.0;

        zeros[6].re := -1.0;
        zeros[6].im := 0.0;

        zeros[9].re := 0.0;
        zeros[9].im := -1.0;
      end;

    if (s = 4) then
      begin
        zeros[1].re := 0.0;
        zeros[1].im := 1.0;

        zeros[2].re := -1.0;
        zeros[2].im := 0.0;

        zeros[3].re := 0.0;
        zeros[3].im := -1.0;
      end;

  end; {SeasonalZeros}

{_________________________________________________}

  function FIRFilterGain (omega: real;
                  var filterSpec: FilterRecord;
                  var firVec: vector): real;

 {This is genuinely the gain and not the squared gain}
    var
      gain: real;
      j, m: integer;

  begin
    m := (filterSpec.filterSpan - 1) div 2;
    gain := firVec[0];
    for j := 1 to m do
      gain := gain + 2 * firVec[j] * Cos(omega * j);

    if (filterSpec.gainChar = 'A') and (gain < 0.0) then
      gain := -gain;

    FIRFilterGain := gain;
  end; {FIRFilterGain}

 {_________________________________________________}

  function DigitalButterGain (omega: real): real;
 {Gain of SquareGain}
    var
      x, pi, gain: real;
      n, twoN: integer;

  begin
    pi := 4 * Arctan(1.0);
    with filterSpec do
      begin{filterSpec }
        n := filterOrder;
        case filterType of
          LowPass: 
            x := Tan(omega / 2) * CoTan(CutOff / 2);
          HighPass: 
            x := CoTan(omega / 2) * CoTan((pi - cutOff) / 2);
        end;{case}

        twoN := 2 * n;
        gain := 1 / (1 + Power(x, twoN));
      end;{FilterRecord}
    DigitalButterGain := gain;

  end; {DigitalButterGain}

 {_________________________________________________}

  function HPGain (omega, lambda: real): real;

    var
      x, gain: real;

  begin
    x := 2 * Sin(omega / 2);
    gain := 1 / (1 + lambda * Power(x, 4));
    if (filterSpec.filterType = highpass) then
      gain := 1 - gain;
    HPGain := gain;

  end; {HPGain}

 {_________________________________________________}

  function SeasFilterGain (omega: real;
                  s: integer;
                  var numerator, denominator: vector): real;

{we need to pass phi and chi as parameters}

    var
      j: integer;
      numerSum, denoSum: extended;

  begin
    numerSum := numerator[0];
    for j := 1 to s - 1 do
      numerSum := numerSum + 2 * numerator[j] * Cos(omega * j);
    denoSum := denominator[0];
    for j := 1 to s - 1 do
      denoSum := denoSum + 2 * denominator[j] * Cos(omega * j);
    SeasFilterGain := (numerSum / denoSum);
  end; {SeasFilterGain}

 {_________________________________________________}

  procedure PrescottFilter (lambda: real;
                  var gamma, delta: vector;
                  filterSpec: filterRecord);

{This seems to be detached. It is subsumed in <HPFilterSpec>}

  begin
    delta[0] := 1;
    gamma[0] := lambda;
    gamma[1] := -4 * lambda;
    gamma[2] := 1 + 6 * lambda;
    gamma[3] := -4 * lambda;
    gamma[4] := lambda;

    if filterSpec.filterType = HighPass then
      begin {Residue Extraction}
        delta[0] := gamma[0];
        delta[1] := gamma[1];
        delta[2] := gamma[2] - 1;
        delta[3] := gamma[3];
        delta[4] := gamma[4];
      end; {Residue Extraction}
  end; {PrescottFilter}

{_________________________________________________}

  procedure CombinedCoefficients (var lambda, rho, factor: real;
                  var s: integer;
                  var numerator, denominator: vector);

{This procedure calculates the coefficients of the  seasonal-adjustment filter.}
{These are the coefficients of a symmetric two sided-filter, so we only store}
{those associated with one side.}

    var
      mu, gamma: vector;
      rhoPower: real;
      j: integer;

  begin
    mu[0] := 1.0;
    rhoPower := rho;
    for j := 1 to s - 1 do
      begin {j}
        mu[j] := rhoPower;
        rhoPower := rhoPower * rho;
      end; {j}
 {Make R(z) = 1 + rho z  + rho^2 z^2 + cdots  rho^(s-1) z^(s-1)}

    Laurent(mu, gamma, s - 1); {gamma(z) = m(z)mu(1/z)}

    for j := 0 to s - 1 do
      begin {j}
        mu[j] := rhoPower; {why do we need mu[j]?}
        denominator[j] := lambda * gamma[j] + (s - j)
      end; {j}
{Make lambda* R(z)R(1/z) + S(z)S(1/z)}

    factor := denominator[0];
    for j := 1 to s - 1 do
      factor := factor + 2 * denominator[j];
    if s = 4 then
      factor := factor / 16.0;
    if s = 12 then
      factor := factor / 144.0;

    for j := 0 to s - 1 do
      numerator[j] := (s - j) * factor;

{n := s - 1;}
{d := s - 1;}

  end;{CombinedCoefficients}

{_________________________________________________}

  procedure DiffCoefficients (var alpha: vector;
                  d: integer);

 {This procedure forms a vector containing the coefficients of}
 {(1 - L)^d, which is the d-th power of the difference operator.}

    var
      i, j: integer;

  begin
    alpha[0] := 1.0;
    for j := 1 to d do
      begin {j}
        alpha[j] := 0;
        for i := j downto 1 do
          alpha[i] := alpha[i] - alpha[i - 1];
      end; {j}

  end; {DiffCoefficients}

 {_________________________________________________}

  procedure DeltaPrimeY (var y: longVector;
                  var Tcap: integer;
                  d: integer);

 {This procedure finds succesive first differences of the elements of a}
 {vector. The leading  element of the vector is lost in the process. The}
 {leading elements of both the original vector and the differenced vector}
 {are indexed by t = 0}

    var
      t, j: integer;

  begin
    for j := 1 to d do
      begin {j: repeated differences}
        Tcap := Tcap - 1;
        for t := 0 to Tcap - 1 do
          y[t] := y[t + 1] - y[t];
      end; {j}
  end; {DeltaPrimeY}

 {_________________________________________________}

  procedure DeltaY (var y: longVector;
                  var Tcap: integer;
                  d: integer);

 {This procedure replaces the elements y(t) of a vector by y(t-1) -y(t)}
 {and it performs this operation a total of d times. In each round, the}
 {vector y of Tcap elements is multiplied by a matrix Q of order Tcap +1}
 {times Tcap, where Q' is the matrix that finds the first differences}
 {of a vector. In each round, the order of the vector is increased by one.}

    var
      t, j: integer;

  begin
    for j := 1 to d do
      begin {j: repeated differences}
        y[Tcap] := y[Tcap - 1];
        for t := Tcap - 1 downto 1 do
          y[t] := y[t - 1] - y[t];
        y[0] := -y[0];
        Tcap := Tcap + 1;
      end; {j}

  end; {DeltaY}

 {_________________________________________________}

  procedure FormGamma (var gamma: vector;
                  mu: vector;
                  r: integer);

 {This procedure forms the Autcovariances  }
 {of an rth-order moving-average process.}

    var
      j, k: integer;

  begin
    for j := 0 to r do
      begin {j}
        gamma[j] := 0.0;
        for k := 0 to r - j do
          gamma[j] := gamma[j] + mu[k] * mu[k + j];
      end; {j}
  end; {FormGamma}

 {_________________________________________________}

  procedure SumDifference (var theta: vector;
                  n: integer;
                  sign: integer);

 {This procedure finds the coefficients of the nth power of}
 {the summation operator or of the difference operator.}

    var
      j, q: integer;

  begin
    theta[0] := 1.0;
    for q := 1 to n do
      begin {q}
        theta[q] := 0.0;
        for j := q downto 1 do
          theta[j] := theta[j] + sign * theta[j - 1];
      end; {q}

  end; {SumDifference}

 {_________________________________________________}

  procedure FormMvec (var mVec: vector;
                  n: integer);

    var
      mu: vector;

  begin
    sumDifference(mu, n, 1);
    FormGamma(mVec, mu, n);
  end; {FormMvec}

 {_________________________________________________}

  procedure FormSVec (var sVec: vector;
                  n: integer);

    var
      mu: vector;

  begin
    sumDifference(mu, n, -1);
    FormGamma(sVec, mu, n);
  end; {FormSvec}

 {_________________________________________________}

  procedure FormWvec (var wVec: vector;
                  n: integer;
                  lamOne, lamTwo: real);

    var
      i: integer;
      sVec, mVec: vector;

  begin
    FormSVec(sVec, n); {sVec = Q'SQ where Q is the 2nd operator}
    FormMVec(mVec, n);
    for i := 0 to n do
      wVec[i] := lamTwo * mVec[i] + lamOne * sVec[i];
  end; {FormwVec}

 {_________________________________________________}

  procedure QprimeY (var y: longVector;
                  Tcap: integer);

 {This procedure finds the second differences of the elements of a vector.}
 {Notice that the first two elements of the vector are lost in the process.}
 {However, we index the leading element of the differenced vector by t = 0.}

    var
      t: integer;

  begin
    for t := 0 to Tcap - 3 do
      y[t] := y[t] - 2 * y[t + 1] + y[t + 2];
  end; {QprimeY}

 {_________________________________________________}

  procedure GammaY (gamma: vector;
                  var y: longVector;
                  Tcap, n: integer);

 {This procedure premultiplies a vector y by a symmetric banded Toeplitz matrix}
 {Gamma with n nonzero sub-diagonal bands and n nonzero supra-dagonal bands.}

    var
      i, t, j: integer;
      left, right: real;
      store: vector;

  begin
    for i := 0 to n do
      store[i] := 0.0;

    for t := 0 to Tcap - 1 do
      begin {t}
        for i := n downto 1 do
          store[i] := store[i - 1];
        store[0] := gamma[0] * y[t];
        for j := 1 to n do
          begin {j}
            if (t - j < 0) then
              left := 0
            else
              left := y[t - j];
            if (t + j > Tcap - 1) then
              right := 0
            else
              right := y[t + j];
            store[0] := store[0] + gamma[j] * (left + right);
          end; {j}
        if t >= n then
          y[t - n] := store[n];
      end; {t}
    for j := 0 to n - 1 do
      y[Tcap - 1 - j] := store[j];
  end; {GammaY}

 {_________________________________________________}

  procedure FormQy (var y: longVector;
                  Tcap: integer);

 {This procedure multiplies the vector y of Tcap -2 elements by matrix}
 {Q of order Tcap times Tcap -2, where Q' is the matrix which finds}
 {the second differences of a vector.}

    var
      t: integer;
      lag1, lag2, store: real;

  begin
    lag1 := 0.0;
    lag2 := 0.0;
    for t := 0 to Tcap - 3 do
      begin {t}
        store := y[t];
        y[t] := y[t] - 2 * lag1 + lag2;
        lag2 := lag1;
        lag1 := store
      end; {t}
    y[Tcap - 2] := -2 * lag1 + lag2;
    y[Tcap - 1] := lag1;

  end; {FormQy}

 {_________________________________________________}

  procedure FormTrend (var y: longVector;
                  var x: longVector;
                  lamOne: real;
                  Tcap: integer);

    var
      t: integer;
      store: real;

  begin

    for t := 0 to Tcap - 1 do
      begin
        store := lamOne * x[t];
        x[t] := y[t] - store; {the trend vector}
        y[t] := store; {the residual vector}
      end;
  end; {FormTrend}

 {_________________________________________________}

  procedure ButterworthFilter (n, Tcap: integer;
                  omegaCut: real;
                  var x: longVector;
                  var y: longVector);

    var
      lamOne, lamTwo: real;
      sVec, wVec: vector;

  begin
    lamOne := 1 / Tan(omegaCut / 2);
    lamOne := Power(lamOne, 2 * n);
    lamTwo := 1.0;
    if lamOne > 1.0 then
      begin
        lamTwo := 1 / lamOne;
        LamOne := 1.0;
      end;

    x := y;
    FormWvec(wVec, n, lamOne, lamTwo); { W = M + lambda*Q'SQ }
    QprimeY(x, Tcap); {d = Q'y: we are loosing two points in taking 2nd differences}
    Toeplitz(wVec, x, Tcap - 3, n); {solve (M + lambda*Q'SQ)g = d for g}
    FormQy(x, Tcap); {Form Qg}
    FormSvec(sVec, n - 2);
    GammaY(sVec, x, Tcap, n - 2); {Form SQg}
    FormTrend(y, x, lamOne, Tcap); {x is trend, y is residual}

  end; {ButterworthFilter}

 {_________________________________________________}

  procedure FormQuinDiag (var wVec: vector;
                  lambda: real);

    var
      mu: vector;

  begin
    sumDifference(mu, 2, -1);
    FormGamma(wVec, mu, 2);
    wVec[0] := 1.0 + lambda * wVec[0];
    wVec[1] := lambda * wVec[1];
    wVec[2] := lambda * wVec[2];
  end; {FormQuinDiag}

 {_________________________________________________}

  procedure HPFilter (Tcap: integer;
                  lambda: real;
                  var x: longVector;
                  var y: longVector);

    var
      wVec, qqVec: vector;

  begin
    x := y;
    FormMvec(qqVec, 2); {Q'Q}
    FormQuinDiag(wVec, lambda);
    QprimeY(x, Tcap); { d = Q'y: we are loosing two dimesions in taking 2nd differences}
    Toeplitz(wVec, x, Tcap - 3, 2); {solve (I + lambda*Q'Q)z = d for z}
    FormQy(x, Tcap); {Form Qz}
    FormTrend(y, x, lambda, Tcap); {x = y - lambda *Qz}
  end; {HPFilter}

{_________________________________________________}

  procedure SprimeY (var y: longVector;
                  s, Tcap: integer);

{is this seasonal cumulation? }
    var
      t, j: integer;
  begin
    for t := 0 to Tcap - s do
      for j := 1 to s - 1 do
        y[t] := y[t] + y[t + j];
  end; {SprimeY}

{_________________________________________________}

  procedure FormkappaSy (var s, Tcap: integer;
                  var factor: real;
                  var y: longVector);
    var
      p, q, t, j: integer;
  begin
    for t := Tcap - 1 downto 0 do
      begin {t}
        p := 1;
        q := s - 1;

        if t < s - 1 then
          q := t;

        if (t > (Tcap - s)) then
          begin
            y[t] := 0.0;
            p := s + t - Tcap;
          end;

        for j := p to q do
          y[t] := y[t] + y[t - j];

        y[t] := y[t] * factor;
      end; {t}

  end; {FormkappaSy}

{_________________________________________________}

  procedure SeasonalFilter (var s, Tcap: integer;
                  var factor: real;
                  var denominator: vector;
                  var x, y: longVector);

{SeasonalFilter(s, Tcap, factor, denominator, seasonalVec, adjustedVec);}

{On entry, y is the dataVector, on exit, it is the seasonally adjusted vector.}
{On exit, x is the vector of the seasonal fluctuations.}
{we need to adjust Tcap in view ofthe number of differnces taken}

    var
      t: integer;

  begin
{x := y;}
    SprimeY(y, s, Tcap);
    Toeplitz(denominator, y, Tcap - s, s - 1); {in MATHS}
    FormkappaSy(s, Tcap, factor, y);
    for t := 0 to Tcap - 1 do
      x[t] := x[t] - y[t]; {Vector of seasonal fluctuations}

{y[t] is the seasonally adjusted data x[t] is the seasonal fluctuations }

  end; {SeasonalFilter}

 {_________________________________________________}

  procedure GetFilterGainVector (var filterSpec: FilterRecord;
                  var filterBOX: RBox;
                  var SCREENBOX: Ibox;
                  var fIRVec: vector;
                  var dataVector: longVector);
    var
      BOXWDT, i, s: integer;
      yMax, yMin, omega, pi, inc: real;
{phi, chi <==> numerator denominator are global vectors}

  begin
    with filterSpec do
      begin{filterSpec}
        pi := 4.0 * Arctan(1.0);
        BOXWDT := SCREENBOX.X2 - SCREENBOX.X1;

        inc := pi / BOXWDT;
        omega := 0.0;

        yMax := 0.0;
        yMin := 0.0;
        for i := 0 to BOXWDT do
          begin {i}
            if filterFamily = Butterworth then
              dataVector[i] := DigitalButterGain(omega);
            if filterFamily = HodrickPrescott then
              dataVector[i] := HPGain(omega, smoothingParameter);
            if filterFamily = DeSeasoning then
              begin
                s := filterSpec.seasonality;
                dataVector[i] := SeasFilterGain(omega, s, numerator, denominator);
{phi, chi <==> numerator denominator are global vectors}
              end;
            if filterFamily in [Henderson, BaxterKing] then
              dataVector[i] := FIRFilterGain(omega, filterSpec, firVec); {in FILTERS.PAS}
            yMax := RMax(yMax, dataVector[i]);
            yMin := RMin(yMin, dataVector[i]);
            omega := omega + inc;
          end; {i}
      end; {with filterSpec}

 {Should the be  filterBOX?}
    filterBOX.NOPUNCT := BOXWDT;
    filterBOX.xMax := pi;
    filterBOX.xMin := 0.0;
    filterBOX.yMax := yMax;
    filterBOX.yMin := yMin;
    filterBOX.xRB := pi;
    filterBOX.xLB := 0.0;
    filterBOX.yUB := yMax;
    filterBOX.yLB := yMin;

  end; {GetFilterGainVector}

 {________________________________________________}

  procedure TrendCycleComponent (var polyParas: polyParameters;
                  var dataBOX, trendBOX, cycleBOX: rBOX;
                  var dataVector, trendVector, cycleVec: longVector;
                  var SCREENBOX, POSTBOX: IBOX);

    var
      z, polyt: real;
      t: integer;
      addPoly: boolean;
      legend: string;

  begin
    Writeln;
    Writeln('    THE TREND-CYCLE COMPONENT');
    Writeln;
    Writeln('    The trend-cycle component can be derived by adding the filtered');
    Writeln('    version of the residual sequence, which has been subjected to a');
    Writeln('    lowpass filter, to the polynomial function interpolated through');
    Writeln('    the data.');

    Writeln;
    YesOrNo('    Do you wish to plot the Trend/Cycle component', addPoly);

      {On entry CycleVec contains the data and cycleBOX is the corresponding rBOX}
      {We can begin by putting CycleVec --> dataVector, cycleBOX --> dataBOX.}
      {Also, trendVector --> cycleVec and trendBOX --> cycleBOX.}

    if addPoly then
      begin {add the polynomial: the above conversions should happen in the calling segment}

        for t := 0 to cycleBOX.NOPUNCT - 1 do
          begin {t}
            z := t;
            polyt := PolyOrdinate(z, polyParas.alpha, polyParas.gamma, polyParas.delta, polyParas.polyDegree);
            trendVector[t] := cycleVec[t] + polyt;{trendVec contains the trendcycles, cycleVec contains the data}
          end; {t}

        legend := TrendAndCycle(cycleBOX, dataName);
        ScreenOverplot(dataBOX, SCREENBOX, dataVector, trendVector, legend);

        if (PSPict = true) then
          WritePostOverFile(dataBOX, POSTBOX, dataVector, trendVector, legend);

        MakeDataBox(trendBOX, trendVector);
        weHaveATrend := true;

      end; {add the polynomial}

  end; {TrendCycleComponet}

 {________________________________________________}

  procedure TrendPlusSeason (var polyParas: polyParameters;
                  var dataBOX, residueBOX: rBOX;
                  var dataVector, residueVec: longVector;
                  var SCREENBOX, POSTBOX: IBOX);

    var
      z, polyt: real;
      t: integer;
      addPoly: boolean;
      legend: string;

  begin
    Writeln;
    Writeln('    SEASONALLY ADJUSTED TRENDED DATA');
    Writeln;
    Writeln('    A seasonally  adjusted version of the original data can be derived');
    Writeln('    by adding the seasonally adjusted version of the residual sequence');
    Writeln('    to the polynomial function that has  been interpolated through the');
    Writeln('    the data.');
    Writeln;
    YesOrNo('    Do you wish to generate the trended sequence', addPoly);

    if addPoly then
      begin {add the polynomial}

        for t := 0 to dataBOX.NOPUNCT - 1 do
          begin {t}
            z := t;
            polyt := PolyOrdinate(z, polyParas.alpha, polyParas.gamma, polyParas.delta, polyParas.polyDegree);
            residueVec[t] := residueVec[t] + polyt;{trendVec contains the trendcycles, cycleVec contains the data}
          end; {t}

        legend := DeSeasonalisedOverlay(dataBOX, dataName);
        ScreenOverplot(dataBOX, SCREENBOX, dataVector, residueVec, legend);

        if (PSPict = true) then
          WritePostOverFile(dataBOX, POSTBOX, dataVector, residueVec, legend);

      end; {add the polynomial}

  end; {TrendPlusSeason}

 {________________________________________________}

end. {FILTERS.PAS: Unit 13 of NEWIDEOLOG.PAS}

