unit FIRFILT; {Unit 14 of NEWIDEOLOG.PAS}

interface

  uses
    GLOBALS, UTILS, MATHS, SCREEN, POST, FILTERS, DATASEG, LEGENDS, Graph, Dos, Crt;

  procedure ReduceTheFilter (var filterSpec: FilterRecord;
                  var fIRVec: vector);

  procedure GetHendersonWeights (var filterSpec: FilterRecord;
                  var fIRVec: vector);

  procedure GetMusgraveWeights (var FilterSpec: FilterRecord;
                  var m: integer;
                  var alpha, beta: real;
                  var uVec: vector);

  procedure GetRIRfilterEndWeights (var r, p: integer;
                  var RIRVec, uVec: vector);
  {This is not used}

  procedure LowpassTruncWeights (var weightsVec: vector;
                  var cutoff: real;
                  var span: integer);

  procedure GetTruncIdealWeights (var filterSpec: FilterRecord;
                  var fIRVec: vector);

  procedure GetFurlongWeights (var FilterSpec: FilterRecord;
                  var dataBOX: rBOX;
                  var furlongVec: longVector);

  procedure HendersonFilterSpec (var filterSpec: FilterRecord);

  procedure BaxterKingFilterSpec (var filterSpec: FilterRecord);

  procedure ChristFitzFilterSpec (var filterSpec: FilterRecord);

  procedure ApplyMusgaveFilter (var dataBOX, trendBOX, residueBOX: rBOX;
                  var filterSpec: FilterRecord;
                  var dataVector, trendVector, residueVec: longVector);

  {--------------------------------------------------------------}
  procedure ApplyFilterDirectly (var dataBOX, residueBOX: rBOX;
                  var filterSpec: FilterRecord;
                  var fIRVec: vector;
                  var dataVector, residueVec: longVector);

  procedure DiffenceFilterAndReinflate (var dataBOX, residueBOX: rBOX;
                  var filterSpec: FilterRecord;
                  var fIRVec: vector;
                  var dataVector, residueVec: longVector);

  procedure DifferenceReduceAndFilter (var dataBOX, residueBOX: rBOX;
                  var filterSpec: FilterRecord;
                  var fIRVec: vector;
                  var dataVector, residueVec: longVector);

  procedure ApplyTruncFilter (var dataBOX, cycleBOX: rBOX;
                  var filterSpec: filterRecord;
                  var dataVector, cycleVec: longVector);
  {--------------------------------------------------------------}

  procedure ApplyChristFitzFilter (var filterSpec: FilterRecord;
                  var dataBOX, cycleBOX: rBOX;
                  var dataVector, cycleVec: longVector);

  procedure PlotFIRBPassGain (var SCREENBOX: IBOX;
                  var filterSpec: FilterRecord);

  procedure PlotHendersonGain (var SCREENBOX: IBOX;
                  var filterSpec: FilterRecord);

implementation

   {_________________________________________________}

  procedure ReduceTheFilter (var FilterSpec: FilterRecord;
                  var fIRVec: vector);
    var
      numerator, divisor, remainder: vector;
      quotient: jVector;
      m, p, i, j: integer;

  begin
    p := FilterSpec.filterSpan;
    m := (p - 1) div 2; {p := 2m + 1}
    for i := 0 to m do
      numerator[i] := fIRVec[m - i];
    for i := 1 to m do
      numerator[m + i] := fIRVec[i];
  {numerator is the symmetric filter vector indexed}
  {from  i = 0 to i = 2m}

    divisor[0] := 1.0;
    divisor[1] := -2.0;
    divisor[2] := 1.0;
    p := p - 1; {the polynomial order is one less than the filter span}
    DivisionAlgorithm(numerator, divisor, p, 2, quotient, remainder);

    for j := 0 to m - 1 do
      FIRVec[j] := quotient[m + j - 1];
    FIRVec[m] := 0.0;

  {N.B. m still relates to the original unreduced filter}
  {also, we do not alter the value of FilterSpec.filterSpan}

  end; {ReduceTheFilter}

  {________________________________________________}

  procedure GetHendersonWeights (var FilterSpec: FilterRecord;
                  var fIRVec: vector);
    var
      m, j: integer;
      sum: real;

  begin {GetHenderson}
    m := (FilterSpec.FilterSpan - 1) div 2;
    for j := 0 to m do
      begin {j}
        fIRVec[j] := (m + 1) * (m + 1) - j * j;
        fIRVec[j] := fIRVec[j] * ((m + 2) * (m + 2) - j * j);
        fIRVec[j] := fIRVec[j] * ((m + 3) * (m + 3) - j * j);
        fIRVec[j] := fIRVec[j] * (3 * (m + 2) * (m + 2) - 16 - 11 * j * j);
      end; {j}

    sum := fIRVec[0];
    for j := 1 to m do
      sum := sum + 2 * fIRVec[j];
    for j := 0 to m do
      fIRVec[j] := fIRVec[j] / sum;

  end; {GetHendersonWeights}

  {________________________________________________}

  procedure GetMusgraveWeights (var FilterSpec: FilterRecord;
                  var m: integer;
                  var alpha, beta: real;
                  var uVec: vector);

   {This procedure implements the formula given by Dougherty (2001).}
   {It returns the asymmetric weights in  in uVec: uVec[1] to uVec[m] }

    var
      oneStore, twoStore, factor, denom, pi: real;
      mplusOneOverTwo, betaSig, EyeOverC: real;
      n, p, i, r, j: integer;
      wVec, firVec: vector;

  begin
    GetHendersonWeights(FilterSpec, fIRVec);
    n := FilterSpec.FilterSpan;

  {n as the full filter span}
  {m is the span of the asymmetic filter u_r, r = 1 to m}

    pi := 4.0 * ArcTan(1.0);
    if n = 9 then
      EyeOverC := 1;
    if n = 13 then
      EyeOverC := 3.5;
    if n = 23 then
      EyeOverC := 4.5;
    betaSig := 4.0 / pi;
    betaSig := betaSig / (EyeOverC * EyeOverC);

    p := (n - 1) div 2;
    mplusOneOverTwo := (m + 1) / 2;

  {Here we are storing the necessary Henderson Weights,}
  {indexed from 1 to 2p+1, in the vector wVec}

    wVec[p + 1] := FIRVec[0];
    for j := 1 to p do
      begin {j}
        wVec[j] := FIRVec[p + 1 - j];
        wVec[p + 1 + j] := FIRVec[j];
      end; {j}

    for r := 1 to m do
      begin {r  = 1 to m is the index on uVec]}
        uVec[r] := wVec[r];
        oneStore := 0.0;
        twoStore := 0.0;

        for i := m + 1 to n do
          begin {i}
            oneStore := oneStore + wVec[i];
            factor := i - mplusOneOverTwo;
            twoStore := twoStore + factor * wVec[i];
          end; {i}

        oneStore := oneStore / m;
        uVec[r] := uVec[r] + oneStore;
  {betaSig := (beta * beta) / (sigma * sigma);}
        factor := betaSig * (r - mplusOneOverTwo);
        denom := (m - 1) * m * (m + 1);
        denom := (denom * betaSig) / 12;
        denom := denom + 1;
        factor := factor / denom;
        uVec[r] := uVec[r] + factor * twoStore;
      end;{r}

  end; {GetMusgraveWeights}

  {________________________________________________}

  procedure GetRIRfilterEndWeights (var r, p: integer;
                  var RIRVec, uVec: vector);
    var
      i, j: integer;
      wVec: vector;
      store, twoStore: real;

  begin
    store := 0.0;
    wVec[p + 1] := FIRVec[0];
    for j := 1 to p do
      begin {j}
        wVec[j] := FIRVec[p + 1 - j];
        wVec[p + 1 + j] := FIRVec[j];
        store := store + wVec[j]
      end; {j}
    WRITELN(' The sum of WVec = ', store : 6 : 4);

    store := 0.0;
    for i := 1 to r do
      store := store + wVec[i];
    store := store / r;
    twoStore := 0.0;
    for j := 1 to r do
      begin {temp}
        UVec[j] := wVec[j] - store;
        twoStore := twoStore + UVec[j];
      end; {temp}

  end; {GetRIRfilterEndWeights}

  {________________________________________________}

  procedure LowpassTruncWeights (var weightsVec: vector;
                  var cutoff: real;
                  var span: integer);
    var
      pi, psi, sum, adjust: real;
      j, m: integer;

  begin
    pi := 4.0 * Arctan(1.0);
    cutoff := cutoff * pi / 180;
    m := (span - 1) div 2;
    weightsVec[0] := cutoff / pi;

    sum := weightsVec[0];
    for j := 1 to m do
      begin {j}
        psi := Sin(cutoff * j);
        psi := psi / (pi * j);
        weightsVec[j] := psi;
        sum := sum + 2 * weightsVec[j];
      end; {j}
    adjust := (1 - sum) / span;

    if filterSpec.truncAdjustment then
      begin
        for j := 0 to m do
          weightsVec[j] := weightsVec[j] + adjust; {coefficients sum to unity}
      end;

  end; {LowpassTruncWeights}

   {________________________________________________}

  procedure GetTruncIdealWeights (var filterSpec: FilterRecord;
                  var fIRVec: vector);
    var
      omega, omega1, omega2: real;
      span, m, j: integer;
      StoreVector: vector;

  begin
    span := FilterSpec.filterSpan;
    m := (span - 1) div 2;
    if filterSpec.filterType = lowPass then
      begin
        omega := FilterSpec.cutOff;
        LowpassTruncWeights(fIRVec, omega, span);
      end {lowPass}
    else if filterSpec.filterType = highPass then
      begin
        omega := FilterSpec.cutOff;
        LowpassTruncWeights(fIRVec, omega, span);
        fIRVec[0] := 1 - fIRVec[0];
        for j := 1 to m do
          fIRVec[j] := -fIRVec[j];
      end {highPass}
    else if filterSpec.filterType in [bandPass, bandStop] then
      begin
        omega1 := FilterSpec.lowerCut;
        omega2 := FilterSpec.upperCut;
        LowpassTruncWeights(fIRVec, omega2, span);
        LowpassTruncWeights(StoreVector, omega1, span);
        for j := 0 to m do
          fIRVec[j] := fIRVec[j] - StoreVector[j];
      end; {bandPass}
  {Band pass filter is the diffence of two lowpass filters}

    if filterSpec.filterType = bandStop then
      for j := 0 to m do
        fIRVec[j] := 1 - fIRVec[j];

  {Now we should adjust the weights to sum to unity or to zero}
  {We have done this in the previous routine <LowpassTruncWeights>}

  end; {GetTruncIdealWeights}

   {_________________________________________________}

  procedure AltGetTruncIdealWeights (var FilterSpec: FilterRecord;
                  var fIRVec: vector);
    var
      pi, a, b, omega, omega1, omega2, psi, sum, adjust: real;
      m, j: integer;

  begin
    pi := 4.0 * Arctan(1.0);

    m := (FilterSpec.filterSpan - 1) div 2;
    omega := FilterSpec.cutOff * pi / 180;
    omega1 := FilterSpec.lowerCut * pi / 180;
    omega2 := FilterSpec.upperCut * pi / 180;

    if filterSpec.filterType = LowPass then
      begin
        fIRVec[0] := omega / pi;
        sum := fIRVec[0];
        for j := 1 to m do
          begin {j}
            a := Sin(omega * j);
            psi := a / (pi * j);
            fIRVec[j] := psi;
            sum := sum + 2 * fIRVec[j];
          end; {j}
        adjust := -sum / (2 * m + 1); {coefficients sum to unity}
      end; {LowPass}

    if FilterSpec.filterType = highPass then
      begin
        fIRVec[0] := 1 - (omega) / pi;
        sum := fIRVec[0];
        for j := 1 to m do
          begin {j}
            a := Sin(omega * j);
            psi := -a / (pi * j);
            fIRVec[j] := psi;
            sum := sum + 2 * fIRVec[j];
          end; {j}
        adjust := (1.0 - sum) / (2 * m + 1); {coefficients sum to zero}

      end; {HighPass}

    if FilterSpec.filterType = bandPass then
      begin
        fIRVec[0] := (omega2 - omega1) / pi;
        sum := fIRVec[0];
        for j := 1 to m do
          begin {j}
            b := Sin(omega2 * j);
            a := Sin(omega1 * j);
            psi := (b - a) / (pi * j);
            fIRVec[j] := psi;
            sum := sum + 2 * fIRVec[j];
          end; {j}
        adjust := (1.0 - sum) / (2 * m + 1); {coefficients sum to zero}
      end; {BandPass}

    if FilterSpec.filterType = bandStop then
      begin
        fIRVec[0] := 1 - (omega2 - omega1) / pi;
        sum := fIRVec[0];
        for j := 1 to m do
          begin {j}
            b := Sin(omega2 * j);
            a := Sin(omega1 * j);
            psi := (a - b) / (pi * j);
            fIRVec[j] := psi;
            sum := sum + 2 * fIRVec[j];
          end; {j}
        adjust := (1.0 - sum) / (2 * m - 1); {coefficients sum to unity}
      end; {BandStop}

    if filterSpec.truncAdjustment then
      begin
        for j := 0 to m do
          firVec[j] := firVec[j] + adjust;
      end;

  end; {AltGetTruncIdealWeights}

  {________________________________________________}

  procedure GetFurlongWeights (var FilterSpec: FilterRecord;
                  var dataBOX: rBOX;
                  var furlongVec: longVector);

  {Called by <ApplyChristFitzFilter> within <ImplementTheFIRFilter> in FIRFFILT.PAS}

    var
      pi, a, b, omega1, omega2, psi: real;
      j, Tcap: integer;
   {FILENAME: string;}

  begin
    Tcap := dataBOX.NOPUNCT;
    pi := 4.0 * Arctan(1.0);
    omega1 := FilterSpec.lowerCut * pi / 180;
    omega2 := FilterSpec.upperCut * pi / 180;

    furlongVec[0] := (omega2 - omega1) / pi;
    for j := 1 to Tcap - 1 do
      begin {j}
        b := Sin(omega2 * j);
        a := Sin(omega1 * j);
        psi := (b - a) / (pi * j);
        furlongVec[j] := psi;
      end; {j}

  end; {GetFurlongWeights }

  {_________________________________________________}

  procedure HendersonFilterSpec (var filterSpec: FilterRecord);

  begin
    Writeln;
    Writeln('    The Henderson  moving-average filter is employed in the Census Bureau');
    Writeln('    X-11 program to obtain trend estimates. A set of supplmentatey filter');
    Writeln('    weights, which  are the  so-called Musgrave  weights, are used at the');
    Writeln('    ends of the sample  when there are insufficient data points  to apply');
    Writeln('    the normal symmetric filter.');
    Writeln;
    Writeln('    What is the order of the Henderson filter ? ( 5 , 7 , 13 or 23)');
    GetBoundedInteger(5, 23, filterSpec.filterSpan);
    Writeln(' ' : 62, '<RETURN>');
    Readln;

  end; {HendersonFilterSpec}

   {_________________________________________________}

  procedure BaxterKingFilterSpec (var filterSpec: FilterRecord);

  begin
    if filterSpec.filterType in [lowPass, highPass] then
      begin
        Writeln;
        Writeln('         Specify the cut-off point in degrees');
        ReadANumber(filterSpec.cutOff);
        Writeln('         Specify the filter span');
        GetBoundedInteger(5, 33, filterSpec.filterSpan);
      end; {lowPass, highPass}

    if filterSpec.filterType in [bandPass, bandStop] then
      begin
        Writeln;
        Writeln('        Specify the lower cut-off point in degrees.');
        Writeln;
        Writeln('        For quarterly data, the Burns-Mitchell period of ');
        Writeln('        eight years corresponds to 11.25  degrees.');
        ReadANumber(FilterSpec.lowerCut);
        Writeln;
        Writeln('        Specify the upper cut-off point in degrees.');
        Writeln;
        Writeln('        For quarterly data, the Burns-Mitchell period of  ');
        Writeln('        one-and-a-half years corresponds to 60  degrees.');
        ReadANumber(FilterSpec.upperCut);
        Writeln;
        Writeln('        Specify the filter span (this must be an odd number.');
        Writeln;
        Writeln('        For quarterly data, the Baxter and King propose a filter');
        Writeln('        of 25 coefficients.');
        Writeln;
        GetBoundedInteger(5, 33, FilterSpec.filterSpan);
        if not odd(FilterSpec.filterSpan) then
          FilterSpec.filterSpan := FilterSpec.filterSpan + 1;
      end; {bandPass, bandStop}

    filterSpec.truncAdjustment := true;
  {GetTruncIdealWeights(filterSpec, fIRVec); }
   {in FIRFILT.PAS}

  {Should we include <GetTruncIdealWeights> instead in the filtering procedures?}

  {This  has  also gone  into  PlotFilterGain in WIENER}
  {the firVec contains only the coefficients associated with the causal side of the filter}

  end; {BaxterKingFilterSpec}

  {_________________________________________________}

  procedure ChristFitzFilterSpec (var filterSpec: FilterRecord);

  {Called by SpecifyFIRFilter in ORGANISE}
    var
      theChoice: char;

  begin
    Writeln('     The Chrisitiano-Fitzgerald filter is  obtained  by truncating the');
    Writeln('     sequence of  the  coefficients of  an ideal bandpass  filter. The');
    Writeln('     symmetric filter sequence  is centred on the current index of the');
    Writeln('     sample, and its branches are truncated at the ends of the sample.');
    Writeln('     The filter can be adopted to the following cases:');
    Writeln;
    Writeln('        (a) The sample is from a stationary stochastic process,');
    Writeln;
    Writeln('        (b) The sample is from a process with a single unit root.');
    Writeln;
    Writeln('        (c) The sample is from a process with a drifting random walk.');
    Writeln;
    theChoice := MenuChar('a', 'c');

    case theChoice of
      'a': 
        filterSpec.ChristFitzData := Stationary;
      'b': 
        filterSpec.ChristFitzData := RandomWalk;
      'c': 
        begin {case c}
          filterSpec.ChristFitzData := DriftingWalk;
          if (dataBOX.dataFrequency in [Quarterly, Monthly]) then
            begin {quarterly or monthly}
              ClrScr;
              Writeln;
              Writeln('    REMOVING THE DRIFT FROM THE DATA');
              Writeln;
              Writeln('    The drift can be removed by taking the deviations from an linear trend');
              Writeln('    interpolated  throught the first and the last of the data points. When');
              Writeln('    the data has seasonal fluctuations, it is advisable to i ntepolate the');
              Writeln('    trend through the data of the first and the last years. Please choose');
              Writeln;
              Writeln('          (a) Endpoint interpolation');
              Writeln;
              Writeln('          (b) End-year interpolation');
              Writeln;
              theChoice := MenuChar('a', 'b');
              if theChoice = 'a' then
                filterSpec.ChristFitzData := EndPointDrift;
              if theChoice = 'b' then
                filterSpec.ChristFitzData := EndYearDrift;
              ClrScr;
              Writeln;
            end; {quarterly or monthly}
        end; {case c}
    end; {cases}

    filterSpec.filterType := bandPass;
    Writeln;
    Writeln('    SPECIFY THE CUT-OFF FREQUENCIES OF THE BANDPASS FILTER');
    Writeln;
    Writeln('        Specify the lower cut-off point in degrees.');
    Writeln;
    Writeln('        For quarterly data, the Burns-Mitchell period of ');
    Writeln('        eight years corresponds to 11.25  degrees.');
    ReadANumber(FilterSpec.lowerCut);
    Writeln;
    Writeln('        Specify the upper cut-off point in degrees.');
    Writeln;
    Writeln('        For quarterly data, the Burns-Mitchell period of  ');
    Writeln('        one-and-a-half years corresponds to 60  degrees.');
    ReadANumber(FilterSpec.upperCut);

  end; {ChristFitzFilterSpec}

  {_________________________________________________}

  procedure ApplyMusgaveFilter (var dataBOX, trendBOX, residueBOX: rBOX;
                  var filterSpec: FilterRecord;
                  var dataVector, trendVector, residueVec: longVector);

  {This procedure applies the Henderson filter with }
  {the endweights supplied by the Musgrave formula.}

    var
      r, p, m, t, j, Tcap: integer;
      fIRVec, uVec: vector;
      alpha, beta: real;
      legend: string;

  begin
    p := FilterSpec.filterSpan;
    m := (p - 1) div 2;
    Tcap := dataBOX.NOPUNCT;

    GetHendersonWeights(filterSpec, fIRVec);

    for t := 0 to Tcap - 1 do
      begin {t}
        if (t - m < 0) then
          begin {beginning weights}
  {At time t=0 we look forward to x[m+1]}
            r := t + m + 1;
            GetMusgraveWeights(FilterSpec, r, alpha, beta, uVec);

   {Presumably uVec contains the Musgrave endWeights}

            trendVector[t] := 0.0;
            for j := 1 to r do
              trendVector[t] := trendVector[t] + dataVector[j - 1] * uVec[r - j + 1];
          end {beginning weights}
        else if (t + m > Tcap - 1) then
          begin {ending weights}
            r := Tcap - t + m;
            GetMusgraveWeights(FilterSpec, r, alpha, beta, uVec);
            trendVector[t] := 0.0;
            for j := 1 to r do
              trendVector[t] := trendVector[t] + dataVector[t - m + j - 1] * uVec[j];
          end {ending weights}
        else
          begin {central weights}
            trendVector[t] := dataVector[t] * fIRVec[0];
            for j := 1 to m do
              trendVector[t] := trendVector[t] + fIRVec[j] * (dataVector[t - j] + dataVector[t + j]);
          end; {central weights}
        residueVec[t] := dataVector[t] - trendVector[t];
      end; {t}

    residueBOX := dataBOX;
    MakeDataBox(residueBOX, residueVec);

    legend := HendersonTrend(dataBOX, filterSpec, dataName);

    ScreenOverplot(dataBOX, SCREENBOX, dataVector, trendVector, legend);

    if (PSPict = true) then
      WritePostOverFile(dataBOX, POSTBOX, dataVector, trendVector, legend);

    legend := 'The deviations of the data sequence from the interpolated trend';

    ScreenMakePlot(residueBOX, SCREENBOX, residueVec, legend);

    weHaveATrend := true;
    weHaveTrendResiduals := true;

  end; {ApplyMusgraveFilter}

  {_________________________________________________}

  procedure ApplyFilterDirectly (var dataBOX, residueBOX: rBOX;
                  var filterSpec: FilterRecord;
                  var fIRVec: vector;
                  var dataVector, residueVec: longVector);

  {We use This  procedure to apply the Baxter-King bandpass filter to trended data}
  {Should we include <GetTruncIdealWeights(filterSpec, fIRVec);> instead of in BaxterKingFilterSpec;}

    var
      p, m, t, j, Tcap: integer;

  begin

    p := FilterSpec.filterSpan;
    m := (p - 1) div 2;

    Tcap := dataBOX.NOPUNCT;
    for t := 0 to Tcap - 1 do
      begin {t}
        if (t - m < 0) or (t + m > Tcap - 1) then
          residueVec[t] := 0.0 {i.e. the filter cannot reach the ends}
        else
          begin {if}
            residueVec[t] := dataVector[t] * fIRVec[0];
            for j := 1 to m do
              residueVec[t] := residueVec[t] + fIRVec[j] * (dataVector[t - j] + dataVector[t + j]);
          end; {if}
      end; {t}
    residueBOX := dataBOX;
    MakeDataBox(residueBOX, residueVec);

  end; {ApplyFilterDirectly}

  {_________________________________________________}

  procedure DiffenceFilterAndReinflate (var dataBOX, residueBOX: rBOX;
                  var filterSpec: FilterRecord;
                  var fIRVec: vector;
                  var dataVector, residueVec: longVector);
    var
      p, m, t, j, Tcap: integer;
      dataVectmj, dataVectpj: real;
      storeVec: longVector;
      storeBOX: rBOX;

  begin
  {We need to preserve the Data Vector for the reason that it is used in the process of reinflating}
    p := FilterSpec.filterSpan;
    m := (p - 1) div 2;
    storeVec := dataVector;
    storeBOX := dataBOX;
    DiffOfVector(storeBOX, storeVec);
    DiffOfVector(storeBOX, storeVec);
    Tcap := dataBOX.NOPUNCT;

    for t := 0 to Tcap - 1 do
      begin {what about padding the ends with zeros}
        residueVec[t] := 0.0;
        begin {if}
          residueVec[t] := storeVec[t] * fIRVec[0];
          for j := 1 to m do
            begin {j}

              if t - j < 0 then
                dataVectmj := 0.0
              else
                dataVectmj := storeVec[t - j];
              if t + j > Tcap - 1 then
                dataVectpj := 0.0
              else
                dataVectpj := storeVec[t + j];

              residueVec[t] := residueVec[t] + 2 * (dataVectmj + dataVectpj);
            end; {j}
        end; {if}
      end; {t}
  {Reinflate}
  {DataCumulation(dataBOX, trendBOX, trendDiffOrder, dataVector, trendVector);}
  {We need to move this From Fourier to DataSeg}
  {Should we use datacumultion of Cycumulation?}

  end; {DiffenceFilterAndReinflate}

  {_________________________________________________}

  procedure DifferenceReduceAndFilter (var dataBOX, residueBOX: rBOX;
                  var filterSpec: FilterRecord;
                  var fIRVec: vector;
                  var dataVector, residueVec: longVector);

  {We apply a reduced bandpass filter to twice differenced data}
  {residueVec contains the putitative business cycle}
  {Should we include <GetTruncIdealWeights(filterSpec, fIRVec);> instead of in BaxterKingFilterSpec;}
    var
      p, m, t, j, Tcap: integer;
      storeVectMj, storeVectPj: real;
      RIRVec: vector;
      storeVec: longVector;
      storeBOX: rBOX;
    label
      999;

  begin
    Tcap := dataBOX.NOPUNCT;
    p := FilterSpec.filterSpan;
    m := (p - 1) div 2;
    storeVec := dataVector;
    storeBOX := dataBOX;
    DoubleDifference(storeBOX, storeVec);{leave the original data undisturbed}

    RIRVec := fIRVec; {Reduced FIR vec}
    ReduceTheFilter(filterSpec, RIRVec);
    Tcap := storeBOX.NOPUNCT;

  {Tcap should be the same as original data order since }
  {we have placed zeros at either end of the differnced data}

  {-------------------------------------------------------}
    for t := 0 to Tcap - 1 do
      begin {t}
        if (t - m < 0) or (t + m > Tcap - 1) then
          residueVec[t] := 0.0 {i.e. the filter cannot reach the ends}
        else
          begin {else}
            residueVec[t] := storeVec[t] * RIRVec[0];
            for j := 1 to m - 1 do {m-1 because we are using the reduced filter}
              residueVec[t] := residueVec[t] + RIRVec[j] * (storeVec[t - j] + storeVec[t + j]);
          end; {else}
      end; {t}

    goto 999;
  {-------------------------------------------------------}
    for t := 0 to Tcap - 1 do
      begin {t}
        residueVec[t] := storeVec[t] * RIRVec[0];
        for j := 1 to m - 1 do {m-1 because we are using the reduced filter}
          begin {j}
            if (t - j < 0) then
              storeVectMj := 0.0
            else
              storeVectMj := storeVec[t - j];
            if (t + j > Tcap - 1) then
              storeVectPj := 0.0
            else
              storeVectPj := storeVec[t + j];
            residueVec[t] := residueVec[t] + RIRVec[j] * (storeVectMj + storeVectPj);
          end; {j}
      end; {t}
  {-------------------------------------------------------}
999:
    residueBOX := storeBOX;
    MakeDataBox(residueBOX, residueVec);

  end; {DifferenceReduceAndFilter}

  {_________________________________________________}

  procedure ApplyTruncFilter (var dataBOX, cycleBOX: rBOX;
                  var filterSpec: FilterRecord;
                  var dataVector, cycleVec: longVector);

    var
      firVec: vector;
      postFileName, legend: string;
      F: text;

  begin
    if thereIsData = false then
      begin
        SysBeep(3);
        ReadTheData(dataBOX, dataVector);
      end;

    if noPageParameters then
      begin
        SysBeep(3);
        PageSetUp(SCREENBOX, POSTBOX, PSPict, TeXFormat);
      end;

    GetTruncIdealWeights(filterSpec, fIRVec);

    if filterSpec.trendMethod = ApplyDirect then
      ApplyFilterDirectly(dataBOX, cycleBOX, filterSpec, fIRVec, dataVector, cycleVec);
    if filterSpec.trendMethod = DiffAndReinflate then
      DiffenceFilterAndReinflate(dataBOX, cycleBOX, filterSpec, fIRVec, dataVector, cycleVec);
    if filterSpec.trendMethod = DiffAndReduce then
      DifferenceReduceAndFilter(dataBOX, cycleBOX, filterSpec, fIRVec, dataVector, cycleVec);

    GetGraphics(graphicsOn, SCREENBOX, currentMode);
    ScreenFrame(dataBOX, SCREENBOX);
    ScreenPlot(dataBOX, SCREENBOX, dataVector);

    legend := DataLegend(dataBOX, dataName);
    ScreenCaption(legend);

    CallForReturn;
    Readln;
    CloseGraph;
    RestoreCrtMode;
    ClrScr;

    GetGraphics(graphicsOn, SCREENBOX, currentMode);
    ScreenFrame(cycleBOX, SCREENBOX);
    MarkFilterLimits(filterSpec, cycleBOX, SCREENBOX);
    ScreenPlot(cycleBOX, SCREENBOX, cycleVec);

    legend := BaxterKingCycle(dataBOX, dataName);
    ScreenCaption(legend);

    CallForReturn;
    Readln;
    CloseGraph;
    RestoreCrtMode;
    ClrScr;

    if (PSPict = true) then
      begin {PSPict}
        Write('    Name the PostScript File = ');
        Readln(postFileName);
        if postFileName <> '' then
          begin {Postfile}
            Assign(F, postFileName);
            Rewrite(F);
            PostHeader(F, POSTBOX, TeXFormat);
            POSTBOX.xAxis := true;
            POSTBOX.yAxis := true;
            POSTBOX.doFrame := true;
            POSTBOX.doMarks := true;
            PostFrame(F, cycleBOX, POSTBOX);
            PostPlot(F, cycleBOX, POSTBOX, cycleVec);
            PostFilterLimits(F, filterSpec, cycleBOX, POSTBOX);
            PostFooter(F, POSTBOX, legend, TeXFormat);
            Close(F);
          end; {Postfile}
      end; {PSPict}

    weHaveABusinessCycle := true;

  end; {ApplyTruncFilter}

  {________________________________________________}

  procedure ApplyChristFitzFilter (var filterSpec: FilterRecord;
                  var dataBOX, cycleBOX: rBOX;
                  var dataVector, cycleVec: longVector);
    var
      t, j, q, Tcap: integer;
      A, B, alpha, beta: real;
      furlongVec, dataStore: longVector;
      storeBOX: rBOX;
      legend: string;

  begin

    if thereIsData = false then
      begin
        SysBeep(3);
        ReadTheData(dataBOX, dataVector);
      end;

    if noPageParameters then
      begin
        SysBeep(3);
        PageSetUp(SCREENBOX, POSTBOX, PSPict, TeXFormat);
      end;

    Tcap := dataBOX.NOPUNCT;

    legend := DataLegend(dataBOX, dataName);
    ScreenMakePlot(dataBOX, SCREENBOX, dataVector, legend);

    if (filterSpec.ChristFitzData in [DriftingWalk, EndPointDrift]) then
      begin
        beta := (dataVector[Tcap - 1] - dataVector[0]) / (Tcap - 1);
        alpha := dataVector[0];

        for t := 0 to Tcap - 1 do
          dataStore[t] := dataVector[t] - t * beta - alpha;
      end;
    if (filterSpec.ChristFitzData = EndYearDrift) then
      begin
        EndYearInterpolation(alpha, beta, dataBox, dataVector); {in DATASEG}
        for t := 0 to Tcap - 1 do
          dataStore[t] := dataVector[t] - t * beta - alpha;
      end;

    storeBOX := dataBOX;
    MakeDataBox(storeBOX, dataStore);

    legend := 'The residual sequence from the linear detrending of the data';
    ScreenMakePlot(storeBOX, SCREENBOX, dataStore, legend);

    GetFurlongWeights(filterSpec, dataBOX, FurlongVec);

    for t := 0 to Tcap - 1 do
      begin {t}
        cycleVec[t] := 0.0;
        A := furlongVec[0] / 2;
        B := A;
        for j := 0 to Tcap - 1 do
          begin {j}
            q := Abs(t - j);
            if j < t then
              A := A + furlongVec[j + 1];
            if j > t then
              B := B + furlongVec[j];
            cycleVec[t] := cycleVec[t] + dataStore[j] * furlongVec[q];
          end; {j}
        if filterSpec.ChristFitzData = RandomWalk then
          cycleVec[t] := cycleVec[t] - A * dataVector[0] - B * dataVector[Tcap - 1];
      end; {t}

    cycleBOX := dataBOX;
    MakeDataBox(cycleBOX, cycleVec);

    legend := ChristFitzCycle(dataBOX, dataName);
    ScreenMakePlot(cycleBOX, SCREENBOX, cycleVec, legend);

    if (PSPict = true) then
      WritePostFile(cycleBOX, POSTBOX, cycleVec, legend);

  {+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
  {We wish to overaly the cycleVec upon the storeVec}


    legend := 'The cyclical fluctuations extracted by the Christiano-Fitzgerald';
    legend := Concat(legend, 'bandpass filter superimposed upon the data sequence');
    ScreenOverplot(storeBOX, SCREENBOX, dataStore, cycleVec, legend);


    if (PSPict = true) then
      begin {PSPict}
        legend := 'A filtered sequence obtained from';
        legend := Concat(legend, dataName);
        legend := Concat(legend, ' by applying the Christiano--Fitzgerald bandpass filter.');
        WritePostOverFile(storeBOX, POSTBOX, dataStore, cycleVec, legend);
      end; {PSPict}
  {+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}

    weHaveABusinessCycle := true;

  end; {ChristFitzFilter}

  {________________________________________________}

  procedure PlotFIRBPassGain (var SCREENBOX: IBOX;
                  var filterSpec: FilterRecord);

  {This procedure Organises the Plotting of the Gain of the Filters}

    var
      firVec: vector;
      filterVector: longVector;
      filterBOX: rBOX;
      postFileName, legend: string;
      F: text;
      theMenuItem: integer;

  {is FIRVector Global? Yes it is, But we shodul declare it in the integface or make it Local}
  begin
    ClrScr;

    if noPageParameters then
      begin
        SysBeep(3);
        PageSetUp(SCREENBOX, POSTBOX, PSPict, TeXFormat);
        ClrScr;
      end;

    Writeln;
    Writeln('    FREQUENCY RESPONSE OF THE TRUNCATED BANDPASS FILTER');
    Writeln;
    Writeln('    The  coefficients of the truncated bandpass filter may be adjusted');
    Writeln('    to ensure that there is zero gain  at zero frequency. The adjusted');
    Writeln('    filter incorporates the twofold differencing factor (1 - z)^2. The');
    Writeln('    differencing  operator may be  applied directly to the data, after');
    Writeln('    which a reduced filter may be applied to the differenced data. The');
    Writeln('    reduced filter is obtained  by dividing the adjusted filter by the');
    Writeln('    factor (1 - z)^2.');
    Writeln;
    Writeln('        1. Plot the frequency respose of the trucated filter');
    Writeln;
    Writeln('        2. Plot the frequency response of the adjusted filter');
    Writeln;
    Writeln('        3. Plot the frequency response of the reduced filter');
    Writeln;
    Writeln('                      Select a number and press RETURN');
    Writeln;
    GetBoundedInteger(1, 3, theMenuItem);

    case theMenuItem of
      1: 
        begin {1}
          filterSpec.truncAdjustment := false;
          GetTruncIdealWeights(filterSpec, fIRVec);  {in FIRFLTR.PAS}
        end; {1}
      2: 
        begin {2}
          filterSpec.truncAdjustment := true;
          GetTruncIdealWeights(filterSpec, fIRVec);  {in FIRFLTR.PAS}
        end; {2}
      3: 
        begin {3}
          filterSpec.truncAdjustment := true;
          GetTruncIdealWeights(filterSpec, fIRVec); {in FIRFLTR,PAS}
          ReduceTheFilter(filterSpec, fIRVec);  {in FIRFLTR.PAS}
        end; {3}
    end;{case}

    Writeln;
    Writeln('    FREQUENCY RESPONSE OR AMPLITUDE  SPECTRUM');
    Writeln;
    Writeln('    Do you wish to plot the real-valued frequency response (F) of the ');
    Writeln('    symmetric filter or its non-negative amplitude response (A)? F/A');
    ReadAChar(FilterSpec.gainChar);
    FilterSpec.gainChar := UpCase(FilterSpec.gainChar);

    GetFilterGainVector(filterSpec, filterBOX, SCREENBOX, firVec, filterVector); {in FILTRS}

    GetGraphics(graphicsOn, SCREENBOX, currentMode);

    MakeDataBox(filterBOX, filterVector); {THIS SHOULD be achieved in <GetFilterGainVector>}

    SCREENBOX.xAxis := false;
    ScreenFrame(filterBOX, SCREENBOX);
    ZeroPiAxis(SCREENBOX);
    ScreenPlot(filterBOX, SCREENBOX, filterVector);
    if theMenuItem in [1, 2] then
      BoxCarPlot(filterBOX, SCREENBOX, filterSpec);

    legend := BaxterKingGain(filterSpec);
    ScreenCaption(legend);

    CallForReturn;
    Readln;
    CloseGraph;
    RestoreCrtMode;
    ClrScr;

    if PSPict then
      begin {PSPict}
        Write('    Name the PostScript File = ');
        Readln(postFileName);
        if postFileName <> '' then
          begin {Postfile}
            Assign(F, postFileName);
            Rewrite(F);
            PostHeader(F, POSTBOX, TeXFormat);
            POSTBOX.xAxis := false;
            PostFrame(F, filterBOX, POSTBOX);
            PostZeroPiAxis(F, POSTBOX);
            PostPlot(F, filterBOX, POSTBOX, filterVector);
            if filterSpec.filterType = bandPass then
              PostCarPlot(F, filterBOX, POSTBOX, filterSpec);
            PostGainFooter(F, POSTBOX, filterSpec, TeXFormat);
            Close(F);
          end;
      end; {PSPict}

    SCREENBOX.xAxis := true; {Restore the flag}
    POSTBOX.xAxis := true;

  end; {PlotFIRBPassGain}

  {________________________________________________}

  procedure PlotHendersonGain (var SCREENBOX: IBOX;
                  var filterSpec: FilterRecord);

    var
      firVec: vector;
      gainVector: longVector;
      gainBOX: rBOX;
      postFileName, legend: string;
      F: text;

  begin
    ClrScr;
    if noPageParameters then
      begin
        SysBeep(3);
        PageSetUp(SCREENBOX, POSTBOX, PSPict, TeXFormat);
        ClrScr;
      end;

    Writeln;
    Writeln('    FREQUENCY RESPONSE OR AMPLITUDE  SPECTRUM');
    Writeln;
    Writeln('    Do you wish to plot the real-valued frequency response (F) of the ');
    Writeln('    symmetric filter or its non-negative amplitude response (A)? F/A');
    ReadAChar(FilterSpec.gainChar);
    FilterSpec.gainChar := UpCase(FilterSpec.gainChar);

    GetHendersonWeights(filterSpec, fIRVec);
    GetFilterGainVector(filterSpec, gainBOX, SCREENBOX, firVec, gainVector); {in FILTERS}

    GetGraphics(graphicsOn, SCREENBOX, currentMode);

    SCREENBOX.xAxis := false;
    ScreenFrame(gainBOX, SCREENBOX);
    ZeroPiAxis(SCREENBOX);
    ScreenPlot(gainBOX, SCREENBOX, gainVector);
    if (filterSpec.gainChar = 'F') then
      begin{Draw the horizontal axis}
        MoveTo(SCREENBOX.X1, SCREENBOX.Y0);
        LineTo(SCREENBOX.X2, SCREENBOX.Y0);
      end;

    legend := HendersonGain(filterSpec);
    ScreenCaption(legend);

    CallForReturn;
    Readln;
    CloseGraph;
    RestoreCrtMode;
    ClrScr;

    if PSPict then
      begin {PSPict}
        Write('    Name the PostScript File = ');
        Readln(postFileName);
        if postFileName <> '' then
          begin {Postfile}
            Assign(F, postFileName);
            Rewrite(F);
            PostHeader(F, POSTBOX, TeXFormat);
            POSTBOX.xAxis := false;
            PostFrame(F, gainBOX, POSTBOX);
            PostZeroPiAxis(F, POSTBOX);
            if (filterSpec.gainChar = 'F') then
              begin{Draw the horizontal axis}
                Writeln(F, 'newpath');
                Writeln(F, POSTBOX.X0 : 3, '  ', POSTBOX.Y0 : 3, '   moveto');
                Writeln(F, POSTBOX.X0 : 3, '  ', POSTBOX.Y0 : 3, '   lineto');
                Writeln(F, 'stroke');
              end;
            PostPlot(F, gainBOX, POSTBOX, gainVector);
            PostGainFooter(F, POSTBOX, filterSpec, TeXFormat);
            Close(F);
          end;
      end; {PSPict}

    SCREENBOX.xAxis := true; {Restore the flag}
    POSTBOX.xAxis := true;

  end; {PlotHendersonGain}

  {________________________________________________}

end. {FIRFLTR: Unit 14 of NEWIDEOLOG.PAS}

