unit FOURIER; {Unit 10 of NEWIDEOLOG.PAS}

interface

  uses
    GLOBALS, MATHS, UTILS;

  procedure FourierCoefficients (var dataBOX: rBOX;
                  var dataVector: longVector;
                  FourierInc: real;
                  cutOffInt: integer;
                  var alpha: fourierVec;
                  var beta: fourierVec);

  procedure BandPassCoefficients (var dataBOX: rBOX;
                  var dataVector: longVector;
                  FourierInc: real;
                  lowerCutInt, upperCutInt: integer;
                  var alpha: fourierVec;
                  var beta: fourierVec);

  procedure InflateFourierCoefficients (diffOrder: integer;
                  lowerCutInt, upperCutInt: integer;
                  fourierInc: real;
                  var alpha: fourierVec;
                  var beta: fourierVec);

  procedure FourierSynthesis (var dataBOX: rBOX;
                  var dataVector: longVector;
                  FourierInc: real;
                  cutOffInt: integer;
                  var alpha: fourierVec;
                  var beta: fourierVec);

  procedure BandpassSynthesis (var dataBOX: rBOX;
                  var dataVector: longVector;
                  FourierInc: real;
                  lowerCutInt, upperCutInt: integer;
                  var alpha: fourierVec;
                  var beta: fourierVec);

  procedure DataCumulation (var dataBOX, diffBOX: rBOX;
                  diffOrder: integer;
                  var dataVector, diffVector: longVector);

  procedure Cycumulation (var dataBOX: rBOX;
                  var diffBOX: rBOX;
                  diffOrder: integer;
                  var dataVector, diffVector: longVector);

  procedure StandardPoints (var dataBOX: rBOX;
                  var frequencyPoints: vector;
                  var numbofPoints: integer);

  procedure SupplementaryPoints (var dataBOX: rBOX;
                  var fourierInc: real;
                  var frequencyPoints: vector;
                  var numbofPoints: integer);

  procedure SeasonalCoefficients (var dataBOX: rBOX;
                  var dataVector: longVector;
                  var frequencyPoints: vector;
                  var numbofPoints: integer;
                  var alpha: fourierVec;
                  var beta: fourierVec);

  procedure InflateSeasonalCoefficients (var frequencyPoints: vector;
                  var numbofPoints: integer;
                  var alpha: fourierVec;
                  var beta: fourierVec);

  procedure SeasonalSynthesis (dataBOX: rBOX;
                  var seasonalVector: longVector;
                  var frequencyPoints: vector;
                  var numbofPoints: integer;
                  var alpha: fourierVec;
                  var beta: fourierVec);

  procedure SeasonalCumulation (var dataBOX: rBOX;
                  var diffBOX: rBOX;
                  diffOrder: integer;
                  var diffVector: longVector);

  procedure Variance (var dataBOX: rBOX;
                  var dataVector: longVector);

implementation

  {_________________________________________________}

  procedure FourierCoefficients (var dataBOX: rBOX;
                  var dataVector: longVector;
                  FourierInc: real;
                  cutOffInt: integer;
                  var alpha: fourierVec;
                  var beta: fourierVec);
  {Lowpass Fourier Coefficients}
    var
      t, j: integer;
      omegaj, omegajt: real;

  begin
    alpha[0] := 0.0;
    for t := 0 to dataBOX.NOPUNCT - 1 do
      alpha[0] := alpha[0] + dataVector[t];
    alpha[0] := alpha[0] / dataBOX.NOPUNCT;

    for j := 1 to cutOffInt do
      begin
        omegaj := FourierInc * j;
        alpha[j] := 0.0;
        beta[j] := 0.0;
        for t := 0 to dataBOX.NOPUNCT - 1 do
          begin
            omegajt := omegaj * t;
            alpha[j] := alpha[j] + Cos(omegajt) * dataVector[t];
            beta[j] := beta[j] + Sin(omegajt) * dataVector[t];
          end;
        alpha[j] := alpha[j] * 2 / dataBOX.NOPUNCT;
        beta[j] := beta[j] * 2 / dataBOX.NOPUNCT;
      end;

  end; {FourierCoefficients}

  {_________________________________________________}

  procedure BandPassCoefficients (var dataBOX: rBOX;
                  var dataVector: longVector;
                  FourierInc: real;
                  lowerCutInt, upperCutInt: integer;
                  var alpha: fourierVec;
                  var beta: fourierVec);

    var
      t, j: integer;
      omegaj, omegajt: real;

  begin

    for j := 0 to upperCutInt do
      begin
        alpha[j] := 0.0;
        beta[j] := 0.0;
      end; {This is unnecssary}

    alpha[0] := 0.0;
    if lowerCutInt = 0 then
      begin
        for t := 0 to dataBOX.NOPUNCT - 1 do
          alpha[0] := alpha[0] + dataVector[t];
        alpha[0] := alpha[0] / dataBOX.NOPUNCT;
      end;

    for j := lowerCutInt to upperCutInt do
      begin {j}
        omegaj := FourierInc * j;
        alpha[j] := 0.0;
        beta[j] := 0.0;
        for t := 0 to dataBOX.NOPUNCT - 1 do
          begin {t}
            omegajt := omegaj * t;
            alpha[j] := alpha[j] + Cos(omegajt) * dataVector[t];
            beta[j] := beta[j] + Sin(omegajt) * dataVector[t];
          end; {t}
        alpha[j] := alpha[j] * 2 / dataBOX.NOPUNCT;
        beta[j] := beta[j] * 2 / dataBOX.NOPUNCT;
      end; {j}
    if (2 * upperCutInt) = dataBOX.NOPUNCT then
      begin
        j := upperCutInt;
        alpha[j] := alpha[j] / 2;
        beta[j] := 0.0;
      end;

  end; {BandPassCoefficients}

  {_________________________________________________}

  procedure InflateFourierCoefficients (diffOrder: integer;
                  lowerCutInt, upperCutInt: integer;
                  fourierInc: real;
                  var alpha: fourierVec;
                  var beta: fourierVec);
    var
      omegaj, gainFactor: real;
      j: integer;

  {This procedure applies the gain factor of the cumulation}
   {operator to the Fourier coefficients of the data}

  begin
    if diffOrder > 0 then
      begin
        for j := lowerCutInt to upperCutInt do
          begin {j}
            omegaj := fourierInc * j;
            if diffOrder = 2 then
              gainFactor := 2 * Cos(omegaj) - 2;
            gainFactor := 1 / gainFactor;
            alpha[j] := alpha[j] * gainFactor;
            beta[j] := beta[j] * gainFactor;
          end; {j}
      end; {diffOrder >0}

  end; {InflateFourierCoefficients}
  {________________________________________________}

  procedure FourierSynthesis (var dataBOX: rBOX;
                  var dataVector: longVector;
                  fourierInc: real;
                  cutOffInt: integer;
                  var alpha: fourierVec;
                  var beta: fourierVec);

    var
      t, j: integer;
      omegat, omegajt, yMax, yMin: real;

  begin
    yMax := 0.0;
    yMin := 0.0;

    for t := 0 to dataBOX.NOPUNCT - 1 do
      begin {t}
        dataVector[t] := alpha[0];
        omegat := fourierInc * t;
        for j := 1 to cutOffInt do
          begin {j}
            omegajt := omegat * j;
            dataVector[t] := dataVector[t] + alpha[j] * Cos(omegajt);
            dataVector[t] := dataVector[t] + beta[j] * Sin(omegajt);
          end; {j}
        yMax := Rmax(ymax, dataVector[t]);
        yMin := Rmin(ymin, dataVector[t]);
      end;{t}

    dataBOX.yMax := yMax;
    dataBOX.yMin := yMin;
    dataBOX.xMax := dataBOX.NOPUNCT - 1;  {the maximum value of x in the dataVector}
    dataBOX.xMin := 0.0;   {the minimum value of x in the dataVector}
    dataBOX.xLB := dataBOX.xMin;   {xLB <= xMin: a left (lower) bound for x}
    dataBOX.xRB := dataBOX.xMax;    {xRB >= xMax: a right (upper) bound for x}

  end; {FourierSynthesis}

  {_________________________________________________}

  procedure BandPassSynthesis (var dataBOX: rBOX;
                  var dataVector: longVector;
                  fourierInc: real;
                  lowerCutInt, upperCutInt: integer;
                  var alpha: fourierVec;
                  var beta: fourierVec);

    var
      t, j: integer;
      omegat, omegajt, yMax, yMin: real;

  begin
    yMax := 0.0;
    yMin := 0.0;

    for t := 0 to dataBOX.NOPUNCT - 1 do
      begin {t}
        omegat := fourierInc * t;
        dataVector[t] := 0.0;
        for j := lowerCutInt to upperCutInt do
          begin {j}
            omegajt := omegat * j;
            dataVector[t] := dataVector[t] + alpha[j] * Cos(omegajt);
            dataVector[t] := dataVector[t] + beta[j] * Sin(omegajt);
          end; {j}
        yMax := Rmax(ymax, dataVector[t]);
        yMin := Rmin(ymin, dataVector[t]);
      end;{t}

    dataBOX.yMax := yMax;
    dataBOX.yMin := yMin;
    dataBOX.xMax := dataBOX.NOPUNCT - 1;  {the maximum value of x in the dataVector}
    dataBOX.xMin := 0.0;   {the minimum value of x in the dataVector}
    dataBOX.xLB := dataBOX.xMin;   {xLB <= xMin: a left (lower) bound for x}
    dataBOX.xRB := dataBOX.xMax;    {xRB >= xMax: a right (upper) bound for x}

  end; {BandPassSynthesis}

 {_________________________________________________}

  procedure DataCumulation (var dataBOX, diffBOX: rBOX;
                  diffOrder: integer;
                  var dataVector, diffVector: longVector);

 {diffBOX.NOPUNCT is increased by the order of cumulation}
 {dataBOX.NOPUNCT is unaltered}
 {diffVector is cumulated, dataVector is unaltered }

    var
      t: integer;
      sum0, sum1: real;
      det, s11, s12, s21, s22, b11, b12, b21, b22, Tcap: double;
      diff, zStar0, zStar1: real;

  begin

   {----------------------------------------------------------------}
    if (diffOrder = 1) then
      begin
       {Do recursive cumulation}
        for t := 1 to diffBOX.NOPUNCT - 1 do
          diffVector[t] := diffVector[t] + diffVector[t - 1];

         {Shift the cumulated data to the right}
        for t := diffBOX.NOPUNCT downto 1 do
          diffVector[t] := diffVector[t - 1];
        diffVector[0] := 0;

         {Find the initial condition}
        sum0 := 0.0;
        for t := 0 to dataBOX.NOPUNCT - 1 do
          sum0 := sum0 + dataVector[t] - diffVector[t];

        zStar0 := sum0 / dataBOX.NOPUNCT;

         {Add  S*z* to Sz}
        for t := 0 to dataBOX.NOPUNCT - 1 do
          diffVector[t] := diffVector[t] + zStar0;

        diffBOX.NOPUNCT := diffBOX.NOPUNCT + 1;
      end; {diffOrder = 1}
  {----------------------------------------------------------------}

  {----------------------------------------------------------------}
    if (diffOrder = 2) then
      begin
       {Do recursive cumulation}
        diffVector[1] := diffVector[1] + 2 * diffVector[0];
        for t := 2 to diffBOX.NOPUNCT - 1 do
          diffVector[t] := diffVector[t] + 2 * diffVector[t - 1] - diffVector[t - 2];

         {Shift the cumulated data two steps to the right}
        for t := diffBOX.NOPUNCT + 1 downto 2 do
          diffvector[t] := diffvector[t - 2];
        diffvector[1] := 0;
        diffvector[0] := 0;

       {Find the initial condition: observe that this depdens upon}
       {our having already cumulated the data}

        sum0 := dataVector[0];
        sum1 := 0.0;
        for t := 1 to dataBOX.NOPUNCT - 1 do
          begin {t}
            diff := dataVector[t] - diffVector[t]; {the vector y-Sz}
            sum0 := sum0 + (t + 1) * diff;
            sum1 := sum1 + t * diff;
          end; {t}

        Tcap := dataBOX.NOPUNCT;

        s11 := Tcap * (Tcap + 1) * (2 * Tcap + 1) / 6.0;
        Tcap := Tcap - 1;
        s12 := Tcap * (Tcap + 1) * (2 * Tcap + 1) / 6.0 + Tcap * (Tcap + 1) / 2;
        s21 := s12;
        s22 := Tcap * (Tcap + 1) * (2 * Tcap + 1) / 6.0;
        det := s11 * s22 - s12 * s21;

        b11 := s22 / det;
        b12 := -s12 / det;
        b21 := -s21 / det;
        b22 := s11 / det;

        zStar0 := b11 * sum0 + b12 * sum1;
        zStar1 := b21 * sum0 + b22 * sum1;

         {Add  S*z* to Sz}
        for t := 0 to dataBOX.NOPUNCT - 1 do
          diffVector[t] := diffVector[t] + zStar0 * (t + 1) + zStar1 * t;

        diffBOX.NOPUNCT := diffBOX.NOPUNCT + 2;
      end; {diffOrder = 2}
  {----------------------------------------------------------------}

  end; {DataCumulation}

  {_________________________________________________}

  procedure Cycumulation (var dataBOX: rBOX;
                  var diffBOX: rBOX;
                  diffOrder: integer;
                  var dataVector, diffVector: longVector);

  {This procedure cumulates the data in a manner that omits}
  {the polynomial component that is present in a cumulated tend.}

    var
      t: integer;
      sum0, sum1: real;
      det, s11, s12, s21, s22, b11, b12, b21, b22, Tcap: double;
      diff, zStar0, zStar1: real;

  begin
    if (diffOrder = 1) then
      begin
        for t := 1 to diffBOX.NOPUNCT - 1 do
          diffVector[t] := diffVector[t] + diffVector[t - 1];

        for t := diffBOX.NOPUNCT downto 1 do {Shift the registers}
          diffVector[t] := diffVector[t - 1];
        diffVector[0] := 0;

        sum0 := 0.0;
        for t := 0 to dataBOX.NOPUNCT - 1 do
          sum0 := sum0 - diffVector[t];
  {sum0 := sum0 + dataVector[t] - diffVector[t];}

        zStar0 := sum0 / dataBOX.NOPUNCT;

        for t := 0 to dataBOX.NOPUNCT - 1 do
          diffVector[t] := diffVector[t] + zStar0;
        DiffBOX.NOPUNCT := dataBOX.NOPUNCT;  {i.e. DiffBOX.NOPUNCT := DiffBOX.NOPUNCT +1}
      end; {diffOrder = 1}

    if (diffOrder = 2) then
      begin
        diffVector[1] := diffVector[1] + 2 * diffVector[0];
        for t := 2 to diffBOX.NOPUNCT - 1 do
          diffVector[t] := diffVector[t] + 2 * diffVector[t - 1] - diffVector[t - 2];

        for t := diffBOX.NOPUNCT + 1 downto 2 do {Shift the registers}
          diffvector[t] := diffvector[t - 2];
        diffvector[1] := 0;
        diffvector[0] := 0;

  {sum0 := dataVector[0];}
        sum0 := 0.0;
        sum1 := 0.0;
        for t := 1 to dataBOX.NOPUNCT - 1 do
          begin {t}
            diff := -diffVector[t];
  {diff := dataVector[t] - diffVector[t];}
            sum0 := sum0 + (t + 1) * diff;
            sum1 := sum1 + t * diff;
          end; {t}

        Tcap := dataBOX.NOPUNCT;

        s11 := Tcap * (Tcap + 1) * (2 * Tcap + 1) / 6.0;
        Tcap := Tcap - 1;
        s12 := Tcap * (Tcap + 1) * (2 * Tcap + 1) / 6.0 + Tcap * (Tcap + 1) / 2;
        s21 := s12;
        s22 := Tcap * (Tcap + 1) * (2 * Tcap + 1) / 6.0;
        det := s11 * s22 - s12 * s21;

        b11 := s22 / det;
        b12 := -s12 / det;
        b21 := -s21 / det;
        b22 := s11 / det;

        zStar0 := b11 * sum0 + b12 * sum1;
        zStar1 := b21 * sum0 + b22 * sum1;

        for t := 0 to dataBOX.NOPUNCT - 1 do
          diffVector[t] := diffVector[t] + zStar0 * (t + 1) + zStar1 * t;
        DiffBOX.NOPUNCT := dataBOX.NOPUNCT; {i.e. DiffBOX.NOPUNCT := DiffBOX.NOPUNCT + 2}
      end; {diffOrder = 2}

  end; {Cycumulation}

  {________________________________________________}

  procedure StandardPoints (var dataBOX: rBOX;
                  var frequencyPoints: vector;
                  var numbofPoints: integer);

  {This procedure relates to the extraction of the seasonal component}

    var
      pi, piDiv6: real;
      j: integer;

  begin
    pi := 4.0 * ArcTan(1.0);
    piDiv6 := pi / 6.0;

    if (dataBOX.dataFrequency = Quarterly) then
      begin {quarterly data}
        frequencyPoints[1] := pi / 2.0;
        frequencyPoints[2] := pi;
        numbofPoints := 2
      end; {quarterly data}

    if (dataBOX.dataFrequency = Monthly) then
      begin {monthly data}
        for j := 1 to 6 do
          frequencyPoints[j] := j * piDiv6;
        numbofPoints := 6;
      end; {monthly data}

  end; {StandardPoints}

  {________________________________________________}

  procedure SupplementaryPoints (var dataBOX: rBOX;
                  var fourierInc: real;
                  var frequencyPoints: vector;
                  var numbofPoints: integer);

  {This procedure relates to the extraction of the seasonal component}

    var
      pi, piDiv6, fHarmonic: real;
      j, k, q, addPoints: integer;
      mHarmonic: integer;

  begin
    q := 0;
    pi := 4.0 * ArcTan(1.0);
    piDiv6 := pi / 6.0;

    if (dataBOX.dataFrequency = Quarterly) then
      begin {quarterly data}

        fHarmonic := pi / 2.0;
        Writeln('   How many additional Fourier points below 90 degrees?');
        GetBoundedInteger(0, 5, addPoints);
        for j := addPoints downto 0 do
          begin
            q := q + 1;
            frequencyPoints[q] := fHarmonic - j * fourierInc;
          end;

        Writeln('   How many additional Fourier points above 90 degrees?');
        GetBoundedInteger(0, 5, addPoints);
        for j := 1 to addPoints do
          begin {j}
            q := q + 1;
            frequencyPoints[q] := fHarmonic + j * fourierInc;
          end; {j}

        fHarmonic := pi;
        Writeln('   How many additional Fourier points below 180 degrees?');
        GetBoundedInteger(0, 5, addPoints);
        for j := addPoints downto 0 do
          begin {j}
            q := q + 1;
            frequencyPoints[q] := fHarmonic - j * fourierInc;
          end; {j}

      end; {quarterly data}

    if (dataBOX.dataFrequency = Monthly) then
      begin {monthly data}
        for k := 1 to 6 do
          begin {k}
            mHarmonic := k * 30;
            fHarmonic := k * piDiv6;
            Writeln('    How many additional Fourier points below ', mHarmonic : 2, ' degrees?');
            GetBoundedInteger(0, 5, addPoints);
            for j := addPoints downto 0 do
              begin {below}
                q := q + 1;
                frequencyPoints[q] := fHarmonic - j * fourierInc;
              end; {below}

            if k < 6 then
              begin {k < 5}
                Writeln('    How many additional Fourier points above ', mHarmonic : 2, ' degrees?');
                GetBoundedInteger(0, 5, addPoints);
                for j := 1 to addPoints do
                  begin {above}
                    q := q + 1;
                    frequencyPoints[q] := fHarmonic + j * fourierInc;
                  end; {above}
              end; {k < 5}

          end; {k}
      end; {monthly data}

    numbofPoints := q;

  end; {SupplementaryPoints}

  {________________________________________________}

  procedure SeasonalCoefficients (var dataBOX: rBOX;
                  var dataVector: longVector;
                  var frequencyPoints: vector;
                  var numbofPoints: integer;
                  var alpha: fourierVec;
                  var beta: fourierVec);

    var
      omegaj, omegajt: real;
      j, p, t, sign: integer;

  begin
    p := numbofPoints;

    for j := 1 to p - 1 do
      begin
        omegaj := frequencyPoints[j];
        alpha[j] := 0.0;
        beta[j] := 0.0;
        for t := 0 to dataBOX.NOPUNCT - 1 do
          begin
            omegajt := omegaj * t;
            alpha[j] := alpha[j] + Cos(omegajt) * dataVector[t];
            beta[j] := beta[j] + Sin(omegajt) * dataVector[t];
          end;
        alpha[j] := alpha[j] * 2 / dataBOX.NOPUNCT;
        beta[j] := beta[j] * 2 / dataBOX.NOPUNCT;
      end;

    alpha[p] := 0.0;
    sign := -1;
    for t := 0 to dataBOX.NOPUNCT - 1 do
      begin
        sign := -sign;
        alpha[p] := alpha[p] + sign * dataVector[t];
      end;
    alpha[p] := alpha[p] / dataBOX.NOPUNCT;

  end; {SeasonalCoefficients}

  {________________________________________________}

  procedure InflateSeasonalCoefficients (var frequencyPoints: vector;
                  var numbofPoints: integer;
                  var alpha: fourierVec;
                  var beta: fourierVec);
    var
      omegaj, gainFactor: real;
      j, p: integer;

 {This procedure applies the gain factor of the twofold cumulation}
 {operator to the Fourier coefficients of the seasonal component}

  begin
    p := numbofPoints; {The number of elements in the Fourier synthesis}

    for j := 1 to p do
      begin {j}
        omegaj := frequencyPoints[j];
        gainFactor := 2 * Cos(omegaj) - 2;
        gainFactor := 1 / gainFactor;
        alpha[j] := alpha[j] * gainFactor;
        beta[j] := beta[j] * gainFactor;
      end; {j}

  end; {InflateSeasonalCoefficients}
 {________________________________________________}

  procedure SeasonalSynthesis (dataBOX: rBOX;
                  var seasonalVector: longVector;
                  var frequencyPoints: vector;
                  var numbofPoints: integer;
                  var alpha: fourierVec;
                  var beta: fourierVec);

    var
      p, t, j, sign: integer;
      omegaj, omegajt, yMax, yMin: real;

  begin
    p := numbofPoints;
    yMax := 0.0;
    yMin := 0.0;

    sign := -1;
    for t := 0 to dataBOX.NOPUNCT - 1 do
      begin {t}
        seasonalVector[t] := 0.0;
        sign := -sign;
        for j := 1 to p - 1 do
          begin {j}
            omegaj := frequencyPoints[j];
            omegajt := omegaj * t;
            seasonalVector[t] := seasonalVector[t] + alpha[j] * Cos(omegajt);
            seasonalVector[t] := seasonalVector[t] + beta[j] * Sin(omegajt);
          end; {j}
        seasonalVector[t] := seasonalVector[t] + sign * alpha[p];
        yMax := Rmax(ymax, seasonalVector[t]);
        yMin := Rmin(ymin, seasonalVector[t]);
      end;{t}

    dataBOX.yMax := yMax;
    dataBOX.yMin := yMin;
    dataBOX.xMax := dataBOX.NOPUNCT - 1;  {the maximum value of x in the dataVector}
    dataBOX.xMin := 0.0;   {the minimum value of x in the dataVector}
    dataBOX.xLB := dataBOX.xMin;   {xLB <= xMin: a left (lower) bound for x}
    dataBOX.xRB := dataBOX.xMax;    {xRB >= xMax: a right (upper) bound for x}

  end; {SeasonalSynthesis}

  {________________________________________________}

  procedure SeasonalCumulation (var dataBOX: rBOX;
                  var diffBOX: rBOX;
                  diffOrder: integer;
                  var diffVector: longVector);

    var
      t: integer;
      sum0, sum1: real;
      det, s11, s12, s21, s22, b11, b12, b21, b22, Tcap: double;
      diff, zStar0, zStar1, yMax, yMin: real;

  begin
    if (diffOrder = 1) then
      begin
        for t := 1 to diffBOX.NOPUNCT - 1 do {cumulate the vector}
          diffVector[t] := diffVector[t] + diffVector[t - 1];

        for t := diffBOX.NOPUNCT downto 1 do {Shift the registers}
          diffVector[t] := diffVector[t - 1];
        diffVector[0] := 0;

        sum0 := 0.0;
        for t := 0 to dataBOX.NOPUNCT - 1 do
          sum0 := sum0 - diffVector[t];

        zStar0 := sum0 / dataBOX.NOPUNCT;

        yMax := diffVector[0];
        yMin := diffVector[0];
        for t := 0 to dataBOX.NOPUNCT - 1 do
          begin
            diffVector[t] := diffVector[t] + zStar0;
            yMax := Rmax(yMax, diffVector[t]);
            yMin := Rmin(yMin, diffVector[t]);
          end;
      end; {diffOrder = 1}

    if (diffOrder = 2) then
      begin
        diffVector[1] := diffVector[1] + 2 * diffVector[0];
        for t := 2 to diffBOX.NOPUNCT - 1 do
          diffVector[t] := diffVector[t] + 2 * diffVector[t - 1] - diffVector[t - 2];

        for t := diffBOX.NOPUNCT + 1 downto 2 do {Shift the registers}
          diffvector[t] := diffvector[t - 2];
        diffvector[1] := 0;
        diffvector[0] := 0;

        sum0 := 0.0;
        sum1 := 0.0;
        for t := 1 to dataBOX.NOPUNCT - 1 do
          begin {t}
            diff := -diffVector[t];
            sum0 := sum0 + (t + 1) * diff;
            sum1 := sum1 + t * diff;
          end; {t}

        Tcap := dataBOX.NOPUNCT;

        s11 := Tcap * (Tcap + 1) * (2 * Tcap + 1) / 6.0;
        Tcap := Tcap - 1;
        s12 := Tcap * (Tcap + 1) * (2 * Tcap + 1) / 6.0 + Tcap * (Tcap + 1) / 2;
        s21 := s12;
        s22 := Tcap * (Tcap + 1) * (2 * Tcap + 1) / 6.0;
        det := s11 * s22 - s12 * s21;

        b11 := s22 / det;
        b12 := -s12 / det;
        b21 := -s21 / det;
        b22 := s11 / det;

        zStar0 := b11 * sum0 + b12 * sum1;
        zStar1 := b21 * sum0 + b22 * sum1;

        yMax := diffVector[0];
        yMin := diffVector[0];
        for t := 0 to dataBOX.NOPUNCT - 1 do
          begin {t}
            diffVector[t] := diffVector[t] + zStar0 * (t + 1) + zStar1 * t;
            yMax := Rmax(yMax, diffVector[t]);
            yMin := Rmin(yMin, diffVector[t]);
          end; {t}
      end; {diffOrder = 2}

    diffBOX.yMax := yMax;
    diffBOX.yMin := yMin;
    diffBOX.NOPUNCT := dataBOX.NOPUNCT;  {i.e. DiffBOX.NOPUNCT := DiffBOX.NOPUNCT + 1,2}
    diffBOX.xMax := dataBOX.NOPUNCT - 1;  {the maximum value of x in the dataVector}
    diffBOX.xMin := 0.0;   {the minimum value of x in the dataVector}
    diffBOX.xLB := dataBOX.xMin;   {xLB <= xMin: a left (lower) bound for x}
    diffBOX.xRB := dataBOX.xMax;    {xRB >= xMax: a right (upper) bound for x}

  end; {SeasonalCumulation}

  {_________________________________________________}

  procedure Variance (var dataBOX: rBOX;
                  var dataVector: longVector);

    var
      t: integer;

  begin
    dataBOX.yMean := 0;
    for t := 0 to dataBOX.NOPUNCT - 1 do
      begin {t}
        dataBOX.yMean := dataBOX.yMean + dataVector[t];
      end; {t}
    dataBOX.yMean := dataBOX.yMean / dataBOX.NOPUNCT;

    dataBOX.yVar := 0;
    for t := 0 to dataBOX.NOPUNCT - 1 do
      begin {t}
        dataBOX.yVar := dataBOX.yVar + (dataVector[t] - dataBOX.yMean) * (dataVector[t] - dataBOX.yMean);
      end; {t}
    dataBOX.yVar := dataBOX.yVar / dataBOX.NOPUNCT;

  end; {Variance}

  {_________________________________________________}

end. {FOURIER: Unit 10 of NEWIDEOLOG.PAS}

