unit GLOBALS; {Unit 1 of NEWIDEOLOG.PAS}

interface

  const
    left = 0;
    right = 1;
    dim = 100;
    assumedZero = 0.00001;
    maxOrder = 12;
    maxArray = 2500;
    halfmaxArray = 1250;

  const {ASCII Codes}
    TAB = 9;
    CR = 13;
    SPACE = 32;

  const {Filter Types}
    noProfile = 0;
    triangle = 1;
    square = 2;
    bell = 3;
    Henderson = 1;
    BaxterKing = 2;
    Butterworth = 3;
    ChristFitz = 10;
    HodrickPrescott = 4;
    DeSeasoning = 5;
    Frequency = 5;
    highPass = 6;
    lowPass = 7;
    bandPass = 8;
    bandStop = 9;
    ApplyDirect = 1;
    DiffAndReinflate = 2;
    DiffAndReduce = 3;
    AsymmeticWeights = 4;
    ExtrapolateData = 5;
    Stationary = 1;
    RandomWalk = 2;
    DriftingWalk = 3;
    EndPointDrift = 4;
    EndYearDrift = 5;
    Annual = 1;
    Quarterly = 4;
    Monthly = 12;
    Other = 9;
    PulseResponse = 1;
    Coefficients = 2;
    lagged = 1;
    central = 0;

  type
    complex = record
        re, im: real;
      end;

  type
    polar = record
        argu, modu: real;
      end;

  type
    points = record
        x, y: real
      end;

  type
    vector = array[-2..200] of real;

  type
    ivector = array[-10..50] of integer;

  type
    jVector = array[-20..20] of real;

  type
    longVector = array[-2..maxArray] of real;

  type
    fourierVec = array[0..halfmaxArray] of real;

  type
    matrix = array[0..25] of array[0..25] of real;

  type
    pointsVector = array[0..maxArray] of points;

  type
    complexVector = array[-20..20] of complex;

  type
    polarVector = array[-20..20] of polar;

  type
    sPoint = array[0..10] of real;

  type
    sMatrix = array[0..10] of sPoint;

  type
    triIntVec = array[0..3] of integer;

  type
    triRealVec = array[0..3] of real;

  type
    IBOX = record
        X0: integer;  {the horizontal value of the graphics origin}
        Y0: integer;  {the vertical value of the graphics origin}
        X1: integer; {left of the graphics bounding box}
        Y1: integer; {top or bottom of the graphics bounding box}
        X2: integer; {right of the graphics bounding box}
        Y2: integer; {bottom or top of graphics the bounding box}
        XC: integer; {horizontal value of the centre of window}
        YC: integer; {vertical value of the centre of window}
        xAxis: boolean;
        yAxis: boolean;
        doFrame: boolean;
        doMarks: boolean;
        lollipops: boolean;
      end;

  type
    rBOX = record
        xMax: real;  {the maximum value of x in the dataVector}
        xMin: real;  {the minimum value of x in the dataVector}
        yMax: real;  {the maximum value of y in the dataVector}
        yMin: real;  {the minimum value of y in the dataVector}
        xMean: real; {the mean value of x in the dataVector}
        yMean: real; {the mean value of y in the dataVector}
        xVar: real; {the mean value of x in the dataVector}
        yVar: real; {the mean value of y in the dataVector}
        yUB: real;   {yUB >= yMax: an upper bound for y}
        yLB: real;   {yLB <= yMin: a lower bound for y}
        xLB: real;   {xLB <= xMin: a left (lower) bound for x}
        xRB: real;    {xRB >= xMax: a right (upper) bound for x}
        NOPUNCT: integer; {number of data points}
        EXTRA: integer; {number of extrapolated points at each end}
        diffOrder: integer; {the number of differences taken}
        diffMode: integer; {lagged or central, or other}
        dataFrequency: integer; {Annual, Quarterly, Monthly, Other}
        segmentedData: boolean; {is the data divided into segments?}
        segLength: integer; {the length of the data segments}
        numbOfSegs: integer; {the number of the data segments}
      end;

  type
    polyParameters = record
        alpha: vector;
        gamma: vector;
        delta: vector;
        polyDegree: integer;
        tStart: integer; {the initial index of the polynomial time trend}
      end;

  type
    realRect = record
        top: real;
        left: real;
        bottom: real;
        right: real;
      end;

  type
    integerRect = record
        VTS: integer;
        HRS: integer;
        VTF: integer;
        HRF: integer;
      end;

  type
    FilterRecord = record
        filterFamily: integer; {Henderson , BaxterKing, ChristFitz.  Butterworth , HodrickPrescott }
        filterType: integer;  {lowPass, highPass, bandPass, bandStop}
        filterSpan: integer; {Span of FIR filter}
        filterOrder: integer;   {Order of Wiener-Komogorov Filter}
        cutOff: real;   {Cut-off for HighPass or Lowpass Prototype}
        upperCut: real;  {Upper Cut-off for BandPass or BandStop Filter}
        lowerCut: real;  {Lower Cut-off for BandPass or BandStop Filter}
        dataDiff: integer; {degreee of data differencing}
        trendMethod: integer; {ApplyDirect, DiffAndReinflate, DiffAndReduce, AsymmeticWeights, ExtrapolateData}
        smoothingParameter: real; {Specific to the H-P filter}
        numeratorOrder: integer;   {for IIR filter: zero for an H-P lowpass Filter}
        denominatorOrder: integer; {for IIR filter}
        gainChar: char; {F for for  frequency response, A amplitude spectrum}
        truncAdjustment: boolean; {to make filter coefficients sum to unity or zero}
        ChristFitzData: integer; {Stationary, RandomWalk, DriftingWalk}
        seasonality: integer; {Monthly Quarterly, for the seasonal adjustment filter}
        filterFactor: real; {factor for adjustiing filter gain}
      end;

     {All the data vectors}
  type
    DataRecord = record
        dataVector: longVector;
        trendVector: longVector;
        cycleVec: longVector;
        residueVec: longVector;
        seasonVec: longVector;
        dataBOX: rBOX;
        trendBOX: rBOX;
        cycleBOX: rBOX;
        residueBOX: rBOX;
        seasonBOX: rBOX;
      end;

    {Global Flags}
  var
    PSPict, TeXFormat: boolean;
    allsDone, noPageParameters: boolean;
    {----------------------------------------------}
    {Flags for Data Components}
    thereIsData: boolean;
    weHaveATrend: boolean;
    weHaveTrendResiduals: boolean;
    weHaveABusinessCycle: boolean;
    weHaveSeasonalComponent: boolean;
    weHaveDeasonalisedData: boolean;
    ResidualsInData: boolean;
    SubsampledData: boolean;

     {----------------------------------------------}
    {Flags for Flexible Smoothing}
    noSmoothingRegions: boolean;
    noGlobalLambda: boolean;
    noLocalLambdas: boolean;
    noSmoothingProfile: boolean;
    imposeBand: boolean; {this imposes a band of the peridogram}
     {----------------------------------------------}
     {Other Flags}
    dataExtrapolated: boolean;
    logsTaken: boolean;
    suppressReplace: boolean;
     {This suppresses the option of putting the residuals in place of the data}

  {Graphics Prameters}
  var
    SCREENBOX, POSTBOX: IBOX;
    graphicsOn, noSCREENBOX: boolean;
    dataBOX, trendBOX, cycleBOX, residueBOX, seasonBOX: rBOX;
    currentMode: integer;
    figNumber: integer;

   {Polynomial Parameters}
  var
    polyParas: polyParameters;
    {polyDegree is in dataBOX}

  {Filter Parameters: should all of these become part of FilterRecord?:}
  var
    filterSpec: FilterRecord;
    poles, zeros: complexVector; {Poles and zeros of the filter}
    gamma, delta: complexVector; {Coefficient vectors of the filter}
    denominator, numerator: vector; {these are used by HPFilterSpec. }
{phi, chi <==> numerator denominator: filter coefficient vectors}
    fIRVec: vector;

  {Smoothing Parameters}
  var
    sProfile: integer;
  {smoothLambda: triRealVec}
  {sBandWidth: triIntVec}
  {sBandCentre: triIntVec}
  {sBandCentre[0] records the number of bands of adapted smoothing}

  {Data Vectors}
  var
    dataVector, trendVector, cycleVec, residueVec, seasonVec: longVector;
    dataName: string;

implementation

end. {GLOBALS: Unit 1 of IDEOLOG.PAS}

