unit IMPULSE; {Unit 16 of NEWIDEOLOG.PAS}

interface

  uses
    GLOBALS, UTILS, MATHS, SCREEN, POST, DATASEG, LEGENDS, FILTERS, FIRFILT, Graph, Dos, Crt;

  procedure BaxterKingImpulseResponse (var filterSpec: FilterRecord);

  procedure ChristFitzImpulseResponse (var filterSpec: FilterRecord);

  procedure GetButterPulse (theChoice: integer;
                  var filterSpec: FilterRecord;
                  var dataSpan, impulsePoint: integer;
                  var impulseVec: longVector);

  procedure ButterworthImpulse (var filterSpec: FilterRecord);

  procedure GetHPImpulse (var filterSpec: FilterRecord;
                  var dataSpan, impulsePoint: integer;
                  var impulseVec: longVector);

  procedure HodrickPrescottImpulse (var filterSpec: FilterRecord);

implementation

  {_________________________________________________}

  procedure BaxterKingImpulseResponse (var filterSpec: FilterRecord);

    var
      dataSpan, impulsePoint, t, j, m, span: integer;
      impulseBOX: rBOX;
      fIRVec: vector;
      impulseVec: longVector;
      legend: string;

  begin

    if noPageParameters then
      PageSetUp(SCREENBOX, POSTBOX, PSPict, TeXFormat);

    ClrScr;
    Writeln;
    Writeln;
    Writeln('    THE IMPULSE RESPONSE OF THE BAXTER-KING FILTER');
    Writeln;
    Writeln('    The Baxter-King filter produces no output unless it falls entirely');
    Writeln('    within the  bounds of  the sample. Thus, if  the filter has 2M + 1');
    Writeln('    coefficients and if the sample index runs from t = 0 to t = T - 1,');
    Writeln('    there will be  no output  unless t >= M and t < T - M. Subject  to');
    Writeln('    these  constraints the response to  an impulse  at time t  will be');
    Writeln('    identical to  the sequence of coefficient that  are applied to the');
    Writeln('    data to generate the output at time t.');
    Writeln;
    Writeln;
    Writeln('    What is the span of the data?');
    Readln(dataSpan);

    span := FilterSpec.filterSpan;
    m := (span - 1) div 2;

    Writeln('    Specify the location of the impulse within the interval [', m : 2, ' ', dataSpan - 1 - m : 2, ']');
    Readln(impulsePoint);

    GetTruncIdealWeights(filterSpec, fIRVec);

    impulseBOX.NOPUNCT := dataSpan;

    for t := 0 to dataSpan - 1 do
      begin {t}
        j := Abs(t - impulsePoint);
        if j > m then
          impulseVec[t] := 0.0
        else
          impulseVec[t] := fIRVec[j];
      end; {t}

    MakeDataBox(impulseBOX, impulseVec);

    GetGraphics(graphicsOn, SCREENBOX, currentMode);

    ScreenFrame(impulseBOX, SCREENBOX);
    ImpulsePlot(impulseBOX, SCREENBOX, impulseVec);

    legend := BaxterKingImpulse(filterSpec, dataSpan, impulsePoint);
    ScreenCaption(legend);

    CallForReturn;
    Readln;
    CloseGraph;
    RestoreCrtMode;
    ClrScr;

    if PSPict = true then
      PostImpulsePlot(impulseBOX, POSTBOX, impulseVec, legend);

  end; {BaxterKingImpulseResponse}

  {_________________________________________________}

  procedure ChristFitzImpulseResponse (var filterSpec: FilterRecord);

    var
      dataSpan, halfSpan, impulsePoint, oldImpulsePt, t: integer;
      store: real;
      impulseBOX: rBOX;
      furlongVec: longVector;
      legend: string;
      reverseIt: boolean;

  begin

    if noPageParameters then
      PageSetUp(SCREENBOX, POSTBOX, PSPict, TeXFormat);

    ClrScr;
    Writeln;
    Writeln;
    Writeln('    THE IMPULSE RESPONSE OF THE CHRISTIANO-FITZGERALD FILTER');
    Writeln;
    Writeln('    The progam plots the response of the filter to  an isolated impulse');
    Writeln('    located at a specific point in time within a wider time span, which');
    Writeln('    might correspond to the duration of a typical data sample.');
    Writeln;
    Writeln('    For the Christiano-Fitzgerald filter, the  response to an isolated');
    Writeln('    impulse  at time t  is identical to  the  sequence of coefficients');
    Writeln('    involved  in  generating  the  filtered  output. Equivalently, the');
    Writeln('    filter  matrix  which maps  from the  data vector to the filtered');
    Writeln('    vector is a  symmetric Toeplitz  matrix of  which the  elements of');
    Writeln('    t-th column, which  correspond to the impulse response, are  equal');
    Writeln('    to those of the t-th row, which correspond the filter coefficients.');
    Writeln;
    Writeln('    What is the span of the impulse response?');
    Readln(dataSpan);

    Writeln('    Specify the location of the impulse within the interval; [0, ', dataSpan : 2, ']');
    Readln(impulsePoint);

    impulseBOX.NOPUNCT := dataSpan;

    GetFurlongWeights(FilterSpec, impulseBOX, furlongVec);

    MakeDataBox(impulseBOX, furlongVec);
    halfSpan := dataSpan div 2;

    reverseIt := false;
    if (impulsePoint > halfSpan) then
      begin
        oldImpulsePt := impulsePoint;
        impulsePoint := dataspan - 1 - impulsePoint;
        reverseIt := true;
      end;

    for t := dataSpan - 1 downto 0 do
      begin {t}
        if (t > impulsePoint) then
          furlongVec[t] := furlongVec[t - impulsepoint]
        else if (t = impulsePoint) then
          furlongVec[t] := furlongVec[0]
        else if (t < impulsePoint) then
          furlongVec[t] := furlongVec[2 * impulsepoint - t]
      end; {t}

    if reverseIt then
      begin
        for t := 0 to (dataSpan - 1) div 2 do
          begin {t: reverse the sequence}
            store := furlongVec[t];
            furlongVec[t] := furlongVec[dataSpan - 1 - t];
            furlongVec[dataSpan - 1 - t] := store;
          end; {t: reverse the sequence}
        impulsePoint := oldImpulsePt;
      end;

    GetGraphics(graphicsOn, SCREENBOX, currentMode);

    ScreenFrame(impulseBOX, SCREENBOX);
    ImpulsePlot(impulseBOX, SCREENBOX, furlongVec);

    legend := ChristFitzImpulse(filterSpec, dataSpan, impulsePoint);
    ScreenCaption(legend);

    CallForReturn;
    Readln;
    CloseGraph;
    RestoreCrtMode;
    ClrScr;

    if PSPict = true then
      PostImpulsePlot(impulseBOX, POSTBOX, furlongVec, legend);


  end; {ChristFitzImpulseResponse}

  {_________________________________________________}

  procedure FormSe (var n, dataSpan, impulsePoint: integer;
                  var impulseVec: longvector);

  {This procedure forms Se, where e is the impulse vector}
  {n = filterSpec.filterOrder}

    var
      j, t: integer;
      sVec: vector;

  begin
    FormSvec(sVec, n - 2); {S}
    impulseVec[impulsePoint] := sVec[0];

    for j := 1 to n - 2 do
      begin {j}
        t := impulsePoint + j;
        if (t < dataSpan) then
          impulseVec[t] := sVec[j];
        t := impulsePoint - j;
        if t >= 0 then
          impulseVec[t] := sVec[j];
      end {j}
  end; {FormQprimeSe}

  {_________________________________________________}

  procedure GetButterPulse (theChoice: integer;
                  var filterSpec: FilterRecord;
                  var dataSpan, impulsePoint: integer;
                  var impulseVec: longVector);

    var
      n, t, Tcap: integer;
      lamOne, lamTwo: real;
      sVec, wVec: vector;

  begin
    n := filterSpec.filterOrder;
    Tcap := dataSpan;

    lamOne := 1 / Tan(filterSpec.cutOff / 2);
    lamOne := Power(lamOne, 2 * n);
    lamTwo := 1.0;
    if lamOne > 1.0 then
      begin
        lamTwo := 1 / lamOne;
        LamOne := 1.0;
      end;

    for t := 0 to Tcap - 1 do
      impulseVec[t] := 0.0;

    if (theChoice = Coefficients) then {coefficient sequence}
      FormSe(n, Tcap, impulsePoint, impulseVec)
    else
      impulseVec[impulsePoint] := 1.0;

    QprimeY(impulseVec, Tcap); {d = Q'e we are loosing two points in taking 2nd differences}
    FormWvec(wVec, n, lamOne, lamTwo); { W = M + lambda*Q'SQ }
    Toeplitz(wVec, impulseVec, Tcap - 3, n); {solve (M + lambda*Q'SQ)g = d for g}
    FormQy(impulseVec, Tcap); {Form Qg}

    if (theChoice = PulseResponse) then
      begin {impulse response}
        FormSvec(sVec, n - 2);
        GammaY(sVec, impulseVec, Tcap, n - 2); {Form SQg}
      end; {impulse response}

    for t := 0 to Tcap - 1 do
      impulseVec[t] := -impulseVec[t];
    impulseVec[impulsePoint] := impulseVec[impulsePoint] + 1.0;

  end; {GetButterPulse}

  {_________________________________________________}

  procedure ButterworthImpulse (var filterSpec: FilterRecord);

    var
      theChoice: integer;
      dataSpan, impulsePoint: integer;
      impulseBOX: rBOX;
      impulseVec: longVector;
      legend: string;

  begin

    if noPageParameters then
      PageSetUp(SCREENBOX, POSTBOX, PSPict, TeXFormat);

    ClrScr;
    Writeln;
    Writeln;
    Writeln('    THE IMPULSE RESPONSE OF THE BUTTERWORTH FILTER');
    Writeln;
    Writeln('    In the case of the  finite-sample Butterworth filter, the response');
    Writeln('    to an impulse at time t differs from the  sequence of coefficients');
    Writeln('    involved in generating  the output. The former  corresponds to the');
    Writeln('    t-th column  of the filter matrix  that transforms the data vector');
    Writeln('    to the vector of  the filtered output, and the latter  corresponds');
    Writeln('    to the t-th row');
    Writeln;

    repeat
      Writeln('    Do you wish to plot ');
      Writeln;
      Writeln('        (1) the impulse response, or');
      Writeln;
      Writeln('        (2) the coefficient sequence');
      Writeln;
      Writeln('    Type 1 or 2 and Press <RETURN>.');
      Readln(theChoice);
    until theChoice in [1, 2];

    Writeln;
    Writeln('    What is the span of the impulse data?');
    Readln(dataSpan);

    Writeln('    Specify the location of the impulse within the interval; [0, ', dataSpan : 2, ']');
    Readln(impulsePoint);

    impulseBOX.NOPUNCT := dataSpan;

    GetButterPulse(theChoice, filterSpec, dataSpan, impulsePoint, impulseVec);

    MakeDataBox(impulseBOX, impulseVec);

    GetGraphics(graphicsOn, SCREENBOX, currentMode);

    ScreenFrame(impulseBOX, SCREENBOX);
    ImpulsePlot(impulseBOX, SCREENBOX, impulseVec);

    if (theChoice = PulseResponse) then
      legend := ButterPulseLegend(filterSpec);
    if (theChoice = Coefficients) then
      legend := ButterCoefficients(filterSpec);
    ScreenCaption(legend);

    CallForReturn;
    Readln;
    CloseGraph;
    RestoreCrtMode;
    ClrScr;

    if PSPict = true then
      PostImpulsePlot(impulseBOX, POSTBOX, impulseVec, legend);

  end; {ButterworthImpulse}

  {_________________________________________________}

  procedure GetHPImpulse (var filterSpec: FilterRecord;
                  var dataSpan, impulsePoint: integer;
                  var impulseVec: longVector);

    var
      lambda: real;
      wVec: vector;
      t, Tcap: integer;
       {W: text;}

  begin
    lambda := filterSpec.smoothingParameter;
    Tcap := dataSpan;

    for t := 0 to Tcap - 1 do
      impulseVec[t] := 0.0;
    impulseVec[impulsePoint] := 1.0;

    QprimeY(impulseVec, Tcap); { d = Q'e: we are loosing two dimesions in taking 2nd differences}
    FormQuinDiag(wVec, lambda);
    Toeplitz(wVec, impulseVec, Tcap - 3, 2); {solve (I + lambda*Q'Q)z = d for z}

    FormQy(impulseVec, Tcap); {Form Qz}

    for t := 0 to Tcap - 1 do
      impulseVec[t] := -lambda * impulseVec[t];
    impulseVec[impulsePoint] := impulseVec[impulsePoint] + 1.0;

  end; {GetHPImpulse}

  {_________________________________________________}

  procedure HodrickPrescottImpulse (var filterSpec: FilterRecord);

    var
      dataSpan, impulsePoint: integer;
      impulseBOX: rBOX;
      impulseVec: longVector;
      legend: string;

  begin

    if noPageParameters then
      PageSetUp(SCREENBOX, POSTBOX, PSPict, TeXFormat);

    ClrScr;
    Writeln;
    Writeln;
    Writeln('    THE IMPULSE RESPONSE OF THE HODRICK-PRESCOTT FILTER');
    Writeln;
    Writeln('    In the case of the finite-sample Hodrick-Prescott filter, the matrix');
    Writeln('    of  the filter  is symmetric. Therefore, the  filters response to an ');
    Writeln('    impulse  at  time t, which  corresponds to  the t-th  column  of the');
    Writeln('    matrix, is  identical to the  sequence of filter coefficients in the');
    Writeln('    t-th row of the matrix.');

    Writeln;
    Writeln('    What is the span of the impulse data?');
    Readln(dataSpan);

    Writeln('    Specify the location of the impulse within the interval; [0, ', dataSpan : 2, ']');
    Readln(impulsePoint);

    impulseBOX.NOPUNCT := dataSpan;

    GetHPImpulse(filterSpec, dataSpan, impulsePoint, impulseVec);

    MakeDataBox(impulseBOX, impulseVec);

    GetGraphics(graphicsOn, SCREENBOX, currentMode);

    ScreenFrame(impulseBOX, SCREENBOX);
    ImpulsePlot(impulseBOX, SCREENBOX, impulseVec);

    legend := HPImpulseResponse(filterSpec);
    ScreenCaption(legend);

    CallForReturn;
    Readln;
    CloseGraph;
    RestoreCrtMode;
    ClrScr;

    if PSPict = true then
      PostImpulsePlot(impulseBOX, POSTBOX, impulseVec, legend);

  end;{HodrickPrescottImpulse}
  {_________________________________________________}

end. {IMPULSE: Unit 16 of NEWIDEOLOG.PAS}

