unit LEGENDS; {Unit 4 of NEWIDEOLOG.PAS}

interface

  uses
    GLOBALS, UTILS, Crt, Graph;

  function Sgn (number: real): char;

  procedure ScreenCaption (legend: string);

  function DataLegend (var dataBOX: rBOX;
                  var dataName: string): string;

  function DeSeasonalisedOverlay (var dataBOX: rBOX;
                  var dataName: string): string;

  function SeasonallyAdjustedData (var dataBOX: rBOX;
                  var dataName: string): string;

  function DataPeriodogram (var dataBOX: rBOX;
                  var dataName: string): string;

  function PolynomialWeights: string;

  function PolynomialTrend (var polyOrder: integer;
                  var dataBOX: rBOX;
                  var dataName: string): string;

  function PolyWeightTrend (var polyOrder: integer;
                  var dataBOX: rBOX;
                  var dataName: string): string;

  function PolynomialResiduals (var polyOrder: integer;
                  var dataBOX: rBOX;
                  var dataName: string): string;

  function HPtrend (var lambda: real;
                  var dataBOX: rBOX;
                  var dataName: string): string;

  function HPresiduals (var lambda: real;
                  var dataBOX: rBOX;
                  var dataName: string): string;

  function VariSmoothTrend (var dataBOX: rBOX;
                  var dataName: string): string;

  function VariSmoothResiduals (var dataBOX: rBOX;
                  var dataName: string): string;

  function TrigonometricalTrend (var degrees: real;
                  var dataBOX: rBOX;
                  var dataName: string): string;

  function TrendResiduals (var dataBOX: rBOX;
                  var dataName: string): string;

  function TurnPoints: string;

  function HendersonTrend (var dataBOX: rBOX;
                  var filterSec: FilterRecord;
                  var dataName: string): string;

  function HendersonGain (var filterSpec: FilterRecord): string;

  function BaxterKingCycle (var dataBOX: rBOX;
                  var dataName: string): string;

  function BaxterKingGain (var filterSpec: FilterRecord): string;

  function BaxterKingImpulse (var filterSpec: FilterRecord;
                  var dataSpan, impulsePoint: integer): string;

  function ChristFitzImpulse (var filterSpec: FilterRecord;
                  var dataSpan, impulsePoint: integer): string;

  function ChristFitzCycle (var dataBOX: rBOX;
                  var dataName: string): string;

  function ButterworthTrend (var dataBOX: rBOX;
                  var dataName: string;
                  var filterSpec: FilterRecord): string;

  function ButterworthResiduals (var dataBOX: rBOX;
                  var dataName: string;
                  var filterSpec: FilterRecord): string;

  function ButterworthGain (var filterSpec: FilterRecord): string;

  function ButterPulseLegend (var filterSpec: FilterRecord): string;

  function ButterCoefficients (var filterSpec: FilterRecord): string;

  function ButterPoles (var filterSpec: FilterRecord): string;

  function HPFilterGain (var filterSpec: FilterRecord): string;

  function HodPoleZero (var filterSpec: FilterRecord): string;

  function HPImpulseResponse (var filterSpec: FilterRecord): string;

  function SeasonalFilterGain (var filterSpec: FilterRecord): string;

  function FourierLowPass (var dataBOX: rBOX;
                  var dataName: string): string;

  function FourierBandPass (var dataBOX: rBOX;
                  var dataName: string): string;

  function TrendAndCycle (var dataBOX: rBOX;
                  var dataName: string): string;

  function SeasonalComponent (var dataBOX: rBOX;
                  var dataName: string): string;

  function SeasonallyAdjusted (var dataBOX: rBOX;
                  var dataName: string): string;

  function SpectralBand (var lowerCut, upperCut: real): string;

  procedure PoleZeroLegend (var F: text);

  function SeasonPoleZero (var filterSpec: FilterRecord): string;

implementation

  {________________________________________________}

  function Sgn (number: real): char;
  begin
    if number >= 0 then
      Sgn := '+'
    else
      Sgn := ' ';
  end; {Sgn}

  {_________________________________________________}

  procedure ScreenCaption (legend: string);

    var
      i, q, width, distance, breakPoint, XS, XQ, YS: integer;
      firstString, secondString: string;

  begin
    XS := GetMaxX div 2;
    YS := (GetMaxY) - 30;

    width := Length(legend);

    if width > 65 then
      begin {Breakit }
        distance := width;
        for i := 1 to width do
          begin {i}
            if legend[i] = ' ' then
              begin {if legend}
                q := Abs(width div 2 - i);
                if distance > q then
                  begin {if distance}
                    breakPoint := i;
                    distance := q;
                  end; {if distance}
              end; {if legend}
          end; {i}

        firstString := Copy(legend, 1, breakPoint);
        secondString := Copy(legend, breakPoint + 1, width);
        width := TextWidth(firstString);
        XQ := XS - (width div 2);
        YS := YS - 35;
        YS := SCREENBOX.Y2 + 60; {Alternative}
        MoveTo(XQ, YS);
        OutText(firstString);
        width := TextWidth(secondString);
        XQ := XS - (width div 2);
        YS := YS + 15;
        MoveTo(XQ, YS);
        OutText(secondString);
      end {Breakit}
    else
      begin
        width := TextWidth(legend);
        XS := XS - (width div 2);
        YS := YS - 30;
        YS := SCREENBOX.Y2 + 60; {Alternative}
        MoveTo(XS, YS);
        OutText(legend);
      end;

  end; {ScreenCaption}

  {________________________________________________}

  function DataLegend (var dataBOX: rBOX;
                  var dataName: string): string;

    var
      legend, addString: string;

  begin
    legend := 'The plot of ';
    addString := IntegerToString(dataBOX.NOPUNCT);
    legend := Concat(legend, addString, ' points of ');
    legend := Concat(legend, dataName);
    DataLegend := legend;
  end; {DataLegend}

{________________________________________________}

  function DeSeasonalisedOverlay (var dataBOX: rBOX;
                  var dataName: string): string;
    var
      legend, addString: string;

  begin
    legend := 'The plot of ';
    addString := IntegerToString(dataBOX.NOPUNCT);
    legend := Concat(legend, addString);
    if (dataBOX.dataFrequency = Monthly) then
      addString := ' monthly ';
    if (dataBOX.dataFrequency = Quarterly) then
      addString := ' quarterly ';
    legend := Concat(legend, addstring, 'observations on ');
    legend := Concat(legend, dataName);
    addString := ' with a superimposed seasonally-adjusted sequence.';
    legend := Concat(legend, addstring);
    DeSeasonalisedOverlay := legend;
  end; {DeSeasonalisedOverlay}

{________________________________________________}

  function SeasonallyAdjustedData (var dataBOX: rBOX;
                  var dataName: string): string;
    var
      legend, addString: string;

  begin
    legend := 'The plot of a seasonally adjusted sequence of ';
    addString := IntegerToString(dataBOX.NOPUNCT);
    legend := Concat(legend, addString);
    if (dataBOX.dataFrequency = Monthly) then
      addString := ' monthly ';
    if (dataBOX.dataFrequency = Quarterly) then
      addString := ' quarterly ';
    legend := Concat(legend, addstring, 'observations on ');
    legend := Concat(legend, dataName);
    SeasonallyAdjustedData := legend;
  end; {SeasonallyAdjustedData}

{________________________________________________}

  function DataPeriodogram (var dataBOX: rBOX;
                  var dataName: string): string;

    var
      legend, addString: string;

  begin
    legend := 'The periodogram of ';
    addString := IntegerToString(dataBOX.NOPUNCT);
    legend := Concat(legend, addString, ' points of ', dataName);
    DataPeriodogram := legend;
  end; {DataPeridogram}

  {________________________________________________}

  function PolynomialWeights: string;

    var
      legend: string;
  begin
    legend := 'The profile of the weights applied to the data points ';
    PolynomialWeights := Concat(legend, 'in the polynomial regression');
  end; {PolynomialWeights}

  {________________________________________________}

  function PolynomialTrend (var polyOrder: integer;
                  var dataBOX: rBOX;
                  var dataName: string): string;

    var
      legend, addString: string;

  begin
    legend := 'A polynomial of degree ';
    addString := IntegerToString(polyOrder);
    legend := Concat(legend, addString, ' interpolated through ');
    addString := IntegerToString(dataBOX.NOPUNCT);
    legend := Concat(legend, addString, ' points of ', dataName);
    PolynomialTrend := legend;
  end; {PolynomialTrend}

  {________________________________________________}

  function PolyWeightTrend (var polyOrder: integer;
                  var dataBOX: rBOX;
                  var dataName: string): string;

    var
      legend, addString: string;

  begin
    legend := 'A polynomial of degree ';
    addString := IntegerToString(polyOrder);
    legend := Concat(legend, addString, ' interpolated by weighted ');
    legend := Concat(legend, 'least-squares regression through ');
    addString := IntegerToString(dataBOX.NOPUNCT);
    legend := Concat(legend, addString, ' points of ', dataName);
    PolyWeightTrend := legend;
  end; {PolynomialTrend}

  {________________________________________________}

  function PolynomialResiduals (var polyOrder: integer;
                  var dataBOX: rBOX;
                  var dataName: string): string;

    var
      legend, addString: string;

  begin
    legend := 'The residual sequence from fitting a polynomial of degree ';
    addString := IntegerToString(polyOrder);
    legend := Concat(legend, addString, ' to ');
    addString := IntegerToString(dataBOX.NOPUNCT);
    legend := Concat(legend, addString, ' points of ', dataName);
    PolynomialResiduals := legend;
  end; {PolynomialResiduals}

  {________________________________________________}

  function HPtrend (var lambda: real;
                  var dataBOX: rBOX;
                  var dataName: string): string;

    var
      legend, addString: string;

  begin
    legend := 'A trend determined by Hodrick-Prescott filter with a smoothing parameter of  ';
    addString := IntegerToString(Round(lambda));
    legend := Concat(legend, addString, ', interpolated through ');
    addString := IntegerToString(dataBOX.NOPUNCT);
    legend := Concat(legend, addString, ' points of ', dataName);
    HPtrend := legend;
  end; {HPtrend}

  {________________________________________________}

  function HPresiduals (var lambda: real;
                  var dataBOX: rBOX;
                  var dataName: string): string;

    var
      legend, paraString, addString: string;

  begin
    legend := 'The residual sequence from fitting a trend via a ';
    addString := 'Hodrick-Prescott filter with a smoothing parameter of ';
    paraString := IntegerToString(Round(lambda));
    legend := Concat(legend, addString, paraString, ' to ');
    addString := IntegerToString(dataBOX.NOPUNCT);
    legend := Concat(legend, addString, ' points of ', dataName);
    HPresiduals := legend;
  end; {HPresiduals}

  {________________________________________________}

  function VariSmoothTrend (var dataBOX: rBOX;
                  var dataName: string): string;

    var
      legend, addString, frequency: string;

  begin
    legend := 'A trend determined by a filter with a variable ';
    addString := 'smoothing parameter applied to ';
    legend := Concat(legend, addString);
    addString := IntegerToString(dataBOX.NOPUNCT);
    if dataBOX.dataFrequency = monthly then
      frequency := ' monthly ';
    if dataBOX.dataFrequency = Quarterly then
      frequency := ' quarterly ';
    if dataBOX.dataFrequency = Annual then
      frequency := ' annual ';
    legend := Concat(legend, addString, frequency, 'points of ', dataName);
    VariSmoothTrend := legend;

  end; {VariSmoothTrend}

  {________________________________________________}

  function VariSmoothResiduals (var dataBOX: rBOX;
                  var dataName: string): string;

    var
      legend, addString: string;

  begin
    legend := 'The residual sequence from fitting a trend via a ';
    addString := 'filter with a variable smoothing parameter to ';
    legend := Concat(legend, addString);
    addString := IntegerToString(dataBOX.NOPUNCT);
    legend := Concat(legend, addString, ' points of ', dataName);
    VariSmoothResiduals := legend;
  end; {VariSmoothResiduals}

  {________________________________________________}

  function TrigonometricalTrend (var degrees: real;
                  var dataBOX: rBOX;
                  var dataName: string): string;

    var
      legend, addString: string;

  begin
    legend := 'A trend comprising Fourier components of frequencies less than ';
    addString := TalyToString(degrees);
    legend := Concat(legend, addString, ' degrees per period interpolated through ');
    addString := IntegerToString(dataBOX.NOPUNCT);
    legend := Concat(legend, addString, ' points of ', dataName);
    TrigonometricalTrend := legend;

  end; {TrigonometricalTrend}

  {________________________________________________}

  function TrendResiduals (var dataBOX: rBOX;
                  var dataName: string): string;

    var
      legend, addString: string;

  begin
    legend := 'The residual sequence from detrending ';
    addString := IntegerToString(dataBOX.NOPUNCT);
    legend := Concat(legend, addString, ' points of ', dataName, ' via the Fourier method');
    TrendResiduals := legend;
  end; {TrendResiduals}

{________________________________________________}

  function TurnPoints: string;

    var
      legend, addString: string;

  begin
    legend := 'The turning points of the low-frequency cycle (red)';
    addString := ' identified by the derivative of the function (blue).';
    legend := Concat(legend, addString);
    TurnPoints := legend;

  end; {TurnPoints}

{________________________________________________}

  function HendersonTrend (var dataBOX: rBOX;
                  var filterSec: FilterRecord;
                  var dataName: string): string;

    var
      legend, addString: string;

  begin
    legend := 'A trend, determined by a Henderson filter with ';
    addString := IntegerToString(filterSpec.filterSpan);
    legend := Concat(legend, addString, ' coefficients, interpolated through ');
    addString := IntegerToString(dataBOX.NOPUNCT);
    legend := Concat(legend, addString, ' points of ', dataName);
    HendersonTrend := legend;

  end; {HendersonTrend}

  {________________________________________________}

  function HendersonGain (var filterSpec: FilterRecord): string;

    var
      legend, addString: string;

  begin
    legend := 'The frequency response of the Henderson lowpass filter of ';
    addString := IntegerToString(filterSpec.filterSpan);
    legend := Concat(legend, addString, ' coefficients');
    HendersonGain := legend;

  end; {HendersonGain}

  {________________________________________________}

  function BaxterKingCycle (var dataBOX: rBOX;
                  var dataName: string): string;
    var
      legend, addString: string;

  begin
    legend := 'Cyclical fluctuations extracted from ';
    addString := IntegerToString(dataBOX.NOPUNCT);
    legend := Concat(legend, addString, ' points of ', dataName, ' by the Baxter-King filter');
    BaxterKingCycle := legend;

  end; {BaxterKingCycle}

  {________________________________________________}

  function BaxterKingImpulse (var filterSpec: FilterRecord;
                  var dataSpan, impulsePoint: integer): string;

    var
      legend, addString: string;
  begin
    legend := 'The response of the bandpass filter of Baxter and King to an';
    addString := IntegerToString(impulsePoint);
    legend := Concat(legend, ' isolated impulse occuring at the ', addString);
    addString := IntegerToString(dataSpan);
    legend := Concat(legend, 'th point within a sequence of ', addString, ' points');
    BaxterKingImpulse := legend;
  end; {BaxterKingImpulse}

  {________________________________________________}

  function BaxterKingGain (var filterSpec: FilterRecord): string;
    var
      legend, addString: string;

  begin
    legend := 'The frequency response of the truncated bandpass';
    legend := Concat(legend, ' filter of Baxter and King, comprising ');
    addString := IntegerToString(filterSpec.filterSpan);
    legend := Concat(legend, addString, ' coefficients');
    BaxterKingGain := legend;

  end; {BaxterKingGain}

  {________________________________________________}

  function ChristFitzImpulse (var filterSpec: FilterRecord;
                  var dataSpan, impulsePoint: integer): string;

    var
      legend, addString: string;
  begin
    legend := 'The response of the bandpass filter of Christiano and Fitzgerald to an';
    addString := IntegerToString(impulsePoint);
    legend := Concat(legend, ' isolated impulse occuring at the ', addString);
    addString := IntegerToString(dataSpan);
    legend := Concat(legend, 'th point within a sequence of ', addString, ' points');
    ChristFitzImpulse := legend;
  end; {ChristFitzImpulse}

  {________________________________________________}

  function ChristFitzCycle (var dataBOX: rBOX;
                  var dataName: string): string;

    var
      legend, addString: string;

  begin
    legend := 'Cyclical fluctuations extracted from ';
    addString := IntegerToString(dataBOX.NOPUNCT);
    legend := Concat(legend, addString, ' points of ', dataName, ' by the Christiano-Fitzgerald filter');
    ChristFitzCycle := legend;

  end; {ChristFitzCycle}

  {________________________________________________}

  function ButterworthTrend (var dataBOX: rBOX;
                  var dataName: string;
                  var filterSpec: FilterRecord): string;

    var
      legend, addString: string;
  {pi, degrees: real;}

  begin
  {pi := 4.0 * Arctan(1.0);}
    legend := 'The trend estimated by a Butterworth lowpass filter of order ';
    addString := IntegerToString(filterSpec.filterOrder);
    legend := Concat(legend, addString, ' interpolated through ');
    addString := IntegerToString(dataBOX.NOPUNCT);
    legend := Concat(legend, addString, ' points of ', dataName);

  {legend := Concat(legend, addString, ', with a nominal cut-off point at ');}
  {degrees := 180.0 / pi;}
  {degrees := degrees * filterSpec.cutOff;}
  {addString := IntegerToString(Round(degrees));}
  {legend := Concat(legend, addString, ' degrees');}
    ButterworthTrend := legend;

  end; {ButterworthTrend}

  {________________________________________________}

  function ButterworthResiduals (var dataBOX: rBOX;
                  var dataName: string;
                  var filterSpec: FilterRecord): string;

    var
      legend, addString: string;
  {pi, degrees: real;}

  begin
  {pi := 4.0 * Arctan(1.0);}
    legend := 'The residual sequence from detrending ';
    addString := IntegerToString(dataBOX.NOPUNCT);
    legend := Concat(legend, addString, ' points of ', dataName);
    legend := Concat(legend, ' via a Butterworth lowpass filter of order ');
    addString := IntegerToString(filterSpec.filterOrder);
    legend := Concat(legend, addString);
  {legend := Concat(legend, addString, ', with a nominal cut-off point at ');}
  {degrees := 180.0 / pi;}
  {degrees := degrees * filterSpec.cutOff;}
  {addString := IntegerToString(Round(degrees));}
  {legend := Concat(legend, addString, ' degrees');}
    ButterworthResiduals := legend;

  end; {ButterworthResiduals}

  {________________________________________________}

  function ButterworthGain (var filterSpec: FilterRecord): string;

    var
      legend, addString: string;
      pi, degrees: real;

  begin
    pi := 4.0 * Arctan(1.0);
    legend := 'The frequency response of the Butterworth lowpass';
    legend := Concat(legend, ' filter of order ');
    addString := IntegerToString(filterSpec.filterOrder);
    legend := Concat(legend, addString, ', with a nominal cut-off point at ');
    degrees := 180.0 / pi;
    degrees := degrees * filterSpec.cutOff;
    addString := IntegerToString(Round(degrees));
    legend := Concat(legend, addString, ' degrees');
    ButterworthGain := legend;

  end; {ButterworthGain}

  {________________________________________________}

  function ButterPulseLegend (var filterSpec: FilterRecord): string;

    var
      legend, addString: string;
      pi, degrees: real;

  begin
    pi := 4.0 * Arctan(1.0);
    legend := 'The impulse response of the Butterworth lowpass';
    legend := Concat(legend, ' filter of order ');
    addString := IntegerToString(filterSpec.filterOrder);
    legend := Concat(legend, addString, ', with a nominal cut-off point at ');
    degrees := 180.0 / pi;
    degrees := degrees * filterSpec.cutOff;
    addString := IntegerToString(Round(degrees));
    legend := Concat(legend, addString, ' degrees');
    ButterPulseLegend := legend;

  end;{ButterPulseLegend}

  {________________________________________________}

  function ButterCoefficients (var filterSpec: FilterRecord): string;

    var
      legend, addString: string;
      pi, degrees: real;

  begin
    pi := 4.0 * Arctan(1.0);
    legend := 'A coefficient sequence of the Butterworth lowpass';
    legend := Concat(legend, ' filter of order ');
    addString := IntegerToString(filterSpec.filterOrder);
    legend := Concat(legend, addString, ', with a nominal cut-off point at ');
    degrees := 180.0 / pi;
    degrees := degrees * filterSpec.cutOff;
    addString := IntegerToString(Round(degrees));
    legend := Concat(legend, addString, ' degrees');
    ButterCoefficients := legend;

  end;{ButterCoefficients}

  {________________________________________________}

  function ButterPoles (var filterSpec: FilterRecord): string;

    var
      legend, addString: string;
      pi, degrees: real;

  begin
    pi := 4.0 * Arctan(1.0);
    legend := 'The pole-zero diagram of the Butterworth lowpass';
    legend := Concat(legend, ' filter of order ');
    addString := IntegerToString(filterSpec.filterOrder);
    legend := Concat(legend, addString, ', with a nominal cut-off point at ');
    degrees := 180.0 / pi;
    degrees := degrees * filterSpec.cutOff;
    addString := IntegerToString(Round(degrees));
    legend := Concat(legend, addString, ' degrees');
    ButterPoles := legend;

  end;{ButterPoles}

  {________________________________________________}

  function HPFilterGain (var filterSpec: FilterRecord): string;

    var
      legend, addString: string;

  begin
    legend := 'The frequency response of the Hodrick-Prescott lowpass';
    legend := Concat(legend, ' filter with a smoothing parameter of ');
    addString := IntegerToString(Round(filterSpec.smoothingParameter));
    legend := Concat(legend, addString);
    HPFilterGain := legend;

  end; {HPFilterGain}

  {________________________________________________}

  function HodPoleZero (var filterSpec: FilterRecord): string;

    var
      legend, addString: string;

  begin
    legend := 'The pole-zero diagram of the Hodrick-Prescott lowpass';
    legend := Concat(legend, ' filter with a smoothing parameter of ');
    addString := IntegerToString(Round(filterSpec.smoothingParameter));
    legend := Concat(legend, addString);
    HodPoleZero := legend;

  end; {HodPoleZero}

  {________________________________________________}

  function HPImpulseResponse (var filterSpec: FilterRecord): string;

    var
      legend, addString: string;

  begin
    legend := 'An impulse response of the Hodrick-Prescott lowpass filter ';
    legend := Concat(legend, 'with a smoothing parameter of lambda = ');
    addString := IntegerToString(Round(filterSpec.smoothingParameter));
    legend := Concat(legend, addString);
    HPImpulseResponse := legend;

  end; {HPImpulseResponse}

  {________________________________________________}

  function SeasonalFilterGain (var filterSpec: FilterRecord): string;

    var
      legend, addString: string;

  begin
    legend := 'The frequency response function of a time-domain filter ';
    addString := 'for the seasonal adjustment of a ';
    legend := Concat(legend, addString);
    if (filterSpec.seasonality = Monthly) then
      addString := ' monthly ';
    if (filterSpec.seasonality = Quarterly) then
      addString := ' quarterly ';
    legend := Concat(legend, addString, 'data sequence.');
    SeasonalFilterGain := legend;
  end; {SeasonalFilterGain}

  {________________________________________________}

  function FourierLowPass (var dataBOX: rBOX;
                  var dataName: string): string;

    var
      legend, addString: string;

  begin
    legend := 'A low-frequency cyclical component extracted from ';
    addString := IntegerToString(dataBOX.NOPUNCT);
    legend := Concat(legend, addString, ' points of ', dataName, ' by a Fourier Method');
    FourierLowPass := legend;

  end; {FourierLowPass}

  {________________________________________________}

  function FourierBandPass (var dataBOX: rBOX;
                  var dataName: string): string;

    var
      legend, addString: string;

  begin
    legend := 'Cyclical fluctations extracted from ';
    addString := IntegerToString(dataBOX.NOPUNCT);
    legend := Concat(legend, addString, ' points of ', dataName, ' by a Fourier Bandpass Method');
    FourierBandPass := legend;

  end; {FourierBandPass}

  {________________________________________________}

  function TrendAndCycle (var dataBOX: rBOX;
                  var dataName: string): string;

    var
      legend, addString: string;

  begin
    legend := 'The trend-cycle component derived by adding the interpolated ';
    addString := 'polynomial to the low-frequency components of the residual sequence';
    legend := Concat(legend, addString);
    TrendAndCycle := legend;

  end; {TrendAndCycle}

   {________________________________________________}

  function SeasonalComponent (var dataBOX: rBOX;
                  var dataName: string): string;

    var
      legend, addString: string;

  begin
    legend := 'The seasonal component extracted from ';
    addString := IntegerToString(dataBOX.NOPUNCT);
    legend := Concat(legend, addString, ' points of ');
    legend := Concat(legend, dataName);
    SeasonalComponent := legend;

  end; {SeasonalComponent}

 {________________________________________________}

  function SeasonallyAdjusted (var dataBOX: rBOX;
                  var dataName: string): string;

    var
      legend, addString: string;

  begin
    legend := 'A seasonally adjusted version of a sequence of ';
    addString := IntegerToString(dataBOX.NOPUNCT);
    legend := Concat(legend, addString, ' points of ');
    legend := Concat(legend, dataName);
    SeasonallyAdjusted := legend;

  end; {SeasonallyAdjusted}

 {________________________________________________}

  function SpectralBand (var lowerCut, upperCut: real): string;

    var
      legend, addString: string;

  begin
    legend := 'A spectral band, with a lower bound of ';
    addString := TalyToString(lowerCut);
    legend := Concat(legend, addString, ' degrees and an upper bound of ');
    addString := TalyToString(upperCut);
    legend := Concat(legend, addString, ' degrees, masking the periodogram');
    SpectralBand := legend;
  end; {SpectralBand}

  {________________________________________________}

  procedure PoleZeroLegend (var F: text);

  begin
    Writeln(F, '\par{\narrower\noindent{\bf Figure ', figNumber : 1, '.}');
    Writeln(F, ' The pole--zero diagram of the Filter');
    Writeln(F, '.\par} ');
    Writeln(F, '\medskip');
    figNumber := figNumber + 1;

  end; {PoleZeroLegend}

{________________________________________________}

  function SeasonPoleZero (var filterSpec: FilterRecord): string;

    var
      legend: string;

  begin
    legend := 'The pole-zero diagram of the seasonal adjustment filter';
    if filterSpec.seasonality = Monthly then
      legend := Concat(legend, ' appropriate to monthly data.');
    if filterSpec.seasonality = Quarterly then
      legend := Concat(legend, ' appropriate to quarterly data.');
    SeasonPoleZero := legend;

  end;{SeasonPoleZero}

{________________________________________________}

end. {LEGENDS: Unit 4 of NEWIDEOLOG.PAS}

