program IDEOLOG;

  uses
    GLOBALS, UTILS, ORGANISE, DATASEG, Dos, Crt, Graph;

  var
    theMenuItem: integer;

  {________________________________________________}

begin
  SetTheText;
  Writeln;
  Writeln('      IDEOLOG.PAS (D.S.G. Pollock, <stephen_pollock@sigmapi.u-net.com>)');
  Writeln;
  Writeln('      This program investigates some frequency-domain filtering methods by');
  Writeln('      comparing  their effects  with those  of  other filters. The filters');
  Writeln('      of Baxter and King  and of  Christiano and Fitzgerald  are used  for');
  Writeln('      comparison. These  are truncated  versions  of  the ideal frequency-');
  Writeln('      selective filter. The Henderson  filter, the  Butterworth filter and');
  Writeln('      the Hodrick-Prescott filter are also demonstrated.');
  Writeln;
  Writeln('      There are several alternative ways of accommodating  the trends that');
  Writeln('      are  typical of economic time series. Two of the methods require the');
  Writeln('      data to be reduced to stationarity by differencing before filtering.');
  Writeln;
  Writeln('      In the first method, ordinary filters are applied to the differenced');
  Writeln('      data and, thereafter, the filtered sequences are re-inflated via the');
  Writeln('      summation operator, which is the inverse of the difference operator.');
  Writeln;
  Writeln('      In the second method, the filter that  is applied to the differenced');
  Writeln('      data is formed from  a highpass  or  bandpass filter by removing  an');
  Writeln('      implicit difference operator  contained therein. With the removal of');
  Writeln('      this operator from  the filter, there is no  need to  re-inflate the');
  Writeln('      data after it has been filtered.');
  Writeln(' ' : 66, '<RETURN>');
  Readln;

  {Global Flags}
  graphicsOn := false;
  noSCREENBOX := true;
  SCREENBOX.XC := 0;
  SCREENBOX.YC := 0;
  allsDone := false;
  noPageParameters := true;
  logsTaken := false;

  {-------------------------------------}
  thereIsData := false;
  weHaveATrend := false;
  weHaveTrendResiduals := false;
  weHaveABusinessCycle := false;
  weHaveSeasonalComponent := false;
  weHaveDeasonalisedData := false;
  ResidualsInData := false;
  {--------------------------------------}

  noSmoothingRegions := true;
  noGlobalLambda := true;
  noLocalLambdas := true;
  noSmoothingProfile := true;
  suppressReplace := false;
  figNumber := 1;

  repeat
    ClrScr;
    Writeln;
    Writeln('    IDEOLOG.PAS: Ideal Filters and their Approximations');
    Writeln;
    Writeln('         1. Page Parameters');
    Writeln;
    Writeln('         2. Get the Data and/or Transform the Data');
    Writeln;
    Writeln('         3. Plot the Series');
    Writeln;
    Writeln('         4. Periodogram Analysis');
    Writeln;
    Writeln('         5. Polynomial Regression');
    Writeln;
    Writeln('         6. Variable Smoothing');
    Writeln;
    Writeln('         7. FIR Time-Domain Filters');
    Writeln;
    Writeln('         8. Wiener-Kolmogorov IIR Filters');
    Writeln;
    Writeln('         9. Frequency-Domain Filters');
    Writeln;
    Writeln('        10. Save and Exit');
    Writeln;
    Writeln('                      Select a number and press RETURN');

    GetBoundedInteger(1, 10, theMenuItem);

    case theMenuItem of
      1: 
        PageSetUp(SCREENBOX, POSTBOX, PSPict, TeXFormat); {in ORGANISE}
      2: 
        GetTheData(dataBOX, SCREENBOX, POSTBOX, dataVector); {in ORGANISE}
      3: 
        PlotTheSeries(dataBOX, trendBOX, cycleBOX, residueBOX, seasonBOX, SCREENBOX, POSTBOX, dataVector, trendVector, cycleVec, residueVec, seasonVec);
  {in ORGANISE}
      4: 
        PeriodogamChoice(dataBOX, trendBOX, cycleBOX, residueBOX, seasonBOX, SCREENBOX, POSTBOX, dataVector, trendVector, cycleVec, residueVec, seasonVec);
  {in ORGANISE}
      5: 
        FitPolyTrend(polyParas, dataBOX, trendBOX, residueBOX, SCREENBOX, POSTBOX, dataVector, trendVector, residueVec);
  {in DATASEG: This also should go in ORGANISE}
      6: 
        VariSmooth(dataBOX, trendBOX, residueBOX, SCREENBOX, POSTBOX, dataVector, trendVector, residueVec);
  {in ORGANISE}
      7: 
        ImplementTheFIRFilter(filterSpec, dataBOX, trendBOX, residueBOX, cycleBOX, SCREENBOX, POSTBOX, dataVector, trendVector, residueVec, cycleVec);
  {in ORGANISE}
      8: 
        ImplementTheIIRFilter(filterSpec, polyParas, dataBOX, trendBOX, residueBOX, SCREENBOX, POSTBOX, dataVector, trendVector, residueVec);
  {in ORGANISE}
      9: 
        ImplementFrequencyFilter(filterSpec, polyParas, dataBOX, trendBOX, residueBOX, cycleBOX, SCREENBOX, POSTBOX, dataVector, trendVector, residueVec, cycleVec);
  {in ORGANISE}
      10: 
        SaveAndExit(dataBOX, trendBOX, cycleBOX, residueBOX, seasonBOX, dataVector, trendVector, cycleVec, residueVec, seasonVec);
  {in ORGANISE}
    end;{case}
  until allsDone;

  RevertText;

end.{IDEOLOG Main Unit}

{________________________________________________}

  {Compile the Units in the Following Order:}

  {1. GLOBALS}
  {2. MATHS}
  {3. UTILS}
  {4. LEGENDS}
  {5. SCREEN}
  {6. POSTBIS}
  {7. POST}
  {8. POLYNOME}
  {9. PERIGRAM}
  {10. FOURIER}
  {11. DATASEG}
  {12. TAPER}
  {13. FILTERS}
  {14. FIRFLT}
  {15. SMOOTHER}
  {16. IMPULSE}
  {17. TRIGONOM}
  {18. WIENER}
  {19. ORGANISE}

