unit MATHS; {Unit 2 of NEWIDEOLOG.PAS}

interface

  uses
    GLOBALS;

  function Even (q: integer): boolean;

  function Max (p, q: integer): integer;

  function Min (p, q: integer): integer;

  function Rmax (x, y: real): real;

  function Rmin (x, y: real): real;

  function Power (x: real;
                  n: integer): real;

  function Tan (theta: real): real;

  function CoTan (theta: real): real;

  function Sign (alpha: real): integer;

  function Arg (psi: complex): real;

  {function Cadd (a, b: complex): complex;}

  procedure ComplexAdd (a, b: complex;
                  var result: complex);

  {function Csubtract (a, b: complex): complex;}

  procedure ComplexSub (a, b: complex;
                  var result: complex);

  function Cmod (a: complex): real;

  function Cmodsqr (a: complex): real;

  {function Cmultiply (a, b: complex): complex;}

  procedure ComplexMult (a, b: complex;
                  var result: complex);

  {procedure ComplexPoly (a: complexvector;    p : integer;z: complex;    var    g0: complex;    var    b: complexvector    );}

  {function Cinverse (a: complex): complex;}

  {function Cdivide (a, b: complex): complex;}

  procedure ComplexDiv (a, b: complex;
                  var result: complex);

  {function Csqrt (a: complex): complex;}

  procedure ComplexSqrt (a: complex;
                  var result: complex);

  procedure ComplexPoly (a: complexVector;
                  p: integer;
                  z: complex;
                  var g0: complex;
                  var b: complexvector);

  procedure Mueller (p: integer;
                  poly: complexVector;
                  var root: complex;
                  var quotient: complexVector);

  procedure ComplexRoots (p: integer;
                  var alpha, lambda: complexVector);

  function TheArgument (thepoint: complex): real;

  function TheModulus (apoint: complex): real;

  {function ComplexToPolar (apoint: complex): polar;}

  procedure ComplexToPolar (complexPoint: complex;
                  var polarPoint: polar);

  {function PolarToComplex (polarpoint: polar): complex;}

  procedure PolarToComplex (polarPoint: polar;
                  var complexPoint: complex);

  procedure QuadraticRoots (a, b, c: real;
                  var x1, x2: complex);

  procedure LUsolve (start, n: integer;
                  var a: matrix;
                  var x, b: vector);

  procedure Toeplitz (gamma: vector;
                  var y: longVector;
                  n, q: integer);

  procedure QinDiag (n: integer;
                  var d, s, q, y: longvector);

  procedure Minit (var mu: vector;
                  var varEpsilon: real;
                  gamma: vector;
                  q: integer);

  procedure DivisionAlgorithm (alpha, delta: vector;
                  p, q: integer;
                  var beta: jVector;
                  var rho: vector);

  procedure Convolution (var alpha, beta: vector;
                  p, k: integer);

implementation

  {_________________________________________________}

  function Even (q: integer): boolean;

  begin
    Even := false;
    if q mod 2 = 0 then
      Even := true;
  end; {Even}

  {_________________________________________________}

  function Max (p, q: integer): integer;

  begin
    if p > q then
      Max := p
    else
      Max := q
  end; {Max}

  {_________________________________________________}

  function Min (p, q: integer): integer;

  begin
    if p < q then
      Min := p
    else
      Min := q
  end; {Min}

  {_________________________________________________}

  function Rmax (x, y: real): real;

  begin
    if x > y then
      Rmax := x
    else
      Rmax := y;
  end; {Rmax}

  {_________________________________________________}

  function Rmin (x, y: real): real;

  begin
    if x > y then
      Rmin := y
    else
      Rmin := x;
  end; {Rmin}

  {_________________________________________________}

  function Power (x: real;
                  n: integer): real;

    var
      logPower: real;

  begin
    x := Abs(x);
    if x < 1E-35 then
      Power := 0
    else
      begin {else}
        logPower := n * Ln(x);
        if (logPower < -80) then
          Power := 0
        else if (logPower > 80) then
          Power := Exp(80)
        else
          Power := Exp(logPower);
      end; {else}
  end; {Power}

  {_________________________________________________}

  function Tan (theta: real): real;

    var
      c, s: real;

  begin
    c := Cos(theta);
    s := Sin(theta);
    Tan := s / c;
  end; {Tan}

  {_________________________________________________}

  function CoTan (theta: real): real;

    const
      almostZero = 1E-35;

    var
      c, s: real;

  begin
    c := Cos(theta);
    s := Sin(theta);
    if Abs(s) < almostZero then
      s := Sign(s) * almostZero;
    CoTan := c / s;
  end; {CoTan}

  {_________________________________________________}

  function Sign (alpha: real): integer;

  begin
    Sign := 0;
    if alpha < 0 then
      Sign := -1
    else
      Sign := 1;
  end; {Sign}

  {_________________________________________________}

  function Arg (psi: complex): real;

    var
      theta: real;

  begin
    if psi.re > 0 then
      theta := Arctan(psi.im / psi.re)
    else if (psi.re < 0) and (psi.im > 0) then
      theta := pi + Arctan(psi.im / psi.re)
    else if (psi.re < 0) and (psi.im < 0) then
      theta := -pi + Arctan(psi.im / psi.re);
    Arg := theta
  end; {Arg}

  {_________________________________________________}

  function TheArgument (thepoint: complex): real;

    var
      theta: real;

  begin
    if thepoint.re = 0 then
      theta := Sign(thepoint.im) * pi / 2;
    if thepoint.re <> 0 then
      theta := Arctan(thepoint.im / thepoint.re);
    if thepoint.re < 0 then
      theta := theta + Sign(thepoint.im) * pi;
    TheArgument := theta;
  end; {TheArgument}

  {_________________________________________________}

  function TheModulus (apoint: complex): real;
  begin
    TheModulus := Sqrt(apoint.re * apoint.re + apoint.im * apoint.im);
  end; {TheModulus}

  {_________________________________________________}

  {function ComplexToPolar (apoint: complex): polar;}

  {_________________________________________________}

  procedure ComplexToPolar (complexPoint: complex;
                  var polarPoint: polar);
  begin
    polarPoint.modu := TheModulus(complexPoint);
    polarPoint.argu := TheArgument(complexPoint);
  end; {ComplexToPolar}

  {_________________________________________________}

  {function PolarToComplex (polarPoint: polar): complex;}

  {_________________________________________________}

  procedure PolarToComplex (polarPoint: polar;
                  var complexPoint: complex);
  begin
    complexPoint.re := polarPoint.modu * Cos(polarPoint.argu);
    complexPoint.im := polarPoint.modu * Sin(polarPoint.argu);
  end; {PolarToComplex}

  {_________________________________________________}

  function Cmod (a: complex): real;

  begin
    Cmod := Sqrt(Sqr(a.re) + Sqr(a.im));
  end; {ComplexModulus}

  {_________________________________________________}

  function CmodSqr (a: complex): real;

  begin
    CmodSqr := Sqr(a.re) + Sqr(a.im);
  end; {CmodSqr: Complex Squared Modulus}

  {_________________________________________________}

  {function Cadd (a, b: complex): complex;}
    {This routine cannot be compiled by Turbo Pascal}

  {_________________________________________________}

  procedure ComplexAdd (a, b: complex;
                  var result: complex);
    var
      c: complex;
  begin
    c.re := a.re + b.re;
    c.im := a.im + b.im;
    result := c;
  end; {ComplexAdd: Complex Addition}

   {_________________________________________________}

  {function Csubtract (a, b: complex): complex;}
    {This routine cannot be compiled by Turbo Pascal}

  {_________________________________________________}

  procedure ComplexSub (a, b: complex;
                  var result: complex);
    var
      c: complex;
  begin
    c.re := a.re - b.re;
    c.im := a.im - b.im;
    result := c;
  end; {ComplexSub: Complex Subtraction }

   {_________________________________________________}

  {function Cmultiply (a, b: complex): complex;}
    {This routine cannot be compiled by Turbo Pascal}

  {_________________________________________________}

  procedure ComplexMult (a, b: complex;
                  var result: complex);
    var
      c: complex;
  begin
    c.re := a.re * b.re - a.im * b.im;
    c.im := a.im * b.re + b.im * a.re;
    result := c;
  end; {Complex Multiplication}

   {_________________________________________________}

  {function Cdivide (a, b: complex): complex;}

  {_________________________________________________}

  procedure ComplexDiv (a, b: complex;
                  var result: complex);
    var
      c: complex;
  begin
    c.re := (a.re * b.re + a.im * b.im) / (Sqr(b.re) + Sqr(b.im));
    c.im := (a.im * b.re - b.im * a.re) / (Sqr(b.re) + Sqr(b.im));
    result := c;
  end; {Complex Division}

   {_________________________________________________}

  {function Csqrt (a: complex): complex;}
    {This routine cannot be compiled by Turbo Pascal}

  {_________________________________________________}

  procedure ComplexSqrt (a: complex;
                  var result: complex);

    var
      b: complex;
      rho: real;

  begin
    rho := Sqrt(a.re * a.re + a.im * a.im);
    b.re := Sqrt((rho + a.re) / 2);
    b.im := Sign(a.im) * Sqrt((rho - a.re) / 2);
    result := b;
  end; {ComplexSqrt}

   {_________________________________________________}

    {_________________________________________________}

  procedure ComplexPoly (a: complexVector;
                  p: integer;
                  z: complex;
                  var g0: complex;
                  var b: complexvector);

    var
      i, q: integer;
      beta: real;

  begin
    b[p - 1] := a[p];
    for i := 1 to p - 1 do
      begin
        ComplexMult(b[p - i], z, b[p - i - 1]);
        ComplexAdd(b[p - i - 1], a[p - i], b[p - i - 1]);
      end;
    ComplexMult(b[0], z, g0);
    ComplexAdd(g0, a[0], g0);
  end; {ComplexPoly}

  {_________________________________________________}

  procedure CpolyValue (a: complexvector;
                  p: integer;
                  z: complex;
                  var f: complex;
                  var b: complexvector);

    var
      i: integer;

  begin {CpolyValue}
    b[p - 1].re := a[p].re;
    b[p - 1].im := a[p].im;
    for i := 1 to p - 1 do
      begin
        b[p - i - 1].re := b[p - i].re * z.re - b[p - i].im * z.im + a[p - i].re;
        b[p - i - 1].im := b[p - i].im * z.re + b[p - i].re * z.im + a[p - i].im;
      end;
    f.re := b[0].re * z.re - b[0].im * z.im + a[0].re;
    f.im := b[0].im * z.re + b[0].re * z.im + a[0].im;
  end; {CpolyValue}

  {_________________________________________________}

  procedure Mueller (p: integer;
                  poly: complexVector;
                  var root: complex;
                  var quotient: complexVector);

    const
      almostZero = 1E-15;

    var
      iterations, exit: integer;
      convergence: boolean;
      a, b, c, z0, z1, z2, z3, h, h1, h2, h3, f0, f1, f2, f3: complex;
      delta1, delta2, discrim, denom, store1, store2: complex;

  begin{Mueller}

     {set the intial values}
    z2.re := -0.75 * root.re - 1;
    z2.im := root.im;
    z1.re := 0.75 * root.re;
    z1.im := 1.2 * root.im + 1;
    z0 := root;
    ComplexPoly(poly, p, z0, f0, quotient);
    ComplexPoly(poly, p, z1, f1, quotient);
    ComplexPoly(poly, p, z2, f2, quotient);

    iterations := 0;
    exit := 0;
    convergence := false;

    while (not convergence) and (exit = 0) and (iterations < 60) do
      begin
        ComplexSub(f1, f0, delta1);
        ComplexSub(f2, f0, delta2);
        ComplexSub(z1, z0, h1);
        ComplexSub(z2, z0, h2);
        ComplexSub(z1, z2, h3);
        ComplexMult(h1, h2, h);
        ComplexMult(h, h3, h);
        if Cmod(h) < almostZero then
          exit := 1;      {Can't fit a quadratic to these points}

        if exit = 0 then
          begin  {calculate coefficients of the quadratic}
            ComplexMult(h2, delta1, store1);
            ComplexMult(h1, delta2, store2);
            ComplexSub(store1, store2, a);
            ComplexDiv(a, h, a); { a = (h_2\delta_1 - h_1\delta_2)/h }
            ComplexMult(store1, h2, store1);
            ComplexMult(store2, h1, store2);
            ComplexSub(store2, store1, b);
            ComplexDiv(b, h, b); { b = (h_1^2\delta_2 - h_2^2\delta_1)/h }
            c := f0;
          end;

        if (Cmod(a) <= almostZero) and (Cmod(b) <= almostZero) then
          exit := 2;     {Test if parabola is really a constant }

        ComplexMult(b, b, discrim);  { b^2 }
        ComplexMult(a, c, h);
        discrim.re := discrim.re - 4 * h.re;  { b^2 - 4ac }
        discrim.im := discrim.im - 4 * h.im;
        ComplexSqrt(discrim, discrim); { Sqrt( b^2 - 4ac) }

        if (discrim.re * b.re + discrim.im * b.im > 0) then
          ComplexAdd(b, discrim, denom)
        else
          ComplexSub(b, discrim, denom);

        if Cmod(denom) < almostZero then
          begin  { if b \pm \Sqrt(b^2 - 4ac) \simeq 0 }
            z3.re := 0;
            z3.im := 0;
          end
        else
          begin
            ComplexAdd(c, c, h);
            ComplexDiv(h, denom, h);
            ComplexSub(z0, h, z3);
          end;

        ComplexPoly(poly, p, z3, f3, quotient);
        if (Cmod(h) < 1E-10) or (Cmod(f3) < 1E-10) then
          convergence := true;

        z2 := z1;
        f2 := f1;
        z1 := z0;
        f1 := f0;
        z0 := z3;
        f0 := f3;

        iterations := iterations + 1;
      end; {while}
    root := z3;

{if not convergence then}
{Writeln('    The program failed to converge');}

{Writeln('    root.re  = ', root.re : 8 : 4);}
{Writeln('    root.im  = ', root.im : 8 : 4);}
{Writeln('    Exit = ', exit : 1);}
{Writeln('    iterations = ', iterations : 5);}
{Writeln}

  end; {Mueller}

  {_________________________________________________}

  procedure ComplexRoots (p: integer;
                  var alpha, lambda: complexVector);

    var
      i: integer;
      root: complex;
      quotient, beta: complexVector;

  begin {ComplexRoots}

    beta := alpha;
    for i := 0 to p - 2 do
      begin
        root.re := 0;
        root.im := 0;
        Mueller(p - i, beta, root, quotient);
        lambda[i + 1] := root;
        beta := quotient;
      end;
    lambda[p].re := -beta[0].re / alpha[p].re;
    lambda[p].im := -beta[0].im / alpha[p].re;

  end; {ComplexRoots}

 {_________________________________________________}

  procedure QuadraticRoots (a, b, c: real;
                  var x1, x2: complex);

    var
      delta: real;

  begin
    x1.im := 0;
    x2.im := 0;
    delta := Sqr(b) - 4 * a * c;
    if (delta > 0) and (a <> 0) then
      begin
        x1.re := (-b + Sqrt(delta)) / (2 * a);
        x2.re := (-b - Sqrt(delta)) / (2 * a);
      end;
    if (delta = 0) and (a <> 0) then
      begin
        x1.re := -b / (2 * a);
        x1.re := -b / (2 * a);
      end;
    if (delta < 0) and (a <> 0) then
      begin
        x1.re := -b / (2 * a);
        x1.im := Sqrt(-delta) / (2 * a);
        x2.re := -b / (2 * a);
        x2.im := -Sqrt(-delta) / (2 * a);
      end;
  end; {QuadraticRoots}

  {_________________________________________________}

  procedure LUsolve (start, n: integer;
                  var a: matrix;
                  var x, b: vector);

    type
      ivector = array[0..20] of integer;

    var
      v, w, pivot, lambda: real;
      i, j, k, pivotRow, finish: integer;
      p: ivector;
      d: vector;

  begin {LUsolve}

    finish := start + n - 1;
    for i := start to finish do
      begin {i; determine the scale factors}
        p[i] := i;
        d[i] := 0.0;
        for j := start to finish do
          if d[i] < Abs(a[i, j]) then
            d[i] := Abs(a[i, j]);
      end; {i}

    for i := start to finish - 1 do
      begin {i; begin the process of reduction}
        pivot := a[p[i], i];
        for k := i + 1 to finish do
          begin { k; search for a better pivot}
            v := Abs(pivot) / d[p[i]];
            w := Abs(a[p[k], i]) / d[p[k]];
            if v < w then
              begin  {interchange rows if a better pivot is found}
                pivot := a[p[k], i];
                pivotRow := p[k];
                p[k] := p[i];
                p[i] := pivotRow;
              end; {end interchange}
          end; {k; end the search for a pivot}

        for k := i + 1 to finish do
          begin {k; eliminate a[k, i]}
            lambda := a[p[k], i] / pivot;
            for j := i + 1 to finish do
              a[p[k], j] := a[p[k], j] - lambda * a[p[i], j];
            a[p[k], i] := lambda   {save the multiplier }
          end; {k}
      end; {i; reduction completed}

    for i := start to finish do
      begin {i; forward substitution}
        x[i] := b[p[i]];
        for j := i - 1 downto start do
          x[i] := x[i] - a[p[i], j] * x[j];
      end; {i; forward substitution}

    for i := finish downto start do
      begin {i; back substitution}
        for j := i + 1 to finish do
          x[i] := x[i] - a[p[i], j] * x[j];
        x[i] := x[i] / a[p[i], i];
      end; {i; back substitution}

  end; {LUsolve}

  {__________________________________________}

  procedure Toeplitz (gamma: vector;
                  var y: longVector;
                  n, q: integer);

  {This procedure uses a Cholesky decomposition to find the solution of}
  {the equation Gx = y,  where  G is a symmetric Toeplitz matrix of order}
  {n with q supra-diagonal and q sub-diagonal bands . The coefficients of}
  {G are contained in gamma. The RHS vector y contains the elements of}
  {the solution vector x on completion.}

    var
      t, j, k: integer;
      mu: array[0..maxOrder, 0..maxArray] of real; {wideMatrix}

  begin {Toeplitz}

  {Factorise}
    for t := 0 to q - 1 do
      for j := t + 1 to q do
        mu[j, t] := 0.0;

    for t := 0 to n do
      begin {t}
        for k := Min(q, t) downto 0 do
          begin {k}
            mu[k, t] := gamma[k];
            for j := 1 to q - k do
              mu[k, t] := mu[k, t] - mu[j, t - k] * mu[j + k, t] * mu[0, t - k - j];
            if k > 0 then
              mu[k, t] := mu[k, t] / mu[0, t - k];
          end;{k}
      end; {t}

  {ForwardSolve;}
    for t := 0 to n do
      for j := 1 to Min(t, q) do
        y[t] := y[t] - mu[j, t] * y[t - j];

  {Divide by the diagonal}
    for t := 0 to n do
      y[t] := y[t] / mu[0, t];

  {BackSolve;}
    for t := n downto 0 do
      for j := 1 to Min(q, n - t) do
        y[t] := y[t] - mu[j, t + j] * y[t + j];

  end; {Toeplitz}

  {_________________________________________________}

  procedure QinDiag (n: integer;
                  var d, s, q, y: longvector);
    var
      t: integer;

  begin

  {Factorisation}
    d[-2] := 0.0;
    d[-1] := 0.0;
    s[-1] := 0.0;
    q[-2] := 0.0;
    q[-1] := 0.0;
    for t := 0 to n do
      begin
        d[t] := d[t] - d[t - 1] * Sqr(s[t - 1]) - d[t - 2] * Sqr(q[t - 2]);
        s[t] := (s[t] - d[t - 1] * s[t - 1] * q[t - 1]) / d[t];
        q[t] := q[t] / d[t];
      end;

  {Forward Substitution}
    y[-2] := 0.0;
    y[-1] := 0.0;
    for t := 0 to n do
      y[t] := y[t] - s[t - 1] * y[t - 1] - q[t - 1] * y[t - 2];
    for t := 0 to n do
      y[t] := y[t] / d[t];

  {Back Substitution}
    y[n + 2] := 0.0;
    y[n + 1] := 0.0;
    for t := n downto 0 do
      y[t] := y[t] - s[t] * y[t + 1] - q[t] * y[t + 2];

  end;{QinDiag}

  {________________________________________________}

  procedure Minit (var mu: vector;
                  var varEpsilon: real;
                  gamma: vector;
                  q: integer);

    var
      D: matrix;
      delta, f: vector;
      i, j, iterations: integer;
      convergence: boolean;


  {-------------------------------------------}

    function CheckDelta (q: integer;
                    var delta, mu: vector): boolean;

      var
        i: integer;
        muNorm, deltaNorm: real;

    begin
      muNorm := 0.0;
      deltaNorm := 0.0;
      for i := 0 to q do
        begin
          muNorm := muNorm + sqr(mu[i]);
          deltaNorm := deltaNorm + Sqr(delta[i])
        end;
      if (deltaNorm / muNorm) > 1.0E-20 then
        CheckDelta := false
      else
        CheckDelta := true;

    end; {Checkdelta}

  {------------------------------------------}

  begin {PROCEDURE Minit}

  {initialise the vector mu }
    for i := 0 to q do
      mu[i] := 0.0;
    mu[0] := Sqrt(gamma[0]);
    convergence := false;
    iterations := 0;

    while (convergence = false) and (iterations < 25) do
      begin

  {Form the matrix of derivatives}
        for i := 0 to q do
          for j := 0 to q do
            begin
              D[i, j] := 0.0;
              if (j - i) >= 0 then
                D[i, j] := mu[j - i];
              if (i + j) <= q then
                D[i, j] := D[i, j] + mu[i + j];
            end;

  {Find the function value}
        for j := 0 to q do
          begin
            f[j] := gamma[j];
            for i := 0 to q - j do
              f[j] := f[j] - mu[i] * mu[i + j]
          end;

  {Find the updating vector}
        LUsolve(0, q + 1, D, delta, f);

  {Update the value of mu}
        for i := 0 to q do
          mu[i] := mu[i] + delta[i];
        iterations := iterations + 1;

  {Check for convergence}
        convergence := CheckDelta(q, delta, mu);
      end; {while}
{Writeln('The number of iterations was ', iterations : 3);}

  {Renomalise the results}
    varEpsilon := 1.0;
    for i := 1 to q do
      begin {i}
        mu[i] := mu[i] / mu[0];
        varEpsilon := varEpsilon + mu[i] * mu[i];
      end; {i}
    mu[0] := 1;
    varEpsilon := gamma[0] / varEpsilon;

  end; {Minit}
  {_________________________________________________}

  procedure DivisionAlgorithm (alpha, delta: vector;
                  p, q: integer;
                  var beta: jVector;
                  var rho: vector);

    var
      store: real;
      s, j, i: integer;

  begin

  {for i := p downto 0 do}
  {Writeln('    a(', I : 1, ') = ', alpha[i] : 10 : 5);}
  {Writeln;}
  {for i := q downto 0 do}
  {Writeln('    d(', I : 1, ') = ', delta[i] : 10 : 5);}
  {Writeln;}

    for j := p downto 0 do
      begin {j}
        s := Min(j, p - q);
        store := 0.0;
        for i := j - q + 1 to s do
          store := store + beta[i] * delta[j - i];
        beta[j - q] := (alpha[j] - store) / delta[q]
      end; {j}

    for j := 0 to q - 1 do
      begin {j}
        rho[j] := 0.0;
        for i := j - q to -1 do
          rho[j] := rho[j] + beta[i] * delta[j - i];
      end; {j}

  {for i := p - q downto 0 do}
  {Writeln('    b(', i : 1, ') = ', beta[i] : 10 : 5);}
  {Writeln;}
  {for i := q - 1 downto 0 do}
  {Writeln('    r(', i : 1, ') = ', rho[i] : 10 : 5);}
  {Writeln;}

  end; {Division Algorithm}

  {_________________________________________________}

  procedure Convolution (var alpha, beta: vector;
                  p, k: integer);
    var
      gamma: real;
      i, j, r, s: integer;

  begin

    for j := p + k downto 0 do
      begin {j}
        s := Min(j, p);
        r := Max(0, j - k);
        gamma := 0.0;
        for i := r to s do
          gamma := gamma + alpha[i] * beta[j - i];
        beta[j] := gamma;
      end; {j}

  end; {Convolution}

  {_________________________________________________}

end. {MATHS: Unit 2 of IDEOLOG.PAS}

