unit ORGANISE; {Unit 19 of NEWIDEOLOG.PAS}

interface

  uses
    GLOBALS, MATHS, UTILS, SCREEN, POST, WIENER, FIRFILT, SMOOTHER, DATASEG, TAPER, TRIGONOM, IMPULSE, Dos, Crt, Graph;

  procedure GetTheData (var dataBOX: rBOX;
                  var SCREENBOX, POSTBOX: IBOX;
                  var dataVector: longVector);

  procedure TransformTheData (var dataBOX: rBOX;
                  var dataVector: longVector);

  procedure PlotTheSeries (var dataBOX, trendBOX, cycleBOX, residueBOX, seasonBOX: rBOX;
                  var SCREENBOX, POSTBOX: IBOX;
                  var dataVector, trendVector, cycleVec, residueVec, seasonVec: longVector);

  procedure PeriodogamChoice (var dataBOX, trendBOX, cycleBOX, residueBOX, seasonBOX: rBOX;
                  var SCREENBOX, POSTBOX: IBOX;
                  var dataVector, trendVector, cycleVec, residueVec, seasonVec: longVector);

  procedure PeriodogramAnalysis (var dataBOX, trendBOX, cycleBOX, residueBOX, seasonBOX: rBOX;
                  var SCREENBOX, POSTBOX: IBOX;
                  var dataVector, trendVector, cycleVec, residueVec, seasonVec: longVector);

  procedure VariSmooth (var dataBOX, trendBOX, residueBOX: rBOX;
                  var SCREENBOX, POSTBOX: IBOX;
                  var dataVector, trendVector, residueVec: longVector);

  procedure ChooseFIRTrendMethod (var filterSpec: FilterRecord);

  procedure SpecifyFIRFilter (var filterSpec: FilterRecord;
                  var fIRVec: vector);

  procedure ImplementTheFIRFilter (var filterSpec: FilterRecord;
                  var dataBOX, trendBOX, residueBOX, cycleBOX: rBOX;
                  var SCREENBOX, POSTBOX: IBOX;
                  var dataVector, trendVector, residueVec, cycleVec: longVector);

  procedure SpecifyIIRFilter (var FilterSpec: FilterRecord);

  procedure ImplementTheIIRFilter (var FilterSpec: FilterRecord;
                  var polyParas: polyParameters;
                  var dataBOX, trendBOX, residueBOX: rBOX;
                  var SCREENBOX, POSTBOX: IBOX;
                  var dataVector, trendVector, residueVec: longVector);

  procedure ImplementFrequencyFilter (var filterSpec: FilterRecord;
                  var polyParas: polyParameters;
                  var dataBOX, trendBOX, residueBOX, cycleBOX: rBOX;
                  var SCREENBOX, POSTBOX: IBOX;
                  var dataVector, trendVector, residueVec, cycleVec: longVector);

  procedure SaveAndExit (var dataBOX, trendBOX, cycleBOX, residueBOX, seasonBOX: rBOX;
                  var dataVector, trendVector, cycleVec, residueVec, seasonVec: longVector);

implementation

  {________________________________________________}

  procedure GetTheData (var dataBOX: rBOX;
                  var SCREENBOX, POSTBOX: IBOX;
                  var dataVector: longVector);

    var
      theMenuItem: integer;
      legend: string;

  begin
    repeat
      ClrScr;
      Writeln;
      Writeln('    GET THE DATA');
      Writeln;
      Writeln('         1. Read the Data');
      Writeln;
      Writeln('         2. Plot the Data');
      Writeln;
      Writeln('         3. Transform the Data');
      Writeln;
      Writeln('         4. Subsample the Data');
      Writeln;
      Writeln('         5. Discard Some Data');
      Writeln;
      Writeln('         6. Segment the Data/Desegment the Data');
      Writeln;
      Writeln('         7. Extrapolate and/or Taper the Data');
      Writeln;
      Writeln('         8. Return to the Main Menu');
      Writeln;
      Writeln('                      Select a number and press RETURN');

      GetBoundedInteger(1, 9, theMenuItem);

      case theMenuItem of
        1: 
          ReadTheData(dataBOX, dataVector); {in UTILS}
        2: 
          PlotTheData(dataBOX, SCREENBOX, POSTBOX, dataVector); {in DATASEG}
        3: 
          TransformtheData(dataBOX, dataVector); {in ORGANISE}
        4:
          SubsampleTheData(dataBOX, dataVector); {in DATASEG}
        5:
          DiscardData(SCREENBOX, dataBOX, dataVector); {in DATASEG}
        6:
          SegmentTheData(SCREENBOX, POSTBOX, dataBOX, dataVector); {in DATASEG}
        7:
          MakeExtrapolation(SCREENBOX, POSTBOX, dataBOX, trendBox, residueBOX, dataVector, trendVector, residueVec);
  {in TAPER}
      end;{case}

    until theMenuItem = 8;

  end;{GetTheData}

  {________________________________________________}

  procedure TransformtheData (var dataBOX: rBOX;
                  var dataVector: longVector);

    var
      theMenuItem: integer;
      takeDiffs: boolean;

  begin
    if (thereIsData = false) then
      begin
        SysBeep(3);
        ReadTheData(dataBOX, dataVector);
      end;

    ClrScr;
    Writeln;
    Writeln;
    Writeln('    TRANSFORM THE DATA');
    Writeln;
    Writeln;
    Writeln('         1.  Take Differences');
    Writeln;
    Writeln('         2.  Take Natural Logarithms');
    Writeln;
    Writeln('         3.  Take Running Sums of the Data Points');
    Writeln;
    Writeln('         4.  Take Alternating Running Sums');
    Writeln;
    Writeln('         5.  Return to Main Menu');
    Writeln;
    Writeln('                      Select a number and press RETURN');

    GetBoundedInteger(1, 5, theMenuItem);

    case theMenuItem of
      1: 
        begin {Differences}
          if (dataBOX.diffOrder = 2) then
            begin
              SysBeep(3);
              Writeln('     You have already taken second differences of the data.');
              Writeln;
              Writeln(' ' : 64, '<RETURN>');
              Readln;
            end;
          if (dataBOX.diffOrder = 1) then
            begin
              SysBeep(3);
              Writeln('     You have already taken first differences of the data.');
              YesorNo('     Do you wish to take second differences', takeDiffs);
              if takeDiffs then
                begin
                  DiffOfVector(dataBOX, dataVector);
                  Writeln('     The 1st differences have been replaced by 2nd differences.');
                  Writeln;
                  Writeln(' ' : 64, '<RETURN>');
                  Readln;
                end;
            end;
          if ((dataBOX.diffOrder = 0)) then
            begin
              DiffOfVector(dataBOX, dataVector);
              Writeln('     The original data has been replaced by its first differences. ');
              Writeln;
              Writeln(' ' : 64, '<RETURN>');
              Readln;
            end;
        end; {Differences}
      2: 
        begin {Logarithms}
          if logsTaken then
            begin
              SysBeep(3);
              Writeln('     You have already taken logarithms of the data.');
              Writeln;
              Writeln(' ' : 64, '<RETURN>');
              Readln;
            end
          else
            begin
              LogOfVector(dataBOX, dataVector);
              Writeln('     The  original data has been replaced by its logarithms.');
              Writeln(' ' : 64, '<RETURN>');
              Readln;
            end;
        end; {Logarithms}
      3: 
        SumOfVector(dataBOX, dataVector);
      4: 
        AlternatingVectorSum(dataBOX, dataVector);
    end;{case}

  end;{TransformtheData}

 {________________________________________________}

  procedure PlotTheSeries (var dataBOX, trendBOX, cycleBOX, residueBOX, seasonBOX: rBOX;
                  var SCREENBOX, POSTBOX: IBOX;
                  var dataVector, trendVector, cycleVec, residueVec, seasonVec: longVector);

    var
      theMenuChar: char;
      makeMenu: boolean;

  begin
    if (not thereIsData) then
      ReadTheData(dataBOX, dataVector);

    if noPageParameters then
      PageSetUp(SCREENBOX, POSTBOX, PSPict, TeXFormat);

    makeMenu := false;
    theMenuChar := 'x';

    repeat
      ClrScr;
      Writeln;
      Writeln('    PLOT THE SERIES');
      Writeln;
      if thereIsData then
        begin
          Writeln('         (d) Plot the Data');
          Writeln;
        end;
      if weHaveATrend then
        begin
          Writeln('         (t) Plot the Trend');
          Writeln;
          makeMenu := true;
        end; {trend}
      if weHaveABusinessCycle then
        begin
          Writeln('         (b) Plot the Business Cycle');
          Writeln;
          makeMenu := true;
        end; {trend}
      if weHaveSeasonalComponent then
        begin
          Writeln('         (s) Plot the Seasonal Component');
          Writeln;
          makeMenu := true;
        end;
      if weHaveTrendResiduals then
        begin
          Writeln('         (r) Plot the Trend Residuals ');
          Writeln;
          makeMenu := true;
        end;
      Writeln('         (e) Return to the Main Menu');
      Writeln;
      Writeln('                      Select a letter and press RETURN');

      ReadAChar(theMenuChar);
      theMenuChar := DownCase(theMenuChar);

      case theMenuChar of
        'd': 
          if thereIsData then
            begin
              PlotTheData(dataBOX, SCREENBOX, POSTBOX, dataVector); {DataSeg}
              if (makeMenu = false) then
                theMenuChar := 'e';
            end;
        't': 
          if weHaveATrend then
            PlotTheData(trendBOX, SCREENBOX, POSTBOX, trendVector); {DataSeg}
        'b': 
          if weHaveABusinessCycle then
            PlotTheData(cycleBOX, SCREENBOX, POSTBOX, cycleVec); {DataSeg}
        'r': 
          if weHaveTrendResiduals then
            PlotTheData(residueBOX, SCREENBOX, POSTBOX, residueVec); {DataSeg}
        's': 
          if weHaveSeasonalComponent then
            PlotTheData(seasonBOX, SCREENBOX, POSTBOX, seasonVec); {DataSeg}
      end;{case}
    until theMenuChar = 'e';

  end;{PlotTheSeries}

{________________________________________________}

  procedure PeriodogamChoice (var dataBOX, trendBOX, cycleBOX, residueBOX, seasonBOX: rBOX;
                  var SCREENBOX, POSTBOX: IBOX;
                  var dataVector, trendVector, cycleVec, residueVec, seasonVec: longVector);

    var
      segments, superimpose: boolean;

  begin

    if (dataBOX.segmentedData = true) then
      begin {segmentedData}
        ClrScr;
        Writeln;
        Writeln('    SEGMENTED PERIODOGRAM ');
        Writeln;
        if (dataBOX.numbOfSegs = 1) then
          begin {if}
            Writeln('    Your have segmented the data with only one segment.');
            Writeln('    Your are advised to resegment or to desegment the data.');
            segments := false;
            Writeln(' ' : 60, '<RETURN>');
            Readln;
          end {if}
        else
          begin {else}
            Writeln('    Your have divided the data into ', dataBOX.numbOfSegs : 2, ' segments of ', dataBOX.segLength : 3, ' points each. ');
            YesOrNo('    Do you wish to plot the  periodograms of the segments?', segments);
          end; {else}

        if segments then
          begin {plot the segments}
            YesOrNo('    Do you wish to superimpose the  segments?', superimpose);
            MultiPeriodogram(superimpose, SCREENBOX, dataBOX, dataVector)
          end;
        if not segments then
          PeriodogramAnalysis(dataBOX, trendBOX, cycleBOX, residueBOX, seasonBOX, SCREENBOX, POSTBOX, dataVector, trendVector, cycleVec, residueVec, seasonVec);

      end;{segmentedData}

    if (dataBOX.segmentedData = false) then
      PeriodogramAnalysis(dataBOX, trendBOX, cycleBOX, residueBOX, seasonBOX, SCREENBOX, POSTBOX, dataVector, trendVector, cycleVec, residueVec, seasonVec);

  end; {PeriodogamChoice}

 {________________________________________________}

  procedure PeriodogramAnalysis (var dataBOX, trendBOX, cycleBOX, residueBOX, seasonBOX: rBOX;
                  var SCREENBOX, POSTBOX: IBOX;
                  var dataVector, trendVector, cycleVec, residueVec, seasonVec: longVector);

    var
      theMenuChar: char;
      legend: string;

  begin
    if (not thereIsData) then
      ReadTheData(dataBOX, dataVector);

    if noPageParameters then
      PageSetUp(SCREENBOX, POSTBOX, PSPict, TeXFormat);

    theMenuChar := 'x';
    repeat
      ClrScr;
      Writeln;
      Writeln('    PERIODOGRAM ANALYSIS');
      Writeln;
      if thereIsData then
        begin
          Writeln('         (d) Plot the Periodogram of the Data');
          Writeln;
        end; {data}
      if weHaveATrend then
        begin
          Writeln('         (t) Plot the Periodogram of the Trend Component');
          Writeln;
        end; {trend}
      if weHaveABusinessCycle then
        begin
          Writeln('         (b) Plot the Periodogram of the Business Cycle');
          Writeln;
        end; {trend}
      if weHaveSeasonalComponent then
        begin
          Writeln('         (s) Plot the Periodogram of the Seasonal Component');
          Writeln;
        end; {seasonalComponent}
      if weHaveTrendResiduals then
        begin
          Writeln('         (r) Plot the Periodogram of the Trend Residuals');
          Writeln;
        end; {trendResiduals}
      Writeln('         (e) Return to the Main Menu');
      Writeln;
      Writeln('                      Select a letter and press RETURN');

      ReadAChar(theMenuChar);
      theMenuChar := DownCase(theMenuChar);

      case theMenuChar of
        'd': 
          if thereIsData then
            begin
              legend := 'The periodogram of ';
              if weHaveDeasonalisedData then
                legend := Concat(legend, 'the seasonally adjusted version of ');
              legend := Concat(legend, dataName, '.');
              PlotThePeriodogam(dataBOX, SCREENBOX, POSTBOX, dataVector, legend); {in DATASEG}
            end;
        't': 
          if weHaveATrend then
            begin
              legend := 'The periodogram of the trend in ';
              legend := Concat(legend, dataName, '.');
              PlotThePeriodogam(trendBOX, SCREENBOX, POSTBOX, trendVector, legend); {in DATASEG}
            end;
        'b': 
          if weHaveABusinessCycle then
            begin
              legend := 'The periodogram of the business cycle in ';
              legend := Concat(legend, dataName, '.');
              PlotThePeriodogam(cycleBOX, SCREENBOX, POSTBOX, cycleVec, legend); {in DATASEG}
            end;
        'r': 
          if weHaveTrendResiduals then
            begin
              YesOrNo('    Do you wish to impose a highlighted band upon the peridogram', imposeBand);
              legend := 'The periodogram of residuals from fitting a trend to ';
              legend := Concat(legend, dataName, '.');
              PlotThePeriodogam(residueBOX, SCREENBOX, POSTBOX, residueVec, legend); {in DATASEG}
            end;
        's': 
          if weHaveSeasonalComponent then
            begin
              legend := 'The periodogram of the seasonal cycle in ';
              legend := Concat(legend, dataName, '.');
              PlotThePeriodogam(seasonBOX, SCREENBOX, POSTBOX, seasonVec, legend); {in DATASEG}
            end;
      end;{case}
    until theMenuChar = 'e';

  end;{PeriodogramAnalysis}

  {________________________________________________}

  procedure VariSmooth (var dataBOX, trendBOX, residueBOX: rBOX;
                  var SCREENBOX, POSTBOX: IBOX;
                  var dataVector, trendVector, residueVec: longVector);

  {This Goes into ORGANISE}
    var
      theMenuItem: integer;
      smoothLambda: triRealVec;
      sBandCentre, sBandWidth: triIntVec;

  begin
    ClrScr;
    Writeln;
    Writeln('    SMOOTHING WITH LOCAL VARIATIONS');
    Writeln;
    Writeln;
    Writeln('   This  segment of the program implements a version of Hodrick--Prescott');
    Writeln('   filter in which the value of the  smoothing parameter can be varied in');
    Writeln('   as many as three sub intervals of the  period spanned by the data. The');
    Writeln('   purpose  of reducing the  value of the  smoothing  parameter within  a');
    Writeln('   restricted interval is to allow the trend to undergo a change in slope');
    Writeln('   or  a rapid shift in level  of a sort that corresponds to a structural');
    Writeln('   break in the processes generating the data');
    Writeln(' ' : 65, '<RETURN>');
    Readln;

    if (not thereIsData) then
      ReadTheData(dataBOX, dataVector);

    if noPageParameters then
      begin
        SysBeep(3);
        PageSetUp(SCREENBOX, POSTBOX, PSPict, TeXFormat);
      end;

    repeat
      ClrScr;
      Writeln;
      Writeln;
      Writeln('    SMOOTHING WITH LOCAL VARIATIONS');
      Writeln;
      Writeln('         1.  Specify the Regular Smoothing Parameter');
      Writeln;
      Writeln('         2.  Specify the Regions of Smoothing Variation');
      Writeln;
      Writeln('         3.  Specify the Local Smoothing Parameters');
      Writeln;
      Writeln('         4.  Specify the Profile of the Smoothing Variation');
      Writeln;
      Writeln('         5.  Smooth the Data');
      Writeln;
      Writeln('         6.  Return to the Main Menu');
      Writeln;
      Writeln('                      Select a number and press RETURN');
      GetBoundedInteger(1, 6, theMenuItem);

      case theMenuItem of
        1: 
          GetGlobalLambda(smoothLambda);
        2: 
          GetLocalSmoothingRegions(sBandWidth, sBandCentre, SCREENBOX, POSTBOX, dataBOX, dataVector);
        3: 
          begin {case 3}
            if noSmoothingRegions then
              GetLocalSmoothingRegions(sBandWidth, sBandCentre, SCREENBOX, POSTBOX, dataBOX, dataVector);
            if noGlobalLambda then
              GetGlobalLambda(smoothLambda);
            GetLocalLambdas(smoothLambda, sBandWidth, sBandCentre, SCREENBOX, POSTBOX, dataBOX, dataVector);
          end; {case 3}
        4: 
          begin {case 4}
            if noSmoothingRegions then
              GetLocalSmoothingRegions(sBandWidth, sBandCentre, SCREENBOX, POSTBOX, dataBOX, dataVector);
            if noGlobalLambda then
              GetGlobalLambda(smoothLambda);
            if noLocalLambdas then
              GetLocalLambdas(smoothLambda, sBandWidth, sBandCentre, SCREENBOX, POSTBOX, dataBOX, dataVector);
            GetSmoothingProfile(sProfile);
          end; {case 4}
        5: 
          begin {case 5}
            if noSmoothingRegions then
              GetLocalSmoothingRegions(sBandWidth, sBandCentre, SCREENBOX, POSTBOX, dataBOX, dataVector);
            if noGlobalLambda then
              GetGlobalLambda(smoothLambda);
            if noLocalLambdas then
              GetLocalLambdas(smoothLambda, sBandWidth, sBandCentre, SCREENBOX, POSTBOX, dataBOX, dataVector);
            if noSmoothingProfile then
              GetSmoothingProfile(sProfile);
            MakeSmoothVector(smoothLambda, sBandWidth, sBandCentre, dataBOX, trendBOX, residueBOX, SCREENBOX, POSTBOX, dataVector, trendVector, residueVec);
          end; {case 5}
      end;{case}

    until theMenuItem = 6;

  end; {VariSmooth}

  {________________________________________________}

  procedure ChooseFIRTrendMethod (var FilterSpec: FilterRecord);

    var
      theMenuChar: Char;

  begin
    ClrScr;
    Writeln;
    Writeln;
    Writeln('    APPLY A HIGHPASS OR A BANDPASS FILTER TO THE DATA');
    Writeln;
    Writeln('    There are two ways of applying the filters');
    Writeln;
    Writeln('    A.) The filter  can  be applied directly to the original data  sequence at');
    Writeln('    the  cost of loosing a number of data points at  the beginning and the end');
    Writeln('    of the data that are equal to half the number of filter coefficients.');
    Writeln;
     {Writeln('    B.) The  filter can  be applied to data that  have been reduced to station');}
     {Writeln('    -arity by differencing. Twofold  differencing  is usually appropriate. The');}
     {Writeln('    extra -sample values which are needed to enable the filter to reach to the');}
     {Writeln('    ends  of  the data  sequence  are provided by  zeros. After filtering, the');}
     {Writeln('    processed  sequence is  subjected  to  an anti-differncing, or cumulation,');}
     {Writeln('    operation.');}
     {Writeln; }
    Writeln('    B.) The  symmetric  highpass  or  bandpass  filter  embodies   a  two-fold');
    Writeln('    difference  operator. This operator may  be factored  from the filter  and');
    Writeln('    applied directly to the data. A reduced  filter,  which correspond  to the');
    Writeln('    quotient obtained by dividing the z-transform of  the  original filter  by');
    Writeln('    the z-transform of  twofold  difference  operator, is then applied  to the');
    Writeln('    differenced data. There is no need to  reinflate the data after filtering.');
    Writeln;
    theMenuChar := MenuChar('A', 'B');
    theMenuChar := DownCase(theMenuchar);

    case theMenuChar of
      'a': 
        filterSpec.trendMethod := ApplyDirect;
       {'b':}
 {filterSpec.trendMethod := DiffAndReinflate;}
      'b': 
        filterSpec.trendMethod := DiffAndReduce;
    end; {cases}

  end; {ChooseFIRTrendMethod}

  {_________________________________________________}

  procedure SpecifyFIRFilter (var FilterSpec: FilterRecord;
                  var fIRVec: vector);

  {Called by ImplementFIRFilter in ORGANISE}

    var
      theMenuItem: integer;

  begin
    ClrScr;
    Writeln;
    Writeln('    SELECT AN FIR FILTER');
    Writeln;
    Writeln('         1. Henderson Lowpass Filter');
    Writeln;
    Writeln('         2. Baxter-King Bandpass Filter');
    Writeln;
    Writeln('         3. Christiano--Fitgerald Bandpass Filter');
    Writeln;
    Writeln('                      Select a number and press RETURN');

    GetBoundedInteger(1, 3, theMenuItem);
    case theMenuItem of
      1: 
        begin {1}
          FilterSpec.filterFamily := Henderson;
          FilterSpec.filterType := lowPass;
          ClrScr;
          Writeln;
          Writeln('    SPECIFY THE HENDERSON LOWPASS FILTER');
          Writeln;
          HendersonFilterSpec(FilterSpec); {in FIRFLTR}
        end; {1}
      2: 
        begin {2}
          FilterSpec.filterFamily := BaxterKing;
          FilterSpec.filterType := bandPass;
          ClrScr;
          Writeln;
          Writeln('    SPECIFY THE BAXTER-KING BANDPASS FILTER');
          Writeln;
          BaxterKingFilterSpec(filterSpec);{in FIRFLTR}
        end; {2}
      3: 
        begin {3}
          FilterSpec.filterFamily := ChristFitz;
          FilterSpec.filterType := bandPass;
          ClrScr;
          Writeln;
          Writeln('    SPECIFY THE CHRISTIANO-FITZGERALD BANDPASS FILTER');
          Writeln;
          ChristFitzFilterSpec(filterSpec); {in FIRFLTR}
        end; {3}
      33: 
        begin {33}
          FilterSpec.filterFamily := BaxterKing;
          FilterSpec.filterType := highPass;
          ClrScr;
          Writeln;
          Writeln('    SPECIFY THE TRUNCATED IDEAL HIGHPASS FILTER');
          Writeln;
          Writeln;
          BaxterKingFilterSpec(filterSpec); {in FIRFLTR}
        end; {33}
      44: 
        begin {44}
          FilterSpec.filterFamily := BaxterKing;
          FilterSpec.filterType := lowPass;
          ClrScr;
          Writeln;
          Writeln('    SPECIFY THE TRUNCATED IDEAL LOWPASS FILTER');
          Writeln;
          BaxterKingFilterSpec(filterSpec);{in FIRFLTR}
        end; {44}
      55: 
        begin {55}
          FilterSpec.filterFamily := BaxterKing;
          FilterSpec.filterType := bandStop;
          ClrScr;
          Writeln;
          Writeln('    SPECIFY THE TRUNCATED IDEAL BANDSTOP FILTER');
          Writeln;
          BaxterKingFilterSpec(filterSpec);{in FIRFLTR}
        end; {55}
    end;{cases}

  end;{SpecifyFIRFilter}

  {________________________________________________}

  procedure ExplainChristFitz;
  begin
    ClrScr;
    Writeln;
    Writeln;
    Writeln('    THE FREQUENCY RESPONSE OF THE CHRISTIANO FITZGERALD FILTER.');
    Writeln;
    Writeln('    The frequency response is defined only for linear time-invariant filters.');
    Writeln('    The frequency  response the  bandpass filter of Christiano and Fitzgerald');
    Writeln('    is undefined, since the filter coefficients  vary  thoughout the sample.');
    Writeln;
    Writeln(' ' : 68, '<RETURN>');
    Readln
  end;

  {________________________________________________}

  procedure ImplementTheFIRFilter (var filterSpec: FilterRecord;
                  var dataBOX, trendBOX, residueBOX, cycleBOX: rBOX;
                  var SCREENBOX, POSTBOX: IBOX;
                  var dataVector, trendVector, residueVec, cycleVec: longVector);
    var
      theMenuItem: integer;

  begin
    SpecifyFIRFilter(filterSpec, fIRVec);  {in ORGANISE.PAS}
  {fIRVec is Global}

    if filterSpec.FilterFamily in [Henderson, BaxterKing, ChristFitz] then
      repeat {Henderson, BaxterKing, ChristFitz}
        ClrScr;
        Writeln;
        Writeln('    INVESTIGATE AND APPLY THE FILTER');
        Writeln;
        Writeln('         1. Plot the Frequency Response');
        Writeln;
        Writeln('         2. Plot the Impulse Response');
        Writeln;
        Writeln('         3. Apply the Filter');
        Writeln;
        Writeln('         4. Return to the Previous Menu');
        Writeln;
        Writeln('                      Select a number and press RETURN');
        Writeln;
        GetBoundedInteger(1, 4, theMenuItem);
        case theMenuItem of
          1: 
            begin {1: Frequency Response}
              if filterSpec.FilterFamily = BaxterKing then
                PlotFIRBPassGain(SCREENBOX, filterSpec);
              if (filterSpec.FilterFamily = ChristFitz) then
                ExplainChristFitz;
              if (filterSpec.FilterFamily = Henderson) then
                PlotHendersonGain(SCREENBOX, filterSpec);
              if (filterSpec.FilterFamily = ChristFitz) then
                ExplainChristFitz;
            end; {1: Frequency Response}
          2: 
            begin {2: impulse response}
              if (filterSpec.FilterFamily = BaxterKing) then
                BaxterKingImpulseResponse(filterSpec);
              if (filterSpec.FilterFamily = ChristFitz) then
                ChristFitzImpulseResponse(filterSpec);
            end;{2: impulse response}
  {Plot the Impulse Response}
          3: 
            begin {3: Apply The Filter}
              if (filterSpec.filterFamily = BaxterKing) then
                begin {BaxterKing}
                  ChooseFIRTrendMethod(filterSpec); {in ORGANISE}
                  ApplyTruncFilter(dataBOX, cycleBOX, filterSpec, dataVector, cycleVec); {in FIRFILT.PAS}
                end; {BaxterKing}
              if (filterSpec.filterFamily = Henderson) then
                ApplyMusgaveFilter(dataBOX, trendBOX, residueBOX, filterSpec, dataVector, trendVector, residueVec);
              if (filterSpec.FilterFamily = ChristFitz) then
                ApplyChristFitzFilter(filterSpec, dataBOX, cycleBOX, dataVector, cycleVec);
            end; {3:Apply The Filter}
        end;{case}
      until theMenuItem = 4; {end Henderson, BaxterKing}

  end; {ImplementTheFIRFilter}

{________________________________________________}

  procedure SpecifyIIRFilter (var FilterSpec: FilterRecord);

    var
      theMenuItem: integer;

  begin
    ClrScr;
    Writeln;
    Writeln('    SELECT AN IIR FILTER');
    Writeln;
    Writeln('         1. Hodrick--Prescott  Filter');
    Writeln;
    Writeln('         2. Butterworth Filter');
    Writeln;
    Writeln('         3. Seasonal Adjustment Filter');
    Writeln;
    Writeln('                      Select a number and press RETURN');

    GetBoundedInteger(1, 3, theMenuItem);

    case theMenuItem of
      1: 
        begin {1}
          filterSpec.filterFamily := HodrickPrescott;
          filterSpec.filterType := lowPass;
          ClrScr;
          Writeln;
          Writeln('    SPECIFY THE HODRICK-PRESCOTT FILTER');
          Writeln;
          HPFilterSpec(filterSpec, poles, zeros, gamma, delta, numerator, denominator); {in WIENER}
        end; {1}
      2: 
        begin {2}
          FilterSpec.filterFamily := Butterworth;
          FilterSpec.filterType := lowPass;
          ClrScr;
          Writeln;
          Writeln('    SPECIFY THE LOWPASS BUTTERWORTH FILTER');
          Writeln;
          ButterSpec(filterSpec, poles, zeros, gamma, delta); {in WIENER}
        end; {2}
      3: 
        begin {3}
          FilterSpec.filterFamily := DeSeasoning;
          FilterSpec.filterType := bandStop;
          ClrScr;
          Writeln;
          Writeln('    SPECIFY THE SEASONAL ADJUSTMENT FILTER');
          Writeln;
          SeasonalFilterSpec(filterSpec, poles, zeros, gamma, delta, numerator, denominator); {in WIENER}
        end; {3}
    end; {cases}

  end; {SpecifyIIRFilter}

  {________________________________________________}

  procedure ImplementTheIIRFilter (var filterSpec: FilterRecord;
                  var polyParas: polyParameters;
                  var dataBOX, trendBOX, residueBOX: rBOX;
                  var SCREENBOX, POSTBOX: IBOX;
                  var dataVector, trendVector, residueVec: longVector);

    var
      theMenuItem: integer;

  begin
    SpecifyIIRFilter(filterSpec);
  {We could localise some of the parameters. i.e. poles, zeros gamma, delta}
    repeat
      ClrScr;
      Writeln;
      Writeln('    INVESTIGATE AND APPLY FILTER');
      Writeln;
      Writeln('         1. Plot the Pole-Zero Diagram');
      Writeln;
      Writeln('         2. Plot the Frequency Response');
      Writeln;
      Writeln('         3. Plot the Impulse Response');
      Writeln;
      Writeln('         4. Apply the Filter ');
      Writeln;
      Writeln('         5. Return to the Previous Menu');
      Writeln;
      Writeln('                      Select a number and press RETURN');
      Writeln;
      GetBoundedInteger(1, 5, theMenuItem);

      case theMenuItem of
        1: 
          PoleZeroPlot(filterSpec, poles, zeros); {in WIENER}
        2: 
          PlotWienerFilterGain(SCREENBOX, filterSpec); {in WIENER}
        3: 
          begin {3: Plot the Impulse Response}
            if filterSpec.filterfamily = Butterworth then
              ButterworthImpulse(filterSpec);
            if filterSpec.filterfamily = HodrickPrescott then
              HodrickPrescottImpulse(filterSpec);
          end; {3: Plot the Impulse Response}
        4: 
          begin {4: Apply the Filter}
            if filterSpec.filterFamily = HodrickPrescott then  {in WIENER}
              ApplyHPFilter(dataBOX, residueBOX, cycleBOX, dataVector, trendVector, residueVec, cycleVec, polyParas, filterSpec);
            if filterSpec.filterFamily = Butterworth then {in WIENER}
              ApplyButterFilter(dataBOX, residueBOX, cycleBOX, dataVector, trendVector, residueVec, cycleVec, polyParas, filterSpec);
            if filterSpec.filterFamily = DeSeasoning then {in WIENER: chi is the denomiator coefficient vector}
              DoSeasAdjust(polyParas, denominator, dataBOX, cycleBOX, seasonBOX, dataVector, cycleVec, seasonVec, filterSpec); {in WIENER}
{PeriodogramPlots(dataBOX, residueBOX, SCREENBOX, POSTBOX, dataVector, residueVec);}
          end; {4: Apply the Filter}
      end;{case}
    until theMenuItem = 5;

  end; {ImplementTheIIRFilter}

  {________________________________________________}

  procedure ImplementFrequencyFilter (var filterSpec: FilterRecord;
                  var polyParas: polyParameters;
                  var dataBOX, trendBOX, residueBOX, cycleBOX: rBOX;
                  var SCREENBOX, POSTBOX: IBOX;
                  var dataVector, trendVector, residueVec, cycleVec: longVector);

    {Called by MAIN}
    var
      theMenuItem: integer;
      choice: char;

  begin

    if (dataBOX.diffOrder > 0) then
      begin
        ClrScr;
        Writeln;
        Writeln('    SPECIFY AND APPLY A FREQUENCY-DOMAIN  FILTER');
        Writeln;
        Writeln('        To implement these procedures you require undifferenced data');
        Writeln('        Read the data afresh  before returning to this segment');
        Writeln;
        Writeln(' ' : 60, '<RETURN>');
        Readln;
      end
    else
      repeat
        ClrScr;
        Writeln;
        Writeln('    SPECIFY AND APPLY A FREQUENCY-DOMAIN FILTER');
        Writeln;
        Writeln('         1. Lowpass  Filter');
        Writeln;
        Writeln('         2. BandPass Filter');
        Writeln;
        Writeln('         3. De-sesonalising BandStop Filter');
        Writeln;
        Writeln('         4. Return to the Main Menu');
        Writeln;
        Writeln('                      Select a number and press RETURN');

        GetBoundedInteger(1, 4, theMenuItem);

        case theMenuItem of
          1: 
            begin {1}
              filterSpec.filterFamily := Frequency;
              filterSpec.filterType := lowPass;
              ClrScr;
              Writeln;
              Writeln('    SPECIFY THE FREQUENCY DOMAIN LOWPASS FILTER');
              Writeln;
              Writeln('    In order to apply the Fourier method, data  must be reduced to station-');
              Writeln('    arity. Twofold  differencing  is usually  appropriate  for the purpose.');
              Writeln('    There are two ways of proceeding:');
              Writeln;
              Writeln('    A. The  frequency filter  may  be applied to the  differenced data and,');
              Writeln('       thereafter, the filtered sequence can  be reinflated via a summation');
              Writeln('       operation.');
              Writeln;
              Writeln('       This option  also provides  a  facilty  for identifying  the turning');
              Writeln('       points of the low-frequency  cycle  by locating the  points at which');
              Writeln('       the derivative function cuts the horizontal axis.');
              Writeln;
              Writeln('    B. A lowpass filter can be realised by subtracting a highpass component');
              Writeln('       from the  data.  The  differencing  factor that is  contained within');
              Writeln('       the higpass filter can  be cancelled  with the summation operator to');
              Writeln('       avoid the need for reinflating the filtered sequence.');
              Writeln;
              choice := MenuChar('A', 'B');

              case choice of
                'A', 'a': 
                  filterSpec.trendMethod := DiffAndReinflate;
                'B', 'b': 
                  filterSpec.trendMethod := DiffAndReduce;
              end; {case}

              Writeln;
              Writeln('    Specify the cut-off point in degrees');
              ReadANumber(filterSpec.cutOff);
              if (filterSpec.trendMethod = DiffAndReinflate) then
                LowpassFrequency(filterSpec, polyParas, dataBOX, trendBOX, residueBOX, cycleBOX, SCREENBOX, POSTBOX, dataVector, trendVector, residueVec, cycleVec);
              if (filterSpec.trendMethod = DiffAndReduce) then
                AltLowpassFrequency(filterSpec, polyParas, dataBOX, trendBOX, residueBOX, cycleBOX, SCREENBOX, POSTBOX, dataVector, trendVector, residueVec, cycleVec);
            end; {1}
          2: 
            begin {2}
              filterSpec.filterFamily := Frequency;
              filterSpec.filterType := bandPass;
              ClrScr;
              Writeln;
              Writeln('    SPECIFY THE FREQUENCY DOMAIN BANDPASS FILTER');
              Writeln;
              Writeln;
              Writeln('         Specify the lower cut-off point in degrees');
              ReadANumber(filterSpec.lowerCut);
              Writeln('         Specify the upper cut-off point in degrees');
              ReadANumber(filterSpec.upperCut);
              BandpassFrequency(filterSpec, dataBOX, cycleBOX, SCREENBOX, POSTBOX, dataVector, cycleVec); {in TRIGONOM}
            end; {2}
          3: 
            begin {3}
              filterSpec.filterFamily := Frequency;
              filterSpec.filterType := bandStop;
              if (dataBOX.diffOrder > 0) then
                begin
                  ClrScr;
                  Writeln;
                  Writeln('    SPECIFY THE FREQUENCY DOMAIN SEASONAL ADJUSTMENT FILTER');
                  Writeln;
                  Sysbeep(3);
                  Writeln('    The data have been differenced. Therefore, they cannot be seasonally adjusted');
                  Writeln;
                  Writeln(' ' : 60, '<RETURN>');
                  Readln;
                end
              else {in TRIGONOM}
                AdjustForSeasons(polyParas, dataBOX, trendBOX, seasonBOX, residueBOX, SCREENBOX, POSTBOX, dataVector, trendVector, seasonVec, residueVec);
            end; {3}
        end; {cases}

      until theMenuItem = 4;

  end;{ImplementFrequencyFilter}

  {________________________________________________}

  procedure SaveAndExit (var dataBOX, trendBOX, cycleBOX, residueBOX, seasonBOX: rBOX;
                  var dataVector, trendVector, cycleVec, residueVec, seasonVec: longVector);

    var
      theMenuChar: char;

  begin
    repeat
      ClrScr;
      Writeln;
      Writeln('    SAVE AND EXIT');
      Writeln;
      if thereIsData then
        begin
          Writeln('         (f) Save the Data');
          Writeln;
        end;
      if weHaveATrend then
        begin
          Writeln('         (t) Save the Trend Component');
          Writeln;
        end;
      if weHaveABusinessCycle then
        begin
          Writeln('         (b) Save the Business-Cycle Component');
          Writeln;
        end;
      if weHaveTrendResiduals then
        begin
          Writeln('         (d) Save the Trend Residuals');
          Writeln;
        end;
      if weHaveSeasonalComponent then
        begin
          Writeln('         (s) Save the Seasonal Component');
          Writeln;
        end;
      Writeln('         (r) RETURN to the Main Menu');
      Writeln;
      Writeln('         (e) EXIT the Program');
      Writeln;
      Writeln('                      Select a letter and press RETURN');

      ReadAChar(theMenuChar);
      theMenuChar := DownCase(theMenuChar);

      case theMenuChar of
        'f': 
          if thereIsData then
            WriteData(dataBOX, dataVector); {in UTILS}
        't': 
          if weHaveATrend then
            WriteData(trendBOX, trendVector); {in UTILS}
        'b': 
          if weHaveABusinessCycle then
            WriteData(cycleBOX, cycleVec); {in UTILS}
        'd': 
          if weHaveTrendResiduals then
            WriteData(residueBOX, residueVec); {in UTILS}
        's': 
          if weHaveSeasonalComponent then
            WriteData(seasonBOX, seasonVec); {in UTILS}
        'e': 
          allsDone := true;
      end;{case}
    until theMenuChar in ['r', 'e'];

  end;{SaveAndExit}

  {________________________________________________}

end. {ORGANISE: Unit 18 of NEWIDEOLOG.PAS}

