unit PERIGRAM; {Unit 9 of NEWIDEOLOG.PAS}

interface

  uses
    GLOBALS, MATHS;

  procedure PrimeFactors (Tcap: integer;
                  var g: integer;
                  var N: ivector;
                  var palindrome: boolean);

  function tOfj (j, g: integer;
                  P, Q: ivector): integer;

  procedure ReOrder (P, Q: ivector;
                  Tcap, g: integer;
                  var yImag, yReal: longVector);

  procedure MixedRadixCore (var yReal, yImag: longVector;
                  var N, P, Q: ivector;
                  Tcap, g: integer);

  procedure RealMixedRadix (x: longVector;
                  var xi: longVector;
                  var Ncap: integer);

  procedure Periodogram (xi: longVector;
                  var spec: pointsVector;
                  var maxi: real;
                  Ncap: integer);

  procedure SlowPeriodogram (var dataBOX, periodoBOX: rBOX;
                  var dataVector: longVector;
                  var periodoVec: longVector);

implementation

  {_________________________________________________}

  procedure PrimeFactors (Tcap: integer;
                  var g: integer;
                  var N: ivector;
                  var palindrome: boolean);

    var
      i, p, T, first, last, store: integer;

    function Next (p: integer): integer;
    begin {Next}
      if p = 2 then
        Next := 3
      else
        Next := p + 2
    end; {Next}

  begin {PrimeFactors}
    palindrome := false;
    g := 1;
    p := 2;
    T := Tcap;

  {Find the prime factors}
    while sqr(p) <= T do
      begin {while}
        if T mod p = 0 then
          begin
            T := T div p;
            N[g] := p;
            g := g + 1
          end
        else
          p := Next(p);
      end;{while}
    N[g] := T;

    first := 1;
    last := g;

  {Rearrange the factors}
    while (N[first] <= N[first + 1]) do
      begin {while}
        store := N[first];
        for i := first to last - 1 do
          N[i] := N[i + 1];
        N[last] := store;
        if (N[first] = N[last]) then
          first := first + 1;
        last := g + 1 - first;
      end; {while}
    if (last - first <= 0) and (g <> 1) then
      palindrome := true

  end; {PrimeFactors}

  {_________________________________________________}

  function tOfj (j, g: integer;
                  P, Q: ivector): integer;

    var
      i, t: integer;

  begin
    t := (j div Q[g]) * P[g];
    for i := g - 1 downto 1 do
      t := t + ((j mod Q[i + 1]) div Q[i]) * P[i];
    tOfj := t
  end; {tOfj}

  {_________________________________________________}

  procedure ReOrder (P, Q: ivector;
                  Tcap, g: integer;
                  var yImag, yReal: longVector);

    var
      h, t, j, Pcount: integer;

    procedure Pcycle (h: integer;
                    var yImag, yReal: longVector;
                    var Pcount: integer);

      var
        j, t: integer;
        Rstore, Istore: real;

    begin {Pcycle}
      j := h;
      t := tOfj(j, g, P, Q);
      if t = j then
        Pcount := Pcount + 1
      else
        begin {else}
          Rstore := yReal[j];
          Istore := yImag[j];
          repeat
            yReal[j] := yReal[t];
            yImag[j] := yImag[t];
            Pcount := Pcount + 1;
            j := t;
            t := tOfj(j, g, P, Q);
          until t = h;
          yReal[j] := Rstore;
          yImag[j] := Istore;
          Pcount := Pcount + 1;
        end; {else}
    end; {Pcycle}

  begin {ReOrder}
    h := 1;
    Pcount := 1;
    Pcycle(h, yImag, yReal, Pcount);

    repeat {h}
      h := h + 1;
      j := h;
      repeat {j}
        t := tOfj(j, g, P, Q);
        j := t;
      until j <= h;
      if j = h then
        Pcycle(h, yImag, yReal, Pcount);
    until (h = Tcap - 1) or (Pcount = Tcap - 1);

  end; {ReOrder}

  {_________________________________________________}

  procedure MixedRadixCore (var yReal, yImag: longVector;
                  var N, P, Q: ivector;
                  Tcap, g: integer);

    type
      Wvector = array[0..23] of real;
      Wmatrix = array[0..23] of Wvector;

    var
      a, b, c, t, i, j, k, l, r: integer;
      W, Wl, theta, twopi: real;
      yR, yI: Wvector;
      cosine, Sine: Wmatrix;

  begin {MixedRadixCore}
    twopi := 8 * ArcTan(1.0);
    W := twopi / Tcap;
    Q[0] := 1;
    N[0] := 1;
    P[0] := Tcap;
    b := 0;
    c := 0;

    for l := 1 to g do
      begin {l : this is the major loop}
        a := 0;
        t := 0;
        Q[l] := Q[l - 1] * N[l - 1];
        P[l] := P[l - 1] div N[l];

    {Construct the transformation matrix}
        Wl := twopi / N[l];
        for j := 0 to N[l] - 1 do
          for r := j to N[l] - 1 do
            begin {r,j}
              theta := Wl * ((j * r) mod N[l]);
              cosine[j, r] := Cos(theta);
              sine[j, r] := Sin(theta);
              cosine[r, j] := cosine[j, r];
              sine[r, j] := Sine[j, r];
            end; {r, j}

        for i := 0 to Q[l] - 1 do
          begin {i}
            for k := 0 to P[l] - 1 do
              begin {k}
    {Subsequences are indexed by i,k jointly}

    {Copy a subsequence of the data}
                for j := 0 to N[l] - 1 do
                  begin {j}
                    t := a + b + c;
                    yR[j] := yReal[t];
                    yI[j] := yImag[t];
                    b := b + P[l];
                  end; {j}
                b := 0;

    {Transform the subsequence}
                for j := 0 to N[l] - 1 do
                  begin {j}
                    t := a + b + c;
                    yReal[t] := 0.0;
                    yImag[t] := 0.0;
                    for r := 0 to N[l] - 1 do
                      begin {r}
                        yReal[t] := yReal[t] + yR[r] * cosine[j, r] + yI[r] * sine[j, r];
                        yImag[t] := yImag[t] + yI[r] * cosine[j, r] - yR[r] * sine[j, r];
                      end; {r}
                    b := b + P[l];
                  end; {j}
                b := 0;

    {Scale the subsequence by the twiddle factors}
                if l < g then
                  begin {if}
                    for j := 0 to N[l] - 1 do
                      begin {j: twiddle factors}
                        t := a + b + c;
                        theta := W * ((j * c * Q[l]) mod Tcap);
                        yR[0] := yReal[t];
                        yI[0] := yImag[t];
                        yReal[t] := yR[0] * Cos(theta) + yI[0] * Sin(theta);
                        yImag[t] := yI[0] * Cos(theta) - yR[0] * Sin(theta);
                        b := b + P[l];
                      end; {j: twiddle factors}
                    b := 0;
                  end; {if}

                c := c + 1;
              end; {k}
            c := 0;
            a := a + N[l] * P[l];
          end; {i}
      end; {l : the major loop}

  end; {MixedRadixCore}

  {_________________________________________________}

  procedure RealMixedRadix (x: longVector;
                  var xi: longVector;
                  var Ncap: integer);

    var
      t, i, j, g, Nmax, Tcap: integer;
      Q, N, P: ivector;
      yReal, yImag: longVector;
      palindrome: boolean;
      zetaReven, zetaRodd, zetaIeven, zetaIodd: real;
      pi, theta, increment, sine, cosine: real;

  begin
    pi := 4 * ArcTan(1.0);
    g := 0;
    Tcap := Ncap div 2 + 1;

    repeat {find a composite number for T}
      Tcap := Tcap - 1;
      palindrome := false;
      PrimeFactors(Tcap, g, N, palindrome);

      Nmax := 0;
      for i := 1 to g do
        if N[i] > Nmax then
          Nmax := N[i];
    until (g >= 3) and (Nmax <= 23);

    Ncap := 2 * Tcap;
    increment := pi / Tcap;
    theta := 0;

    for t := 0 to Tcap - 1 do
      begin
        yReal[t] := x[2 * t] / Ncap;
        yImag[t] := x[2 * t + 1] / Ncap;
      end;

    MixedRadixCore(yReal, yImag, N, P, Q, Tcap, g);
    ReOrder(P, Q, Tcap, g, yImag, yReal);

    for j := 1 to (Tcap div 2) do
      begin{j}
        theta := theta + increment;
        cosine := Cos(theta);
        sine := Sin(theta);

        zetaReven := (yReal[j] + yReal[Tcap - j]) / 2;
        zetaRodd := (yReal[j] - yReal[Tcap - j]) / 2;
        zetaIeven := (yImag[j] + yImag[Tcap - j]) / 2;
        zetaIodd := (yImag[j] - yImag[Tcap - j]) / 2;

        xi[j] := zetaReven + cosine * zetaIeven - sine * zetaRodd;
        xi[Tcap - j] := zetaReven - cosine * zetaIeven + sine * zetaRodd;
        xi[Tcap + j] := zetaIodd - cosine * zetaRodd - sine * zetaIeven;
        xi[Ncap - j] := -zetaIodd - cosine * zetaRodd - sine * zetaIeven;
      end;{j}
    xi[0] := yReal[0] + yImag[0];
    xi[Tcap] := yReal[0] - yImag[0];

  end; {RealMixedRadix}

  {_________________________________________________}

  procedure Periodogram (xi: longVector;
                  var spec: pointsVector;
                  var maxi: real;
                  Ncap: integer);

    var
      j, Nover2: integer;
      omega, pi, increment, scale: real;

  begin
    pi := 4.0 * ArcTan(1.0);
    Nover2 := Ncap div 2;
    scale := 2 * Ncap;
  {the scale for the periodogram}
  {scale := Ncap / (2 * pi);}
  {the scale for the sample spectrum}
    increment := pi / Nover2;
    omega := 0;
    spec[0].x := 0;
    spec[0].y := scale * Sqr(xi[0]);
    spec[Nover2].x := pi;
    spec[Nover2].y := scale * Sqr(xi[Nover2]);
    maxi := Rmax(spec[0].y, spec[Nover2].y);
    for j := 1 to Nover2 - 1 do
      begin {j}
        omega := omega + increment;
        spec[j].x := omega;
        spec[j].y := scale * (Sqr(xi[j]) + Sqr(xi[Nover2 + j]));
        maxi := Rmax(maxi, spec[j].y);
      end; {j}
  end; {Periodogram}

  {_________________________________________________}

  procedure SlowPeriodogram (var dataBOX, periodoBOX: rBOX;
                  var dataVector: longVector;
                  var periodoVec: longVector);

    var
      alpha, beta: longVector;
      j, t, Tcap, Ncap, sign, stop: integer;
      pi, fourierInc, omegaj, omegajt, scale: double;
      maxi: real;
{The data should be mean adjusted, In which case alpha[0] = beta[0] = 0.}

  begin
    Tcap := dataBOX.segLength;
    Ncap := Tcap div 2;
    pi := 4.0 * Arctan(1.0);
    scale := 1 / (2.0 * Tcap * pi);
    fourierInc := (2 * pi) / Tcap;
    maxi := 0.0;

    omegaj := 0.0;
    for j := 0 to Ncap do
      begin {j}
        alpha[j] := 0.0;
        beta[j] := 0.0;
        omegajt := 0.0;
        for t := 0 to Tcap - 1 do
          begin {t}
            alpha[j] := alpha[j] + Cos(omegajt) * dataVector[t];
            beta[j] := beta[j] + Sin(omegajt) * dataVector[t];
            omegajt := omegajt + omegaj;
          end; {t}
        periodoVec[j] := scale * (Sqr(alpha[j]) + Sqr(beta[j]));
        if j > 0 then
          maxi := Rmax(maxi, periodoVec[j]);
        omegaj := omegaj + fourierInc;
      end; {j}

    periodoVec[0] := Sqr(alpha[0]) * scale;
    if Tcap = 2 * Ncap then
      begin {if}
        periodoVec[Ncap] := Sqr(alpha[Ncap]) * scale;
      end; {if}
    periodoVec[0] := 0.0; {This is tantamount the mean-adjustment of the data}

    periodoBOX.xMax := pi;
    periodoBOX.xMin := 0;
    periodoBOX.yMax := maxi;
    periodoBOX.yMin := 0;
    periodoBOX.NOPUNCT := Ncap; {Ncap +1?}

  end; {SlowPeriodogram}

{________________________________________________}


end. {PERIGRAM: Unit 9 of NEWIDEOLOG.PAS}

