unit POLYNOME; {Unit 8 of NEWIDEOLOG.PAS}

interface

  uses
    GLOBALS, UTILS, MATHS, SCREEN, LEGENDS, POST, Crt, Graph;


  procedure PolyRegress (var x, y: longVector;
                  var alpha, gamma, delta: vector;
                  var poly, wgt: longVector;
                  q, n: integer);

  procedure ExePolyRegress (var x, y: longVector;
                  var alpha, gamma, delta: vector;
                  var poly, wgt: longVector;
                  q, n: integer);

  procedure OrthoToPower (alpha, gamma, delta: vector;
                  var beta: vector;
                  q: integer);

  function PolyOrdinate (z: real;
                  var alpha, gamma, delta: vector;
                  q: integer): real;

  procedure MakePolyTrend (var polyParas: polyParameters;
                  var dataBOX, trendBox, residueBOX: rBOX;
                  var dataVector, trendVector, residueVec, PolyWgts: longVector);

  procedure QuadraticWgts (var PolyWgts: longVector;
                  var minWgt, maxWgt: real;
                  var dataBOX: rBOX;
                  var SCREENBOX: IBOX);

  procedure CrenelatedWgts (var PolyWgts: longVector;
                  var minWgt, maxWgt: real;
                  var dataBOX: rBOX;
                  var SCREENBOX: IBOX);

  procedure CosBellWgts (var PolyWgts: longVector;
                  var minWgt, maxWgt: real;
                  var dataBOX: rBOX;
                  var SCREENBOX: IBOX);

  procedure UniformWgts (var PolyWgts: longVector;
                  var dataBOX: rBOX);

  procedure GetTheWeights (var PolyWgts: longVector;
                  var dataBOX: rBOX;
                  var SCREENBOX: IBOX);

implementation

  {_________________________________________________}

  procedure PolyRegress (var x, y: longVector;
                  var alpha, gamma, delta: vector;
                  var poly, wgt: longVector;
                  q, n: integer);

    var
      phi, philag: longVector;
      denom, lagdenom, temp: real;
      k, t: integer;

  begin {PolyRegress}
    alpha[0] := 0.0;
    gamma[1] := 0.0;
    denom := 0.0;

    for t := 0 to n do
      begin
        alpha[0] := alpha[0] + y[t] * wgt[t];
        gamma[1] := gamma[1] + x[t] * wgt[t];
        denom := denom + wgt[t];
      end;

    alpha[0] := alpha[0] / denom;
    gamma[1] := gamma[1] / denom;
    lagdenom := denom;

    for t := 0 to n do
      begin
        philag[t] := 1;
        phi[t] := x[t] - gamma[1];
        poly[t] := alpha[0];
      end;

    for k := 1 to q do
      begin {k}

        alpha[k] := 0.0;
        gamma[k + 1] := 0;
        denom := 0.0;

        for t := 0 to n do
          begin {t}
            alpha[k] := alpha[k] + y[t] * phi[t] * wgt[t];
            denom := denom + Sqr(phi[t]) * wgt[t];
            gamma[k + 1] := gamma[k + 1] + Sqr(phi[t]) * x[t] * wgt[t];
          end; {t}

        alpha[k] := alpha[k] / denom;
        gamma[k + 1] := gamma[k + 1] / denom;
        delta[k + 1] := denom / lagdenom;
        lagdenom := denom;

        for t := 0 to n do
          begin {t}
            poly[t] := poly[t] + alpha[k] * phi[t];
            temp := phi[t];
            phi[t] := (x[t] - gamma[k + 1]) * phi[t] - delta[k + 1] * philag[t];
            philag[t] := temp;
          end; {t}

      end; {k}

  end; {PolyRegress}

  {________________________________________________}

  procedure ExePolyRegress (var x, y: longVector;
                  var alpha, gamma, delta: vector;
                  var poly, wgt: longVector;
                  q, n: integer);

  {It looks  as if this is now redundant}

    var
      phi, philag: longVector;
      denom, lagdenom, temp: real;
      k, t: integer;

  begin {PolyRegress}

    alpha[0] := 0.0;
    gamma[1] := 0.0;
    denom := 0.0;

    for t := 0 to n do
      begin
        alpha[0] := alpha[0] + y[t] * wgt[t];
        gamma[1] := gamma[1] + x[t] * wgt[t];
        denom := denom + wgt[t];
      end;

    alpha[0] := alpha[0] / denom;
    gamma[1] := gamma[1] / denom;
    lagdenom := denom;

    for t := 0 to n do
      begin
        philag[t] := 1;
        phi[t] := x[t] - gamma[1];
        poly[t] := alpha[0];
      end;

    for k := 1 to q do
      begin {k}

        alpha[k] := 0.0;
        gamma[k + 1] := 0;
        denom := 0.0;

        for t := 0 to n do
          begin {t}
            alpha[k] := alpha[k] + y[t] * phi[t] * wgt[t];
            denom := denom + Sqr(phi[t]) * wgt[t];
            gamma[k + 1] := gamma[k + 1] + Sqr(phi[t]) * x[t] * wgt[t];
          end; {t}

        alpha[k] := alpha[k] / denom;
        gamma[k + 1] := gamma[k + 1] / denom;
        delta[k + 1] := denom / lagdenom;
        lagdenom := denom;

        for t := 0 to n do
          begin {t}
            poly[t] := poly[t] + alpha[k] * phi[t];
            temp := phi[t];
            phi[t] := (x[t] - gamma[k + 1]) * phi[t] - delta[k + 1] * philag[t];
            philag[t] := temp;
          end; {t}

      end; {k}

  end; {PolyRegress}

  {________________________________________________}

  procedure OrthoToPower (alpha, gamma, delta: vector;
                  var beta: vector;
                  q: integer);

    var
      phiplus: real;
      phi, philag: vector;
      i, k: integer;

  begin
    phi[-1] := 0;
    phi[0] := -gamma[1];
    phi[1] := 1; {redundant}
    philag[0] := 1;
    philag[1] := 0;
    beta[0] := alpha[0];

  {find the power-form coefficients}
    for k := 1 to q do
      begin {k}
        beta[k] := 0.0;
        for i := k downto 0 do
          begin
            beta[i] := beta[i] + alpha[k] * phi[i];
            phiplus := phi[i - 1] - gamma[k + 1] * phi[i] - delta[k + 1] * philag[i];
            philag[i] := phi[i];
            phi[i] := phiplus;
          end;
        phi[k + 1] := 1;
        philag[K + 1] := 0;
      end;{k}

  end; {OrthoToPower}

  {________________________________________________}

  function PolyOrdinate (z: real;
                  var alpha, gamma, delta: vector;
                  q: integer): real;

    {alpha, gamma, delta and q--> polyDegree should be subsumbed under polyParas}

    var
      d: vector;
      j: integer;

  begin
    d[q] := alpha[q];
    d[q - 1] := alpha[q - 1] + d[q] * (z - gamma[q]);
    for j := q - 2 downto 0 do
      d[j] := alpha[j] + d[j + 1] * (z - gamma[j + 1]) - d[j + 2] * delta[j + 2];
    PolyOrdinate := d[0];

  end; {PolyOrdinate}

  {________________________________________________}

  procedure MakePolyTrend (var polyParas: polyParameters;
                  var dataBOX, trendBox, residueBOX: rBOX;
                  var dataVector, trendVector, residueVec, PolyWgts: longVector);

 {Called by FitPolyTrend in DATASEG: alpha, gamma, delta adn polyOrder--> polyDegree}
  {should be subsumbed under polyParas}

    var
      x: longVector;
      i: integer;
      beta: vector;

  begin
    with PolyParas do
      begin {polyParas}
        tStart := 0;
        residueBOX := dataBOX;
        trendBOX := dataBOX;

        for i := 0 to dataBOX.NOPUNCT - 1 do
          x[i] := i;

        PolyRegress(x, dataVector, alpha, gamma, delta, trendVector, PolyWgts, polyDegree, dataBOX.NOPUNCT - 1);
{++++++++++++++++++++++++++++++++++++++++++++++++++}
        if polyDegree <= 10 then
          begin
            OrthoToPower(alpha, gamma, delta, beta, polyDegree);
            Writeln;
            Writeln(' These are the cofficients of the powers of x:');
            Writeln;
            for i := 0 to polyDegree do
              Writeln('   beta[', i : 1, '] = ', beta[i] : 10 : 5);
          end;
        Writeln;
        Writeln(' Press <RETURN> to Continue');
        Readln;
{++++++++++++++++++++++++++++++++++++++++++++++++++}

        for i := 0 to dataBOX.NOPUNCT - 1 do
          residueVec[i] := dataVector[i] - trendVector[i];

      end; {polyParas}

    MakeDataBox(trendBOX, trendVector);
    MakeDataBox(residueBOX, residueVec);
    weHaveATrend := true;
    weHaveTrendResiduals := true;

  end; {MakePolyTrend}

  {_______________________________________________}

  procedure QuadraticWgts (var PolyWgts: longVector;
                  var minWgt, maxWgt: real;
                  var dataBOX: rBOX;
                  var SCREENBOX: IBOX);
    var
      answer: boolean;
      f0, f1, z1, z2, a, b, c: real;
      det, quadratic, midRange: real;
      t, Tcap, cutOff: integer;
      quadBOX: rBOX;
      legend: string;

  begin
    ClrScr;
    Writeln;
    Writeln('    QUADRATIC WEIGHTING FUNCTION');
    Writeln;
    Writeln('    The option exists for splitting the quadratic weighting function');
    Writeln('    by  interpolating a constant segment in  the middle that has the');
    Writeln('    minimum weighting');
    Writeln;

    Tcap := dataBOX.NOPUNCT;
    YesorNo('   Do you wish to interpolate a middle segment?', answer);
    if answer then
      begin
        Writeln('    Specify the relative length of the middle range as a percentage');
        ReadBoundedNumber(0.0, 100.0, midRange);
        midRange := Midrange / 100.0;
        cutOff := Round(Tcap * (1.0 - MidRange) / 2.0);
        z1 := cutOff;
        z2 := 2 * cutOff;
      end
    else
      begin
        cutoff := Tcap div 2;
        z1 := (Tcap - 1) / 2.0;
        z2 := (Tcap - 1);
      end;

  {Find the quadratic parameters}

    f0 := maxWgt;
    f1 := minWgt;
    c := f0;
    det := z1 * (z1 * z2 - z2 * z2);
    det := 1.0 / det;
    c := f0;
    a := z2 * (f1 - f0) * det;
    b := -z2 * a;

    for t := 0 to Tcap div 2 do
      begin
        if (t <= cutoff) then
          begin
            quadratic := a * t + b;
            quadratic := quadratic * t + c;
          end
        else
          quadratic := minWgt;
        PolyWgts[t] := quadratic;
        PolyWgts[Tcap - 1 - t] := quadratic;
      end; {t}

    quadBOX := dataBOX;
    quadBOX.ymax := maxWgt;
    quadBOX.ymin := minWgt;

    GetGraphics(graphicsOn, SCREENBOX, currentMode);
    ScreenFrame(quadBOX, SCREENBOX);
    ScreenPlot(quadBOX, SCREENBOX, PolyWgts);

    legend := PolynomialWeights;
    ScreenCaption(legend);

    CallForReturn;
    Readln;
    CloseGraph;
    RestoreCrtMode;
    ClrScr;

  end; {QuadraticWgts}

  {_______________________________________________}

  procedure CrenelatedWgts (var PolyWgts: longVector;
                  var minWgt, maxWgt: real;
                  var dataBOX: rBOX;
                  var SCREENBOX: IBOX);
    var
      answer: boolean;
      crenWeight, midRange: real;
      t, Tcap, cutOff: integer;
      crenBOX: rBOX;
      legend: string;

  begin
    ClrScr;
    Writeln;
    Writeln('    CRENELATED WEIGHTING FUNCTION');
    Writeln;
    Writeln('    By default, the first quarter of the sample has the maximum');
    Writeln('    weighting, the  middle half has the  minimum  weighting and');
    Writeln('    the final quarter has the maximum weighting');
    Writeln;

    Tcap := dataBOX.NOPUNCT;

    YesorNo('    Do you wish to vary the length of the middle range?', answer);
    if answer then
      begin
        Writeln('    Specify the relative length of the middle range as a percentage');
        ReadBoundedNumber(0.0, 100.0, midRange);
        midRange := Midrange / 100.0;
        cutOff := Round(Tcap * (1.0 - MidRange) / 2.0);
      end
    else
      cutoff := Tcap div 4;

    for t := 0 to Tcap div 2 do
      begin
        if (t <= cutoff) then
          begin
            crenWeight := MaxWgt;
          end
        else
          crenWeight := MinWgt;
        PolyWgts[t] := crenWeight;
        PolyWgts[Tcap - 1 - t] := crenWeight;
      end; {t}

    crenBOX := dataBOX;
    crenBOX.ymax := maxWgt;
    crenBOX.ymin := minWgt;

    GetGraphics(graphicsOn, SCREENBOX, currentMode);
    ScreenFrame(crenBOX, SCREENBOX);
    ScreenPlot(crenBOX, SCREENBOX, PolyWgts);

    legend := PolynomialWeights;
    ScreenCaption(legend);

    CallForReturn;
    Readln;
    CloseGraph;
    RestoreCrtMode;
    ClrScr;

  end; {CrenelatedWgts}

  {_______________________________________________}

  procedure CosBellWgts (var PolyWgts: longVector;
                  var minWgt, maxWgt: real;
                  var dataBOX: rBOX;
                  var SCREENBOX: IBOX);
    var
      answer: boolean;
      pi, omega, gamma, cosWeight, midRange: real;
      t, Tcap, cutOff: integer;
      cosBOX: rBOX;
      legend: string;

  begin
    ClrScr;
    Writeln;
    Writeln('    COSINE-BELL WEIGHTING FUNCTION');
    Writeln;
    Writeln('    The option exists for  splitting the cosine-bell weighting');
    Writeln('    function by interpolating a constant segment in the middle');
    Writeln('    that has the minimum weighting');
    Writeln;

    pi := 4.0 * ArcTan(1.0);
    Tcap := dataBOX.NOPUNCT;
    YesorNo('   Do you wish to interpolate a middle segment?', answer);
    if answer then
      begin
        Writeln('    Specify the relative length of the middle range as a percentage');
        ReadBoundedNumber(0.0, 100.0, midRange);
        midRange := Midrange / 100.0;
        cutOff := Round(Tcap * (1.0 - MidRange) / 2.0);
      end
    else
      cutoff := Tcap div 2;

    for t := 0 to Tcap div 2 do
      begin
        if (t <= cutoff) then
          begin
            omega := (t * pi) / cutOff;
            gamma := (maxWgt - minWgt) / 2.0;
            cosWeight := minWgt + gamma * (1 + Cos(omega));
          end
        else
          cosWeight := minWgt;
        PolyWgts[t] := cosWeight;
        PolyWgts[Tcap - 1 - t] := cosWeight;
      end; {t}

    cosBOX := dataBOX;
    cosBOX.ymax := maxWgt;
    cosBOX.ymin := minWgt;

    GetGraphics(graphicsOn, SCREENBOX, currentMode);
    ScreenFrame(cosBOX, SCREENBOX);
    ScreenPlot(cosBOX, SCREENBOX, PolyWgts);

    legend := PolynomialWeights;
    ScreenCaption(legend);

    CallForReturn;
    Readln;
    CloseGraph;
    RestoreCrtMode;
    ClrScr;

  end; {CosBellWgts}

  {_______________________________________________}

  procedure UniformWgts (var PolyWgts: longVector;
                  var dataBOX: rBOX);

    var
      t, Tcap: integer;

  begin
    Tcap := dataBOX.NOPUNCT;
    for t := 0 to Tcap - 1 do
      PolyWgts[t] := 1.0;

  end; {UniformWgts}

  {_______________________________________________}

  procedure GetTheWeights (var PolyWgts: longVector;
                  var dataBOX: rBOX;
                  var SCREENBOX: IBOX);

    var
      acceptance: boolean;
      theMenuItem: integer;
      minWgt, maxWgt: real;

  begin
    ClrScr;
    Writeln;
    Writeln('    SPECIFY THE WEIGHTING FUNCTION');
    Writeln;
    YesorNo('    Do you accept that the minimum weight should be unity', acceptance);
    if acceptance then
      minWgt := 1.0
    else
      begin
        Writeln('    Specify the minimum value for the weights');
        ReadBoundedNumber(0.0, 100.0, minWgt);
      end;
    Writeln('    Specify the maximum value for the weights');
    ReadBoundedNumber(minWgt, 100.0, maxWgt);

    ClrScr;
    Writeln;
    Writeln('    SPECIFY THE WEIGHTING FUNCTION');
    Writeln;
    Writeln('         1.  Quadratic Weighting Function');
    Writeln;
    Writeln('         2.  Crenelated Weighting Function');
    Writeln;
    Writeln('         3.  Cosine-Bell Weighting Function');
    Writeln;
    Writeln('         4.  Uniform Weighting Function');
    Writeln;
    Writeln('                      Select a number and press <RETURN>');

    GetBoundedInteger(1, 4, theMenuItem);
    case theMenuItem of
      1: 
        QuadraticWgts(PolyWgts, minWgt, maxWgt, dataBOX, SCREENBOX);
      2: 
        CrenelatedWgts(PolyWgts, minWgt, maxWgt, dataBOX, SCREENBOX);
      3: 
        CosBellWgts(PolyWgts, minWgt, maxWgt, dataBOX, SCREENBOX);
      4: 
        UniformWgts(PolyWgts, dataBOX);
    end; {case}

  end; {GetTheWeights}

  {_________________________________________________}


end. {POLYNOME: Unit 8 of NEWIDEOLOG}

