unit POST; {Unit 7 of NEWIDEOLOG.PAS}

interface

  uses
    GLOBALS, UTILS, MATHS, POSTBIS;

  procedure PostHeader (var F: text;
                  var POSTBOX: IBOX;
                  TeXFormat: boolean);

  procedure PostFooter (var F: text;
                  var POSTBOX: IBOX;
                  legend: string;
                  TexFormat: boolean);

  procedure PostMarks (var F: text;
                  XSTART, YSTART: real;
                  taly, firstMark, farBound, STEP: real;
                  Axis: string);

  procedure PostFrame (var F: text;
                  var realBOX: rBOX;
                  var POSTBOX: IBOX);

  procedure PostVertAxis (var F: text;
                  scaleSide: integer;
                  XSTART, YSTART: integer;
                  taly, firstMark, farBound, VSTEP: real);

  procedure PostZeroPiAxis (var F: text;
                  var POSTBOX: IBOX);

  procedure PostFrameLite (var F: text;
                  var realBOX: rBOX;
                  var POSTBOX: IBOX;
                  scaleSide: integer);

  procedure PostPlot (var F: text;
                  var realBOX: rBOX;
                  var POSTBOX: IBOX;
                  var dataVector: longVector);

  procedure PostPlot2 (var F: text;
                  var realBOX: rBOX;
                  var POSTBOX: IBOX;
                  var dataVector: longVector);

  procedure PostBiPlot (var F: text;
                  var realBOX: rBOX;
                  var POSTBOX: IBOX;
                  var dataVector: pointsVector);

  procedure PostPlotMod (var F: text;
                  var realBOX: rBOX;
                  var POSTBOX: IBOX;
                  var dataVector: pointsVector);

  procedure PostPlotMod2 (var F: text;
                  var realBOX: rBOX;
                  var POSTBOX: IBOX;
                  var dataVector: longVector);

  procedure PostImpulsePlot (var impulseBOX: rBOX;
                  var POSTBOX: IBOX;
                  var impulseVec: longVector;
                  var legend: string);

  procedure PostGainFooter (var F: text;
                  var POSTBOX: IBOX;
                  filterSpec: filterRecord;
                  TeXFormat: boolean);

  procedure WritePostFile (var dataBOX: rBOX;
                  var POSTBOX: IBOX;
                  var dataVector: longVector;
                  var legend: string);

  procedure WritePostSpectral (var dataBOX: rBOX;
                  var POSTBOX: IBOX;
                  var dataVector: longVector;
                  var legend: string);

  procedure WritePostOverFile (var dataBOX: rBOX;
                  var POSTBOX: IBOX;
                  var dataVector, trendVector: longVector;
                  var legend: string);

  procedure PostCarPlot (var F: text;
                  var filterBOX: RBox;
                  var POSTBOX: Ibox;
                  var filterSpec: FilterRecord);

  procedure PostFilterLimits (var F: text;
                  var filterSpec: FilterRecord;
                  var residueBOX: rBOX;
                  var POSTBOX: IBOX);

  procedure MakePostLine (var F: text;
                  var Tvert: integer;
                  var POSTBOX: IBOX;
                  var dataBOX: rBOX);

implementation

{________________________________________________}

  procedure PostHeader (var F: text;
                  var POSTBOX: IBOX;
                  TeXFormat: boolean);
    var
      BOXHGT, BOXWDT, INDENT: integer;
      LLx, LLy, URx, URy: integer;
      xOrigin, yOrigin: integer;

  begin
     {xOrigin := 160;}
     {yOrigin := 560;}

    BOXWDT := POSTBOX.X2 - POSTBOX.X1;
    BOXHGT := POSTBOX.Y2 - POSTBOX.Y1;

    xOrigin := (595 - BOXWDT) div 2;
    yOrigin := 730 - BOXHGT;

    if TeXFormat then
      Writeln(F, '\special{postscript')
    else
      begin
        LLx := xOrigin - 40;
        LLy := yOrigin - 25;
        URx := xOrigin + BOXWDT + 40;
        URy := yOrigin + BOXHGT + 10;
        Writeln(F, '%!PS-Adobe-2.0');
        Writeln(F, '%%Creator: SigmaPi');
        Writeln(F, '%%BoundingBox:', LLx : 3, '  ', LLy : 3, '  ', URx : 3, '  ', URy : 3);
      end;

    Writeln(F, '/showxhatch');
    Writeln(F, '{/label exch def');
    Writeln(F, '/y exch def');
    Writeln(F, '/x exch def');
    Writeln(F, 'x  y  moveto');
    Writeln(F, '0  5 rlineto');
    Writeln(F, 'stroke');
    Writeln(F, 'x  y  moveto');
    Writeln(F, '0 -20 rmoveto');
    Writeln(F, 'label dup stringwidth pop 2 div neg 0 rmoveto show} def');

    Writeln(F, '/showyhatch');
    Writeln(F, '{/label exch def');
    Writeln(F, '/y exch def');
    Writeln(F, '/x exch def');
    Writeln(F, 'x  y  moveto');
    Writeln(F, ' 5 0 rlineto');
    Writeln(F, 'stroke');
    Writeln(F, 'x  y  moveto');
    Writeln(F, '-15 -2 rmoveto');
    Writeln(F, 'label dup stringwidth pop neg 0 rmoveto show} def');

    Writeln(F, '/showqhatch');
    Writeln(F, '{/label exch def');
    Writeln(F, '/y exch def');
    Writeln(F, '/x exch def');
    Writeln(F, 'x  y moveto');
    Writeln(F, ' -5 0 rlineto');
    Writeln(F, 'stroke');
    Writeln(F, 'x  y  moveto');
    Writeln(F, '30 -2 rmoveto');
    Writeln(F, 'label dup stringwidth pop neg 0 rmoveto show} def');

    if TeXFormat then
      begin {TeXFormat}
        INDENT := Round(((12 * 72) / 2.54 - BOXWDT) / 2);
        Writeln(F, INDENT : 2, '  ', -(BOXHGT + 15) : 2, '  translate');
      end {TeXFormat}
    else
      Writeln(F, xOrigin : 3, '  ', yOrigin : 3, ' translate ');

    Writeln(F, 'newpath');
    Writeln(F, '0.5 setlinewidth');

  end; {PostHeader}

  {_________________________________________________}

  procedure PostFooter (var F: text;
                  var POSTBOX: IBOX;
                  legend: string;
                  TexFormat: boolean);

    var
      BOXHGT: integer;

  begin

    if TexFormat then
      begin
        Writeln(F, 'showpage}');
        BOXHGT := POSTBOX.Y2 - POSTBOX.Y1;
        Writeln(F, '\vbox to ', BOXHGT + 35 : 3, ' pt');
        Writeln(F, ' {\hrule width 12cm height 0cm depth 0cm \vfill}');
        Writeln(F, '\medskip');
        Writeln(F, '{\par\noindent{\bf Figure ', figNumber : 1, '.} ');
        Write(F, legend);
        Writeln(F, '\par}');
        Writeln(F, '\medskip');
      end
    else
      begin
        Writeln(F, 'showpage');
        Writeln(F, '%%Trailer');
        Writeln(F, '%%EOF');
        Writeln(F, '%% Figure ', fignumber : 1, '. ', legend);
      end;

    figNumber := figNumber + 1;

  end; {PostFooter}

  {_________________________________________________}

  procedure PostMarks (var F: text;
                  XSTART, YSTART: real;
                  taly, firstMark, farBound, STEP: real;
                  Axis: string);

    var
      toly, HR, VT: real;
      tolyString: string;

  begin {PostMarks}

    toly := firstMark;
    HR := XSTART;
    VT := YSTART;

    while Abs(toly) <= Abs(farBound) do

      begin {while toly}
        tolyString := TalyToString(toly);
        if Axis = 'xAxis' then
          begin
            Writeln(F, HR : 5 : 2, '  ', VT : 5 : 2, '  (', tolyString, ') showxhatch');
            HR := HR + STEP;
          end;

        if Axis = 'yAxis' then
          begin
            Writeln(F, HR : 5 : 2, '  ', VT : 5 : 2, '  (', tolyString, ') showyhatch');
            VT := VT + STEP;
          end;

        toly := toly + taly;
      end; {while TOLY}

  end; {PostMarks: Marks and Numbers}

  {_________________________________________________}

  procedure PostFrame (var F: text;
                  var realBOX: rBOX;
                  var POSTBOX: IBOX);

    var
      sign: integer;
      XS, YS, XF, YF, BOXWDT, BOXHGT: real;
      FIRSTHATCH, XZERO, YZERO: real;
      HATCHMAX, HATCHMIN, VSTEP, HSTEP: real;
      RX, RY, rxPos, rxNeg, ryPos, ryNeg, taly: real;
      xMax, xMin, yMax, yMin, xRange: real;
      lowerBound, upperBound, boundRange, increment: real;
      nearBound, farBound, startMark: real;

  begin{PostFrame}
    xMax := realBOX.xMax;{the maximum value of x in the data vector}
    xMin := realBOX.xMin; {the minmum value  of x in the data vector}
    yMax := realBOX.yMax; {the maximum value of y in the Data vector}
    yMin := realBOX.yMin; {the minimum value of y in the Data vector}

    XS := POSTBOX.X1; {Left of the  Bounding Box}
    YS := POSTBOX.Y1; {Bottom of the Bounding Box}
    XF := POSTBOX.X2; {Right of the  Bounding Box}
    YF := POSTBOX.Y2; {Top of the Bounding Box}
    XZERO := XS; {Default for Graphics Origin}
    YZERO := YS; {Default for Graphics Origin}
    BOXWDT := XF - XS; {Width of Bounding Box}
    BOXHGT := YF - YS; { Height of Bounding Box}

    if POSTBOX.doMarks then
      begin
        Writeln(F, '/Symbol findfont');
        Writeln(F, '10 scalefont  setfont');
      end;
    if POSTBOX.doFrame then
      begin
        Writeln(F, XS : 5 : 2, '  ', YS : 5 : 2, '  moveto');
        Writeln(F, XS : 5 : 2, '  ', YF : 5 : 2, '  lineto');
        Writeln(F, XF : 5 : 2, '  ', YF : 5 : 2, '  lineto');
        Writeln(F, XF : 5 : 2, '  ', YS : 5 : 2, '  lineto');
        Writeln(F, 'closepath');
        Writeln(F, 'stroke');
        Writeln(F, 'newpath');
      end;

    if POSTBOX.yAxis then
      begin {Do the yAxis}

  {...............................................................................................}

        if (yMin >= 0.0) and (yMax > 0.0) then
          begin {yMin = 0.0}
            if ((yMax - yMin) / yMax) < 0.3 then
              begin {small boundRange}
                lowerBound := yMin - 0.2 * (yMax - yMin);
                upperBound := yMax + 0.2 * (yMax - yMin);
                boundRange := upperBound - lowerBound;
                TallyPoints(boundRange, taly);
                lowerBound := Trunc(lowerBound / taly) * taly;
                boundRange := upperBound - lowerBound;
                YZERO := YS - lowerBound * (BOXHGT / boundRange);
              end {small boundRange}

            else{large boundRange}
              begin
                lowerBound := 0.0;
                upperBound := 1.2 * yMax;
                boundRange := upperBound - lowerBound;
                TallyPoints(boundRange, taly);
                YZERO := YS;
              end; {large boundRange}

            VSTEP := taly * (BOXHGT / boundRange);
            HSTEP := 0.0;
            if POSTBOX.doMarks then
              PostMarks(F, XS, YS, taly, lowerBound, upperBound, VSTEP, 'yAxis');

            HATCHMIN := YZERO;
            HATCHMAX := YS + Trunc(boundRange / taly) * VSTEP;
          end;{yMin >= 0.0 and yMax > 0.0}

  {...............................................................................................}

        if (yMin < 0.0) and (yMax > 0.0) then
          begin {yMin < 0.0}
            increment := 0.1 * (yMax - yMin);
            if yMax > Abs(yMin) then
              begin
                ryPos := 1.2 * yMax;
                ryNeg := yMin - increment;
              end
            else
              begin
                ryNeg := 1.2 * yMin;
                ryPos := yMax + increment
              end;
            YZERO := YS - BOXHGT * ryNeg / (ryPos - ryNeg);

            if POSTBOX.doMarks and POSTBOX.xAxis then
              begin
                Writeln(F, XS : 5 : 2, '  ', YZERO : 5 : 2, '  moveto');
                Writeln(F, XF : 5 : 2, '  ', YZERO : 5 : 2, '  lineto')
              end;
            if ryPos > Abs(ryNeg) then
              begin
                TallyPoints(ryPos, taly);
                VSTEP := (YF - YZERO) * (taly / ryPos);
              end
            else {if ryPos < Abs(ryNeg)}
              begin
                RY := Abs(ryNeg);
                TallyPoints(RY, taly);
                VSTEP := (YZERO - YS) * (taly / RY);
              end;
            HSTEP := 0.0;
            if POSTBOX.doMarks then
              begin {doMarks}
                PostMarks(F, XS, YZERO, taly, 0.0, ryPos, VSTEP, 'yAxis');
                PostMarks(F, XS, YZERO, -taly, 0.0, ryNeg, -VSTEP, 'yAxis');
              end; {doMarks}
            HATCHMAX := YZERO + Trunc(ryPos / taly) * VSTEP;
            HATCHMIN := YZERO + Trunc(ryNeg / taly) * VSTEP;

            lowerBound := ryNeg;
            upperBound := ryPos;
          end; {yMin < 0.0 and yMax > 0.0}

  {...............................................................................................}

        if (yMin < 0.0) and (yMax <= 0.0) then
          begin {yMin = 0.0}
            if ((yMin - yMax) / yMin) < 0.3 then
              begin {small boundRange}
                nearBound := yMax + 0.2 * (yMax - yMin);
                farBound := yMin - 0.2 * (yMax - yMin);
                boundRange := nearBound - farBound;
                TallyPoints(boundRange, taly);
                nearBound := Trunc(nearBound / taly) * taly;
                boundRange := nearBound - farBound;
                {store := nearBound * (BOXHGT / boundRange);}
                YZERO := YF + nearBound * (BOXHGT / boundRange);

              end {small boundRange}

            else{large boundRange}
              begin
                nearBound := 0.0;
                farBound := 1.2 * yMin;
                boundRange := nearBound - farBound;
                TallyPoints(boundRange, taly);
                YZERO := YF;
              end; {large boundRange}

            upperBound := nearBound;
            lowerBound := farBound;

            VSTEP := taly * (BOXHGT / boundRange);
            HSTEP := 0.0;
            PostMarks(F, XS, YF, -taly, nearBound, farBound, -VSTEP, 'yAxis');
            HATCHMIN := YZERO;
            HATCHMAX := YF - Trunc(boundRange / taly) * VSTEP;
          end;{yMin < 0.0 and yMax<= 0.0}

  {...............................................................................................}

      end; {Do the yAxis}

    if POSTBOX.doMarks and (not POSTBOX.doFrame) then
      begin
        Writeln(F, XS : 5 : 2, '  ', HATCHMIN : 5 : 2, '  moveto');
        Writeln(F, XS : 5 : 2, '  ', HATCHMAX : 5 : 2, '  lineto');
      end;
    Writeln(F, 'stroke');
    Writeln(F, 'newpath');

    realBOX.yUB := upperBound;
    realBOX.yLB := lowerBound;

    if POSTBOX.xAxis then
      begin {Do the xAxis}

        if not POSTBOX.doFrame then
          begin
            Writeln(F, XS : 5 : 2, '  ', YS : 5 : 2, '  moveto');
            Writeln(F, XF : 5 : 2, '  ', YS : 5 : 2, '  lineto');
            Writeln(F, 'stroke');
            Writeln(F, 'newpath');
          end;

  {...............................................................................................}

        if (xMin <= 0.0) and (xMax >= 0) then
          begin {xMin <= 0.0}
            xRange := xMax - xMin;
            rxPos := xMax;
            rxNeg := 0.0;
            if xMin < 0.0 then
              begin
                rxNeg := xMin;
                XZERO := XS - BOXWDT * rxNeg / (rxPos - rxNeg);
                Writeln(F, XZERO : 5 : 2, '  ', YS : 5 : 2, '  moveto');
                Writeln(F, XZERO : 5 : 2, '  ', YF : 5 : 2, '  lineto');
              end
            else
              XZERO := XS;

            if rxPos > Abs(rxNeg) then
              begin
                TallyPoints(rxPos, taly);
                HSTEP := (XF - XZERO) * (taly / rxPos);
              end
            else
              begin
                RX := Abs(rxNeg);
                TallyPoints(RX, taly);
                HSTEP := (XZERO - XS) * (taly / RX);
              end;
            VSTEP := 0;
            if xMax > 0.0 then
              PostMarks(F, XZERO, YS, taly, 0, rxPos, HSTEP, 'xAxis');
            if xMin < 0.0 then
              PostMarks(F, XZERO, YS, -taly, 0, rxNeg, -HSTEP, 'xAxis');
          end; {xMin <= 0.0 and xMax >= 0}

  {...............................................................................................}

        if xMin * xMax > 0.0 then
          begin {xMin, xMax < 0 or xMin, xMax >0}
            sign := 1;
            if xMax < 0 then
              sign := -1;
            xRange := xMax - xMin;
            {store := xMin * BOXWDT / xRange;}
            XZERO := XS - xMin * BOXWDT / xRange;
            TallyPoints(xRange, taly);
            if xMin > 0 then
              begin
                farBound := xMax;
                nearBound := xMin;
              end;
            if xMax < 0 then
              begin
                farBound := xMin;
                nearBound := xMax;
              end;
            startMark := Trunc(nearBound / taly) * taly;
            if (Abs((startMark - nearBound) / nearBound) > 0.001) then
              startMark := startMark + sign * taly;
            HSTEP := (BOXWDT) * (taly / xRange);
            VSTEP := 0.0;
            if sign = 1 then
              begin
                FIRSTHATCH := XS + (startMark - xMin) * BOXWDT / xRange;
              end
            else
              begin
                FIRSTHATCH := XF - (xMax - startMark) * BOXWDT / xRange;
              end;
            PostMarks(F, FIRSTHATCH, YS, sign * taly, startMark, farBound, sign * HSTEP, 'xAxis');
          end;

  {...............................................................................................}

        Writeln(F, 'stroke');
        Writeln(F, 'newpath');

      end; {Do the xAxis}

    realBOX.xLB := xMin;
    realBOX.xRB := xMax;
    POSTBOX.X0 := Round(XZERO);
    POSTBOX.Y0 := Round(YZERO);

  end; {PostFrame: PostScript Framework}

  {_________________________________________________}

  procedure PostVertAxis (var F: text;
                  scaleSide: integer;
                  XSTART, YSTART: integer;
                  taly, firstMark, farBound, VSTEP: real);

    var
      toly, HR, VT: real;
      tolyString: string;

  begin {PostVertAxis}

    toly := firstMark;
    HR := XSTART;
    VT := YSTART;

    while Abs(toly) <= Abs(farBound) do
      begin
        tolyString := TalyToString(toly);
        if (scaleSide = left) then
          Writeln(F, HR : 3 : 2, '  ', VT : 3 : 2, '  (', tolyString, ') showyhatch');
        if (scaleSide = right) then
          Writeln(F, HR : 3 : 2, '  ', VT : 3 : 2, '  (', tolyString, ') showqhatch');

        VT := VT + VSTEP;
        toly := toly + taly;
      end;

  end; {PostVertAxis}

  {_________________________________________________}

  procedure PostZeroPiAxis (var F: text;
                  var POSTBOX: IBOX);

    var
      XR, YT, BOXWDT: real;

  begin{ZeroPiAxis}
    BOXWDT := POSTBOX.X2 - POSTBOX.X1;

    XR := POSTBOX.X1;
    YT := POSTBOX.Y1; {type conversion}
    Writeln(F, XR : 5 : 2, '  ', YT : 5 : 2, '  (0) showxhatch');

    XR := POSTBOX.X1 + BOXWDT / 4;
    Writeln(F, XR : 5 : 2, '  ', YT : 5 : 2, '  (p/4) showxhatch');

    XR := POSTBOX.X1 + BOXWDT / 2;
    Writeln(F, XR : 5 : 2, '  ', YT : 5 : 2, '  (p/2) showxhatch');

    XR := POSTBOX.X2 - BOXWDT / 4;
    Writeln(F, XR : 5 : 2, '  ', YT : 5 : 2, '  (3p/4) showxhatch');

    XR := POSTBOX.X2;
    Writeln(F, XR : 5 : 2, '  ', YT : 5 : 2, '  (p) showxhatch');
    Writeln(F, 'stroke');
    Writeln(F, 'newpath');

  end;{PostZeroPiAxis}

  {_________________________________________________}

  procedure PostFrameLite (var F: text;
                  var realBOX: rBOX;
                  var POSTBOX: IBOX;
                  scaleSide: integer);

    var
      BOXHGT: integer;
      VSTEP, taly, yMax: real;
      lowerBound, upperBound: real;

  begin{PostFrameLite}
    yMax := realBOX.yMax;
    BOXHGT := POSTBOX.Y2 - POSTBOX.Y1;

    if (scaleSide = left) then
      begin
        Writeln(F, '/Symbol findfont');
        Writeln(F, '10 scalefont  setfont');
        Writeln(F, POSTBOX.X1 : 2, '  ', POSTBOX.Y1 : 2, '  moveto');
        Writeln(F, POSTBOX.X1 : 2, '  ', POSTBOX.Y2 : 3, '  lineto');
        Writeln(F, POSTBOX.X2 : 3, '  ', POSTBOX.Y2 : 3, '  lineto');
        Writeln(F, POSTBOX.X2 : 2, '  ', POSTBOX.Y1 : 3, '  lineto');
        Writeln(F, 'closepath');
        Writeln(F, 'stroke');
        Writeln(F, 'newpath');
      end;

    lowerBound := 0.0;
    upperBound := 1.2 * yMax;
    Tallypoints(upperBound, taly);

    VSTEP := taly * (BOXHGT / upperBound);

    if scaleSide = left then
      POSTBOX.X0 := POSTBOX.X1
    else
      POSTBOX.X0 := POSTBOX.X2;

    PostVertAxis(F, scaleSide, POSTBOX.X0, POSTBOX.Y1, taly, lowerBound, upperBound, VSTEP);
    Writeln(F, 'stroke');
    Writeln(F, 'newpath');

    realBOX.yUB := upperBound;
    realBOX.yLB := lowerBound;
    realBOX.xLB := realBOX.xMin;
    realBOX.xRB := realBOX.xMax;

    POSTBOX.X0 := POSTBOX.X1;
    POSTBOX.Y0 := POSTBOX.Y1;

  end; {PostFrameLite}

  {_________________________________________________}

  procedure PostPlot (var F: text;
                  var realBOX: rBOX;
                  var POSTBOX: IBOX;
                  var dataVector: longVector);
    var
      i: integer;
      XZERO, YZERO, BOXWDT, BOXHGT: real;
      HR, VT, xRange, yRange: real;

  begin
    BOXHGT := POSTBOX.Y2 - POSTBOX.Y1;
    BOXWDT := POSTBOX.X2 - POSTBOX.X1;
    XZERO := POSTBOX.X0;
    YZERO := POSTBOX.Y0;
    yRange := realBOX.yUB - realBOX.yLB;
  {xRange := realBOX.xRB - realBOX.xLB;}
    xRange := realBOX.NOPUNCT - 1;

    for i := 0 to realBOX.NOPUNCT - 1 do
      begin {i}
        HR := XZERO + (i * BOXWDT / xRange);
        VT := YZERO + (dataVector[i] * BOXHGT / yRange);
        if i = 0 then
          Writeln(F, HR : 3 : 2, '  ', VT : 3 : 2, '   moveto')
        else
          Writeln(F, HR : 3 : 2, '  ', VT : 3 : 2, '   lineto');
      end; {i}
    Writeln(F, 'stroke');
    Writeln(F, 'newpath');

  end; {PostPlot}

{________________________________________________}

  procedure PostPlot2 (var F: text;
                  var realBOX: rBOX;
                  var POSTBOX: IBOX;
                  var dataVector: longVector);

{This is for plotting an arbitrary number of data points over}
{the range defined by the width of the bounding box via the}
{index  i := 0 to Tcap - 1: Tcap := realBOX.NOPUNCT - 1}

    var
      i, Tcap: integer;
      XZERO, YZERO, BOXWDT, BOXHGT: real;
      HR, VT, xRange, yRange: real;
      G: text;

  begin
    BOXHGT := POSTBOX.Y2 - POSTBOX.Y1;
    BOXWDT := POSTBOX.X2 - POSTBOX.X1;
    XZERO := POSTBOX.X0;
    YZERO := POSTBOX.Y0;
    yRange := realBOX.yUB - realBOX.yLB;
    xRange := realBOX.NOPUNCT - 1; {real value}
    Tcap := realBOX.NOPUNCT - 1; {Integer Value}
    Writeln(F, 'newpath');
    HR := BOXWDT;
    VT := YZERO + (dataVector[Tcap - 1] * BOXHGT / yRange);
    Writeln(F, HR : 3 : 2, ' ', VT : 3 : 2, ' moveto');

    for i := 0 to Tcap - 1 do
      begin {i}
        HR := XZERO + (i * BOXWDT / xRange);
        VT := YZERO + (dataVector[i] * BOXHGT / yRange);
        if (i mod 8 = 7) or (i = Tcap - 1) then
          Writeln(F, HR : 3 : 2, ' ', VT : 3 : 2)
        else
          Write(F, HR : 3 : 2, ' ', VT : 3 : 2, ' ');
      end; {i}
    Writeln(F, Tcap : 3, ' {lineto} repeat');
    Writeln(F, 'stroke');

  end; {PostPlot2}

{_________________________________________________}

  procedure PostBiPlot (var F: text;
                  var realBOX: rBOX;
                  var POSTBOX: IBOX;
                  var dataVector: pointsVector);
    var
      i, BOXWDT, BOXHGT: integer;
      XZERO, YZERO: real;
      HR, VT, xRange, yRange: real;

  begin
    BOXHGT := POSTBOX.Y2 - POSTBOX.Y1;
    BOXWDT := POSTBOX.X2 - POSTBOX.X1;
    XZERO := POSTBOX.X0;
    YZERO := POSTBOX.Y0;
    yRange := realBOX.yUB - realBOX.yLB;
    xRange := realBOX.xRB - realBOX.xLB;

    for i := 0 to realBOX.NOPUNCT - 1 do
      begin {i}
  {HR := XZERO + (i * BOXWDT / xRange);}
        HR := XZERO + (dataVector[i].x * BOXWDT / xRange);
        VT := YZERO + (dataVector[i].y * BOXHGT / yRange);
        if i = 0 then
          Writeln(F, HR : 3 : 2, '  ', VT : 3 : 2, '   moveto')
        else
          Writeln(F, HR : 3 : 2, '  ', VT : 3 : 2, '   lineto');
      end; {i}
    Writeln(F, 'stroke');
    Writeln(F, 'newpath');

  end; {PostBiPlot}

  {_________________________________________________}

  procedure PostPlotMod (var F: text;
                  var realBOX: rBOX;
                  var POSTBOX: IBOX;
                  var dataVector: pointsVector);
    var
      i, BOXWDT, BOXHGT: integer;
      HR, VT, yRange: real;

  begin
    BOXWDT := POSTBOX.X2 - POSTBOX.X1;
    BOXHGT := POSTBOX.Y2 - POSTBOX.Y1;
    yRange := realBOX.yUB - realBOX.yLB;

    for i := 0 to BOXWDT do
      begin {i}
        HR := POSTBOX.X0 + i;
        VT := POSTBOX.Y0 + (dataVector[i].y * BOXHGT / yRange);
        if i = 0 then
          Writeln(F, HR : 5 : 2, '  ', VT : 5 : 2, '  moveto')
        else
          Writeln(F, HR : 5 : 2, '  ', VT : 5 : 2, '  lineto');
      end; {i}
    Writeln(F, 'stroke');
  end; {PostPlotMod}

  {_________________________________________________}

  procedure PostPlotMod2 (var F: text;
                  var realBOX: rBOX;
                  var POSTBOX: IBOX;
                  var dataVector: longVector);

{This is for plotting a series of function evaluations that are equal}
{in number to the width to the bounding box measured in pixelpoints}
{via the index  i := 0 to BOXWDT - 1}

    var
      i, BOXWDT, BOXHGT, YZERO: integer;
      HR, VT, yRange: real;

  begin
    BOXWDT := POSTBOX.X2 - POSTBOX.X1;
    BOXHGT := POSTBOX.Y2 - POSTBOX.Y1;
    YZERO := POSTBOX.Y0;
    yRange := realBOX.yUB - realBOX.yLB;

    Writeln(F, 'newpath');
    Writeln(F, '/abscissa ', BOXWDT : 3, ' def');

    HR := BOXWDT;
    VT := YZERO + (dataVector[BOXWDT - 1] * BOXHGT / yRange);
    Writeln(F, HR : 3 : 2, ' ', VT : 3 : 2, ' moveto');
{we are plotting in the reverse direction of x}

    for i := 0 to BOXWDT - 1 do
      begin {i}
        VT := YZERO + (dataVector[i] * BOXHGT / yRange);
{Here we  impose a limiting value on VT to keep the plot within the  bounding box}
        if VT > POSTBOX.Y2 then
          VT := POSTBOX.Y2;

        if (i mod 8 = 7) then
          Writeln(F, VT : 3 : 2) {this is to break the lines}
        else
          Write(F, VT : 3 : 2, ' ');
      end; {i}
    Write(F, BOXWDT : 3); {repeat these many times}
    Writeln(F, ' {/abscissa abscissa 1 sub def abscissa exch lineto} repeat');
    Writeln(F, 'stroke');

  end; {PostPlotMod2}

{_________________________________________________}


  procedure PostImpulsePlot (var impulseBOX: rBOX;
                  var POSTBOX: IBOX;
                  var impulseVec: longVector;
                  var legend: string);

    var
      X2, Y2, HR1, VT1, HR2, VT2, yRange: real;
      i, BOXWDT, BOXHGT, XINC: integer;
      postFileName: string;
      F: text;

  begin {PostImpulsePlot}
    BOXWDT := POSTBOX.X2 - POSTBOX.X1;
    BOXHGT := POSTBOX.Y2 - POSTBOX.Y1;
    yRange := impulseBOX.yUB - impulseBOX.yLB;

    Writeln;
    Write('    Name the PostScriptFile = ');
    Readln(postFileName);
    if postFileName <> '' then
      begin {Postfile}
        Assign(F, postFileName);
        Rewrite(F);

        PostHeader(F, POSTBOX, TeXFormat);
        BlackDotDef(F);

        POSTBOX.xAxis := true;
        POSTBOX.yAxis := true;
        POSTBOX.doFrame := true;
        POSTBOX.doMarks := true;
        PostFrame(F, impulseBOX, POSTBOX);

        Writeln(F, POSTBOX.X0 : 3, '  ', POSTBOX.Y0 : 3, '  moveto');
        Writeln(F, POSTBOX.X2 : 3, '  ', POSTBOX.Y0 : 3, '  lineto');
        Writeln(F, 'stroke');

        XINC := round(BOXWDT / (impulseBOX.NOPUNCT - 1));

        if (XINC > 12) then
          Writeln(F, '1.0 setlinewidth')
        else
          Writeln(F, '1.0 setlinewidth');

        for i := 0 to impulseBOX.NOPUNCT - 1 do
          begin {i}
            X2 := i * BOXWDT / (impulseBOX.NOPUNCT - 1);
            Y2 := impulseVec[i] * BOXHGT / yRange;
            HR1 := POSTBOX.X0 + X2;
            VT1 := POSTBOX.Y0;
            HR2 := HR1;
            VT2 := POSTBOX.Y0 + Y2;

            if Round(Y2) = 0 then
              begin
                if (XINC > 12) then
                  Writeln(F, HR2 : 3 : 2, '  ', VT2 : 3 : 2, '  2.0 blackdot')
                else
                  Writeln(F, HR2 : 3 : 2, '  ', VT2 : 3 : 2, ' 1.0  blackdot');
                 {Writeln(F, 'stroke');}
              end
            else
              begin
                Writeln(F, HR1 : 5 : 2, '  ', VT1 : 5 : 2, '  moveto');
                Writeln(F, HR2 : 5 : 2, '  ', VT2 : 5 : 2, '  lineto');
              end;
            Writeln(F, 'stroke');
          end; {i}
        PostFooter(F, POSTBOX, legend, TexFormat);
        Close(F);
      end; {Postfile}

  end; {PostImpulsePlot}

{_________________________________________________}

  procedure PostGainFooter (var F: text;
                  var POSTBOX: IBOX;
                  filterSpec: filterRecord;
                  TeXFormat: boolean);

    var
      PTBOXHGT: integer;
      typeString: string;
      RadToDeg: real;

  begin
    RadToDeg := 45.0 / Arctan(1.0);

    if TeXFormat then
      Writeln(F, 'showpage}')
    else
      Writeln(F, 'showpage');

    if TeXFormat then
      begin {TeXFormat}
        PTBOXHGT := Round(POSTBOX.Y2 - POSTBOX.Y1);
        writeln(F, '\vbox to ', PTBOXHGT + 35 : 3, ' pt');
        Writeln(F, '{\hrule width 342pt height 0pt depth 0pt \vfill}');

        with filterSpec do
          begin {filterSpec}

            case filterType of
              LowPass: 
                typeString := 'lowpass';
              HighPass: 
                typeString := 'highpass';
              otherwise
                typeString := 'Not Specified';
            end; {case}

            Writeln(F, '\medskip');
            Writeln(F, '{\par\noindent{\bf Figure.} The gain of the ');
            if filterFamily = Butterworth then
              begin
                Writeln(F, filterOrder : 2, 'th order ', 'Butterworth', ' ');
                Writeln(F, typeString, ' filter with a cut-off frequency of ', cutOff * RadToDeg : 2 : 1, ' degrees.\par }');
              end;
            if filterFamily = HodrickPrescott then
              begin
                Writeln(F, 'Hodrick--Prescott', ' ');
                Writeln(F, typeString, ' filter with a smoothing parameter of ', Round(smoothingParameter) : 4, '.\par }');
              end;
          end; {filterRecord}
      end; {TeXFormat}

  end;{PostGainFooter}

  {_________________________________________________}

  procedure WritePostFile (var dataBOX: rBOX;
                  var POSTBOX: IBOX;
                  var dataVector: longVector;
                  var legend: string);
    var
      F: text;
      postFileName: string;

  begin

    Writeln;
    Write('    Name the PostScriptFile = ');
    Readln(postFileName);
    if postFileName <> '' then
      begin {Postfile}
        Assign(F, postFileName);
        Rewrite(F);
        PostHeader(F, POSTBOX, TeXFormat);
        POSTBOX.xAxis := true;
        POSTBOX.yAxis := true;
        POSTBOX.doFrame := true;
        POSTBOX.doMarks := true;
        PostFrame(F, dataBOX, POSTBOX);
        PostPlot(F, dataBOX, POSTBOX, dataVector);
        PostFooter(F, POSTBOX, legend, TeXFormat);
        Close(F);
      end; {Postfile}
    figNumber := figNumber + 1;
  end; {WritePostFile}

  {_________________________________________________}

  procedure WritePostSpectral (var dataBOX: rBOX;
                  var POSTBOX: IBOX;
                  var dataVector: longVector;
                  var legend: string);
    var
      F: text;
      postFileName: string;

  begin

    Writeln;
    Write('    Name the PostScriptFile = ');
    Readln(postFileName);
    if postFileName <> '' then
      begin {Postfile}
        Assign(F, postFileName);
        Rewrite(F);
        PostHeader(F, POSTBOX, TeXFormat);
        POSTBOX.xAxis := false;
        POSTBOX.yAxis := true;
        POSTBOX.doFrame := true;
        POSTBOX.doMarks := true;
        PostFrame(F, dataBOX, POSTBOX);
        PostZeroPiAxis(F, POSTBOX);
{PostPlot2(F, dataBOX, POSTBOX, dataVector);}
        PostPlot(F, dataBOX, POSTBOX, dataVector);
        PostFooter(F, POSTBOX, legend, TeXFormat);
        Close(F);
      end; {Postfile}
    figNumber := figNumber + 1;
  end; {WritePostSpectral}

  {_________________________________________________}

  procedure WritePostOverFile (var dataBOX: rBOX;
                  var POSTBOX: IBOX;
                  var dataVector, trendVector: longVector;
                  var legend: string);

    var
      F: text;
      postFileName: string;

  begin

    Writeln;
    Write('    Name the PostScriptFile = ');
    Readln(postFileName);
    if postFileName <> '' then
      begin {Postfile}
        Assign(F, postFileName);
        Rewrite(F);
        PostHeader(F, POSTBOX, TeXFormat);
        POSTBOX.xAxis := true;
        POSTBOX.yAxis := true;
        POSTBOX.doFrame := true;
        POSTBOX.doMarks := true;
        PostFrame(F, dataBOX, POSTBOX);
        PostPlot(F, dataBOX, POSTBOX, dataVector);
        PostPlot(F, dataBOX, POSTBOX, trendVector);
        PostFooter(F, POSTBOX, legend, TeXFormat);
        Close(F);
      end; {Postfile}
    figNumber := figNumber + 1;
  end; {WritePostOverFile}

  {_________________________________________________}

  procedure PostCarPlot (var F: text;
                  var filterBOX: RBox;
                  var POSTBOX: IBOX;
                  var filterSpec: FilterRecord);
    var
      BOXHGT, POSTWDT: integer;
      X2, Y2, yRange, pi, omega1, omega2: real;

  begin
    pi := 4 * Arctan(1.0);
    omega1 := FilterSpec.lowerCut * pi / 180;
    omega2 := FilterSpec.upperCut * pi / 180;
    yRange := filterBOX.yUB - filterBOX.yLB;


    BOXHGT := POSTBOX.Y2 - POSTBOX.Y1;
    POSTWDT := POSTBOX.X2 - POSTBOX.X1;

    Y2 := POSTBOX.Y0;
    if (filterSpec.gainChar = 'F') then
      begin{Draw the horizontal axis}
        Writeln(F, 'newpath');
        X2 := POSTBOX.X0;
        Writeln(F, X2 : 3 : 2, '  ', Y2 : 3 : 2, '   moveto');
        X2 := POSTBOX.X2;
        Writeln(F, X2 : 3 : 2, '  ', Y2 : 3 : 2, '   lineto');
        Writeln(F, 'stroke');
      end;

    Writeln(F, 'newpath');
    X2 := POSTBOX.X0 + (POSTWDT * omega1 / pi);
    Writeln(F, X2 : 3 : 2, '  ', Y2 : 3 : 2, '   moveto');
    Y2 := POSTBOX.Y0 + 1.0 * (BOXHGT / yRange);
    Writeln(F, X2 : 3 : 2, '  ', Y2 : 3 : 2, '   lineto');
    X2 := POSTBOX.X0 + (POSTWDT * omega2 / pi);
    Writeln(F, X2 : 3 : 2, '  ', Y2 : 3 : 2, '   lineto');
    Y2 := POSTBOX.Y0;
    Writeln(F, X2 : 3 : 2, '  ', Y2 : 3 : 2, '   lineto');
    Writeln(F, 'stroke');

  end; {PostCarPlot}

  {________________________________________________}

  procedure PostFilterLimits (var F: text;
                  var filterSpec: FilterRecord;
                  var residueBOX: rBOX;
                  var POSTBOX: IBOX);

    var
      m: integer;
      HR, VT, POSTWDT, INC, increment: real;
  begin
    m := (filterSpec.filterSpan - 1) div 2;

    POSTWDT := POSTBOX.X2 - POSTBOX.X1;
    increment := POSTWDT / (residueBOX.NOPUNCT - 1);
    INC := m * increment;

    HR := POSTBOX.X1 + INC;
    VT := POSTBOX.Y1;
    Writeln(F, 'newpath');
    Writeln(F, HR : 3 : 2, '  ', VT : 3 : 2, '   moveto');
    VT := POSTBOX.Y2;
    Writeln(F, HR : 3 : 2, '  ', VT : 3 : 2, '   lineto');
    HR := POSTBOX.X2 - INC;
    Writeln(F, HR : 3 : 2, '  ', VT : 3 : 2, '   moveto');
    VT := POSTBOX.Y1;
    Writeln(F, HR : 3 : 2, '  ', VT : 3 : 2, '   lineto');
    Writeln(F, 'stroke');


  end; {MarkFilterLimits}

 {_________________________________________________}

  procedure MakePostLine (var F: text;
                  var Tvert: integer;
                  var POSTBOX: IBOX;
                  var dataBOX: rBOX);
    var
      HR, VT1, VT2, POSTWDT, store: real;

{This procedure marks a vertical line of the graph of a datplot at the point indexed by Tvert}
  begin
    POSTWDT := POSTBOX.X2 - POSTBOX.X1;

    store := Tvert * POSTWDT;
    store := store / (dataBOX.NOPUNCT - 1);
    HR := POSTBOX.X0 + store;

    VT1 := POSTBOX.Y1;
    VT2 := POSTBOX.Y2;

    Writeln(F, HR : 3 : 2, '  ', VT1 : 3 : 2, '   moveto');
    Writeln(F, HR : 3 : 2, '  ', VT2 : 3 : 2, '   lineto');

    Writeln(F, 'stroke');
    Writeln(F, 'newpath');

  end; {MakePostLine}

 {________________________________________________}


end. {POST: Unit 7 of NEWIDEOLOG.PAS}

