unit POSTBIS; {Unit 6 of NEWIDEOLOG.PAS}

interface

  uses
    GLOBALS, UTILS, MATHS;

  procedure ArrowDef (var F: text);

  procedure PoleCrossDef (var F: text);

  procedure ZeroCircDef (var F: text);

  procedure BlackDotDef (var F: text);

  procedure PostAargandHeader (var F: text;
                  TeXFormat: boolean);

  procedure PostAargand (var F: text;
                  p, q: integer;
                  var poles, zeros: complexVector);

  procedure PostAargandFooter (var F: text;
                  filterSpec: filterRecord;
                  TeXFormat: boolean);


implementation

{________________________________________________}

  procedure ArrowDef (var F: text);

  begin
    Writeln(F, '/arrowdict 14 dict def');
    Writeln(F, 'arrowdict begin');
    Writeln(F, '/mtrx matrix def');
    Writeln(F, 'end');
    Writeln(F, '/arrow');
    Writeln(F, '{ arrowdict begin');
    Writeln(F, '/headlength exch def');
    Writeln(F, '/halfheadthickness exch 2 div def');
    Writeln(F, '/halfthickness exch 2 div def');
    Writeln(F, '/tipy exch def /tipx exch def');
    Writeln(F, '/taily exch def /tailx exch def');
    Writeln(F, '/dx tipx tailx sub def');
    Writeln(F, '/dy tipy taily sub def');
    Writeln(F, '/arrowlength dx dx mul dy dy mul add');
    Writeln(F, 'sqrt def');
    Writeln(F, '/angle dy dx atan def');
    Writeln(F, '/base arrowlength headlength sub def');
    Writeln(F, '/savematrix mtrx currentmatrix def');
    Writeln(F, 'tailx taily translate');
    Writeln(F, 'angle rotate');
    Writeln(F, '0 halfthickness neg moveto');
    Writeln(F, 'base halfthickness neg lineto');
    Writeln(F, 'base halfheadthickness neg lineto');
    Writeln(F, 'arrowlength 0 lineto');
    Writeln(F, 'base halfheadthickness lineto');
    Writeln(F, 'base halfthickness lineto');
    Writeln(F, '0 halfthickness lineto');
    Writeln(F, 'closepath');
    Writeln(F, 'savematrix setmatrix');
    Writeln(F, 'end');
    Writeln(F, '} def');
  end; {ArrowDef}

{________________________________________________}

  procedure PoleCrossDef (var F: text);

  begin
    Writeln(F, '/cross{');
    Writeln(F, '/y exch def');
    Writeln(F, '/x exch def');
    Writeln(F, 'x 4 sub y 4 add moveto');
    Writeln(F, '8 -8  rlineto');
    Writeln(F, 'x 4 sub y 4 sub moveto');
    Writeln(F, '8  8  rlineto');
    Writeln(F, '} def');

    Writeln(F, '/polecross');
    Writeln(F, '{ /modulus exch def');
    Writeln(F, '/argument exch def');
    Writeln(F, 'gsave');
    Writeln(F, 'newpath');
    Writeln(F, '0 0 moveto');
    Writeln(F, 'argument rotate');
    Writeln(F, 'modulus 0 rlineto');
    Writeln(F, 'currentpoint cross');
    Writeln(F, 'stroke');
    Writeln(F, 'grestore');
    Writeln(F, '}def');
  end; {PoleCross Def}

{________________________________________________}

  procedure ZeroCircDef (var F: text);

  begin
    Writeln(F, '/circ{');
    Writeln(F, '/y exch def');
    Writeln(F, '/x exch def');
    Writeln(F, '1 setgray');
    Writeln(F, 'x   y  2.5  0 360 arc ');
    Writeln(F, 'fill');
    Writeln(F, '0 setgray');
    Writeln(F, 'x   y   2.5  0 360 arc ');
    Writeln(F, '} def');

    Writeln(F, '/zerocirc');
    Writeln(F, '{ /y exch def');
    Writeln(F, '/x exch def');
    Writeln(F, 'newpath');
    Writeln(F, '0  0 moveto');
    Writeln(F, 'x  y  lineto');
    Writeln(F, 'stroke');
    Writeln(F, 'x  y circ ');
    Writeln(F, 'stroke');
    Writeln(F, '}def');
  end; {ZeroCirc Def}

{_________________________________________________}

  procedure BlackDotDef (var F: text);

  begin
    Writeln(F, '/blackdot');
    Writeln(F, '{/radius exch def');
    Writeln(F, '/y exch def');
    Writeln(F, '/x exch def');
    Writeln(F, ' x  y  radius  0  360 arc fill');
    Writeln(F, '} def');
  end; {BlackDotDef}

{________________________________________________}

  procedure PostAargandHeader (var F: text;
                  TeXFormat: boolean);


  begin
    if TeXFormat then
      Writeln(F, '\special{postscript')
    else
      begin
        Writeln(F, '%!PS-Adobe-2.0');
        Writeln(F, '%%Creator: SigmaPi');
        Writeln(F, '%%BoundingBox: 150 565 350 750');
      end;

    ArrowDef(F);
    PoleCrossDef(F);
    ZeroCircDef(F);

    if TeXFormat then
      begin
        Writeln(F, 165 : 3, '  ', -100 : 3, '  translate');
      end
    else
      Writeln(F, '250  650 translate');

  end;{PostAargandHeader}

{_________________________________________________}

  procedure PostAargand (var F: text;
                  p, q: integer;
                  var poles, zeros: complexVector);

    var
      i, argument: integer;
      xRayTip, yRayTip, modulus: real;
      RadToDeg: real;
  begin {PostAargand}
    RadToDeg := 45 / ArcTan(1.0);

    Writeln(F, '/tailthick 0.5  def');
    Writeln(F, '/headthick 4  def');
    Writeln(F, '/headlength  6 def');
    Writeln(F, '/units {55 mul} def');
    Writeln(F, '0.5 setlinewidth');

{scribe the unit circle and the axis arrows}
    Writeln(F, '0 0 1 units 0 360 arc');
    Writeln(F, 'stroke');

{scribe the legend -i}
    Writeln(F, '15 neg 1 neg units 12 sub moveto');
    Writeln(F, '/Symbol findfont 10 scalefont setfont');
    Writeln(F, '(-) show 1 0 rmoveto');
    Writeln(F, '/Times-Italic findfont 10 scalefont setfont');
    Writeln(F, '(i) show');

{scribe the legend i}
    Writeln(F, '8 neg 1 units 5 add moveto');
    Writeln(F, '/Times-Italic findfont 10 scalefont setfont');
    Writeln(F, '(i) show');

 {scribe the legend -1}
    Writeln(F, '1 neg units 15 sub 12 neg  moveto');
    Writeln(F, '/Symbol findfont 10 scalefont setfont');
    Writeln(F, '(-1) show');

{scribe the legend 1}
    Writeln(F, '1  units 4 add 12 neg  moveto');
    Writeln(F, '/Symbol findfont 10 scalefont setfont');
    Writeln(F, '(1) show');

    Writeln(F, 'newpath');
    Writeln(F, '/tailx 1.5 neg units  def');
    Writeln(F, '/taily  0 units  def');
    Writeln(F, '/tipx  1.5 units  def');
    Writeln(F, '/tipy  0 units  def');
    Writeln(F, 'tailx taily tipx tipy tailthick headthick headlength arrow');
    Writeln(F, 'fill');

{scribe the legend Re}
    Writeln(F, 'tipx 6 add tipy 3 sub moveto');
    Writeln(F, '/Times-Italic findfont 10 scalefont setfont');
    Writeln(F, '(Re) show');

    Writeln(F, 'newpath');
    Writeln(F, '/tailx  0  units  def');
    Writeln(F, '/taily  1.5 neg units  6 add  def');
    Writeln(F, '/tipx  0  units  def');
    Writeln(F, '/tipy  1.5 units def');
    Writeln(F, 'tailx taily tipx tipy tailthick headthick headlength arrow');
    Writeln(F, 'fill');

{scribe the legend Im}
    Writeln(F, 'tipx 9 add tipy 4 sub moveto');
    Writeln(F, '/Times-Italic findfont 10 scalefont setfont');
    Writeln(F, '(Im) show');

{------------------------------------}

{Construct the rays}
    for i := 1 to p do
      begin
        argument := Round(TheArgument(poles[i]) * RadToDeg);
        modulus := TheModulus(poles[i]);
        Writeln(F, 'newpath');
        Writeln(F, '0  0 moveto');
        Writeln(F, argument : 3, '  ', modulus : 2 : 2, ' units  polecross');
        Writeln(F, 'stroke');
      end;

    for i := 1 to q do
      begin
        xRayTip := zeros[i].re;
        yRayTip := zeros[i].im;
        Writeln(F, 'newpath');
        Writeln(F, '0  0 moveto');
        Writeln(F, xraytip : 2 : 2, ' units ', yraytip : 2 : 2, ' units  zerocirc');
        Writeln(F, 'stroke');
      end;

  end; {PostAargand }

{_________________________________________________}

  procedure PostAargandFooter (var F: text;
                  filterSpec: filterRecord;
                  TeXFormat: boolean);

    var
      typeString: string;
      RadToDeg: real;

  begin
    RadToDeg := 45.0 / Arctan(1.0);

    if TeXFormat then
      begin {TeXFormat}
        Writeln(F, 'showpage}');
        Writeln(F, '\vbox to  180 pt');
        Writeln(F, '{\hrule width 342pt height 0pt depth 0pt \vfill}');

        with filterSpec do
          begin {filterSpec}

            case filterType of
              LowPass: 
                typeString := 'lowpass';
              HighPass: 
                typeString := 'highpass';
              otherwise
                typeString := 'Not Specified';
            end; {case}

            Writeln(F, '\medskip');
            Writeln(F, '{\par\noindent{\bf Figure.} The pole--zero diagram of the ');
            if filterFamily = Butterworth then
              begin
                Writeln(F, filterOrder : 2, 'th order ', 'Butterworth', ' ');
                Writeln(F, typeString, ' filter with a cut-off frequency of ', cutOff * RadToDeg : 2 : 1, ' degrees.\par }');
              end;
            if filterFamily = HodrickPrescott then
              begin
                Writeln(F, 'Hodrick--Prescott', ' ');
                Writeln(F, typeString, ' filter with a smoothing parameter of ', Round(smoothingParameter) : 4, '.\par }');
              end;
          end; {filterRecord}

      end {TeXFormat}
    else
      Writeln(F, 'showpage');
  end; {PostAargandFooter}

  {________________________________________________}

end. {POSTBIS: Unit 6 of NEWIDEOLOG.PAS}

