unit SCREEN; {Unit 4 of IDEOLOG.PAS}

interface

  uses
    GLOBALS, UTILS, MATHS, LEGENDS, Crt, Graph;

  procedure MarksAndNumbers (XSTART, YSTART: integer;
                  taly, firstMark, farBound, STEP: real;
                  Axis: string);

  procedure ScreenFrame (var realBOX: rBOX;
                  var SCREENBOX: IBOX);

  procedure ScreenFrameLite (var realBOX: rBOX;
                  var SCREENBOX: IBOX);

  procedure VertAxis (XSTART, YSTART: integer;
                  taly, firstMark, farBound, VSTEP: real);

  procedure ZeroPiAxis (var SCREENBOX: IBOX);

  procedure ScreenPlot (var realBOX: rBOX;
                  var SCREENBOX: IBOX;
                  var dataVector: longVector);

  procedure ScreenBiPlot (var realBOX: rBOX;
                  var SCREENBOX: IBOX;
                  var dataVector: pointsVector);

  procedure ImpulsePlot (impulseBOX: rBOX;
                  var SCREENBOX: IBOX;
                  impulseVec: longVector);

  procedure Rotate (aPoint: complex;
                  centre: complex;
                  angle: real;
                  var newPoint: complex);

  procedure PolePlot (var SCREENBOX: IBOX;
                  lambda: complex);

  procedure ScreenAargand (var SCREENBOX: IBOX;
                  p, q: integer;
                  var poles, zeros: complexVector);

  procedure MarkFilterLimits (var filterSpec: FilterRecord;
                  var residueBOX: rBOX;
                  var SCREENBOX: IBOX);

  procedure BoxCarPlot (var filterBOX: RBox;
                  var SCREENBOX: Ibox;
                  var filterSpec: FilterRecord);

  procedure ScreenOverplot (var dataBOX: rBOX;
                  var SCREENBOX: IBOX;
                  var dataVector, trendVector: longVector;
                  var legend: string);

  procedure ScreenMakePlot (var dataBOX: rBOX;
                  var SCREENBOX: IBOX;
                  var dataVector: longVector;
                  var legend: string);

  procedure MakeDataLine (var Tvert: integer;
                  var SCREENBOX: IBOX;
                  var dataBOX: rBOX);

implementation

  {__________________________________________________}

  procedure MarksAndNumbers (XSTART, YSTART: integer;
                  taly, firstMark, farBound, STEP: real;
                  Axis: string);
       {This DOS Routine differs from the Macintosh Version}

    var
      toly, HRL, VRL: real;
      HR1, HR2, VT1, VT2, tolyWidth: integer;
      HINC, VINC, PENLEFT, PENDOWN: integer;
      tolyString: string;

  begin {MarksAndNumbers}

    if Axis = 'yAxis' then
      begin {Pixel Parameters}
        HINC := 5;
        VINC := 0;
        PENDOWN := -(TextHeight('NUMBERS') div 2);
      end;
    if Axis = 'xAxis' then
      begin
        HINC := 0;
        VINC := -5;
        PENDOWN := 20;
      end; {Pixel Parameters}

    toly := firstMark;
    HRL := XSTART;
    HR1 := XSTART;
    HR2 := HR1 + HINC;
    VRL := YSTART;
    VT1 := YSTART;
    VT2 := VT1 + VINC;

    while Abs(toly) <= Abs(farBound) do

      begin {while toly}
        tolyString := TalyToString(toly);
        tolyWidth := TextWidth(tolyString);
        if axis = 'xAxis' then
          PENLEFT := Round(tolyWidth / 2.0);
        if axis = 'yAxis' then
          PENLEFT := tolyWidth + 15;

      {Numbers}
        MoveTo(HR1 - PENLEFT, VT1 + PENDOWN);
        OutText(tolyString);

       {Hatchmarks}
        MoveTo(HR1, VT1);
        LineTo(HR2, VT2);

       {Increments}
        if axis = 'xAxis' then
          HRL := HRL + STEP;
        HR1 := Round(HRL);
        HR2 := HR1 + HINC;
        if axis = 'yAxis' then
          VRL := VRL - STEP;
        VT1 := Round(VRL);
        VT2 := VT1 + VINC;

        toly := toly + taly;
      end; {while toly}

  end; {MarksAndNumbers}

  {_________________________________________________}

  procedure ScreenFrame (var realBOX: rBOX;
                  var SCREENBOX: IBOX);

   {This DOS Routine differs from the Macintosh Version}
   {Uses, MarksAndNumbers, TallyPoints, TalyToString}

    var
      BOXWDT, BOXHGT, FIRSTHATCH: integer;
      XORIGIN, YORIGIN, sign: integer;
      HRS, VTS, HRF, VTF, HATCHMAX, HATCHMIN: integer;
      RX, RY, rxPos, rxNeg, ryPos, ryNeg, VSTEP, HSTEP, taly: real;
      xMax, xMin, yMax, yMin, xRange: real;
      lowerBound, upperBound, boundRange, increment: real;
      nearBound, farBound, startMark: real;

  begin{ScreenFrame}
    xMax := realBOX.xMax;{the maximum value of x in the data vector}
    xMin := realBOX.xMin; {the minmum value  of x in the data vector}
    yMax := realBOX.yMax; {the maximum value of y in the Data vector}
    yMin := realBOX.yMin; {the minimum value of y in the Data vector}

    HRS := SCREENBOX.x1; {Left of the  Bounding Box}
    VTS := SCREENBOX.y1; {Top of the Bounding Box}
    HRF := SCREENBOX.x2; {Right of the  Bounding Box}
    VTF := SCREENBOX.y2; {Bottom of the Bounding Box}

    XORIGIN := HRS;{ Default for  XORIGIN}
    YORIGIN := VTF;{ Default for  YORIGIN}
    BOXWDT := HRF - HRS; {Width of Bounding Box}
    BOXHGT := VTF - VTS; { Height of Bounding}

    if SCREENBOX.doFrame then
      Rectangle(HRS, VTS, HRF, VTF);

    if SCREENBOX.yAxis then
      begin {Do the yAxis}
     {...............................................................................................}

        if (yMin >= 0.0) and (yMax > 0.0) then
          begin {yMin = 0.0}
            if ((yMax - yMin) / yMax) < 0.3 then
              begin {small boundRange}
                lowerBound := yMin - 0.2 * (yMax - yMin);
                upperBound := yMax + 0.2 * (yMax - yMin);
                boundRange := upperBound - lowerBound;
                TallyPoints(boundRange, taly);
                lowerBound := Trunc(lowerBound / taly) * taly;
                boundRange := upperBound - lowerBound;
                YORIGIN := VTF + Round(lowerBound * (BOXHGT / boundRange));
              end {small boundRange}

            else{large boundRange}
              begin
                lowerBound := 0.0;
                upperBound := 1.2 * yMax;
                boundRange := upperBound - lowerBound;
                TallyPoints(boundRange, taly);
                YORIGIN := VTF;
              end; {large boundRange}

            VSTEP := taly * (BOXHGT / boundRange);
            HSTEP := 0.0;
            if SCREENBOX.doMarks then
              MarksAndNumbers(HRS, VTF, taly, lowerBound, upperBound, VSTEP, 'yAxis');

            HATCHMIN := VTF;
            HATCHMAX := VTF - Round(Trunc(boundRange / taly) * VSTEP);
          end;{yMin >= 0.0 and yMax > 0.0}
     {.........................................................................}

        if (yMin < 0.0) and (yMax > 0.0) then
          begin {yMin < 0.0}
            increment := 0.1 * (yMax - yMin);
            if yMax > Abs(yMin) then
              begin
                ryPos := 1.2 * yMax;
                ryNeg := yMin - increment;
              end
            else
              begin
                ryNeg := 1.2 * yMin;
                ryPos := yMax + increment
              end;
            YORIGIN := VTF + Round(BOXHGT * ryNeg / (ryPos - ryNeg));

            if SCREENBOX.doMarks and SCREENBOX.xAxis then
              begin
                MoveTo(HRS, YORIGIN);
                LineTo(HRF, YORIGIN)
              end;
            if ryPos > Abs(ryNeg) then
              begin
                TallyPoints(ryPos, taly);
                VSTEP := (YORIGIN - VTS) * (taly / ryPos);
              end
            else {if ryPos < Abs(ryNeg)}
              begin
                RY := Abs(ryNeg);
                TallyPoints(RY, taly);
                VSTEP := (VTF - YORIGIN) * (taly / RY);
              end;
            HSTEP := 0.0;
            if SCREENBOX.doMarks then
              begin {doMarks}
                MarksAndNumbers(HRS, YORIGIN, taly, 0.0, ryPos, VSTEP, 'yAxis');
                MarksAndNumbers(HRS, YORIGIN, -taly, 0.0, ryNeg, -VSTEP, 'yAxis');
              end; {doMarks}
            HATCHMAX := YORIGIN - Round(Trunc(ryPos / taly) * VSTEP);
            HATCHMIN := YORIGIN - Round(Trunc(ryNeg / taly) * VSTEP);
            lowerBound := ryNeg;
            upperBound := ryPos;
          end; {yMin < 0.0 and yMax > 0.0}
     {.........................................................................}

        if (yMin < 0.0) and (yMax <= 0.0) then
          begin {yMin = 0.0}
            if ((yMin - yMax) / yMin) < 0.3 then
              begin {small boundRange}
                nearBound := yMax + 0.2 * (yMax - yMin);
                farBound := yMin - 0.2 * (yMax - yMin);
                boundRange := nearBound - farBound;
                TallyPoints(boundRange, taly);
                nearBound := Trunc(nearBound / taly) * taly;
                boundRange := nearBound - farBound;
                YORIGIN := VTS - Round(nearBound * (BOXHGT / boundRange));
              end {small boundRange}

            else{large boundRange}
              begin
                nearBound := 0.0;
                farBound := 1.2 * yMin;
                boundRange := nearBound - farBound;
                TallyPoints(boundRange, taly);
                YORIGIN := VTS;
              end; {large boundRange}

            upperBound := nearBound;
            lowerBound := farBound;

            VSTEP := taly * (BOXHGT / boundRange);
            HSTEP := 0.0;
            MarksAndNumbers(HRS, VTS, -taly, nearBound, farBound, -VSTEP, 'yAxis');
            HATCHMAX := VTS;
            HATCHMIN := VTS + Round(Trunc(boundRange / taly) * VSTEP);
          end;{yMin < 0.0 and yMax<= 0.0}
     {.........................................................................}
      end; {Do the yAxis}

    if SCREENBOX.doMarks and (not SCREENBOX.doFrame) then
      begin
        MoveTo(HRS, HATCHMIN);
        LineTo(HRS, HATCHMAX)
      end;
    realBOX.yUB := upperBound;
    realBOX.yLB := lowerBound;

     {.........................................................................}
       {=========================================================}
       { In  this segement,  we  determine  whether the interval }
       { [xMin, xMax] lies  within  the  interval [xLB, xRB]. In }
       { that case, [xLB, xRB] determines the Cartesian bounding }
       { box. Otherwise, we set xLB = xMin and xRB = xMax.       }
       {=========================================================}

    if (realBOX.xLB < realBOX.xMin) and (realBOX.xMax < realBOX.xRB) then
      begin
        realBOX.xMin := realBOX.xLB;
        realBOX.xMax := realBOX.xRB;
        xMin := realBOX.xMin;
        xMax := realBOX.xMax;
      end
    else
      begin
        realBOX.xLB := realBOX.xMin;
        realBOX.xRB := realBOX.xMax;
      end;
     {.........................................................................}

    if SCREENBOX.xAxis then
      begin {Do the xAxis}

        if not SCREENBOX.doFrame then
          begin
            MoveTo(HRS, VTF);
            LineTo(HRF, VTF);
          end;
     {.........................................................................}
        if (xMin <= 0.0) and (xMax >= 0) then
          begin {xMin <= 0.0}
            xRange := xMax - xMin;
            rxPos := xMax;
            rxNeg := 0.0;
            if xMin < 0.0 then
              begin
                rxNeg := xMin;
                XORIGIN := HRS - Round(BOXWDT * rxNeg / (rxPos - rxNeg));
                MoveTo(XORIGIN, VTS);
                LineTo(XORIGIN, VTF)
              end
            else
              XORIGIN := HRS;

            if rxPos > Abs(rxNeg) then
              begin
                TallyPoints(rxPos, taly);
                HSTEP := (HRF - XORIGIN) * (taly / rxPos);
              end
            else
              begin
                RX := Abs(rxNeg);
                TallyPoints(RX, taly);
                HSTEP := (XORIGIN - HRS) * (taly / RX);
              end;
            VSTEP := 0;
            if xMax > 0.0 then
              MarksAndNumbers(XORIGIN, VTF, taly, 0, rxPos, HSTEP, 'xAxis');
            if xMin < 0.0 then
              MarksAndNumbers(XORIGIN, VTF, -taly, 0, rxNeg, -HSTEP, 'xAxis');
          end; {xMin <= 0.0}

     {.........................................................................}

        if xMin * xMax > 0.0 then
          begin {xMin, xMax < 0 or xMin, xMax >0}
            sign := 1;
            if xMax < 0 then
              sign := -1;
            xRange := xMax - xMin;
            XORIGIN := HRS - Round(xMin * BOXWDT / xRange);
            TallyPoints(xRange, taly);
            if xMin > 0 then
              begin
                farBound := xMax;
                nearBound := xMin;
              end;
            if xMax < 0 then
              begin
                farBound := xMin;
                nearBound := xMax;
              end;

            startMark := Trunc(nearBound / taly) * taly;
            startMark := startMark + sign * taly;
            HSTEP := (BOXWDT) * (taly / xRange);
            VSTEP := 0.0;
            if sign = 1 then
              FIRSTHATCH := HRS + Round((startMark - xMin) * BOXWDT / xRange)
            else
              FIRSTHATCH := HRF - Round((xMax - startMark) * BOXWDT / xRange);
            MarksAndNumbers(FIRSTHATCH, VTF, sign * taly, startMark, farBound, sign * HSTEP, 'xAxis');
          end;
     {.........................................................................}

      end; {Do the xAxis}

    realBOX.xLB := xMin;
    realBOX.xRB := xMax;

    SCREENBOX.X0 := XORIGIN;
    SCREENBOX.Y0 := YORIGIN;

  end; {ScreenFrame}

   {_________________________________________________}

  procedure ScreenFrameLite (var realBOX: rBOX;
                  var SCREENBOX: IBOX);

    var
      BOXHGT: integer;
      XSTART: integer;
      VSTEP, taly: real;
      yMax: real;
      lowerBound, upperBound: real;

  begin
    yMax := realBOX.yMax; {the maximum value of y in the Data vector}

    BOXHGT := SCREENBOX.Y2 - SCREENBOX.Y1; {Height of Bounding Box}

    Rectangle(SCREENBOX.X1, SCREENBOX.Y1, SCREENBOX.X2, SCREENBOX.Y2);

    lowerBound := 0.0;
    upperBound := 1.2 * yMax;
    Tallypoints(upperBound, taly);

    XSTART := SCREENBOX.X1;

    VSTEP := taly * (BOXHGT / upperBound);
    VertAxis(XSTART, SCREENBOX.Y2, taly, lowerBound, upperBound, VSTEP);

    realBOX.yUB := upperBound;
    realBOX.yLB := lowerBound;
    realBOX.xLB := realBOX.xMin;
    realBOX.xRB := realBOX.xMax;

    SCREENBOX.X0 := SCREENBOX.X1;
    SCREENBOX.Y0 := SCREENBOX.Y2;

  end; {ScreenFrameLite}

  {_________________________________________________}

  procedure VertAxis (XSTART, YSTART: integer;
                  taly, firstMark, farBound, VSTEP: real);

    var
      toly, VRL: real;
      HR1, HR2, VT, tolyWidth: integer;
      HINC, PENLEFT, PENDOWN: integer;
      tolyString: string;

  begin {VertAxis}
    HINC := 5;
    PENDOWN := 4;

    toly := firstMark;
    HR1 := XSTART;
    HR2 := HR1 + HINC;
    VRL := YSTART;
    VT := YSTART;

    while Abs(toly) <= Abs(farBound) do
      begin {while toly}

        tolyString := TalyToString(toly);
        tolyWidth := TextWidth(tolyString);
        PENLEFT := tolyWidth + 15;

      {Numbers}
        MoveTo(HR1 - PENLEFT, VT + PENDOWN);
        OutText(tolyString);

       {Hatchmarks}
        MoveTo(HR1, VT);
        LineTo(HR2, VT);

       {Increments}
        VRL := VRL - VSTEP;
        VT := Round(VRL);

        toly := toly + taly;
      end; {while toly}

  end; {VertAxis}

  {________________________________________________}

  procedure ZeroPiAxis (var SCREENBOX: IBOX);
       {This DOS Routine differs from the Macintosh Version}

    var
      HR, VT1, VT2: integer;
      BOXWDT, PENDOWN, PENLEFT: integer;

  begin{ZeroPiAxis}
    BOXWDT := SCREENBOX.X2 - SCREENBOX.X1;
    PENLEFT := 12;
    PENDOWN := 20;

    VT1 := SCREENBOX.Y2;
    VT2 := VT1 - 5;
    HR := SCREENBOX.X1;
    MoveTo(HR - 2, VT1 + PENDOWN);
    OutText('0');

    HR := SCREENBOX.X1 + Round(BOXWDT / 4.0);
    MoveTo(HR, VT1);
    LineTo(HR, VT2);
    MoveTo(HR - 8, VT1 + PENDOWN);
    OutText('p/4');

    HR := SCREENBOX.X1 + Round(BOXWDT / 2.0);
    MoveTo(HR, VT1);
    LineTo(HR, VT2);
    MoveTo(HR - 8, VT1 + PENDOWN);
    OutText('p/2');

    HR := SCREENBOX.X2 - Round(BOXWDT / 4.0);
    MoveTo(HR, VT1);
    LineTo(HR, VT2);
    MoveTo(HR - PENLEFT, VT1 + PENDOWN);
    OutText('3p/4');

    HR := SCREENBOX.X2;
    MoveTo(HR - 2, VT1 + PENDOWN);
    OutText('p');

  end;{ZeroPiAxis}
  {_________________________________________________}

  procedure ScreenPlot (var realBOX: rBOX;
                  var SCREENBOX: IBOX;
                  var dataVector: longVector);

    var
      i, HR, VT, BOXWDT, BOXHGT: integer;
      yRange: real;
      store: double;

  {The assumptiion is that all are indesed from zero to NOPUNCT-1}
  begin
    BOXWDT := SCREENBOX.X2 - SCREENBOX.X1;
    BOXHGT := SCREENBOX.Y2 - SCREENBOX.Y1;
    yRange := realBOX.yUB - realBOX.yLB;

    for i := 0 to realBOX.NOPUNCT - 1 do
      begin {i}
        store := i * BOXWDT;
        store := store / (realBOX.NOPUNCT - 1);
        HR := SCREENBOX.X1 + Round(Store);

        store := (dataVector[i] - realBOX.yLB) / yRange;
        VT := Round(SCREENBOX.Y2 - store * BOXHGT);

        if i = 0 then
          MoveTo(HR, VT)
        else
          LineTo(HR, VT);
      end; {i}

  end; {ScreenPlot}

  {_________________________________________________}

  procedure ScreenBiPlot (var realBOX: rBOX;
                  var SCREENBOX: IBOX;
                  var dataVector: pointsVector);

   {This procedure  is used when dataVector is of the pointsVector}
   {type as opposed to tthe usual longVector type}

    var
      i, HR, VT, BOXWDT, BOXHGT: integer;
      xRange, yRange: real;
      store: double;

  begin
    BOXWDT := SCREENBOX.X2 - SCREENBOX.X1;
    BOXHGT := SCREENBOX.Y2 - SCREENBOX.Y1;
    yRange := realBOX.yUB - realBOX.yLB;
    xRange := realBOX.xRB - realBOX.xLB;

    for i := 0 to realBOX.NOPUNCT - 1 do
      begin {i}
        store := (dataVector[i].x - realBOX.xLB) / xRange;
        HR := Round(SCREENBOX.X1 + store * BOXWDT);

        store := (dataVector[i].y - realBOX.yLB) / yRange;
        VT := Round(SCREENBOX.Y2 - store * BOXHGT);

        if i = 0 then
          MoveTo(HR, VT)
        else
          LineTo(HR, VT);
      end; {i}

  end; {ScreenBiPlot}

  {________________________________________________}

  procedure ImpulsePlot (impulseBOX: rBOX;
                  var SCREENBOX: IBOX;
                  impulseVec: longVector);
    {This DOS Routine differs from the Macintosh Version}

    var
      i, X2, Y2, HR1, VT1, HR2, VT2: integer;
      BOXWDT, BOXHGT, XINC: integer;
      yRange: real;

  begin {SpikePlot}
    BOXWDT := SCREENBOX.X2 - SCREENBOX.X1;
    BOXHGT := SCREENBOX.Y2 - SCREENBOX.Y1;
    yRange := impulseBOX.yUB - impulseBOX.yLB;

    MoveTo(SCREENBOX.X0, SCREENBOX.Y0);
    LineTo(SCREENBOX.X2, SCREENBOX.Y0);

    XINC := Round(BOXWDT div (impulseBOX.NOPUNCT - 1));

    if (XINC > 12) then
      SetLineStyle(solidLn, 0, thickWidth)
    else
      SetLineStyle(solidLn, 0, normWidth);

    for i := 0 to impulseBOX.NOPUNCT - 1 do
      begin {i}
        X2 := Round(i * BOXWDT div (impulseBOX.NOPUNCT - 1));
        Y2 := Round(impulseVec[i] * BOXHGT / yRange);
        HR1 := SCREENBOX.X0 + X2;
        VT1 := SCREENBOX.Y0;
        HR2 := HR1;
        VT2 := SCREENBOX.Y0 - Y2;

        if Y2 = 0 then
          begin {if}
            if (XINC > 12) then
              PieSlice(HR1, VT1, 0, 360, 2)
            else
              PieSlice(HR1, VT1, 0, 360, 1);
          end  {if}
        else
          begin  {else}
            MoveTo(HR1, VT1);
            LineTo(HR2, VT2);
          end;  {else}
      end; {i}

    SetLineStyle(solidLn, 0, normWidth);

  end; {ImpulsePlot}

  {________________________________________________}

  procedure Rotate (aPoint: complex;
                  centre: complex;
                  angle: real;
                  var newPoint: complex);
    var
      thePolar: polar;
  begin
    newPoint.re := aPoint.re - centre.re;
    newPoint.im := aPoint.im - centre.im;
    ComplexToPolar(newPoint, thePolar);
    thePolar.argu := thePolar.argu + angle;
    PolarToComplex(thePolar, newPoint);
    newPoint.re := newPoint.re + centre.re;
    newPoint.im := newPoint.im + centre.im;
  end; {Rotate}

    {________________________________________________}

  procedure PolePlot (var SCREENBOX: IBOX;
                  lambda: complex);
    var
      P, A, B, C, D, centre: complex;
      thePole: polar;
      theArgument, theModulus: real;
      RADIUS: integer;
  begin
    ComplexToPolar(lambda, thePole);
    theArgument := thePole.argu;
    theModulus := thePole.modu;

     {writeln(theArgument : 3 : 3);}
    centre.re := SCREENBOX.XC;
    centre.im := SCREENBOX.YC;
    RADIUS := 72;
    P.re := theModulus * RADIUS + centre.re;
    P.im := centre.im;
    A.re := P.re + 5;
    A.im := P.im + 5;
    B.re := P.re + 5;
    B.im := P.im - 5;
    C.re := P.re - 5;
    C.im := P.im - 5;
    D.re := P.re - 5;
    D.im := P.im + 5;

    Rotate(P, centre, theargument, P);
    Rotate(A, centre, theargument, A);
    Rotate(B, centre, theargument, B);
    Rotate(C, centre, theargument, C);
    Rotate(D, centre, theargument, D);
    MoveTo(Round(centre.re), Round(centre.im));
    LineTo(Round(P.re), Round(P.im));
    MoveTo(Round(A.re), Round(A.im));
    LineTo(Round(C.re), Round(C.im));
    MoveTo(Round(B.re), Round(B.im));
    LineTo(Round(D.re), Round(D.im));

  end; {PolePlot}

   {________________________________________________}

  procedure ScreenAargand (var SCREENBOX: IBOX;
                  p, q: integer;
                  var poles, zeros: complexVector);

    var
      i, CTX, CTY, XSTART, XEND, YSTART, YEND, RADIUS: integer;

  begin {ScreenAargand }

    CTX := SCREENBOX.XC;
    CTY := SCREENBOX.YC;
    RADIUS := 72;
    Circle(CTX, CTY, RADIUS);

     {Horizontal Axis}
    XSTART := CTX - Round(1.25 * RADIUS);
    XEND := CTX + Round(1.25 * RADIUS);
    YSTART := CTY;
    YEND := CTY;
    MoveTo(XSTART, YSTART);
    LineTo(XEND, YEND);

     {Vertical Axis}
    XSTART := CTX;
    XEND := CTX;
    YSTART := CTY + Round(1.25 * RADIUS);
    YEND := CTY - Round(1.25 * RADIUS);
    MoveTo(XSTART, YSTART);
    LineTo(XEND, YEND);

    for i := 1 to p do {Plot the poles}
      PolePlot(SCREENBOX, poles[i]);

     {NB. plotting poles requires an elaborate procedure, PlotPole, to ensure that}
     {the crosses which mark the poles maintain their alignment with the radii}

    for i := 1 to q do
      begin {Plot the zeros}
        XEND := CTX + Round(zeros[i].re * RADIUS);
        YEND := CTY - Round(zeros[i].im * RADIUS);
        MoveTo(CTX, CTY);
        LineTo(XEND, YEND);
        Circle(XEND, YEND, 4);
      end;

  end; {ScreenAargand}

  {_________________________________________________}

  procedure MarkFilterLimits (var filterSpec: FilterRecord;
                  var residueBOX: rBOX;
                  var SCREENBOX: IBOX);
    var
      m, HR, BOXWDT, INC: integer;
      increment: real;
  begin
    m := (filterSpec.filterSpan - 1) div 2;

    BOXWDT := SCREENBOX.X2 - SCREENBOX.X1;
    increment := BOXWDT div (residueBOX.NOPUNCT - 1);
    INC := Round(m * increment);
    HR := SCREENBOX.X1 + INC;
    MoveTo(HR, SCREENBOX.Y1);
    LineTo(HR, SCREENBOX.Y2);
    HR := SCREENBOX.X2 - INC;
    Moveto(HR, SCREENBOX.Y1);
    Lineto(HR, SCREENBOX.Y2);

  end; {MarkFilterLimits}

  {_________________________________________________}

  procedure BoxCarPlot (var filterBOX: RBox;
                  var SCREENBOX: Ibox;
                  var filterSpec: FilterRecord);
  {This could serve for lowpass badnpass and high pass filters}
    var
      BOXWDT, BOXHGT, YORIGIN, VT, HR: integer;
      yRange, pi, omega1, omega2: real;

  begin
    pi := 4 * Arctan(1.0);
    omega1 := FilterSpec.lowerCut * pi / 180;
    omega2 := FilterSpec.upperCut * pi / 180;
    yRange := filterBOX.yUB - filterBOX.yLB;

    YORIGIN := SCREENBOX.Y0;
    BOXWDT := SCREENBOX.X2 - SCREENBOX.X1; {Width of Bounding Box}
    BOXHGT := SCREENBOX.Y2 - SCREENBOX.Y1; { Height of Bounding}

    if (filterSpec.gainChar = 'F') then
      begin{Draw the horizontal axis}
        MoveTo(SCREENBOX.X1, YORIGIN);
        LineTo(SCREENBOX.X2, YORIGIN);
      end;

    HR := SCREENBOX.X1 + Round(BOXWDT * omega1 / pi);
    MoveTo(HR, YORIGIN);
    VT := YORIGIN - Round(1 * (BOXHGT / yRange));
    LineTo(HR, VT);
    HR := SCREENBOX.X1 + Round(BOXWDT * omega2 / pi);
    LineTo(HR, VT);
    LineTo(HR, YORIGIN);

  end;{BoxCarPlot}

  {________________________________________________}

  procedure ScreenMakePlot (var dataBOX: rBOX;
                  var SCREENBOX: IBOX;
                  var dataVector: longVector;
                  var legend: string);

  begin
    ClrScr;
    Writeln;
    Writeln(' ' : 60, '<RETURN>');
    GetGraphics(graphicsOn, SCREENBOX, currentMode);

    ScreenFrame(dataBOX, SCREENBOX);
    ScreenPlot(dataBOX, SCREENBOX, dataVector);
    ScreenCaption(legend);

    CallForReturn;
    Readln;
    CloseGraph;
    RestoreCrtMode;
    ClrScr;

  end; {ScreenMakePlot}

  {________________________________________________}

  procedure ScreenOverplot (var dataBOX: rBOX;
                  var SCREENBOX: IBOX;
                  var dataVector, trendVector: longVector;
                  var legend: string);

  begin
    ClrScr;
    Writeln;
    Writeln(' ' : 60, '<RETURN>');
    GetGraphics(graphicsOn, SCREENBOX, currentMode);

    ScreenFrame(dataBOX, SCREENBOX);
    ScreenPlot(dataBOX, SCREENBOX, dataVector);

    SetColor(red);
    SetLineStyle(0, 0, ThickWidth);
    ScreenPlot(dataBOX, SCREENBOX, trendVector);
    SetColor(yellow);
    SetLineStyle(0, 0, NormWidth);

    ScreenCaption(legend);

    CallForReturn;
    Readln;
    CloseGraph;
    RestoreCrtMode;
    ClrScr;

  end; {ScreenOverplot}

  {_________________________________________________}

  procedure MakeDataLine (var Tvert: integer;
                  var SCREENBOX: IBOX;
                  var dataBOX: rBOX);
    var
      HR, VT1, VT2: Integer;
      BOXWDT, store: real;

{This procedure marks a vertical line of the graph of a datplot at the point indexed by Tvert}
  begin
    BOXWDT := SCREENBOX.X2 - SCREENBOX.X1;

    store := Tvert * BOXWDT; {We have BOXWDT real to avoid overflow}
    store := store / (dataBOX.NOPUNCT - 1);
    HR := SCREENBOX.X1 + Round(store);

    VT1 := SCREENBOX.Y1;
    VT2 := SCREENBOX.Y2;

    MoveTo(HR, VT1);
    LineTo(HR, VT2);

  end; {MakeDataLine}

 {________________________________________________}



end. {SCREEN: Unit 4 of IDEOLOG.PAS}

