unit SMOOTHER; {Unit 15 of NEWIDEOLOG.PAS}

interface

  uses
    GLOBALS, UTILS, MATHS, FILTERS, DATASEG, SCREEN, POST, LEGENDS, Dos, Crt, Graph;

  procedure GetGlobalLambda (var smoothLambda: triRealVec);

  procedure GetLocalLambdas (var smoothLambda: triRealVec;
                  var sBandWidth, sBandCentre: triIntVec;
                  var SCREENBOX, POSTBOX: IBOX;
                  var dataBOX: rBOX;
                  var dataVector: longVector);

  procedure GetLocalSmoothingRegions (var sBandWidth, sBandCentre: triIntVec;
                  var SCREENBOX, POSTBOX: IBOX;
                  var dataBOX: rBOX;
                  var dataVector: longVector);

  procedure HighLightRegions (var sBandWidth, sBandCentre: triIntVec;
                  var SCREENBOX: IBOX);

  procedure ShowSmoothingRegions (var dataBOX: rBOX;
                  var SCREENBOX: IBOX;
                  var dataVector: longVector;
                  var sBandWidth, sBandCentre: triIntVec);

  procedure GetSmoothingProfile (var sProfile: integer);

  procedure AdaptiveSmoothing (var sBandCentre, sBandWidth: triIntVec;
                  var sProfile: integer;
                  var smoothLambda: triRealVec;
                  var dataBOX: rBOX;
                  var residueVec: longVector);

  procedure PostalRegions (var F: text;
                  var sBandWidth, sBandCentre: triIntVec;
                  var POSTBOX: IBOX);

  procedure MakeSmoothVector (var smoothLambda: triRealVec;
                  var sBandWidth, sBandCentre: triIntVec;
                  var dataBOX, trendBOX, residueBOX: rBOX;
                  var SCREENBOX, POSTBOX: IBOX;
                  var dataVector, trendVector, residueVec: longVector);

implementation

  {_________________________________________________}

  procedure GetGlobalLambda (var smoothLambda: triRealVec);

    var
      AcceptIt: boolean;

  begin
    ClrScr;
    Writeln;
    Writeln;
    Writeln('    SPECIFY THE REGULAR SMOOTHING PARAMETER');
    Writeln;

    noGlobalLambda := true; {We have made thee patemters local}
    AcceptIt := false;
    if (noGlobalLambda = false) then
      begin
        Writeln('    The value previously specified for the Global smoothing parameter is');
        Writeln('        lambda = ', smoothLambda[0] : 6 : 4);
        Writeln;
        YesOrNo('    Do you wish to accept this value', AcceptIt);
      end;
    if (AcceptIt = false) then
      begin
        Writeln('    What is the value of the smoothing parameter?');
        ReadBoundedNumber(0.0, 160000.0, smoothLambda[0]);
      end;

    noGlobalLambda := false;

  end;{GetGlobalLambda}

  {________________________________________________}

  procedure GetLocalLambdas (var smoothLambda: triRealVec;
                  var sBandWidth, sBandCentre: triIntVec;
                  var SCREENBOX, POSTBOX: IBOX;
                  var dataBOX: rBOX;
                  var dataVector: longVector);
    var
      theMenuItem, j: integer;

  begin

    repeat
      ClrScr;
      Writeln;
      Writeln;
      Writeln('    SPECIFY THE LOCAL SMOOTHING PARAMETERS');
      Writeln;
      Writeln;
      Writeln('         1. Show Smoothing Regions');
      Writeln;
      Writeln('         2. Specify Smoothing Parameters');
      Writeln;
  {Writeln('         3. Return to the Main Menu');}
  {Writeln;}
      Writeln('                      Select a number and press RETURN');
      GetBoundedInteger(1, 2, theMenuItem);

      case theMenuItem of
        1: 
          ShowSmoothingRegions(dataBOX, SCREENBOX, dataVector, sBandWidth, sBandCentre);
        2: 
          begin {2}
            for j := 1 to sBandCentre[0] do
              begin {j}
                case j of
                  1: 
                    Writeln('    Specify the value of the smoothing parameter in the FIRST subinteval');
                  2: 
                    Writeln('    Specify the value of the smoothing parameter in the SECOND subinteval');
                  3: 
                    Writeln('    Specify the value of the smoothing parameter in the THIRD subinteval');
                end;
                ReadBoundedNumber(0.0, 20000.0, smoothLambda[j]);
              end; {j}
            noLocalLambdas := false;
          end; {case of 2}
      end;{cases}

    until (noLocalLambdas = false);
  {until theMenuItem = 3;}


  end;{GetLocalLambdas}

  {________________________________________________}

  procedure GetLocalSmoothingRegions (var sBandWidth, sBandCentre: triIntVec;
                  var SCREENBOX, POSTBOX: IBOX;
                  var dataBOX: rBOX;
                  var dataVector: longVector);

    var
      showThem, reviseThem, acceptIt, anotherOne: boolean;
      k: integer;

  begin
    ClrScr;
    Writeln;
    Writeln;
    Writeln('    SPECIFY REGIONS OF SMOOTHING VARIATION');
    Writeln;
    Writeln;
    reviseThem := true;
    showThem := false;

    if (noSmoothingRegions = false) then
      begin
        Writeln('    You have already specified some regions of adapted smoothing');
        YesOrNo('    Do you wish to see the graph of these regions', showThem);
        if (showThem = false) then
          SBandCentre[0] := 0;
      end
    else {No Regions Specified}
      begin
        SBandCentre[0] := 0;
        reviseThem := true;
        Writeln('    The user is asked to specify no more than three sub-intervals within');
        Writeln('    the  period spanned by the data, wherein the  value of the smoothing');
        Writeln('    parameter can  be reduced in order to enhance the flexibility of the');
        Writeln('    trend, so as to allow it to accomodate structural breaks.');
        Writeln;
        Writeln(' ' : 64, '<RETURN>');
        Readln;
      end;

    ShowSmoothingRegions(dataBOX, SCREENBOX, dataVector, sBandWidth, sBandCentre);

    if (noSmoothingRegions = false) then
      YesOrNo('    Do you wish to revise these regions', reviseThem);

    anotherOne := true;
    k := 1;

    if reviseThem then
      begin{ Revise them}

        sBandCentre[0] := 0; {The specifies the number of adapted smoothing bands}
        SBandCentre[1] := -10;
        SBandCentre[2] := -10;
        SBandCentre[3] := -10;

        repeat {until anotherOne = false}
          ClrScr;
          Writeln;

          case k of
            1: 
              Writeln('    SPECIFY THE FIRST REGION OF ADAPTED SMOOTHING');
            2: 
              Writeln('    SPECIFY THE SECOND REGION OF ADAPTED SMOOTHING');
            3: 
              Writeln('    SPECIFY THE THIRD REGION OF ADAPTED SMOOTHING');
          end;
          sBandCentre[0] := k;
          Writeln;
          Writeln('    Specify the data central point');
          GetBoundedInteger(0, dataBOX.NOPUNCT - 1, sBandCentre[k]);
          Writeln('    Specify the number of points above and below the central point');
          GetBoundedInteger(0, dataBOX.NOPUNCT div 3, sBandWidth[k]);
          ShowSmoothingRegions(dataBOX, SCREENBOX, dataVector, sBandWidth, sBandCentre);

          Writeln;
          case k of
            1: 
              Writeln('    SPECIFY THE FIRST REGION OF ADAPTED SMOOTHING');
            2: 
              Writeln('    SPECIFY THE SECOND REGION OF ADAPTED SMOOTHING');
            3: 
              Writeln('    SPECIFY THE THIRD REGION OF ADAPTED SMOOTHING');
          end;
          Writeln;
          YesOrNo('    Do you accept this region ', acceptIt);

          if AcceptIt then
            begin
              k := k + 1;
              if k = 4 then
                anotherOne := false
              else
                begin
                  Writeln;
                  YesOrNo('    Do you wish to specify additional regions', anotherOne);
                end;
            end;{AcceptIt}

        until (anotherOne = false);
      end;{Revise them}

    noSmoothingRegions := false;
    noLocalLambdas := true;

  end;{ GetLocalSmoothingRegions}

  {________________________________________________}

  procedure HighLightRegions (var sBandWidth, sBandCentre: triIntVec;
                  var SCREENBOX: IBOX);
    var
      k, BOXWDT, TOP, BOTM, LEFT, RGHT, CENTR, NPTM1: integer;
      stripRect: integerRect;

  begin
    BOXWDT := SCREENBOX.X2 - SCREENBOX.X1;
    TOP := SCREENBOX.Y1;
    BOTM := SCREENBOX.Y2;
    NPTM1 := dataBOX.NOPUNCT - 1;
    SetFillStyle(closeDotFill, yellow);

    for k := 1 to sBandCentre[0] do
      begin
        CENTR := SCREENBOX.X1 + Round(sBandCentre[k] * BOXWDT / NPTM1);
        RGHT := SCREENBOX.X1 + Round((sBandCentre[k] + sBandWidth[k]) * BOXWDT / NPTM1);
        LEFT := SCREENBOX.X1 + Round((sBandCentre[k] - sBandWidth[k]) * BOXWDT / NPTM1);

        stripRect.HRS := LEFT;
        stripRect.VTS := BOTM;
        stripRect.HRF := RGHT;
        stripRect.VTF := TOP;

        Bar(stripRect.HRS, stripRect.VTS, stripRect.HRF, stripRect.VTF);
        Rectangle(stripRect.HRS, stripRect.VTS, stripRect.HRF, stripRect.VTF);

        MoveTo(CENTR, TOP);
        LineTo(CENTR, BOTM);
      end;

  end; {HighLightRegions}

   {________________________________________________}

  procedure ShowSmoothingRegions (var dataBOX: rBOX;
                  var SCREENBOX: IBOX;
                  var dataVector: longVector;
                  var sBandWidth, sBandCentre: triIntVec);
    var
      legend: string;

  begin
    GetGraphics(graphicsOn, SCREENBOX, currentMode);
    ScreenFrame(dataBOX, SCREENBOX);
    HighLightRegions(sBandWidth, sBandCentre, SCREENBOX);
    ScreenFrame(dataBOX, SCREENBOX);
    ScreenPlot(dataBOX, SCREENBOX, dataVector);

    legend := DataLegend(dataBOX, dataName);
    ScreenCaption(legend);

    CallForReturn;
    Readln;
    CloseGraph;
    RestoreCrtMode;
    ClrScr;

  end;{ShowSmoothingRegions}

   {________________________________________________}

  procedure GetSmoothingProfile (var sProfile: integer);

    var
      theMenuItem: integer;

  begin
    repeat
      ClrScr;
      Writeln;
      Writeln;
      Writeln('    SPECIFY THE PROFILE OF THE SMOOTHING PARAMATER');
      Writeln;
      Writeln('    Local reductions in the  smoothing parameter, which are intended to');
      Writeln('    accomodate  structural  breaks,  must follow  a chosen profile. The');
      Writeln('    effect of a local reduction is to cut a trench in the trajectory of');
      Writeln('    the smoothing parameter, which would otherwise be horizontal. These');
      Writeln('    are the profiles to choose from:');
      Writeln;
      Writeln('         1. Triangular trench');
      Writeln;
      Writeln('         2. Square trough');
      Writeln;
      Writeln('         3. Inverted cosine bell');
      Writeln;
      Writeln('                      Select a number and press RETURN');
      GetBoundedInteger(1, 3, theMenuItem);


      case theMenuItem of
        1: 
          sProfile := triangle;
        2: 
          sProfile := square;
        3: 
          sProfile := bell;
      end;{case}
    until theMenuItem in [1..3];

    noSmoothingProfile := false;

  end;{GetSmoothingProfile}

  {________________________________________________}

  procedure AdaptiveSmoothing (var sBandCentre, sBandWidth: triIntVec;
                  var sProfile: integer;
                  var smoothLambda: triRealVec;
                  var dataBOX: rBOX;
                  var residueVec: longVector);

    var
      d, s, q: longVector;
      trip, trap: triIntVec;
      omega, span, lambda, amplitude, increment: real;
      t, j, k, Tcap: integer;

  begin
    Tcap := dataBox.NOPUNCT;

  {We need to make an adjustment to the location of the sBands}
  {in view of the second differencing}

    for t := 0 to Tcap - 3 do
      begin
        d[t] := 6.0 + 1 / smoothLambda[0];
        s[t] := -4.0;
        q[t] := 1.0;
      end; {t}

    for k := 1 to sBandCentre[0] do
      begin {k}

        trip[k] := sBandCentre[k] - sBandWidth[k];
        trap[k] := sBandCentre[k] + sBandWidth[k];
        span := trap[k] - trip[k];
        amplitude := smoothLambda[0] - smoothLambda[k];
        increment := amplitude / (sBandWidth[k] + 1);{The case of a trianglar trough}

        lambda := smoothLambda[0];
        if sBandCentre[k] > 0 then
          for t := trip[k] to trap[k] do
            begin {t: sBandCentre >  0}

              if sProfile = bell then
                begin {bell}
                  j := t - trip[k];
                  omega := 2 * pi * j / span;
                  increment := amplitude * 0.5 * (cos(omega) - 1.0);
                  lambda := smoothLambda[0] + increment;
                  d[t] := 6.0 + 1 / lambda;
                end; {bell}

              if sProfile = square then
                begin {square}
                  lambda := smoothLambda[k];
                  d[t] := 6.0 + 1 / lambda;
                end; {square}

              if sProfile = triangle then
                begin {triangle}
                  if t <= sBandCentre[k] then
                    lambda := lambda - increment;
                  if t > sBandCentre[k] then
                    lambda := lambda + increment;
                  d[t] := 6.0 + 1 / lambda;
                end; {triangle}

            end; {t: sBandCentre >  0}

      end;{k}

    QinDiag(Tcap - 3, d, s, q, residueVec);

  end;{AdaptiveSmoothing}

  {________________________________________________}

  procedure PostalRegions (var F: text;
                  var sBandWidth, sBandCentre: triIntVec;
                  var POSTBOX: IBOX);
    var
      k, BOXWDT, TOP, BOTM, LEFT, RGHT, CENTR, NPTM1: integer;

  begin
    BOXWDT := POSTBOX.X2 - POSTBOX.X1;
    TOP := POSTBOX.Y2;
    BOTM := POSTBOX.Y1;
    NPTM1 := dataBOX.NOPUNCT - 1;
    for k := 1 to sBandCentre[0] do
      begin {k}
        CENTR := POSTBOX.X1 + Round(sBandCentre[k] * BOXWDT / NPTM1);
        RGHT := POSTBOX.X1 + Round((sBandCentre[k] + sBandWidth[k]) * BOXWDT / NPTM1);
        LEFT := POSTBOX.X1 + Round((sBandCentre[k] - sBandWidth[k]) * BOXWDT / NPTM1);

        Writeln(F, 'newpath');
        Writeln(F, LEFT : 3, '  ', BOTM : 3, ' moveto');
        Writeln(F, LEFT : 3, '  ', TOP : 3, ' lineto');
        Writeln(F, RGHT : 3, '  ', TOP : 3, ' lineto');
        Writeln(F, RGHT : 3, '  ', BOTM : 3, ' lineto');
        Writeln(F, 'closepath');
        Writeln(F, '0.8 setgray');
        Writeln(F, 'fill');
        Writeln(F, '0.0 setgray');
        Writeln(F, 'newpath');
        Writeln(F, CENTR : 3, '  ', TOP : 3, ' moveto');
        Writeln(F, CENTR : 3, '  ', BOTM : 3, ' lineto');
        Writeln(F, 'stroke');
        Writeln(F, 'newpath');
      end; {k}

  end; {PostalRegions}

  {________________________________________________}

  procedure MakeSmoothVector (var smoothLambda: triRealVec;
                  var sBandWidth, sBandCentre: triIntVec;
                  var dataBOX, trendBOX, residueBOX: rBOX;
                  var SCREENBOX, POSTBOX: IBOX;
                  var dataVector, trendVector, residueVec: longVector);
    var
      Tcap, t: integer;
      replace, highLights: boolean;
      legend: string;
      F: text;
      postFileName: string;
  {W: text;}

  begin
    Tcap := dataBox.NOPUNCT;
  {x := y - Q(Q'Q +  Lambda^[-1])^[-1]Q'y = y - QW^[-1]Q'y = y - Qz}
    residueVec := dataVector;
    QprimeY(residueVec, Tcap); {SecondDiffernces}
    AdaptiveSmoothing(sBandCentre, sBandWidth, sProfile, smoothLambda, dataBOX, residueVec);
    FormQy(residueVec, Tcap);  {Form Qz}

    for t := 0 to Tcap - 1 do
      trendVector[t] := dataVector[t] - residueVec[t];

    legend := VariSmoothTrend(dataBOX, dataName);
    ScreenOverplot(dataBOX, SCREENBOX, dataVector, trendVector, legend);

    trendBOX := dataBOX;
    residueBOX := dataBOX;
    MakeDataBox(residueBOX, residueVec);

  {Superimpose the Smoothing regions}
    if sBandCentre[0] > 0 then
      begin {if}
        GetGraphics(graphicsOn, SCREENBOX, currentMode);
        HighLightRegions(sBandWidth, sBandCentre, SCREENBOX);
        ScreenFrame(dataBOX, SCREENBOX);
        ScreenPlot(dataBOX, SCREENBOX, dataVector);
        ScreenPlot(dataBOX, SCREENBOX, trendVector);

        legend := VariSmoothTrend(dataBOX, dataName);
        ScreenCaption(legend);

        CallForReturn;
        Readln;
        CloseGraph;
        RestoreCrtMode;
        ClrScr;
      end; {if}

    legend := ' ';
    if PSPict then
      begin
        Writeln;
        Writeln('     POSTSCRIPT GRAPHICS');
        Writeln;
        Writeln;
        YesOrNo('    Do you wish to highlight the regions of adapted smoothing', highLights);

        Write('    Name the PostScript File = ');
        Readln(postFileName);
        if postFileName <> '' then
          begin {Postfile}
            Assign(F, postFileName);
            Rewrite(F);
            PostHeader(F, POSTBOX, TeXFormat);
            POSTBOX.xAxis := true;
            POSTBOX.yAxis := true;
            POSTBOX.doFrame := true;
            POSTBOX.doMarks := true;
            if highLights then
              PostalRegions(F, sBandWidth, sBandCentre, POSTBOX);
            PostFrame(F, dataBOX, POSTBOX);
            PostPlot(F, dataBOX, POSTBOX, dataVector);
            PostPlot(F, dataBOX, POSTBOX, trendVector);
            PostFooter(F, POSTBOX, legend, TeXFormat);
            Close(F);
          end; {Postfile}
        figNumber := figNumber + 1;
      end;

    legend := VariSmoothResiduals(dataBOX, dataName);
    ScreenMakePlot(residueBOX, SCREENBOX, residueVec, legend);

    if PSPict then
      WritePostFile(residueBOX, POSTBOX, residueVec, legend);

    weHaveATrend := true;
    weHaveTrendResiduals := true;

    if (suppressReplace = false) then
      begin {suppressReplace}
        ClrScr;
        Writeln;
        Writeln('     DATA SMOOTHING');
        Writeln;
        YesOrNo('    Do you wish to replace the data series by the residual series', replace);
        if replace then
          begin {replace}
            cycleVec := dataVector;
            cycleBOX := dataBOX;
            dataBOX := residueBOX;
            dataVector := residueVec;
            residualsInData := true;
          end; {replace}
      end; {suppressReplace}

  end;{MakeSmoothVector}

  {________________________________________________}

end. {SMOOTHER: Unit 12 of NEWIDEOLOG.PAS}

