unit TAPER; {Unit 12 of NEWIDEOLOG.PAS}

interface

  uses
    GLOBALS, MATHS, UTILS, SCREEN, POLYNOME, DATASEG, LEGENDS, POST, Graph, Dos, Crt;

  procedure MakeExtrapolation (var SCREENBOX, POSTBOX: IBOX;
                  var dataBOX, trendBox, residueBOX: rBOX;
                  var dataVector, trendVector, residueVec: longVector);

  procedure ReflectTheData (var SCREENBOX, POSTBOX: IBOX;
                  var dataBOX: rBOX;
                  var dataVector: longVector);

  procedure TaperTheData (var SCREENBOX, POSTBOX: IBOX;
                  var dataBOX: rBOX;
                  var dataVector: longVector);

  procedure ExtrapolateAndTaper (var SCREENBOX, POSTBOX: IBOX;
                  var dataBOX: rBOX;
                  var dataVector: longVector);

  procedure PolynomialExtrapolation (var SCREENBOX, POSTBOX: IBOX;
                  var dataBOX, trendBox, residueBOX: rBOX;
                  var polyParas: polyParameters;
                  var dataVector, trendVector, residueVec: longVector);

  procedure SeasonalPolyExtrapolation (var SCREENBOX, POSTBOX: IBOX;
                  var dataBOX, trendBox, residueBOX: rBOX;
                  var polyParas: polyParameters;
                  var dataVector, trendVector, residueVec: longVector);

  procedure CircularMorphing (var SCREENBOX, POSTBOX: IBOX;
                  var dataBOX: rBOX;
                  var dataVector: longVector);

implementation

  {_________________________________________________}

  procedure MakeExtrapolation (var SCREENBOX, POSTBOX: IBOX;
                  var dataBOX, trendBox, residueBOX: rBOX;
                  var dataVector, trendVector, residueVec: longVector);

    var
      theMenuItem: integer;

  begin
    if (thereIsData = false) then
      begin
        SysBeep(3);
        ReadTheData(dataBOX, dataVector); {in UTILS}
      end;

    if noPageParameters then
      PageSetUp(SCREENBOX, POSTBOX, PSPict, TeXFormat);

    repeat
      ClrScr;
      Writeln;
      Writeln('    EXTRAPOLATE AND/OR TAPER THE DATA');
      Writeln;
      Writeln('         1. Reflect Data Around the Endpoint');
      Writeln;
      Writeln('         2. Taper the Data');
      Writeln;
      Writeln('         3. Extrapolate and Taper');
      Writeln;
      Writeln('         4. Polynomial Extrapolation and Tapering');
      Writeln;
      Writeln('         5. Extrapolate by Circular Morphing');
      Writeln;
      Writeln('         6. Return to the Data Menu');
      Writeln;
      Writeln('                      Select a number and press RETURN');

      GetBoundedInteger(1, 6, theMenuItem);

      case theMenuItem of
        1: 
          ReflectTheData(SCREENBOX, POSTBOX, dataBOX, dataVector);
        2: 
          TaperTheData(SCREENBOX, POSTBOX, dataBOX, dataVector);
        3: 
          ExtrapolateAndTaper(SCREENBOX, POSTBOX, dataBOX, dataVector);
        4: 
          begin
            if dataBOX.dataFrequency in [Annual, Other] then
              PolynomialExtrapolation(SCREENBOX, POSTBOX, dataBOX, trendBox, residueBOX, polyParas, dataVector, trendVector, residueVec);
            if dataBOX.dataFrequency in [Monthly, Quarterly] then
              SeasonalPolyExtrapolation(SCREENBOX, POSTBOX, dataBOX, trendBox, residueBOX, polyParas, dataVector, trendVector, residueVec);
          end;
        5: 
          if (ResidualsInData = false) then
            begin
              Writeln;
              Sysbeep(3);
              Writeln('    This procedure can be applied only to the residuals deviations');
              Writeln('    from an interpolated trend.');
              Writeln(' ' : 60, '<RETURN>');
              Readln;
            end
          else
            CircularMorphing(SCREENBOX, POSTBOX, dataBOX, dataVector);
      end;{case}
    until (theMenuItem = 6);

  end;
  {_________________________________________________}

  procedure ReflectTheData (var SCREENBOX, POSTBOX: IBOX;
                  var dataBOX: rBOX;
                  var dataVector: longVector);

    var
      Tcap, t: integer;

  begin
    Tcap := dataBOX.NOPUNCT;

    ClrScr;
    Writeln;
    Writeln('    REFLECT THE DATA AROUND THE ENDPOINT');
    Writeln;
    Writeln('    This procedure extends the  data by adding the  data sequence in ');
    Writeln('    reverse to the end of the data. The endpoint of the original data');
    Writeln('    is not  repeated. Nor is the  starting point of the data included');
    Writeln('    in the reversed data sequence. The  extended data form a circular');
    Writeln('    sequence such  that the same values are to  be found  at opposite');
    Writeln('    ends of any diameter of the circle.');
    Writeln;
    Writeln(' ' : 60, '<RETURN>');
    Readln;
    for t := 1 to Tcap - 2 do
      dataVector[Tcap - 1 + t] := dataVector[Tcap - 1 - t];

    dataBOX.NOPUNCT := 2 * Tcap - 2;
    MakeDataBox(dataBOX, dataVector);
    PlotTheData(dataBOX, SCREENBOX, POSTBOX, dataVector);

  end; {ReflectTheData}

  {_________________________________________________}

  procedure TaperTheData (var SCREENBOX, POSTBOX: IBOX;
                  var dataBOX: rBOX;
                  var dataVector: longVector);

    var
      perCent, pi, omega, inc, weight: real;
      Tcap, Mcap, t: integer;

  begin
    pi := 4.0 * Arctan(1.0);
    Tcap := dataBOX.NOPUNCT;

    ClrScr;
    Writeln;
    Writeln('    TAPER THE DATA VIA A SPLIT COSINE BELL');
    Writeln;
    Writeln('    This  procedure  subjects  the  sample  to  a  tapering operation');
    Writeln('    based on a  split cosine bell. The latter is found  by separating');
    Writeln('    the two halves of a cosine bell and inserting a stretch of units.');
    Writeln('    The  operation leaves the bulk of the date unmodified  and tapers');
    Writeln('    only the ends. The user is  asked to specify the  fraction of the');
    Writeln('    sample that is to be tapered.');
    Writeln;
    Writeln(' ' : 60, '<RETURN>');
    Readln;

    repeat
      Writeln('    Specify as a percentage the fraction of the sample to be tapered');
      ReadANumber(perCent);
    until (perCent <= 100.0) and (perCent >= 0.0);
    Mcap := Round(perCent * Tcap / 200.0);

    inc := pi / (Mcap + 1);
    omega := 0.0;
    for t := 0 to Mcap - 1 do
      begin
        omega := omega + inc;
        weight := 1 - 0.5 * (Cos(omega) + 1);
        dataVector[t] := weight * dataVector[t];
      end;

    omega := 0.0;
    for t := Tcap - Mcap to Tcap - 1 do
      begin
        omega := omega + inc;
        weight := 0.5 * (Cos(omega) + 1);
        dataVector[t] := weight * dataVector[t];
      end;

    PlotTheData(dataBOX, SCREENBOX, POSTBOX, dataVector);

  end; {TaperTheData}

  {________________________________________________}

  procedure ExtrapolateAndTaper (var SCREENBOX, POSTBOX: IBOX;
                  var dataBOX: rBOX;
                  var dataVector: longVector);

    var
      Tcap, Mcap, Tadd, t: integer;
      inc, pi, omega, weight, perCent: real;

  begin
    pi := 4.0 * Arctan(1.0);
    Tcap := dataBOX.NOPUNCT;

    ClrScr;
    Writeln;
    Writeln('    EXTRAPOLATE AND TAPER THE DATA');
    Writeln;
    Writeln('    The data is extended at both ends by a certain  proportion of the');
    Writeln('    original length. The extrapolation is  achieved by reflecting the');
    Writeln('    sample data around the  endpoints. A  cosine decrement is applied');
    Writeln('    to the extrapolated portions to reduce their extremities to zero.');
    Writeln;
    Writeln(' ' : 60, '<RETURN>');
    Readln;

    repeat
      Writeln('     What proportion of the original data is each extension?');
      Writeln('     Specify the proportion as a percentage');
      ReadANumber(perCent);
    until (perCent <= 100.0) and (perCent >= 0.0);

    Tadd := Round(perCent * Tcap / 100.0);
  
  {Shift the data by Tadd points}
    for t := Tcap - 1 downto 0 do
      dataVector[t + Tadd] := dataVector[t];

    inc := pi / (Tadd + 1);
  {Lower extension by reflection}
    omega := 0;
    for t := 0 to Tadd - 1 do
      begin
        omega := omega + inc;
        weight := 1 - 0.5 * (Cos(omega) + 1);
        dataVector[t] := weight * dataVector[2 * Tadd - t];
      end;

  {Upper extension by reflection}
    omega := 0;
    Mcap := Tcap + Tadd - 1;
    for t := 1 to Tadd do
      begin
        omega := omega + inc;
        weight := 0.5 * (Cos(omega) + 1);
        dataVector[Mcap + t] := weight * dataVector[Mcap - t];
      end;

    dataBOX.NOPUNCT := Tcap + 2 * Tadd;
    dataBOX.EXTRA := Tadd;
    MakeDataBox(dataBOX, dataVector);
    PlotTheData(dataBOX, SCREENBOX, POSTBOX, dataVector);

  end; {ExtapolateAndTaper}

  {________________________________________________}

  procedure PolynomialExtrapolation (var SCREENBOX, POSTBOX: IBOX;
                  var dataBOX, trendBox, residueBOX: rBOX;
                  var polyParas: polyParameters;
                  var dataVector, trendVector, residueVec: longVector);
    var
      Tcap, Mcap, Tadd, t: integer;
      menuChoice: integer;
      z, inc, pi, omega, weight, perCent: real;
      PolyWgts: longVector;
      replace: boolean;

  begin
    pi := 4.0 * Arctan(1.0);
    Tcap := dataBOX.NOPUNCT;

    ClrScr;
    Writeln;
    Writeln('    POLYNOMIAL EXTRAPOLATION AND TAPERING');
    Writeln;
    Writeln('    This procedure fits a  polynomial to the data and it proceeds to');
    Writeln('    extrapolate  the fitted  function  at  either end to  a distance');
    Writeln('    that is chosen  as  a  proportion of the length of  the original');
    Writeln('    data. The  extrapolated  segments are  modified  by the addition');
    Writeln('    of values obtained by reflecting the polynomial residuals around');
    Writeln('    the end  points of the  sample and  tapering  them to  zero. The');
    Writeln('    objective is to avoid  distortionary end-effects in  calculating');
    Writeln('    the frequency-specific components of the data.');
    Writeln;
    Writeln(' ' : 62, '<RETURN>');
    Readln;

    repeat
      Writeln('     What proportion of the original data is each extension?');
      Writeln('     Specify the proportion as a percentage');
      ReadANumber(perCent);
    until (perCent <= 100.0) and (perCent >= 0.0);
    Tadd := Round(perCent * Tcap / 100.0);
    Writeln('    Tadd = ', Tadd : 2);
  {dataBOX.EXTRA := Tadd; See Below}

    ClrScr;
    Writeln;
    Writeln('     FIT A POLYNOMIAL TREND');
    Writeln;
    Writeln('    1.   Fit by Ordinary Least-Squares Regression');
    Writeln;
    Writeln('    2.   Fit by Weighted Least-Squares Regression');
    Writeln;
    Writeln('                      Select a number and press <RETURN>');

    GetBoundedInteger(1, 2, menuChoice);
    case menuChoice of
      1: 
        UniformWgts(PolyWgts, dataBOX);
      2: 
        begin
  {Explain PolyWgts goes here}
          GettheWeights(PolyWgts, dataBOX, SCREENBOX);
        end;
    end; {case}

    Writeln;
    Writeln('    What is the degree of the polynomial?');
    GetBoundedInteger(0, 12, polyParas.polyDegree);

    MakePolyTrend(polyParas, dataBOX, trendBox, residueBOX, dataVector, trendVector, residueVec, PolyWgts);

  {Shift the data by Tadd points}
    Mcap := Tcap + Tadd - 1;
    for t := Mcap downto Tadd do
      begin
        dataVector[t] := dataVector[t - Tadd];
        trendVector[t] := trendVector[t - Tadd];
        residueVec[t] := residueVec[t - Tadd];
      end;

    inc := pi / (Tadd + 1);
  {Lower extension by extrapolation plus tapered reflected residuals}
    omega := 0;
    for t := 0 to Tadd - 1 do
      begin
        z := t - Tadd;
        trendVector[t] := PolyOrdinate(z, polyParas.alpha, polyParas.gamma, polyParas.delta, polyParas.polyDegree);
        dataVector[t] := trendVector[t];
        omega := omega + inc;
        weight := 1 - 0.5 * (Cos(omega) + 1);
        residueVec[t] := weight * residueVec[2 * Tadd - t];
        dataVector[t] := dataVector[t] + residueVec[t];
      end;

  {Upper extension by extrapolation plus tapered reflected residuals}
    omega := 0;
    Mcap := Tcap - 1 + Tadd;
    for t := 1 to Tadd do
      begin
        z := Tcap - 1 + t;
        trendVector[Mcap + t] := PolyOrdinate(z, polyParas.alpha, polyParas.gamma, polyParas.delta, polyParas.polyDegree);
        dataVector[Mcap + t] := trendVector[Mcap + t];
        omega := omega + inc;
        weight := 0.5 * (Cos(omega) + 1);
        residueVec[Mcap + t] := weight * residueVec[Mcap - t];
        dataVector[Mcap + t] := dataVector[Mcap + t] + residueVec[Mcap + t];
      end;

    dataBOX.NOPUNCT := Tcap + 2 * Tadd;

    dataBOX.EXTRA := Tadd;
    residueBOX.EXTRA := Tadd;
    residueBOX.NOPUNCT := dataBOX.NOPUNCT;
    trendBox.NOPUNCT := dataBOX.NOPUNCT;

    MakeDataBox(dataBOX, dataVector);
    PlotTheData(dataBOX, SCREENBOX, POSTBOX, dataVector);

    MakeDataBox(residueBOX, residueVec);
    MakeDataBox(trendBox, trendVector);

    weHaveATrend := true;
    weHaveTrendResiduals := true;

    if (suppressReplace = false) then
      begin {suppressReplace}
        ClrScr;
        Writeln;
        Writeln('     FIT A POLYNOMIAL TREND');
        Writeln;
        YesOrNo('    Do you wish to replace the data series by the residual series', replace);
        if replace then
          begin {replace}
            cycleVec := dataVector;
            cycleBOX := dataBOX;
            dataVector := residueVec;
            dataBOX := residueBOX;
            MakeDataBox(dataBOX, dataVector);
            residualsInData := true;
          end; {replace}
      end; {suppressReplace}

    dataExtrapolated := true;

  end; {PolynomialExtrapolation}

  {________________________________________________}

  procedure SeasonalPolyExtrapolation (var SCREENBOX, POSTBOX: IBOX;
                  var dataBOX, trendBox, residueBOX: rBOX;
                  var polyParas: polyParameters;
                  var dataVector, trendVector, residueVec: longVector);
    var
      q, r, s, t, Tcap, Mcap, Tadd: integer;
      menuChoice: integer;
      z, inc, pi, omega, weight, perCent: real;
      PolyWgts: longVector;
      replace: boolean;

  begin
    pi := 4.0 * Arctan(1.0);
    Tcap := dataBOX.NOPUNCT;

    ClrScr;
    Writeln;
    Writeln('    POLYNOMIAL EXTRAPOLATION AND TAPERING');
    Writeln;
    Writeln('    This procedure fits a  polynomial to the data and it proceeds to');
    Writeln('    extrapolate  the fitted  function  at  either end to  a distance');
    Writeln('    that is chosen  as  a  proportion of the length of  the original');
    Writeln('    data. The  extrapolated  segments are  modified  by the addition');
    Writeln('    of tapered  sequences  based on  repetitions of the residuals of');
    Writeln('    the first  and the final years  of the  sample. The objective is');
    Writeln('    to avoid  distortionary  end-effects in  calculating  components');
    Writeln('    of the data.');
    Writeln;
    Writeln(' ' : 62, '<RETURN>');
    Readln;

    repeat
      Writeln('     What proportion of the original data is each extension?');
      Writeln('     Specify the proportion as a percentage');
      ReadANumber(perCent);
    until (perCent <= 100.0) and (perCent >= 0.0);
    Tadd := Round(perCent * Tcap / 100.0);
    Writeln('    Tadd = ', Tadd : 2);
  {dataBOX.EXTRA := Tadd; See Below}

    ClrScr;
    Writeln;
    Writeln('     FIT A POLYNOMIAL TREND');
    Writeln;
    Writeln('    1.   Fit by Ordinary Least-Squares Regression');
    Writeln;
    Writeln('    2.   Fit by Weighted Least-Squares Regression');
    Writeln;
    Writeln('                      Select a number and press <RETURN>');

    GetBoundedInteger(1, 2, menuChoice);
    case menuChoice of
      1: 
        UniformWgts(PolyWgts, dataBOX);
      2: 
        begin
  {ExplainPolyWgts goes here}
          GettheWeights(PolyWgts, dataBOX, SCREENBOX);
        end;
    end; {case}

    Writeln;
    Writeln('    What is the degree of the polynomial?');
    GetBoundedInteger(0, 12, polyParas.polyDegree);

    MakePolyTrend(polyParas, dataBOX, trendBox, residueBOX, dataVector, trendVector, residueVec, PolyWgts);

  {Shift the data upwards by Tadd points}
    Mcap := Tcap + Tadd - 1;
    for t := Mcap downto Tadd do
      begin
        dataVector[t] := dataVector[t - Tadd];
        trendVector[t] := trendVector[t - Tadd];
        residueVec[t] := residueVec[t - Tadd];
      end;

    if (dataBOX.dataFrequency = Monthly) then
      s := 12;
    if (dataBOX.dataFrequency = Quarterly) then
      s := 4;

    inc := pi / (Tadd + 1);
  {Fill the empty spaces on the left}
    omega := 0;
    r := Tadd mod s;
    for t := Tadd - 1 downto 0 do
      begin
        z := t - Tadd;
        trendVector[t] := PolyOrdinate(z, polyParas.alpha, polyParas.gamma, polyParas.delta, polyParas.polyDegree);
        dataVector[t] := trendVector[t];
        omega := omega + inc;
        weight := 0.5 * (Cos(omega) + 1);
        q := (t - r) mod s;
        residueVec[t] := weight * residueVec[Tadd + q];
        dataVector[t] := dataVector[t] + residueVec[t];
      end;

   {Fill the empty spaces on the right}
    omega := 0;
     {Mcap := Tcap - 1 + Tadd;}
    for t := 1 to Tadd do
      begin
        z := Tcap - 1 + t;
        trendVector[Mcap + t] := PolyOrdinate(z, polyParas.alpha, polyParas.gamma, polyParas.delta, polyParas.polyDegree);
        dataVector[Mcap + t] := trendVector[Mcap + t];
        omega := omega + inc;
        weight := 0.5 * (Cos(omega) + 1);
        q := (Mcap - s) + (t mod s);
        residueVec[Mcap + t] := weight * residueVec[q];
        dataVector[Mcap + t] := dataVector[Mcap + t] + residueVec[Mcap + t];
      end;

    dataBOX.NOPUNCT := Tcap + 2 * Tadd;

    dataBOX.EXTRA := Tadd;
    residueBOX.EXTRA := Tadd;
    residueBOX.NOPUNCT := dataBOX.NOPUNCT;
    trendBox.NOPUNCT := dataBOX.NOPUNCT;

    MakeDataBox(dataBOX, dataVector);
    PlotTheData(dataBOX, SCREENBOX, POSTBOX, dataVector);

    MakeDataBox(residueBOX, residueVec);
    MakeDataBox(trendBox, trendVector);

    weHaveATrend := true;
    weHaveTrendResiduals := true;

    if (suppressReplace = false) then
      begin {suppressReplace}
        ClrScr;
        Writeln;
        Writeln('     FIT A POLYNOMIAL TREND');
        Writeln;
        YesOrNo('    Do you wish to replace the data series by the residual series', replace);
        if replace then
          begin {replace}
            cycleVec := dataVector;
            cycleBOX := dataBOX;
            dataVector := residueVec;
            dataBOX := residueBOX;
            MakeDataBox(dataBOX, dataVector);
            residualsInData := true;
          end; {replace}
      end; {suppressReplace}

    dataExtrapolated := true;

  end; {SasonalPolyExtrapolation}

  {________________________________________________}

  procedure CircularMorphing (var SCREENBOX, POSTBOX: IBOX;
                  var dataBOX: rBOX;
                  var dataVector: longVector);

    var
      Tcap, Tadd, Tplus, Tvert, RoteNo, t, r, s: integer;
      inc, pi, phi, perCent, lambda: real;
      postFileName, legend: string;
      F: text;

  begin
    pi := 4.0 * Arctan(1.0);
    Tcap := dataBOX.NOPUNCT;

    ClrScr;
    Writeln;
    Writeln('    EXTRAPOLATE BY CIRCULAR MORPHING');
    Writeln;
    Writeln('    The procedure works with detrended data. An extension is devised');
    Writeln('    that avoids any disjunctions when the data is wrapped around the');
    Writeln('    circle. The  extension, which is interpolated between the end of');
    Writeln('    the final year of the data  and the beginning of the first year,');
    Writeln('    makes  a gradual transition  between  the  seasonal  patterns of');
    Writeln('    these two years.');
    Writeln;
    Writeln(' ' : 60, '<RETURN>');
    Readln;

    {Check that the orignal data comprises an integral number of years}
    repeat
      Writeln('     What proportion of the original data length is in the extension?');
      Writeln('     Specify the proportion as a percentage');
      ReadANumber(perCent);
    until (perCent <= 100.0) and (perCent >= 0.0);

    if (dataBOX.dataFrequency = Monthly) then
      s := 12;
    if (dataBOX.dataFrequency = Quarterly) then
      s := 4;

    Tadd := Round(perCent * Tcap / 100.0);
    r := Tadd div s;
    if Odd(r) then
      r := r + 1;
    Tadd := r * s;

    Writeln('    The number of points in the data extension is ', Tadd : 2);
    Writeln;
    Readln;
    Writeln(' ' : 60, '<RETURN>');

    inc := pi / Tadd;
    phi := 0;
    for t := 0 to Tadd - 1 do
      begin
        Tplus := TCap + t;
        lambda := 0.5 * (Cos(phi) + 1);
        dataVector[Tplus] := (1.0 - lambda) * dataVector[(t mod s)];
        dataVector[Tplus] := dataVector[Tplus] + lambda * dataVector[Tcap - s + (t mod s)];
        phi := phi + inc;
      end;

    dataBOX.NOPUNCT := Tcap + Tadd;
    dataBOX.EXTRA := Tadd div 2;
    MakeDataBox(dataBOX, dataVector);

    GetGraphics(graphicsOn, SCREENBOX, currentMode);

    ScreenFrame(dataBOX, SCREENBOX);
    ScreenPlot(dataBOX, SCREENBOX, dataVector);
    Tvert := Tcap - 1;
    MakeDataLine(Tvert, SCREENBOX, dataBOX);
    legend := 'The data with the extrapolated segment at the end';
    ScreenCaption(legend);

    CallForReturn;
    Readln;
    CloseGraph;
    RestoreCrtMode;
    ClrScr;

    {Now rotate the data to put the extension into the middle }
    RoteNo := Tadd + (Tcap div 2);
    RotateData(RoteNo, dataBOX, dataVector);

    Writeln;
    Writeln(' ' : 60, '<RETURN>');

    GetGraphics(graphicsOn, SCREENBOX, currentMode);

    ScreenFrame(dataBOX, SCREENBOX);
    ScreenPlot(dataBOX, SCREENBOX, dataVector);
    Tvert := Tcap div 2;
    MakeDataLine(Tvert, SCREENBOX, dataBOX);
    Tvert := Tvert + Tadd;
    MakeDataLine(Tvert, SCREENBOX, dataBOX);
    legend := 'The data rotated to place the extrapolated segment in the middle.';
    legend := Concat(legend, ' The segment will be divided between the two ends.');
    ScreenCaption(legend);

    CallForReturn;
    Readln;
    CloseGraph;
    RestoreCrtMode;
    ClrScr;

    if (PSPict = true) then
      begin {PSPict}
        Write('    Name the Postscript File = ');
        Readln(postFileName);
        if postFileName <> '' then
          begin {postFileName}
            Assign(F, postFileName);
            ReWrite(F);
            PostHeader(F, POSTBOX, TeXFormat);
            POSTBOX.xAxis := true;
            POSTBOX.yAxis := true;
            POSTBOX.doFrame := true;
            POSTBOX.doMarks := true;
            PostFrame(F, dataBOX, POSTBOX);
            PostPlot(F, dataBOX, POSTBOX, dataVector);
            Tvert := Tcap div 2;
            MakePostLine(F, Tvert, POSTBOX, dataBOX);
            Tvert := Tvert + Tadd;
            MakePostLine(F, Tvert, POSTBOX, dataBOX);
            legend := 'A segment of  points interpolated into the circularly wrapped data';
            PostFooter(F, POSTBOX, legend, TeXFormat);
            Close(F);
          end; {Postfile}
      end; {PSPict}

    {Now rotate the data to put half of the extension in either end}
    RoteNo := (Tcap + Tadd) div 2;
    AntiRotateData(RoteNo, dataBOX, dataVector);

  end; {CircularMorphing}

 {________________________________________________}

end. {TAPER: Unit 12 of NEWIDEOLOG.PAS}

