unit TRIGONOM; {Unit 17 of NEWIDEOLOG.PAS}

interface

  uses
    GLOBALS, UTILS, MATHS, SCREEN, POST, LEGENDS, POLYNOME, DATASEG, TAPER, FOURIER, FILTERS, Graph, Dos, Crt;

  procedure ExtrapolateSample (var SCREENBOX, POSTBOX: IBOX;
                  var dataBOX, trendBOX, residueBOX: rBOX;
                  var dataVector, trendVector, residueVec: longVector;
                  var polyParas: polyParameters);

  procedure EnsureIntegralCycles (var cycleBOX, trendBOX, residueBOX: rBOX;
                  var cycleVec, trendVector, residueVec: longVector);

  procedure AltIntegralCycles (var trendBOX, tempBOX: rBOX;
                  var trendVector, tempVector: longVector;
                  var shortenBy: integer);

  procedure FillTheEnds (var trendBOX: rBOX;
                  var trendVector: longVector);

  procedure LowpassFrequency (var filterSpec: FilterRecord;
                  var polyParas: polyParameters;
                  var dataBOX, trendBOX, residueBOX, cycleBOX: rBOX;
                  var SCREENBOX, POSTBOX: IBOX;
                  var dataVector, trendVector, residueVec, cycleVec: longVector);

  procedure AltLowpassFrequency (var filterSpec: FilterRecord;
                  var polyParas: polyParameters;
                  var dataBOX, trendBOX, residueBOX, cycleBOX: rBOX;
                  var SCREENBOX, POSTBOX: IBOX;
                  var dataVector, trendVector, residueVec, cycleVec: longVector);

  procedure BandpassFrequency (var filterSpec: FilterRecord;
                  var dataBOX, cycleBOX: rBOX;
                  var SCREENBOX, POSTBOX: IBOX;
                  var dataVector, cycleVector: longVector);

  procedure ExtractSeasons (var polyParas: polyParameters;
                  var tempBOX, seasonBOX, residueBOX: rBOX;
                  var SCREENBOX, POSTBOX: IBOX;
                  var tempVector, seasonVec, residueVec: longVector);

  procedure AdjustForSeasons (var polyParas: polyParameters;
                  var dataBOX, trendBOX, seasonBOX, residueBOX: rBOX;
                  var SCREENBOX, POSTBOX: IBOX;
                  var dataVector, trendVector, seasonVec, residueVec: longVector);

implementation

 {________________________________________________}

  procedure ExtrapolateSample (var SCREENBOX, POSTBOX: IBOX;
                  var dataBOX, trendBOX, residueBOX: rBOX;
                  var dataVector, trendVector, residueVec: longVector;
                  var polyParas: polyParameters);

    var
      extrapolate: boolean;

  begin
    ClrScr;
    Writeln;
    Writeln('    TREAT THE END-OF-SAMPLE PROBLEM');
    Writeln;
    Writeln('    When the data  show strong seasonality, the estimation of the trend');
    Writeln('    is sensistive to the way in which the end points are handled. Then,');
    Writeln('    it may  be appropriate  extrapolate and to taper the data. A linear');
    Writeln('    extrapolation is appropriate.');
    Writeln;
    Writeln('    A  weighted least-squares regression can be used to ensure that the');
    Writeln('    line passes throught the  midst of  the data points  at either end.');
    Writeln('    The  extapolations may  be discarded after the  trend is estimated.');
    Writeln;

    YesOrNo('    Do you wish to extapolate the data', extrapolate);

    if extrapolate then
      begin
        if dataBOX.dataFrequency in [Annual, Other] then
          PolynomialExtrapolation(SCREENBOX, POSTBOX, dataBOX, trendBox, residueBOX, polyParas, dataVector, trendVector, residueVec);
        if dataBOX.dataFrequency in [Monthly, Quarterly] then
          SeasonalPolyExtrapolation(SCREENBOX, POSTBOX, dataBOX, trendBox, residueBOX, polyParas, dataVector, trendVector, residueVec);
      end;

  end;{ExtrapolateSample}

 {_________________________________________________}

  procedure EnsureIntegralCycles (var cycleBOX, trendBOX, residueBOX: rBOX;
                  var cycleVec, trendVector, residueVec: longVector);

    var
      integerCycles: boolean;
      leftOver: integer;

  begin
    ClrScr;
    Writeln;
    Writeln('    ENSURE INTEGRAL CYCLES');
    Writeln;
    Writeln('    In the  absense of extraplation, it may  be  appropriate to  ensure');
    Writeln('    that the differenced data  comprise  a number of points  that is an');
    Writeln('    integer multiple of the number spanning a seasonal cycle.');
    Writeln;
    Writeln('    Do you wish ensure that  the differenced  data comprise an integral');
    YesOrNo('    number of seasonal cycles', integerCycles);

    leftOver := 0;
    if integerCycles then
      begin {integerCycles}
        if trendBOX.dataFrequency = Quarterly then
          leftOver := trendBOX.NOPUNCT mod 4
        else if trendBOX.dataFrequency = Monthly then
          leftOver := trendBOX.NOPUNCT mod 12;
      end; {integerCycles}

    if (leftOver > 0) then
      begin {leftOver}
        ShortenTheData(leftOver, trendBOX, trendVector);
        ShortenTheData(leftOver, residueBOX, residueVec);
        if residualsInData then
          ShortenTheData(leftOver, cycleBOX, cycleVec);
      end; {leftOver}

  end;{EnsureIntegralCycles}

 {________________________________________________}

  procedure AltIntegralCycles (var trendBOX, tempBOX: rBOX;
                  var trendVector, tempVector: longVector;
                  var shortenBy: integer);

  {This procedure is for the case of centralised second differences}
    var
      integerCycles: boolean;
      leftOver: integer;

  begin
    ClrScr;
    Writeln;
    Writeln('    ENSURE INTEGRAL CYCLES');
    Writeln;
    Writeln('    In the  absense of extraplation, it may  be  appropriate to  ensure');
    Writeln('    that the differenced data  comprise  a number of points  that is an');
    Writeln('    integer multiple of the number spanning a seasonal cycle.');
    Writeln;
    Writeln('    Do you wish ensure that  the differenced  data comprise an integral');
    YesOrNo('    number of seasonal cycles', integerCycles);

     {We subtract the points at the beginning and the end of the dataquence. Then}
     {we determine whether or not the remaining points comprise an integer number}
     {of cycles}

    leftOver := 0;
    if integerCycles then
      begin {integerCycles}
        if trendBOX.dataFrequency = Quarterly then
          leftOver := (trendBOX.NOPUNCT - 2) mod 4
        else if trendBOX.dataFrequency = Monthly then
          leftOver := (trendBOX.NOPUNCT - 2) mod 12;

        shortenBy := leftover + 1;  {the lowermost point is always discarded}
        ShortenTheData(shortenBy, trendBOX, trendVector);
        shortenTheData(shortenBy, tempBOX, tempVector);

        trendBOX.NOPUNCT := trendBOX.NOPUNCT - 1;
        tempBOX.NOPUNCT := tempBOX.NOPUNCT - 1;
      {TrendBOX.NOPUNCT is reduced by one, since the uppermost point is discarded}
      end; {integer Cycles}

  end; {AltIntegralCycles}

 {________________________________________________}

  procedure FillTheEnds (var trendBOX: rBOX;
                  var trendVector: longVector);

  begin
    if trendBOX.dataFrequency = Monthly then
      begin
        trendVector[0] := trendVector[12];
        trendVector[trendBOX.NOPUNCT - 1] := trendVector[trendBOX.NOPUNCT - 13];
      end;

    if dataBOX.dataFrequency = Quarterly then
      begin
        trendVector[0] := trendVector[4];
        trendVector[trendBOX.NOPUNCT - 1] := trendVector[trendBOX.NOPUNCT - 5];
      end;

    if dataBOX.dataFrequency in [Annual, Other] then
      begin
        trendVector[0] := trendVector[1];
        trendVector[trendBOX.NOPUNCT - 1] := trendVector[trendBOX.NOPUNCT - 2];
      end;

  end; {FillTheEnds}

{________________________________________________}

  procedure LowpassFrequency (var filterSpec: FilterRecord;
                  var polyParas: polyParameters;
                  var dataBOX, trendBOX, residueBOX, cycleBOX: rBOX;
                  var SCREENBOX, POSTBOX: IBOX;
                  var dataVector, trendVector, residueVec, cycleVec: longVector);

  {Called by ImplementFrequencyFilter in ORGANISE}
  {The is an Adjunct of DiffandReinflate}

    var
      discard: boolean;
      t, cutOffInt, requiredOrder: integer;
      cutOffRadians: real;
      pi, radiansPerDegree, FourierInc: real;
      alpha, beta: fourierVec;
      legend: string;

  begin
    pi := 4 * ArcTan(1.0);
    radiansPerDegree := pi / 180;

    ClrScr;
    Writeln;
    Writeln('    EXTRACT LOWPASS TREND COMPONENT');
    writeln;

    if (thereIsData = false) then
      begin
        SysBeep(3);
        ReadTheData(dataBOX, dataVector);
      end;

    if noPageParameters then
      PageSetUp(SCREENBOX, POSTBOX, PSPict, TeXFormat);

    ClrScr;
    Writeln;
    Writeln('    EXTRACT LOWPASS TREND COMPONENT');
    Writeln;
    Writeln('    The trend comprises a set of sinusoidal components of');
    Writeln('    frequencies that range from zero to the Fourier frequency');
    Writeln('    nearest to a specified cut-off point.');
    Writeln;

    {The cut-off  has been Specified in <ImplementFrequencyFilter> in ORGANISE.');}

    case dataBOX.dataFrequency of
      Quarterly: 
        Writeln('    The cut-off demarcates cycles of duration longer than ', Round(360 / filterSpec.cutOff) : 2, ' quarters');
      Monthly: 
        Writeln('    The cut-off demarcates cycles of duration longer than ', Round(360 / filterSpec.cutOff) : 2, ' months');
    end; {case}

    cutOffRadians := filterSpec.cutOff * radiansPerDegree;

     {The option is provided to extrapolate the data to assist in handling the end-effects}
     {We suppresses the option of putting the residuals in place of the data}

    suppressReplace := true;
    if (residualsInData = false) then
      ExtrapolateSample(SCREENBOX, POSTBOX, dataBOX, trendBOX, residueBOX, dataVector, trendVector, residueVec, polyParas);
    suppressReplace := false;

    trendVector := dataVector;
    trendBOX := dataBOX;
    residueVec := dataVector;
    residueBOX := dataBOX;

   {We put the data into trendVector to process it. We put the data into residueVec}
   {and we leave it undisturned apart from shortening it, if we require an integral}
   {number of cycles within the differenced data. To create the residue, we subtract}
   {the trend from the data contained in the residueVec. In case we have to put the}
   {residuals from a polynomial detrending into DataVector we must shorten this and}
   {the trended data into cycleVec as well.}

    ClrScr;
    Writeln;
    Writeln('    EXTRACT LOWPASS TREND COMPONENT');
    Writeln;
    Writeln('    Before estimating the trend,  you may require to reduce the  ');
    Writeln('    data to stationarity by at most two differencing operations.');
    Writeln;
    Writeln('    Specify the number of differencing operations.');

    GetBoundedInteger(0, 2, requiredOrder);
    if (requiredOrder = 1) then
      DiffOfVector(trendBOX, trendVector);
    if (requiredOrder = 2) then
      SecondDiffOfVector(trendBOX, trendVector);

     {Observe that taking differences reduces the length of the Vector,}
     {which is recorded in the NOPUNCT field.}

     {In the absense of extrapolation, it pays to have an integral number of}
     {cycles in the differenced data. This is optional.}

    if dataBOX.EXTRA = 0 then
      EnsureIntegralCycles(cycleBOX, trendBOX, residueBOX, cycleVec, trendVector, residueVec);
     {We also shorten the residueVec cycleVec by taking the leftOver points from the LHS}

    fourierInc := 2 * pi / (trendBOX.NOPUNCT); {This a local variable}
    cutOffInt := Round(cutOffRadians / fourierInc);
    FourierCoefficients(trendBOX, trendVector, fourierInc, cutOffInt, alpha, beta);
    beta[0] := 0.0;
    FourierSynthesis(trendBOX, trendVector, fourierInc, cutOffInt, alpha, beta);
    if (requiredOrder > 0) then
      DataCumulation(residueBOX, trendBOX, requiredOrder, residueVec, trendVector);

    if (residueBOX.EXTRA > 0) then  {residue Vec is DataVector}
      begin {EXTRA}
        discard := false;
        DiscardQuestion(discard);
        if discard then
          begin {discard}
            polyParas.tStart := dataBOX.EXTRA;
            DiscardExtraData(dataBOX, dataVector);
            DiscardExtraData(trendBOX, trendVector);
            DiscardExtraData(residueBOX, residueVec);
            if residualsInData then  {because we have put the data into cycleVec}
              DiscardExtraData(cycleBOX, cycleVec);
          end; {discard}
      end; {EXTRA}

    legend := FourierLowPass(residueBOX, dataName);
    ScreenOverplot(residueBOX, SCREENBOX, residueVec, trendVector, legend);

    if PSPict = true then
      WritePostOverFile(residueBOX, POSTBOX, residueVec, trendVector, legend);

    for t := 0 to residueBOX.NOPUNCT do
      residueVec[t] := residueVec[t] - trendVector[t];

    MakeDataBox(residueBOX, residueVec);
    MakeDataBox(trendBOX, trendVector);

    legend := TrendResiduals(residueBOX, dataName);
    ScreenMakePlot(residueBOX, SCREENBOX, residueVec, legend);

    if PSPict = true then
      WritePostFile(residueBOX, POSTBOX, residueVec, legend)
    else
      Writeln(' ' : 60, '<RETURN>');

    weHaveATrend := true;
    weHaveTrendResiduals := true;

     {In case we have put the polynomial residuals into the dataVector, the trendVector}
     {will already contain the cycle. Then, we may add the polynomial component to}
     {create the trendCycle. The original data has gone into cycleVec}

    if residualsInData then
      begin {restore data to dataVector}
        dataBOX := cycleBOX;
        dataVector := cycleVec;
        cycleBOX := trendBOX;
        cycleVec := trendVector;
        MakeDataBox(cycleBOX, cycleVec);
        residualsInData := false;
        weHaveABusinessCycle := true;
        weHaveATrend := false;
        TrendCycleComponent(polyParas, dataBOX, trendBOX, cycleBOX, dataVector, trendVector, cycleVec, SCREENBOX, POSTBOX);
      end;

  end;{LowpassFrequency}

  {________________________________________________}

  procedure AltLowpassFrequency (var filterSpec: FilterRecord;
                  var polyParas: polyParameters;
                  var dataBOX, trendBOX, residueBOX, cycleBOX: rBOX;
                  var SCREENBOX, POSTBOX: IBOX;
                  var dataVector, trendVector, residueVec, cycleVec: longVector);

 {Called by ImplementFrequencyFilter in ORGANISE}
 {This is an Adjunct of DiffAndReduce}
    var
      discard, takeDifferences: boolean;
      t, cutOffInt, lowerCutInt, UpperCutInt, requiredOrder: integer;
      cutOffRadians: real;
      pi, radiansPerDegree, FourierInc: real;
      alpha, beta: fourierVec;
      legend: string;

  begin
    pi := 4 * ArcTan(1.0);
    radiansPerDegree := pi / 180;

    ClrScr;
    Writeln;
    Writeln('    EXTRACT LOWPASS TREND COMPONENT');
    writeln;

    if (thereIsData = false) then
      begin
        SysBeep(3);
        ReadTheData(dataBOX, dataVector);
      end;

    if noPageParameters then
      PageSetUp(SCREENBOX, POSTBOX, PSPict, TeXFormat);

    ClrScr;
    Writeln;
    Writeln('    EXTRACT LOWPASS TREND COMPONENT');
    Writeln;
    Writeln('    The trend comprises a set of sinusoidal components of');
    Writeln('    frequencies that range from zero to the Fourier frequency');
    Writeln('    nearest to a specified cut-off point.');
    Writeln;

     {The cut-off  has been Specified in <ImplementFrequencyFilter> in ORGANISE.');}

    case dataBOX.dataFrequency of
      Quarterly: 
        Writeln('    The cut-off demarcates cycles of duration longer than ', Round(360 / filterSpec.cutOff) : 2, ' quarters');
      Monthly: 
        Writeln('    The cut-off demarcates cycles of duration longer than ', Round(360 / filterSpec.cutOff) : 2, ' months');
    end;

     {The option is provided to extrapolate the data to assist in handling the end-effects}
     {We suppresses the option of putting the residuals in place of the data}

    suppressReplace := true;
    if (residualsInData = false) then
      ExtrapolateSample(SCREENBOX, POSTBOX, dataBOX, trendBOX, residueBOX, dataVector, trendVector, residueVec, polyParas);
    suppressReplace := false;

    trendVector := dataVector;
    trendBOX := dataBOX;
     {We use trendVector initially as a working space}

    ClrScr;
    Writeln;
    Writeln('    EXTRACT LOWPASS TREND COMPONENT');
    Writeln;
    Writeln('    Before estimating the trend,  you may require to reduce the  ');
    Writeln('    data to stationarity by at most two differencing operations.');
    Writeln;
    YesOrNo('    Do you wish to take centralised second differences', takeDifferences);

    requiredOrder := 0;
    if takeDifferences then
      begin
        CentralisedDiffernces(trendBOX, trendVector);
        FillTheEnds(trendBOX, trendVector);
        requiredOrder := 2;
      end;

    fourierInc := 2 * pi / (trendBOX.NOPUNCT); {This a local variable}
    cutOffRadians := filterSpec.cutOff * radiansPerDegree;
    cutOffInt := Round(cutOffRadians / fourierInc);
    lowerCutInt := cutOffInt;
    upperCutInt := dataBOX.NOPUNCT div 2; {frequencey pi}
    BandPassCoefficients(trendBOX, trendVector, fourierInc, lowerCutInt, upperCutInt, alpha, beta);

    if (requiredOrder = 2) then
      InflateFourierCoefficients(requiredOrder, lowerCutInt, upperCutInt, fourierInc, alpha, beta);

    residueBOX := dataBOX;
    MakeDataBox(residueBOX, residueVec);
    BandpassSynthesis(residueBOX, residueVec, fourierInc, lowerCutInt, upperCutInt, alpha, beta);
    for t := 0 to residueBOX.NOPUNCT - 1 do
      trendVector[t] := dataVector[t] - residueVec[t];

    if dataBOX.EXTRA > 0 then
      begin {EXTRA}
        discard := false;
        DiscardQuestion(discard);
        if discard then
          begin {discard}
            polyParas.tStart := dataBOX.EXTRA;
            DiscardExtraData(dataBOX, dataVector);
            DiscardExtraData(trendBOX, trendVector);
            DiscardExtraData(residueBOX, residueVec);
            if residualsInData then  {because we have put the data into cycleVec}
              DiscardExtraData(cycleBOX, cycleVec);
          end; {discard}
      end; {EXTRA}

    legend := TrigonometricalTrend(filterSpec.cutOff, dataBOX, dataName);

    ScreenOverplot(dataBOX, SCREENBOX, dataVector, trendVector, legend);
    trendBOX := dataBOX;
    MakeDataBox(trendBOX, trendVector);

    if PSPict = true then
      WritePostOverFile(dataBOX, POSTBOX, dataVector, trendVector, legend);

    legend := TrendResiduals(dataBOX, dataName);
    ScreenMakePlot(residueBOX, SCREENBOX, residueVec, legend);

    if PSPict = true then
      WritePostFile(residueBOX, POSTBOX, residueVec, legend)
    else
      Writeln(' ' : 60, '<RETURN>');

    weHaveATrend := true;
    weHaveTrendResiduals := true;

     {In case we have put the polynomial residuals into the dataVector, the trendVector}
     {will already contain the cycle. Then, we may add the polynomial component to}
     {create the trendCycle }

    if residualsInData then
      begin
        dataBOX := cycleBOX;
        dataVector := cycleVec;
        cycleBOX := trendBOX;
        cycleVec := trendVector;
        MakeDataBox(cycleBOX, cycleVec);
        weHaveABusinessCycle := true;
        weHaveATrend := false;
        TrendCycleComponent(polyParas, dataBOX, trendBOX, cycleBOX, dataVector, trendVector, cycleVec, SCREENBOX, POSTBOX);
      end;

  end;{AltLowpassFrequency}

  {________________________________________________}

  procedure BandpassFrequency (var filterSpec: FilterRecord;
                  var dataBOX, cycleBOX: rBOX;
                  var SCREENBOX, POSTBOX: IBOX;
                  var dataVector, cycleVector: longVector);
 {also trend and residueVecs}
    var
      discard: boolean;
      upperCutInt, lowerCutInt, requiredDiffs: integer;
      pi, radiansPerDegree: real;
      lowerCutRadians, upperCutRadians, fourierInc: real;
      alpha, beta: fourierVec;
      legend: string;
      tempBOX: rBOX;
      tempVector: longVector;

  begin
    pi := 4 * ArcTan(1.0);
    radiansPerDegree := pi / 180;

    {We may wish to add the facitlity for extrapolating the data and equally one }
    {for ensuring that the differenced data contain an integral number of cycles.}
    {How should we handle the case where the polynomial residuals have replaced}
    {the data? If we are to shorten the data, then we need a tempVector.}

    ClrScr;
    Writeln;
    Writeln('    EXTRACT A BANDPASS COMPONENT');
    Writeln;

    if thereIsData = false then
      begin {Read New Data}
        SysBeep(3);
        ReadTheData(dataBOX, dataVector);
      end; {Read New Data}

    if noPageParameters then
      PageSetUp(SCREENBOX, POSTBOX, PSPict, TeXFormat);

    ClrScr;
    Writeln;
    Writeln('    EXTRACT A BANDPASS BUSINESS-CYCLE COMPONENT');
    Writeln;
    Writeln('    This  procedure  extracts  a cyclical component comprising');
    Writeln('    sinusoidal elements from a range of frequencies bounded by');
    Writeln('    lower and  upper values that  are  specified by the user.');

    tempBOX := dataBOX;
    tempVector := dataVector;

     {The option is provided to extrapolate the data to assist in handling the end-effects}
     {We suppresses the option of putting the residuals in place of the data}

    suppressReplace := true;
    ExtrapolateSample(SCREENBOX, POSTBOX, tempBOX, trendBOX, residueBOX, tempVector, trendVector, residueVec, polyParas);
    suppressReplace := false;

    cycleBOX := tempBOX;
    cycleVector := tempVector;

    ClrScr;
    Writeln;
    Writeln('    EXTRACT A BANDPASS BUSINESS-CYCLE COMPONENT');
    Writeln;
    Writeln('    Before  estimating the cycle, it may be necessary to reduce the');
    Writeln('    data to stationarity  via one or two differencing operations.');
    Writeln;
    Writeln('    Specify the number of differencing operations');

    GetBoundedInteger(0, 2, requiredDiffs);
    if (requiredDiffs = 1) then
      DiffOfVector(cycleBOX, cycleVector);
    if (requiredDiffs = 2) then
      SecondDiffOfVector(cycleBOX, cycleVector);

    if tempBOX.EXTRA = 0 then
      EnsureIntegralCycles(cycleBOX, cycleBOX, tempBOX, cycleVec, cycleVec, tempVector);
     {The first instances of cycleBOX and cycleVec are dummy placeholders}

    fourierInc := 2 * pi / (tempBOX.NOPUNCT);
    lowerCutRadians := filterSpec.lowerCut * radiansPerDegree;
    lowerCutInt := Round(lowerCutRadians / fourierInc);
    upperCutRadians := filterSpec.upperCut * radiansPerDegree;
    upperCutInt := Round(upperCutRadians / fourierInc);

    ClrScr;
    Writeln;
    Writeln('    EXTRACT A BANDPASS BUSINESS-CYCLE COMPONENT');
    Writeln;
    Writeln('    The business cycle comprises a set of sinusoidal components over a');
    Writeln('    narrow  range  of frequencies. A lower cut-off point and  an upper');
    Writeln('    cut-off point have been specified.');
    Writeln;
    case dataBOX.dataFrequency of
      Annual: 
        begin
          Writeln('    The lower cut-off demarcates cycles of duration more than ', Round(360 / filterSpec.lowerCut) : 2, ' years');
          Writeln('    The upper cut-off demarcates cycles of duration less than ', Round(360 / filterSpec.upperCut) : 2, ' years');
        end;
      Quarterly: 
        begin
          Writeln('    The lower cut-off demarcates cycles of duration more than ', Round(360 / filterSpec.lowerCut) : 2, ' quarters');
          Writeln('    The upper cut-off demarcates cycles of duration less than ', Round(360 / filterSpec.upperCut) : 2, ' quarters');
        end;
      Monthly: 
        begin
          Writeln('    The lower cut-off demarcates cycles of duration more than ', Round(360 / filterSpec.lowerCut) : 2, ' months');
          Writeln('    The upper cut-off demarcates cycles of duration less than ', Round(360 / filterSpec.upperCut) : 2, ' months');
        end;
      Other: 
        begin
          Writeln('    The lower cut-off demarcates cycles of duration more than ', Round(360 / filterSpec.lowerCut) : 2, ' periods');
          Writeln('    The upper cut-off demarcates cycles of duration less than ', Round(360 / filterSpec.upperCut) : 2, ' periods');
        end;
    end; {case}
    Writeln;
    Writeln(' ' : 65, '<RETURN>');
    Readln;

  {This is where we can insert our alternative procedure}
  {That is, if we deflate the bandpass coefficients by the frequency response of }
  {the difference operator filter, then we can avoid the <<Cycumulation>> operator}

    BandPassCoefficients(cycleBOX, cycleVector, fourierInc, lowerCutInt, upperCutInt, alpha, beta);
    beta[0] := 0.0;

    BandpassSynthesis(cycleBOX, cycleVector, fourierInc, lowerCutInt, upperCutInt, alpha, beta);
    if (requiredDiffs > 0) then
      Cycumulation(tempBOX, cycleBOX, requiredDiffs, tempVector, cycleVector);
  { <Cycumulation> does not reference the dataVector}
    MakeDataBox(cycleBOX, cycleVector);

    if (tempBOX.EXTRA > 0) then
      begin {Extrapolations}
        DiscardQuestion(discard);
        if discard then
          begin
            DiscardExtraData(tempBOX, tempVector);
            DiscardExtraData(cycleBOX, cycleVector);
          end;
      end; {Extrapolations}

    legend := FourierBandPass(dataBOX, dataName);
    ScreenMakePlot(cycleBOX, SCREENBOX, cycleVector, legend);

    if PSPict = true then
      WritePostFile(cycleBOX, POSTBOX, cycleVector, legend);

    weHaveABusinessCycle := true;


  end;{BandpassFrequency}

  {________________________________________________}

  procedure ExtractSeasons (var polyParas: polyParameters;
                  var tempBOX, seasonBOX, residueBOX: rBOX;
                  var SCREENBOX, POSTBOX: IBOX;
                  var tempVector, seasonVec, residueVec: longVector);

    var
      supplement, replace: boolean;
      t, numbOfPoints, diffOrder: integer;
      pi, fourierInc: real;
      alpha, beta: fourierVec;
      frequencyPoints: vector;
      legend: string;

{On entry, in case of differences, residueVec contains the original data}
{tempVector holds either the differenced data or the residuals from a polynomial}
{detending. On completion, SeasonVec will contain the synthesised seasonal component}
{and residueVec will contain the seasonally adjusted data, which may, at choice, be}
{written into the data vector.}

{N.B. in the case of residualsinData, we should have a facility for adding the}
{seasonally-adjusted residuals to the estimated polynomial trend}

 {We have put the original undifferenced data into the residueVec}
 {At the start of the take difference procedure, If we had rediduals in data}
 {then there should be no effect from failing to take this step}

  begin
    ClrScr;
    Writeln;
    Writeln('    ESTIMATE THE SEASONAL COMPONENT');
    Writeln;
    Writeln('    The seasonal effect comprises a  sinusoidal component at');
    Writeln('    the seasonal frequency together with components at the');
    Writeln('    associated harmonic frequencies.');
    Writeln;

    case tempBOX.dataFrequency of
      Quarterly: 
        Writeln('    The seasonal frequencies occur at 90 and 180 degrees.');
      Monthly: 
        Writeln('    The seasonal frequencies occur at 30, 60, 90, 120, 150 and 180 degrees.');
    end; {case}
    Writeln;
    YesOrNo('    Do you wish to add some supplementary nonharmonic frequencies', supplement);

    pi := 4 * ArcTan(1.0);
    fourierInc := 2 * pi / (tempBOX.NOPUNCT);

    if supplement then
      SupplementaryPoints(tempBOX, fourierInc, frequencyPoints, numbofPoints) {in FOURIER}
    else
      StandardPoints(tempBOX, frequencyPoints, numbofPoints); {in FOURIER}

    SeasonalCoefficients(tempBOX, tempVector, frequencyPoints, numbofPoints, alpha, beta);
    diffOrder := tempBOX.diffOrder;

    if (tempBOX.diffMode = central) then
      InflateSeasonalCoefficients(frequencyPoints, numbofPoints, alpha, beta);

    SeasonalSynthesis(tempBOX, seasonVec, frequencyPoints, numbofPoints, alpha, beta);

    if (tempBOX.diffOrder = 1) or (tempBOX.diffMode = lagged) then
      SeasonalCumulation(tempBOX, seasonBOX, diffOrder, seasonVec);

    seasonBOX := tempBOX;
    MakeDataBOX(seasonBOX, SeasonVec);

    legend := SeasonalComponent(tempBOX, dataName);
    ScreenMakePlot(seasonBOX, SCREENBOX, seasonVec, legend);

    if PSPict = true then
      WritePostFile(seasonBOX, POSTBOX, seasonVec, legend);

    weHaveSeasonalComponent := true;
    residueBOX := tempBOX;

{residueVec contains the original data if differences have been taken (or the polynomial rediduals)}
{What if diffendrende a polynomials not been fitted and residuals are not in the data vector?}
    for t := 0 to residueBOX.NOPUNCT - 1 do
      residueVec[t] := residueVec[t] - seasonVec[t];

    MakeDataBox(residueBOX, residueVec);

    legend := SeasonallyAdjusted(residueBOX, dataName);
    ScreenMakePlot(residueBOX, SCREENBOX, residueVec, legend);

    if PSPict = true then
      WritePostFile(residueBOX, POSTBOX, residueVec, legend);

{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
{Here we should have the option of adding the seasonally adjusted residuals}
{to the fitted polynomial trend. The data has been residing in cycleVec+cycleBOX }
{In that case, we should pass var polyParas: polyParameters; via the procedure heading}

    if residualsInData then
      begin  {restore data to dataVector}
        dataBOX := cycleBOX;
        dataVector := cycleVec;
        residualsInData := false;
        TrendPlusSeason(polyParas, dataBOX, residueBOX, dataVector, residueVec, SCREENBOX, POSTBOX);
      end;
{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}


    Writeln;
    Writeln('    SAVE THE SEASONALLY ADJUSTED DATA');
    Writeln;
    YesOrNo('    Do you wish to replace the data by the seasonally adjusted data', replace);

{The seasonally adjusted data is in residueVec}
    if replace then
      begin {replace}
        dataBOX := residueBOX;
        dataVector := residueVec;
        MakeDataBox(dataBOX, dataVector);
        weHaveDeasonalisedData := true;
      end; {replace}

    weHaveSeasonalComponent := true;
    weHaveTrendResiduals := false;

  end; {ExtractSeasons}

 {________________________________________________}

  procedure AdjustForSeasons (var polyParas: polyParameters;
                  var dataBOX, trendBOX, seasonBOX, residueBOX: rBOX;
                  var SCREENBOX, POSTBOX: IBOX;
                  var dataVector, trendVector, seasonVec, residueVec: longVector);

{On entry, dataVector contains either the raw data or the residuals from a polynomial}
{detrending, which, in that case will also be found in residueVec. trendVector and}
{residueVec will be used in case we need to obtain polynomial residuals for the purpose}
{of investiagating their periodogram}
{tempVector is a temporary local storage for dataVector. It is used in place of}
{dataVector in FitPolyTrend. Then it is used in taking differences of the data.  Finally,}
{it is passed in place of the dataVector to the ExtractSeasons procedure. NB we }
{should include polyparams in the procedure headline so as to pass them on to}
{ExtractSeasons}

    var
      moreDifferences: boolean;
      fittedPoly, examineIt: boolean;
      theMenuItem, requiredOrder: integer;
      menuChar: char;
      legend, alertString: string;
      tempVector: longVector;
      tempBOX: rBOX;

  begin

    if (thereIsData = false) then
      begin
        SysBeep(3);
        ReadTheData(dataBOX, dataVector);
      end;

    if noPageParameters then
      PageSetUp(SCREENBOX, POSTBOX, PSPict, TeXFormat);

    tempBOX := dataBOX;
    tempVector := dataVector;
    fittedPoly := false;

    repeat {until Continue}

      repeat
        ClrScr;
        Writeln;
        Writeln('        REMOVE SEASONAL FLUCTUATIONS FROM THE DATA');
        Writeln;
        Writeln('        In order to  determine the frequency ranges of the sinusoidal');
        Writeln('        elements that of constitute the seasonal fluctuations, it may');
        Writeln('        be helpful to examine the  periodogram of the  detrended data');
        if residualsInData then
          begin
            Writeln('        sequence.');
            Writeln;
            Writeln('           1. Examine the peridogram of the residuals');
            Writeln;
            Writeln('           2. Continue');
            Writeln;
            GetBoundedInteger(1, 2, theMenuItem);
            theMenuitem := 2 * theMenuitem; {i.e if menuitem := 2 then continue}
            fittedpoly := true;
          end
        else
          begin
            Writeln('        sequence. An appropriate sequence  to examine is  provided by');
            Writeln('        the residuals from fitting a polynomial trend to the data');
            Writeln;
            Writeln('           1. Obtain the residuals of a polynomial regression ');
            Writeln;
            Writeln('           2. Examine the peridogram of the residuals');
            Writeln;
            Writeln('           3. Take differences of the data');
            Writeln;
            Writeln('           4. Continue');
            Writeln;
            GetBoundedInteger(1, 4, theMenuItem);
          end;

      until theMenuItem in [1..4];

      case theMenuItem of
        1: 
          begin {Obtain Residuals in residueVec}
            if (tempBOX.diffOrder = 0) then
              begin
                suppressReplace := true;
                FitPolyTrend(polyParas, tempBOX, trendBOX, residueBOX, SCREENBOX, POSTBOX, tempVector, trendVector, residueVec);
                fittedPoly := true;
                suppressReplace := false;
              end
            else
              begin {Already Differenced}
                Sysbeep(3);
                if tempBOX.diffOrder = 1 then
                  alertString := 'once.';
                if tempBOX.diffOrder = 2 then
                  alertString := 'twice.';
                alertString := Concat('    The data has aready been differenced ', alertString);
                Writeln(alertString);
                Writeln('    It is inappropiate to fit a polynomial to the differenced data');
                Writeln;
                Writeln(' ' : 56, '<RETURN>');
                Readln;
              end; {Already Differenced}
          end; {Obtain Residuals}
        2: 
          begin {Plot Residual Periodogram}
            if (fittedPoly = false) then
              begin {noFittedPoly}
                SysBeep(3);
                Writeln('   you have not obtained a residual sequence by fitting a polynomial');
                YesOrNo('   Do you wish to exmine the peridogram of the original data', examineIt);
                legend := 'The periodogram of ';
                legend := Concat(legend, dataName, '.');
                if examineIt then
                  PlotThePeriodogam(tempBOX, SCREENBOX, POSTBOX, tempVector, legend); {in DATASEG}
              end {noFittedPoly}
            else
              begin {PlotThePeriodogam}
                legend := 'The periodogram of residual from fitting a polynomial trend to ';
                legend := Concat(legend, dataName, '.');
                PlotThePeriodogam(residueBOX, SCREENBOX, POSTBOX, residueVec, legend); {in DATASEG}
              end; {PlotThePeriodogam}
          end; {Plot Residual Periodogram}
        3: 
          begin {Take Differences}
{in case of differences. (tempVector<==>dataVector) --> residueVec }
            residueVec := tempVector;     {Perhaps this should happen regardless}
            residueBOX := tempBOX;          {of whether or not we take differnces}
            weHaveTrendResiduals := false;

            requiredOrder := 0;
            Writeln('    Before estimating the seasonal component, you may need to reduce');
            Writeln('    the data to stationarity via one or two differencing operations.');
            Writeln;
            Writeln('    Specify the order of the diffencing operation.');
            GetBoundedInteger(0, 2, requiredOrder);

            if (tempBOX.diffOrder = 2) then
              begin {We have 2nd Diffs already}
                Writeln('    You have already taken twofold differences of the data.');
                Writeln('    These differences will now be centralised');
                Writeln(' ' : 60, '<RETURN>');
                Readln;
                if (tempBOX.diffOrder = lagged) then
                  SecondDiffsCentralised(tempBOX, tempVector);
                FillTheEnds(tempBOX, tempVector);
              end; {We had 2nd Diffs already}

            if (tempBOX.diffOrder = 1) then
              begin {we have first Diffs already}
                Writeln('    You have already taken first differences of the data');
                YesOrNo('    Do you wish to take the second differences', moreDifferences);
                if moreDifferences then
                  begin {one more diff and centralise}
                    Writeln('    Centalised second differences will replace of the first differences');
                    Writeln(' ' : 60, '<RETURN>');
                    Readln;
                    SecondDiffandCentralise(tempBOX, tempVector);
                    FillTheEnds(tempBOX, tempVector);
                  end; {one more diff and centralise}
              end; {we had first Diffs already}

            if (tempBOX.diffOrder = 0) then
              begin {no diffences yet}
                if (requiredOrder = 1) then
                  DiffOfVector(tempBOX, tempVector);
                if (requiredOrder = 2) then
                  begin
                    CentralisedDiffernces(tempBOX, tempVector);
                    FillTheEnds(tempBOX, tempVector);
                  end;
              end; {no diffences yet}

            if tempBOX.diffOrder = 1 then
              alertString := 'first';
            if tempBOX.diffOrder = 2 then
              alertString := 'second-order centralised';

            Writeln('    The data have been replaced by the ', alertString, ' differences');
            Writeln(' ' : 60, '<RETURN>');
            Readln;
          end; {Take Differences}

      end; {cases}

    until (theMenuItem = 4); {Continue}
    menuChar := 'c';

    if tempBOX.dataFrequency in [Annual, Other] then
      begin {not Seasonal}
        ClrScr;
        Sysbeep(3);
        Writeln;
        Writeln('    IDENTIFY A SEASONAL DATA FREQUENCY');
        Writeln;
        Writeln('    To be amenable to  seasonal adjustment, the data must have a quarterly or');
        Writeln('    a monthly frequency of observation. Neither frequency has been identified');
        Writeln('    Please select one of the following options:');
        Writeln;
        repeat
          Writeln('       (q) Indicate that the data have a quarterly frequency');
          Writeln;
          Writeln('       (m) Indicate that the data have a monthly frequency');
          Writeln;
          Writeln('       (e) Terminate the seasonal adjustment procedure');
          Writeln;
          ReadAChar(menuChar);
        until menuChar in ['q', 'Q', 'm', 'M', 'e', 'E'];
        if menuChar in ['q', 'Q'] then
          tempBOX.dataFrequency := Quarterly;
        if menuChar in ['m', 'M'] then
          tempBOX.dataFrequency := Monthly;
      end; {not Seasonal}

    if not (menuChar = 'e') then
      ExtractSeasons(polyParas, tempBOX, seasonBOX, residueBOX, SCREENBOX, POSTBOX, tempVector, seasonVec, residueVec);

  end;{AdjustForSeasons}

 {_________________________________________________}

end. {TRIGONOM: Unit 17 of NEWIDEOLOG.PAS}

