unit WIENER; {Unit 18 of NEWIDEOLOG.PAS}

interface

  uses
    GLOBALS, UTILS, MATHS, SCREEN, POST, POSTBIS, LEGENDS, FILTERS, FIRFILT, POLYNOME, FOURIER, DATASEG, Graph, Dos, Crt;

  procedure ButterSpec (var filterSpec: filterRecord;
                  var poles, zeros: complexVector;
                  var gamma, delta: complexVector);

  procedure HPFilterSpec (var filterSpec: filterRecord;
                  var poles, zeros: complexVector;
                  var gamma, delta: complexVector;
                  var numerator, denominator: vector);

  procedure SeasonalFilterSpec (var filterSpec: filterRecord;
                  var poles, zeros: complexVector;
                  var gamma, delta: complexVector;
                  var numerator, denominator: vector);

  procedure PoleZeroPlot (var filterSpec: filterRecord;
                  var poles, zeros: complexVector);

  procedure PlotWienerFilterGain (var SCREENBOX: IBOX;
                  var filterSpec: filterRecord);

  procedure ApplyHPFilter (var dataBOX, residueBOX, cycleBOX: rBOX;
                  var dataVector, trendVector, residueVec, cycleVec: longVector;
                  var polyParas: polyParameters;
                  var filterSpec: FilterRecord);

  procedure ApplyButterFilter (var dataBOX, residueBOX, cycleBOX: rBOX;
                  var dataVector, trendVector, residueVec, cycleVec: longVector;
                  var polyParas: polyParameters;
                  var filterSpec: FilterRecord);

  procedure MakeHPTrend (var dataBOX, trendBOX, residueBOX: rBOX;
                  var lambda: real;
                  var dataVector, trendVector, residueVec: longVector);

  procedure MakeButterTrend (var dataBOX, trendBOX, residueBOX: rBOX;
                  var filterSpec: filterRecord;
                  var dataVector, trendVector, residueVec: longVector);

  procedure DoSeasAdjust (var polyParas: polyParameters;
                  var denominator: vector;
                  var dataBOX, cycleBOX, seasonBOX: rBOX;
                  var dataVector, cycleVec, seasonVec: longVector;
                  var filterSpec: FilterRecord);

  procedure PeriodogramPlots (var dataBOX, residueBOX: rBOX;
                  var SCREENBOX, POSTBOX: IBOX;
                  var dataVector, residueVec: longVector);

implementation

  {_________________________________________________}

  procedure ButterSpec (var filterSpec: FilterRecord;
                  var poles, zeros: complexVector;
                  var gamma, delta: complexVector);

    var
      i, sgn: integer;
      denomenatorSum, numeratorSum, filterGain: real;

  begin
    denomenatorSum := 0.0;
    numeratorSum := 0.0;

    with filterSpec do
      begin {filterSpec}

        Writeln('    What is the order of the unidirectional filter?');
        GetBoundedInteger(0, 15, filterOrder);
        Writeln;
        numeratorOrder := filterOrder;
        denominatorOrder := filterOrder;

        repeat
          Writeln('    Specify the cut-off of the lowpass digital filter');
          Writeln('    as a number of degrees in the interval (0, 180)');
          ReadANumber(cutOff);
          if (cutOff <= 0) or (cutOff >= 180) then
            begin
              SysBeep(3);
              Write('    THE NUMBER MUST BE IN THE OPEN INTERVAL (0 , 180)! ');
            end;
          Writeln;
        until (cutOff > 0) and (cutOff < 180);
        cutOff := cutOff * pi / 180;

  {Calculate the Poles and Zeros and the Corresponding Coefficients}
        ButterworthDigitalPoles(filterOrder, cutOff, poles);
        ButterworthDigitalZeros(filterOrder, zeros);
        InverseRootsToCoeffs(filterOrder, zeros, delta);
        InverseRootsToCoeffs(filterOrder, poles, gamma);

  {Deflate the Coefficients}
        if filterType = lowPass then
          for i := 0 to filterOrder do
            denomenatorSum := denomenatorSum + gamma[i].re;
        if filterType = highPass then
          begin {HighPass}
            sgn := 1;
            for i := 0 to filterOrder do
              begin
                denomenatorSum := denomenatorSum + sgn * gamma[i].re;
                sgn := -sgn;
              end;
          end; {HighPass}
        numeratorSum := Power(2, filterOrder);
        filterGain := numeratorSum / denomenatorSum;
        for i := 0 to filterOrder do
          delta[i].re := delta[i].re / filterGain;
  {Deflate the Coefficients}

      end; {FilterRecord}

  end; {ButterSpec}

  {________________________________________________}

  procedure HPFilterSpec (var filterSpec: FilterRecord;
                  var poles, zeros: complexVector;
                  var gamma, delta: complexVector;
                  var numerator, denominator: vector);

    var
      lambda, sigsqr: real;
      x1, x2: complex;
      mu: vector;
      q: integer;
  begin
    Writeln('    Specify a value for the smoothing parameter lambda');
    Writeln('    The following values are conventional:');
    Writeln;
    Writeln('        100     for annual data,');
    Writeln;
    Writeln('        1,600   for quarterly data,');
    Writeln;
    Writeln('        14,400  for monthly  data.');

   {Should we be more careful in constraining the value of lambda}
    ReadANumber(lambda);
    filterSpec.smoothingParameter := lambda;

    if lambda < 0.5 then
      begin
        denominator[0] := 6 * lambda + 1;
        denominator[1] := -4 * lambda;
        denominator[2] := lambda;
      end
    else
      begin
        denominator[0] := 6 + 1 / lambda;
        denominator[1] := -4;
        denominator[2] := 1;
      end;

    q := 2;
    Minit(mu, sigsqr, denominator, q);
    delta[0].re := mu[0] + mu[1] + mu[2];
    gamma[0].re := mu[0];
    gamma[1].re := mu[1];
    gamma[2].re := mu[2];

    QuadraticRoots(mu[0], mu[1], mu[2], x1, x2);

  {filterType := lowPass;}
    filterSpec.filterOrder := 2;
    filterSpec.numeratorOrder := 0;
    filterSpec.denominatorOrder := 2;
    poles[1].re := x1.re;
    poles[1].im := x1.im;
    poles[2].re := x2.re;
    poles[2].im := x2.im;
    zeros[1].re := 1;
    zeros[1].im := 0;

  end; {HPFilterSpec;}

{________________________________________________}

  procedure SeasonalFilterSpec (var filterSpec: filterRecord;
                  var poles, zeros: complexVector;
                  var gamma, delta: complexVector;
                  var numerator, denominator: vector);

{gamma delta are not needed, but the could serve a workspace}
{phi, chi are Global are these numerator, denominator?}

    var
      dataChar: char;
      q, s, j: integer;
      lambda, rho, factor, sigsqr: real;
      polyVec: vector;

  begin
{DetectATrend(dataBOX, dataVector);}

    Writeln('    This procedure applies a seasonal-adjustment filter appropriate to');
    Writeln('    the process y(t) = v(t) + {R(L)/S(L)}e(t), where v(t) and e(t) are');
    Writeln('    white noise,  where S(L) = 1 + L + .... + L^{s-1} is  the seasonal');
    Writeln('    summation  operator, which has zeros at  the seasonal frequencies,');
    Writeln('    and where R(L) = 1 + pL + .... + p^{s-1}L^{s-1}, with p in (0, 1),');
    Writeln('    limits the effect of the zeros to those vicinities.');
    Writeln;
    Writeln(' ' : 65, '<RETURN>');
    Readln;

    if (thereIsData = false) or (not (dataBOX.dataFrequency in [Monthly, Quarterly])) then
      begin {Define the seasonal frequency}
        repeat
{Writeln;}
          Writeln('    DEFINE THE SEASONAL FREQUENCY');
          Writeln;
          Writeln('    (q)   Quarterly');
          Writeln;
          Writeln('    (m)   Monthly');
          Readln(dataChar);
          dataChar := DownCase(dataChar);
        until dataChar in ['q', 'm'];

        if dataChar = 'q' then
          filterSpec.Seasonality := Quarterly;
        if dataChar = 'm' then
          filterSpec.Seasonality := Monthly;

      end; {Define the seasonal frequency}

    if thereIsData then
      begin {there is data}
        if (dataBOX.dataFrequency = Quarterly) then
          filterSpec.Seasonality := Quarterly;
        if (dataBOX.dataFrequency = Monthly) then
          filterSpec.Seasonality := Monthly;
{Or else there will be a disparity beween dataFrequency and Seasonality }
{that we shall detect later when we try to apply the filter}
      end; {there is data}

    if (filterSpec.Seasonality = Quarterly) then
      begin {Quarterly spec}
        filterSpec.filterOrder := 3;
        s := 4;
      end; {Quarterly spec}

    if (filterSpec.Seasonality = Monthly) then
      begin {Monthly spec}
        filterSpec.filterOrder := 11;
        s := 12;
      end; {Monthly spec}

    Writeln;
    Writeln('    Specify lambda in (0, 1), which governs the widths of the');
    Writeln('     notches. As lambda --> 0.0 the notches becomes narrower.');
    Writeln('    (lambda = 0.125  works well.)');
    Readln(lambda);
    Writeln('    Specify rho in (0, 1), which is the modulus of the poles.');
    Writeln('     As rho --> 1.0 the notches becomes narrower. ');
    Writeln('    (rho = 0.90  works well.)');
    Readln(rho);

{Calculate the Poles and Zeros and the Corresponding Coefficients}
    SeasonalZeros(s, zeros); {in FILTERS}
    SeasonalPoles(s, rho, poles); {in FILTERS: this is to be replaced by proper calculations}

{++++++++++++++++++++++++++++++++++++++++++++++}
{for j := 1 to s - 1 do}
{Writeln('    zero[', j : 1, '].re = ', zeros[j].re : 4 : 6, ' zero[', j : 1, ']. im = ', zeros[j].im : 4 : 6);}
{Readln;}
{for j := 1 to s - 1 do}
{Writeln('    pole[', j : 1, '].re = ', poles[j].re : 4 : 6, ' pole[', j : 1, '].im = ', poles[j].im : 4 : 6);}
{Readln;}
{++++++++++++++++++++++++++++++++++++++++++++++}

    CombinedCoefficients(lambda, rho, factor, s, numerator, denominator); {in FILTERS}
    filterSpec.filterFactor := factor;

{phi, chi <==> numerator denominator are global vectors}

{denomiator contsins the coefficients associated with  positve powera of z}
{in the Laurent poynomial lambda*R(z)R(1/z) + S(z)S(1/z)}

{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
    q := filterSpec.filterOrder;
    Minit(polyVec, sigsqr, denominator, q);
    for j := 0 to q do
      begin
        gamma[j].re := polyVec[q - j];
        gamma[j].im := 0.0;
      end;
    ComplexRoots(q, gamma, poles);
{we need polyVec and gamma: ComplexVector workspace}
{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++}

{This is where we attempt to calculate the genuine filter poles}

  end; {SeasonalFilterSpec}

{________________________________________________}

  procedure PoleZeroPlot (var filterSpec: FilterRecord;
                  var poles, zeros: complexVector);

  {This procedure organises the plotting of the pole-zero diagram of the filter}

    var
      postFileName, legend: string;
      F: text;
      p, q: integer;

  begin

    if noPageParameters then
      begin
        SysBeep(3);
        PageSetup(SCREENBOX, POSTBOX, PSPict, TeXFormat); {in UTILS}
      end;

      {SCREENBOX.XC := (graphWindow.right - graphWindow.left) div 2;}
      {SCREENBOX.YC := (graphWindow.bottom - graphWindow.top) div 2;}

    if filterSpec.filterfamily = Butterworth then
      begin
        legend := ButterPoles(filterSpec);
        p := filterSpec.filterOrder;
        q := 1;
      end;
    if filterSpec.filterfamily = HodrickPrescott then
      begin
        legend := HodPoleZero(filterSpec);
        p := filterSpec.filterOrder;
        q := 1;
      end;
    if filterSpec.filterfamily = DeSeasoning then
      begin
        p := filterSpec.filterOrder;
        q := filterSpec.filterOrder;
        legend := SeasonPoleZero(filterSpec);
      end;

    GetGraphics(graphicsOn, SCREENBOX, currentMode);

    ScreenAargand(SCREENBOX, p, q, poles, zeros);
    ScreenCaption(legend);

    CallForReturn;
    Readln;
    CloseGraph;
    RestoreCrtMode;
    ClrScr;

    if PSPict then
      begin {PSPict}
        Write('    Name the PostScript File = ');
        Readln(postFileName);
        if postFileName <> '' then
          begin {Postfile}
            Assign(F, postFileName);
            Rewrite(F);
            PostAargandHeader(F, TeXFormat);
            PostAargand(F, filterSpec.filterOrder, 1, poles, zeros);
            PostAargandFooter(F, filterSpec, TeXFormat);
            Close(F);
          end; {if  postFileName <> ''}
      end; {PSPict}

  end; {PoleZeroPlot}

  {________________________________________________}

  procedure OldBoxCarPlot (var realBOX: RBox;
                  var SCREENBOX: Ibox;
                  var filterSpec: FilterRecord);
  {No need for filterSpec}
    var
      BOXWDT, BOXHGT, VT, HR: integer;
      pi, omega1, omega2, yRange: real;

  begin
    pi := 4 * Arctan(1.0);
    omega1 := FilterSpec.lowerCut * pi / 180;
    omega2 := FilterSpec.upperCut * pi / 180;

    BOXWDT := SCREENBOX.X2 - SCREENBOX.X1;
    BOXHGT := SCREENBOX.Y2 - SCREENBOX.Y1;
    yRange := realBOX.yUB - realBOX.yLB;

    VT := SCREENBOX.Y2;
    HR := SCREENBOX.X1 + Round(BOXWDT * omega1 / pi);
    MoveTo(HR, VT);
    VT := SCREENBOX.Y2 - Round(1 * (BOXHGT / yRange));
    LineTo(HR, VT);
    HR := SCREENBOX.X1 + Round(BOXWDT * omega2 / pi);
    LineTo(HR, VT);
    VT := SCREENBOX.Y2;
    LineTo(HR, VT);
  end; {OldBoxCarPlot}

  {________________________________________________}

  procedure PlotWienerFilterGain (var SCREENBOX: IBOX;
                  var filterSpec: FilterRecord);

  {This procedure Organises the Plotting of the Gain of the Filters}

    var
      filterVector: longVector;
      filterBOX: rBOX;
      postFileName, legend: string;
      F: text;

  begin
    ClrScr;
    if noPageParameters then
      begin
        SysBeep(3);
        PageSetUp(SCREENBOX, POSTBOX, PSPict, TeXFormat);
      end;

    GetGraphics(graphicsOn, SCREENBOX, currentMode);

    GetFilterGainVector(filterSpec, filterBOX, SCREENBOX, firVec, filterVector); {in FILTERS}

    SCREENBOX.xAxis := false;
    ScreenFrameLite(filterBOX, SCREENBOX);
    ZeroPiAxis(SCREENBOX);
    ScreenPlot(filterBOX, SCREENBOX, filterVector);

    if (filterSpec.filterFamily = Butterworth) then
      legend := ButterworthGain(filterSpec);

    if (filterSpec.filterFamily = HodrickPrescott) then
      legend := HPFilterGain(filterSpec);

    if (filterSpec.filterFamily = DeSeasoning) then
      legend := SeasonalFilterGain(filterSpec);

    ScreenCaption(legend);

    CallForReturn;
    Readln;
    CloseGraph;
    RestoreCrtMode;
    ClrScr;

    if PSPict then
      begin {PSPict}
        Write('    Name the PostScript File = ');
        Readln(postFileName);
        if postFileName <> '' then
          begin {Postfile}
            Assign(F, postFileName);
            Rewrite(F);
            PostHeader(F, POSTBOX, TeXFormat);
            POSTBOX.xAxis := false;
{PostFrame(F, filterBOX, POSTBOX);}
            PostFrameLite(F, filterBOX, POSTBOX, left);
            PostZeroPiAxis(F, POSTBOX);
            PostPlot(F, filterBOX, POSTBOX, filterVector);
            PostGainFooter(F, POSTBOX, filterSpec, TeXFormat);
            Close(F);
          end;
      end; {PSPict}

    SCREENBOX.xAxis := true; {Restore the flag}
    POSTBOX.xAxis := true;
  end; {PlotWienerFilterGain}

  {________________________________________________}

  procedure ApplyHPFilter (var dataBOX, residueBOX, cycleBOX: rBOX;
                  var dataVector, trendVector, residueVec, cycleVec: longVector;
                  var polyParas: polyParameters;
                  var filterSpec: FilterRecord);

    var
      lambda: real;
      legend: string;

  begin
    if thereIsData = false then
      begin
        SysBeep(3);
        ReadTheData(dataBOX, dataVector);
      end;

    if noPageParameters then
      begin
        SysBeep(3);
        PageSetUp(SCREENBOX, POSTBOX, PSPict, TeXFormat);
      end;

    lambda := filterSpec.smoothingParameter;

    MakeHPTrend(dataBOX, trendBOX, residueBOX, lambda, dataVector, trendVector, ResidueVec);

    legend := HPtrend(lambda, dataBOX, dataName);
    ScreenOverplot(dataBOX, SCREENBOX, dataVector, trendVector, legend);

    if (PSPict = true) then
      WritePostOverFile(dataBOX, POSTBOX, dataVector, trendVector, legend);

    legend := HPResiduals(lambda, dataBOX, dataName);
    ScreenMakePlot(residueBOX, SCREENBOX, residueVec, legend);

    if (PSPict = true) then
      WritePostFile(residueBOX, POSTBOX, residueVec, legend);

    weHaveATrend := true;
    weHaveTrendResiduals := true;

    if residualsInData then
      begin {restore data to dataVector}
        dataBOX := cycleBOX;
        dataVector := cycleVec;
        cycleBOX := trendBOX;
        cycleVec := trendVector;
        MakeDataBox(cycleBOX, cycleVec);
        residualsInData := false;
        weHaveABusinessCycle := true;
        weHaveATrend := false;
        TrendCycleComponent(polyParas, dataBOX, trendBOX, cycleBOX, dataVector, trendVector, cycleVec, SCREENBOX, POSTBOX);
      end;

  end; {ApplyHPFilter}

  {________________________________________________}

  procedure ApplyButterFilter (var dataBOX, residueBOX, cycleBOX: rBOX;
                  var dataVector, trendVector, residueVec, cycleVec: longVector;
                  var polyParas: polyParameters;
                  var filterSpec: FilterRecord);

    var
      legend: string;

  begin
    if thereIsData = false then
      begin
        SysBeep(3);
        ReadTheData(dataBOX, dataVector);
      end;

    if noPageParameters then
      begin
        SysBeep(3);
        PageSetUp(SCREENBOX, POSTBOX, PSPict, TeXFormat);
      end;

    MakeButterTrend(dataBOX, trendBOX, residueBOX, filterSpec, dataVector, trendVector, residueVec);

    legend := ButterworthTrend(dataBOX, dataName, filterSpec);
    ScreenOverplot(dataBOX, SCREENBOX, dataVector, trendVector, legend);

    if (PSPict = true) then
      WritePostOverFile(dataBOX, POSTBOX, dataVector, trendVector, legend);

    legend := ButterworthResiduals(dataBOX, dataName, filterSpec);
    ScreenMakePlot(residueBOX, SCREENBOX, residueVec, legend);

    if (PSPict = true) then
      WritePostFile(residueBOX, POSTBOX, residueVec, legend);

    weHaveATrend := true;
    weHaveTrendResiduals := true;

    if residualsInData then
      begin {restore data to dataVector}
        dataBOX := cycleBOX;
        dataVector := cycleVec;
        cycleBOX := trendBOX;
        cycleVec := trendVector;
        MakeDataBox(cycleBOX, cycleVec);
        residualsInData := false;
        weHaveABusinessCycle := true;
        weHaveATrend := false;
        TrendCycleComponent(polyParas, dataBOX, trendBOX, cycleBOX, dataVector, trendVector, cycleVec, SCREENBOX, POSTBOX);
      end;

  end; {ApplyButterFilter}

  {________________________________________________}

  procedure MakeHPTrend (var dataBOX, trendBOX, residueBOX: rBOX;
                  var lambda: real;
                  var dataVector, trendVector, residueVec: longVector);

    var
      minResidual, maxResidual, trendMean, residueMean: real;
      i: integer;
      x, y: longVector;

  begin
    for i := 0 to dataBOX.NOPUNCT - 1 do
      begin
        x[i] := dataVector[i];
        y[i] := dataVector[i];
      end;
    HPFilter(dataBOX.NOPUNCT, lambda, x, y); {x is Trend, y is Residual}

  {MakeDataBox(residueBOX, residueVec); MakeDataBox(trendBOX, trendVec);}
    minResidual := y[0];
    maxResidual := y[0];
    trendMean := 0.0;
    residueMean := 0.0;
    for i := 0 to dataBOX.NOPUNCT - 1 do
      begin
        trendVector[i] := x[i];
        residueVec[i] := y[i];
        trendMean := trendMean + trendVector[i];
        residueMean := residueMean + residueVec[i];
        maxResidual := Rmax(maxResidual, y[i]);
        minResidual := Rmin(minResidual, y[i]);
      end;
    residueMean := residueMean / dataBOX.NOPUNCT;
    trendMean := trendMean / dataBOX.NOPUNCT;

    residueBOX := dataBOX;
    trendBOX := dataBOX;
    trendBOX.yMean := trendMean;
    residueBOX.yMean := residueMean;
    residueBOX.yMin := minResidual;
    residueBOX.yMax := maxResidual;

    weHaveATrend := true;
    weHaveTrendResiduals := true;

  end; {MakeHPTrend}

  {_________________________________________________}

  procedure MakeButterTrend (var dataBOX, trendBOX, residueBOX: rBOX;
                  var filterSpec: FilterRecord;
                  var dataVector, trendVector, residueVec: longVector);

    var
      minResidual, maxResidual, trendMean, residueMean: real;
      i: integer;
      x, y: longVector;

  begin
    for i := 0 to dataBOX.NOPUNCT - 1 do
      begin
        x[i] := dataVector[i];
        y[i] := dataVector[i];
      end;

    ButterworthFilter(filterSpec.filterOrder, dataBOX.NOPUNCT, filterSpec.cutOff, x, y);

  {MakeDataBox(residueBOX, residueVec); MakeDataBox(trendBOX, trendVec);}
    minResidual := y[0];
    maxResidual := y[0];
    trendMean := 0.0;
    residueMean := 0.0;
    for i := 0 to dataBOX.NOPUNCT - 1 do
      begin
        trendVector[i] := x[i];
        residueVec[i] := y[i];
        trendMean := trendMean + trendVector[i];
        residueMean := residueMean + residueVec[i];
        maxResidual := Rmax(maxResidual, y[i]);
        minResidual := Rmin(minResidual, y[i]);
      end;
    residueMean := residueMean / dataBOX.NOPUNCT;
    trendMean := trendMean / dataBOX.NOPUNCT;

    residueBOX := dataBOX;
    trendBOX := dataBOX;
    trendBOX.yMean := trendMean;
    residueBOX.yMean := residueMean;
    residueBOX.yMin := minResidual;
    residueBOX.yMax := maxResidual;

    weHaveATrend := true;
    weHaveTrendResiduals := true;

  end; {MakeButterTrend}

{________________________________________________}

  procedure DoSeasAdjust (var polyParas: polyParameters;
                  var denominator: vector;
                  var dataBOX, cycleBOX, seasonBOX: rBOX;
                  var dataVector, cycleVec, seasonVec: longVector;
                  var filterSpec: FilterRecord);

{we should put cycleBOX in the headline}
{residualVec can be used in place of adjustedVec}

{This is analogous to ApplyButterFilter and ApplyHPFilter. However, the latter invoke}
{the intermediate routines of MakeButterTrend  and MakeHPTrend which, in turn, invoke}
{ButterworthFilter and HPFilter. The present routine invokes SeasonalFilter directly}

{adjustedVec is local, and it may, at choice get written into dataVector. We could use}
{residueVec in place of the local adjustedVec.}

    var
      adjustBOX: rBOX;
      adjustedVec: longVector;
      s, t, Tcap, requiredOrder: integer;
      factor: real;
      replace: boolean;
      legend: string;

{numerator, denominator <==> phi, chi}

  begin

    if thereIsData = false then
      begin
        SysBeep(3);
        ReadTheData(dataBOX, dataVector);
      end;

    if noPageParameters then
      begin
        SysBeep(3);
        PageSetUp(SCREENBOX, POSTBOX, PSPict, TeXFormat);
      end;

    if (filterSpec.seasonality <> dataBOX.dataFrequency) then
      begin {Disparity}
        ClrScr;
        Writeln;
        Writeln('    THE FILTER DOES NOT MATCH THE DATA');
        Writeln;
        Sysbeep(3);
        Writeln('    There is a disparity between the data frequency and the frequency of');
        Writeln('    the intended  seasonal  adjustment. Therefore, the  filter cannot be');
        Writeln('    applied  to this data. Either respecify the filter or  get new data.');
        Writeln;
        Writeln(' ' : 60, '<RETURN>');
        Readln;
      end {Disparity}
    else
      begin {Process the data}
        Tcap := dataBOX.NOPUNCT;
        adjustBOX := dataBOX;

        s := filterSpec.seasonality;
        seasonVec := dataVector; {Vector of seasonal fluctuations}
        adjustedVec := dataVector; {Seasonally adjusted vector}

        requiredOrder := 0;
        if (not residualsInData) then
          begin {take differnces}
            ClrScr;
            Writeln;
            Writeln('    REDUCE THE DATA TO STATIONARITY');
            Writeln;
            Writeln('    Before applying the seasonal-adjustment filter, you are likely');
            Writeln('    to require to  reduce the data to stationarity by differencing');
            Writeln('    it one or twice.');
            Writeln;
            Writeln('    Specify the number of differencing operations.');

            adjustBOX := dataBOX;
            GetBoundedInteger(0, 2, requiredOrder);
            if (requiredOrder = 1) then
              DiffOfVector(adjustBOX, adjustedVec); {in DATASEG}
            if (requiredOrder = 2) then
              SecondDiffOfVector(adjustBOX, adjustedVec); {in DATASEG}
              {adjustBOX.NOPUNCT is reduced by the order of differencing}
          end; {take Differences}

        Tcap := adjustBOX.NOPUNCT;
        factor := filterSpec.filterFactor;
        SeasonalFilter(s, Tcap, factor, denominator, seasonVec, adjustedVec); {in FILTERS}

        if (requiredOrder > 0) then
          DataCumulation(dataBOX, adjustBOX, requiredOrder, dataVector, adjustedVec); {in FOURIER}

        for t := 0 to dataBOX.NOPUNCT - 1 do
          seasonVec[t] := dataVector[t] - adjustedVec[t];

        seasonBOX := dataBOX;
        MakeDataBox(seasonBOX, seasonVec);
        MakeDataBox(adjustBOX, adjustedVec);
        weHaveSeasonalComponent := true;
      end; {Process the data}

     {Hideall;}
     {ShowDrawing;}

    legend := DeSeasonalisedOverlay(dataBOX, dataName);
    ScreenOverplot(dataBOX, SCREENBOX, dataVector, adjustedVec, legend);

    if (PSPict = true) then
      WritePostOverFile(dataBOX, POSTBOX, dataVector, adjustedVec, legend);

{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}

    if residualsInData then
      begin  {restore data to dataVector}
        dataBOX := cycleBOX;
        dataVector := cycleVec;
        residualsInData := false;
        TrendPlusSeason(polyParas, dataBOX, adjustBOX, dataVector, adjustedVec, SCREENBOX, POSTBOX);
      end;  {if this happens, then we need to have adjustedBOX <- dataBOx}

{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}

    ClrScr;
    Writeln;
    Writeln('     SAVE THE SEASONALLY ADJUSTED DATA');
    Writeln;
    YesOrNo('    Do you wish to replace the data series by the adjusted series', replace);

    if replace then
      begin {replace}
        dataVector := adjustedVec;
        MakeDataBox(dataBOX, dataVector);
        {we need a dataBOX for the deseasonalised detended version i.e. dataBOX = adjustBOX}
        weHaveDeasonalisedData := true; {GLOBAL Flag}

        legend := SeasonallyAdjustedData(dataBOX, dataName);
        ScreenMakePlot(dataBOX, SCREENBOX, dataVector, legend);

        if (PSPict = true) then
          WritePostFile(dataBOX, POSTBOX, dataVector, legend);

      end; {replace}

  end; {DoSeasAdjust}

{_________________________________________________}

  procedure PeriodogramPlots (var dataBOX, residueBOX: rBOX;
                  var SCREENBOX, POSTBOX: IBOX;
                  var dataVector, residueVec: longVector);

{This has become redundant}
    var
      theMenuItem: integer;
      legend: string;

  begin
    repeat
      ClrScr;
      Writeln;
      Writeln('         PERIODOGRAM ANALYSIS');
      Writeln;
      Writeln;
      Writeln('         1. The Periodogram of the Data');
      Writeln;
      Writeln('         2.  The Periodogram of the Residuals');
      Writeln;
      Writeln('         3.  Return to the Main Menu');

      Writeln('                      Select a number and press RETURN');

      GetBoundedInteger(1, 3, theMenuItem);
      case theMenuItem of
        1: 
          begin
            legend := 'The periodogram of ';
            legend := Concat(legend, dataName, '.');
            PlotThePeriodogam(dataBOX, SCREENBOX, POSTBOX, dataVector, legend);
          end;
        2: 
          begin
            legend := 'The periodogram of the residuals obtained by fitting a trend to ';
            legend := Concat(legend, dataName, '.');
            PlotThePeriodogam(residueBOX, SCREENBOX, POSTBOX, residueVec, legend);
          end;
        3: 
      end;{case}
    until (theMenuItem = 3);

  end; {Periodogram Analysis}

  {________________________________________________}

end. {WIENER: Unit 18 of NEWIDEOLOG.PAS}

